@@ -15,7 +15,7 @@ from dagster._cli.workspace.cli_target import (
     get_working_directory_from_kwargs,
     python_origin_target_argument,
 )
-from dagster._core.definitions.metadata import MetadataEntry
+from dagster._core.definitions.metadata import MetadataValue
 from dagster._core.errors import DagsterExecutionInterruptedError
 from dagster._core.events import DagsterEvent, DagsterEventType, EngineEventData
 from dagster._core.execution.api import create_execution_plan, execute_plan_iterator
@@ -33,7 +33,7 @@ from dagster._core.utils import coerce_valid_log_level
 from dagster._grpc import DagsterGrpcClient, DagsterGrpcServer
 from dagster._grpc.impl import core_execute_run
 from dagster._grpc.types import ExecuteRunArgs, ExecuteStepArgs, ResumeRunArgs
-from dagster._serdes import deserialize_as, serialize_dagster_namedtuple
+from dagster._serdes import deserialize_value, serialize_value
 from dagster._utils.error import serializable_error_info_from_exc_info
 from dagster._utils.hosted_user_process import recon_pipeline_from_origin
 from dagster._utils.interrupts import capture_interrupts
@@ -42,8 +42,7 @@ from dagster._utils.log import configure_loggers
 
 @click.group(name="api", hidden=True)
 def api_cli():
-    """
-    [INTERNAL] These commands are intended to support internal use cases. Users should generally
+    """[INTERNAL] These commands are intended to support internal use cases. Users should generally
     not invoke these commands interactively.
     """
 
@@ -58,7 +57,7 @@ def api_cli():
 @click.argument("input_json", type=click.STRING)
 def execute_run_command(input_json):
     with capture_interrupts():
-        args = deserialize_as(input_json, ExecuteRunArgs)
+        args = deserialize_value(input_json, ExecuteRunArgs)
 
         with (
             DagsterInstance.from_ref(args.instance_ref)
@@ -68,7 +67,7 @@ def execute_run_command(input_json):
             buffer = []
 
             def send_to_buffer(event):
-                buffer.append(serialize_dagster_namedtuple(event))
+                buffer.append(serialize_value(event))
 
             return_code = _execute_run_command_body(
                 args.pipeline_run_id,
@@ -99,13 +98,13 @@ def _execute_run_command_body(
 
     pipeline_run: DagsterRun = check.not_none(
         instance.get_run_by_id(pipeline_run_id),
-        "Pipeline run with id '{}' not found for run execution.".format(pipeline_run_id),
+        f"Pipeline run with id '{pipeline_run_id}' not found for run execution.",
     )
 
     check.inst(
         pipeline_run.pipeline_code_origin,
         PipelinePythonOrigin,
-        "Pipeline run with id '{}' does not include an origin.".format(pipeline_run_id),
+        f"Pipeline run with id '{pipeline_run_id}' does not include an origin.",
     )
 
     recon_pipeline = recon_pipeline_from_origin(
@@ -114,7 +113,7 @@ def _execute_run_command_body(
 
     pid = os.getpid()
     instance.report_engine_event(
-        "Started process for run (pid: {pid}).".format(pid=pid),
+        f"Started process for run (pid: {pid}).",
         pipeline_run,
         EngineEventData.in_process(pid),
     )
@@ -148,7 +147,7 @@ def _execute_run_command_body(
                     )
 
         instance.report_engine_event(
-            "Process for run exited (pid: {pid}).".format(pid=pid),
+            f"Process for run exited (pid: {pid}).",
             pipeline_run,
         )
 
@@ -165,7 +164,7 @@ def _execute_run_command_body(
 @click.argument("input_json", type=click.STRING)
 def resume_run_command(input_json):
     with capture_interrupts():
-        args = deserialize_as(input_json, ResumeRunArgs)
+        args = deserialize_value(input_json, ResumeRunArgs)
 
         with (
             DagsterInstance.from_ref(args.instance_ref)
@@ -175,7 +174,7 @@ def resume_run_command(input_json):
             buffer = []
 
             def send_to_buffer(event):
-                buffer.append(serialize_dagster_namedtuple(event))
+                buffer.append(serialize_value(event))
 
             return_code = _resume_run_command_body(
                 args.pipeline_run_id,
@@ -205,12 +204,12 @@ def _resume_run_command_body(
         cancellation_thread, cancellation_thread_shutdown_event = None, None
     pipeline_run = check.not_none(
         instance.get_run_by_id(pipeline_run_id),  # type: ignore
-        "Pipeline run with id '{}' not found for run execution.".format(pipeline_run_id),
+        f"Pipeline run with id '{pipeline_run_id}' not found for run execution.",
     )
     check.inst(
         pipeline_run.pipeline_code_origin,
         PipelinePythonOrigin,
-        "Pipeline run with id '{}' does not include an origin.".format(pipeline_run_id),
+        f"Pipeline run with id '{pipeline_run_id}' does not include an origin.",
     )
 
     recon_pipeline = recon_pipeline_from_origin(
@@ -219,7 +218,7 @@ def _resume_run_command_body(
 
     pid = os.getpid()
     instance.report_engine_event(
-        "Started process for resuming pipeline (pid: {pid}).".format(pid=pid),
+        f"Started process for resuming pipeline (pid: {pid}).",
         pipeline_run,
         EngineEventData.in_process(pid),
     )
@@ -254,7 +253,7 @@ def _resume_run_command_body(
                         pipeline_run,
                     )
         instance.report_engine_event(
-            "Process for pipeline exited (pid: {pid}).".format(pid=pid),
+            f"Process for pipeline exited (pid: {pid}).",
             pipeline_run,
         )
 
@@ -343,7 +342,7 @@ def execute_step_command(input_json, compressed_input_json):
         if compressed_input_json:
             input_json = zlib.decompress(base64.b64decode(compressed_input_json.encode())).decode()
 
-        args = deserialize_as(input_json, ExecuteStepArgs)
+        args = deserialize_value(input_json, ExecuteStepArgs)
 
         with (
             DagsterInstance.from_ref(args.instance_ref)
@@ -359,7 +358,7 @@ def execute_step_command(input_json, compressed_input_json):
                 instance,
                 pipeline_run,
             ):
-                buff.append(serialize_dagster_namedtuple(event))
+                buff.append(serialize_value(event))
 
             for line in buff:
                 click.echo(line)
@@ -377,12 +376,12 @@ def _execute_step_command_body(
         check.inst(
             pipeline_run,
             DagsterRun,
-            "Pipeline run with id '{}' not found for step execution".format(args.pipeline_run_id),
+            f"Pipeline run with id '{args.pipeline_run_id}' not found for step execution",
         )
         check.inst(
             pipeline_run.pipeline_code_origin,
             PipelinePythonOrigin,
-            "Pipeline run with id '{}' does not include an origin.".format(args.pipeline_run_id),
+            f"Pipeline run with id '{args.pipeline_run_id}' does not include an origin.",
         )
 
         location_name = (
@@ -400,11 +399,7 @@ def _execute_step_command_body(
             pipeline_run.pipeline_name,
             message="Step worker started"
             + (f' for "{single_step_key}".' if single_step_key else "."),
-            metadata_entries=(
-                [
-                    MetadataEntry("pid", value=str(os.getpid())),
-                ]
-            ),
+            metadata={"pid": MetadataValue.text(str(os.getpid()))},
             step_key=single_step_key,
         )
 
@@ -721,7 +716,7 @@ def grpc_command(
             if container_context is not None
             else None,
             inject_env_vars_from_instance=inject_env_vars_from_instance,
-            instance_ref=deserialize_as(instance_ref, InstanceRef) if instance_ref else None,
+            instance_ref=deserialize_value(instance_ref, InstanceRef) if instance_ref else None,
             location_name=location_name,
         )
 
