@@ -15,9 +15,7 @@ from dagster._generate.download import AVAILABLE_EXAMPLES
 
 @click.group(name="project")
 def project_cli():
-    """
-    Commands for bootstrapping new Dagster projects and code locations.
-    """
+    """Commands for bootstrapping new Dagster projects and code locations."""
 
 
 scaffold_repository_command_help_text = (
