@@ -8,9 +8,7 @@ from dagster._core.instance import DagsterInstance
 
 @click.group(name="run")
 def run_cli():
-    """
-    Commands for working with Dagster job runs.
-    """
+    """Commands for working with Dagster job runs."""
 
 
 @run_cli.command(name="list", help="List the runs in the current Dagster instance.")
@@ -18,8 +16,8 @@ def run_cli():
 def run_list_command(limit):
     with DagsterInstance.get() as instance:
         for run in instance.get_runs(limit=limit):
-            click.echo("Run: {}".format(run.run_id))
-            click.echo("     Job: {}".format(run.pipeline_name))
+            click.echo(f"Run: {run.run_id}")
+            click.echo(f"     Job: {run.pipeline_name}")
 
 
 @run_cli.command(
