@@ -1,7 +1,7 @@
 import os
 import tempfile
 from contextlib import contextmanager
-from typing import Optional
+from typing import Callable, Iterator, Optional
 
 import click
 
@@ -11,8 +11,10 @@ from dagster._core.instance.config import is_dagster_home_set
 
 @contextmanager
 def get_instance_for_service(
-    service_name, instance_ref: Optional[InstanceRef] = None, logger_fn=click.echo
-):
+    service_name: str,
+    instance_ref: Optional[InstanceRef] = None,
+    logger_fn: Callable[[str], None] = click.echo,
+) -> Iterator[DagsterInstance]:
     if instance_ref:
         with DagsterInstance.from_ref(instance_ref) as instance:
             yield instance
@@ -21,7 +23,7 @@ def get_instance_for_service(
             yield instance
     else:
         # make the temp dir in the cwd since default temp dir roots
-        # have issues with FS notif based event log watching
+        # have issues with FS notify-based event log watching
         with tempfile.TemporaryDirectory(dir=os.getcwd()) as tempdir:
             logger_fn(
                 f"Using temporary directory {tempdir} for storage. This will be removed when"
