@@ -3,12 +3,16 @@ import sys
 from contextlib import contextmanager
 from typing import (
     TYPE_CHECKING,
-    Generator,
+    Any,
+    Callable,
     Iterable,
+    Iterator,
     List,
     Mapping,
     Optional,
+    Sequence,
     Tuple,
+    TypeVar,
     Union,
     cast,
 )
@@ -16,14 +20,14 @@ from typing import (
 import click
 import tomli
 from click import UsageError
-from typing_extensions import TypeAlias
+from typing_extensions import Never, TypeAlias
 
 import dagster._check as check
 from dagster._core.code_pointer import CodePointer
 from dagster._core.definitions.reconstruct import repository_def_from_target_def
 from dagster._core.definitions.repository_definition import RepositoryDefinition
+from dagster._core.host_representation.code_location import CodeLocation
 from dagster._core.host_representation.external import ExternalRepository
-from dagster._core.host_representation.repository_location import RepositoryLocation
 from dagster._core.instance import DagsterInstance
 from dagster._core.origin import (
     DEFAULT_DAGSTER_ENTRY_POINT,
@@ -40,6 +44,7 @@ from dagster._core.workspace.load_target import (
     PyProjectFileTarget,
     PythonFileTarget,
     WorkspaceFileTarget,
+    WorkspaceLoadTarget,
 )
 from dagster._grpc.utils import get_loadable_targets
 from dagster._utils.hosted_user_process import recon_repository_from_origin
@@ -54,21 +59,24 @@ WORKSPACE_TARGET_WARNING = (
     " --grpc-socket."
 )
 
-ClickArgMapping: TypeAlias = Mapping[str, Union[str, Tuple[str]]]
+T_Callable = TypeVar("T_Callable", bound=Callable[..., Any])
+
+ClickArgValue: TypeAlias = Union[str, Tuple[str]]
+ClickArgMapping: TypeAlias = Mapping[str, ClickArgValue]
+ClickOption: TypeAlias = Callable[[T_Callable], T_Callable]
 
 
-def _cli_load_invariant(condition: object, msg=None) -> None:
-    msg = (
+def _raise_cli_usage_error(msg: Optional[str] = None) -> Never:
+    raise UsageError(
         msg
         or "Invalid set of CLI arguments for loading repository/pipeline. See --help for details."
     )
-    if not condition:
-        raise UsageError(msg)
 
 
 def _check_cli_arguments_none(kwargs: ClickArgMapping, *keys: str) -> None:
     for key in keys:
-        _cli_load_invariant(not kwargs.get(key))
+        if kwargs.get(key):
+            _raise_cli_usage_error()
 
 
 def are_all_keys_empty(kwargs: ClickArgMapping, keys: Iterable[str]) -> bool:
@@ -93,7 +101,7 @@ WORKSPACE_CLI_ARGS = (
 )
 
 
-def _has_pyproject_dagster_block(path):
+def _has_pyproject_dagster_block(path: str) -> bool:
     if not os.path.exists(path):
         return False
     with open(path, "rb") as f:
@@ -101,10 +109,10 @@ def _has_pyproject_dagster_block(path):
         if not isinstance(data, dict):
             return False
 
-        return data.get("tool", {}).get("dagster")
+        return "dagster" in data.get("tool", {})
 
 
-def get_workspace_load_target(kwargs: ClickArgMapping):
+def get_workspace_load_target(kwargs: ClickArgMapping) -> WorkspaceLoadTarget:
     check.mapping_param(kwargs, "kwargs")
     if are_all_keys_empty(kwargs, WORKSPACE_CLI_ARGS):
         if kwargs.get("empty_workspace"):
@@ -254,9 +262,7 @@ def get_workspace_load_target(kwargs: ClickArgMapping):
             location_name=None,
         )
     else:
-        _cli_load_invariant(False)
-        # necessary for pyright, does not understand _cli_load_invariant(False) never returns
-        assert False
+        _raise_cli_usage_error()
 
 
 def get_workspace_process_context_from_kwargs(
@@ -282,14 +288,14 @@ def get_workspace_from_kwargs(
     instance: DagsterInstance,
     version: str,
     kwargs: ClickArgMapping,
-) -> Generator[WorkspaceRequestContext, None, None]:
+) -> Iterator[WorkspaceRequestContext]:
     with get_workspace_process_context_from_kwargs(
         instance, version, read_only=False, kwargs=kwargs
     ) as workspace_process_context:
         yield workspace_process_context.create_request_context()
 
 
-def python_file_option(allow_multiple):
+def python_file_option(allow_multiple: bool) -> ClickOption:
     return click.option(
         "--python-file",
         "-f",
@@ -308,7 +314,7 @@ def python_file_option(allow_multiple):
     )
 
 
-def workspace_option():
+def workspace_option() -> ClickOption:
     return click.option(
         "--workspace",
         "-w",
@@ -318,7 +324,7 @@ def workspace_option():
     )
 
 
-def python_module_option(allow_multiple):
+def python_module_option(allow_multiple: bool) -> ClickOption:
     return click.option(
         "--module-name",
         "-m",
@@ -334,7 +340,7 @@ def python_module_option(allow_multiple):
     )
 
 
-def working_directory_option():
+def working_directory_option() -> ClickOption:
     return click.option(
         "--working-directory",
         "-d",
@@ -343,7 +349,7 @@ def working_directory_option():
     )
 
 
-def python_target_click_options(allow_multiple_python_targets):
+def python_target_click_options(allow_multiple_python_targets: bool) -> Sequence[ClickOption]:
     return [
         working_directory_option(),
         python_file_option(allow_multiple=allow_multiple_python_targets),
@@ -365,7 +371,7 @@ def python_target_click_options(allow_multiple_python_targets):
     ]
 
 
-def grpc_server_target_click_options():
+def grpc_server_target_click_options() -> Sequence[ClickOption]:
     return [
         click.option(
             "--grpc-port",
@@ -394,32 +400,28 @@ def grpc_server_target_click_options():
     ]
 
 
-def workspace_target_click_options():
-    return (
-        [
-            click.option("--empty-workspace", is_flag=True, help="Allow an empty workspace"),
-            workspace_option(),
-        ]
-        + python_target_click_options(allow_multiple_python_targets=True)
-        + grpc_server_target_click_options()
-    )
+def workspace_target_click_options() -> Sequence[ClickOption]:
+    return [
+        click.option("--empty-workspace", is_flag=True, help="Allow an empty workspace"),
+        workspace_option(),
+        *python_target_click_options(allow_multiple_python_targets=True),
+        *grpc_server_target_click_options(),
+    ]
 
 
-def python_job_target_click_options():
-    return (
-        python_target_click_options(allow_multiple_python_targets=False)
-        + [
-            click.option(
-                "--repository",
-                "-r",
-                help="Repository name, necessary if more than one repository is present.",
-            )
-        ]
-        + [job_option()]
-    )
+def python_job_target_click_options() -> Sequence[ClickOption]:
+    return [
+        *python_target_click_options(allow_multiple_python_targets=False),
+        click.option(
+            "--repository",
+            "-r",
+            help="Repository name, necessary if more than one repository is present.",
+        ),
+        job_option(),
+    ]
 
 
-def target_with_config_option(command_name):
+def target_with_config_option(command_name: str) -> ClickOption:
     return click.option(
         "-c",
         "--config",
@@ -442,8 +444,8 @@ def target_with_config_option(command_name):
     )
 
 
-def python_job_config_argument(command_name):
-    def wrap(f):
+def python_job_config_argument(command_name: str) -> Callable[[T_Callable], T_Callable]:
+    def wrap(f: T_Callable) -> T_Callable:
         return target_with_config_option(command_name)(f)
 
     return wrap
@@ -481,22 +483,20 @@ def python_origin_target_argument(f):
     return apply_click_params(f, *options)
 
 
-def repository_click_options():
+def repository_click_options() -> Sequence[ClickOption]:
     return [
         click.option(
             "--repository",
             "-r",
             help=(
-                "Repository within the workspace, necessary if more than one repository is present."
+                "Name of the repository, necessary if more than one repository is present in the"
+                " code location."
             ),
         ),
         click.option(
             "--location",
             "-l",
-            help=(
-                "RepositoryLocation within the workspace, necessary if more than one location is"
-                " present."
-            ),
+            help="Name of the code location, necessary if more than one location is present.",
         ),
     ]
 
@@ -507,13 +507,13 @@ def repository_target_argument(f):
     return apply_click_params(workspace_target_argument(f), *repository_click_options())
 
 
-def job_repository_target_argument(f):
+def job_repository_target_argument(f: T_Callable) -> T_Callable:
     from dagster._cli.job import apply_click_params
 
     return apply_click_params(job_workspace_target_argument(f), *repository_click_options())
 
 
-def job_option():
+def job_option() -> ClickOption:
     return click.option(
         "--job",
         "-j",
@@ -522,13 +522,13 @@ def job_option():
     )
 
 
-def job_target_argument(f):
+def job_target_argument(f: T_Callable) -> T_Callable:
     from dagster._cli.job import apply_click_params
 
     return apply_click_params(job_repository_target_argument(f), job_option())
 
 
-def get_job_python_origin_from_kwargs(kwargs):
+def get_job_python_origin_from_kwargs(kwargs: ClickArgMapping) -> PipelinePythonOrigin:
     repository_origin = get_repository_python_origin_from_kwargs(kwargs)
     provided_name = kwargs.get("job_name")
 
@@ -608,8 +608,8 @@ def get_working_directory_from_kwargs(kwargs: ClickArgMapping) -> Optional[str]:
     return check.opt_str_elem(kwargs, "working_directory") or os.getcwd()
 
 
-def get_repository_python_origin_from_kwargs(kwargs: Mapping[str, str]) -> RepositoryPythonOrigin:
-    provided_repo_name = cast(str, kwargs.get("repository"))
+def get_repository_python_origin_from_kwargs(kwargs: ClickArgMapping) -> RepositoryPythonOrigin:
+    provided_repo_name = check.opt_str_elem(kwargs, "repository")
 
     if not (kwargs.get("python_file") or kwargs.get("module_name") or kwargs.get("package_name")):
         raise click.UsageError("Must specify a python file or module name")
@@ -674,58 +674,61 @@ def get_repository_python_origin_from_kwargs(kwargs: Mapping[str, str]) -> Repos
 
 
 @contextmanager
-def get_repository_location_from_kwargs(instance, version, kwargs):
+def get_code_location_from_kwargs(
+    instance: DagsterInstance, version: str, kwargs: ClickArgMapping
+) -> Iterator[CodeLocation]:
     # Instance isn't strictly required to load a repository location, but is included
     # to satisfy the WorkspaceProcessContext / WorkspaceRequestContext requirements
     with get_workspace_from_kwargs(instance, version, kwargs) as workspace:
-        yield get_repository_location_from_workspace(workspace, kwargs.get("location"))
+        location_name = check.opt_str_elem(kwargs, "location")
+        yield get_code_location_from_workspace(workspace, location_name)
 
 
-def get_repository_location_from_workspace(
-    workspace: WorkspaceRequestContext, provided_location_name
-):
+def get_code_location_from_workspace(
+    workspace: WorkspaceRequestContext, provided_location_name: Optional[str]
+) -> CodeLocation:
     if provided_location_name is None:
-        if len(workspace.repository_location_names) == 1:
-            provided_location_name = workspace.repository_location_names[0]
-        elif len(workspace.repository_location_names) == 0:
+        if len(workspace.code_location_names) == 1:
+            provided_location_name = workspace.code_location_names[0]
+        elif len(workspace.code_location_names) == 0:
             raise click.UsageError("No locations found in workspace")
         elif provided_location_name is None:
             raise click.UsageError(
                 (
                     "Must provide --location as there are multiple locations "
                     "available. Options are: {}"
-                ).format(_sorted_quoted(workspace.repository_location_names))
+                ).format(_sorted_quoted(workspace.code_location_names))
             )
 
-    if provided_location_name not in workspace.repository_location_names:
+    if provided_location_name not in workspace.code_location_names:
         raise click.UsageError(
             (
                 'Location "{provided_location_name}" not found in workspace. '
                 "Found {found_names} instead."
             ).format(
                 provided_location_name=provided_location_name,
-                found_names=_sorted_quoted(workspace.repository_location_names),
+                found_names=_sorted_quoted(workspace.code_location_names),
             )
         )
 
-    if workspace.has_repository_location_error(provided_location_name):
+    if workspace.has_code_location_error(provided_location_name):
         raise click.UsageError(
             'Error loading location "{provided_location_name}": {error_str}'.format(
                 provided_location_name=provided_location_name,
-                error_str=str(workspace.get_repository_location_error(provided_location_name)),
+                error_str=str(workspace.get_code_location_error(provided_location_name)),
             )
         )
 
-    return workspace.get_repository_location(provided_location_name)
+    return workspace.get_code_location(provided_location_name)
 
 
-def get_external_repository_from_repo_location(
-    repo_location: RepositoryLocation, provided_repo_name: Optional[str]
+def get_external_repository_from_code_location(
+    code_location: CodeLocation, provided_repo_name: Optional[str]
 ) -> ExternalRepository:
-    check.inst_param(repo_location, "repo_location", RepositoryLocation)
+    check.inst_param(code_location, "code_location", CodeLocation)
     check.opt_str_param(provided_repo_name, "provided_repo_name")
 
-    repo_dict = repo_location.get_repositories()
+    repo_dict = code_location.get_repositories()
     check.invariant(repo_dict, "There should be at least one repo.")
 
     # no name provided and there is only one repo. Automatically return
@@ -737,31 +740,33 @@ def get_external_repository_from_repo_location(
             (
                 "Must provide --repository as there is more than one repository "
                 "in {location}. Options are: {repos}."
-            ).format(location=repo_location.name, repos=_sorted_quoted(repo_dict.keys()))
+            ).format(location=code_location.name, repos=_sorted_quoted(repo_dict.keys()))
         )
 
-    if not repo_location.has_repository(provided_repo_name):
+    if not code_location.has_repository(provided_repo_name):
         raise click.UsageError(
             (
                 'Repository "{provided_repo_name}" not found in location "{location_name}". '
                 "Found {found_names} instead."
             ).format(
                 provided_repo_name=provided_repo_name,
-                location_name=repo_location.name,
+                location_name=code_location.name,
                 found_names=_sorted_quoted(repo_dict.keys()),
             )
         )
 
-    return repo_location.get_repository(provided_repo_name)
+    return code_location.get_repository(provided_repo_name)
 
 
 @contextmanager
-def get_external_repository_from_kwargs(instance, version, kwargs):
+def get_external_repository_from_kwargs(
+    instance: DagsterInstance, version: str, kwargs: ClickArgMapping
+) -> Iterator[ExternalRepository]:
     # Instance isn't strictly required to load an ExternalRepository, but is included
     # to satisfy the WorkspaceProcessContext / WorkspaceRequestContext requirements
-    with get_repository_location_from_kwargs(instance, version, kwargs) as repo_location:
-        provided_repo_name = kwargs.get("repository")
-        yield get_external_repository_from_repo_location(repo_location, provided_repo_name)
+    with get_code_location_from_kwargs(instance, version, kwargs) as code_location:
+        provided_repo_name = check.opt_str_elem(kwargs, "repository")
+        yield get_external_repository_from_code_location(code_location, provided_repo_name)
 
 
 def get_external_job_from_external_repo(
@@ -794,13 +799,13 @@ def get_external_job_from_external_repo(
 
 
 @contextmanager
-def get_external_job_from_kwargs(instance, version, kwargs):
+def get_external_job_from_kwargs(instance: DagsterInstance, version: str, kwargs: ClickArgMapping):
     # Instance isn't strictly required to load an ExternalPipeline, but is included
     # to satisfy the WorkspaceProcessContext / WorkspaceRequestContext requirements
     with get_external_repository_from_kwargs(instance, version, kwargs) as external_repo:
-        provided_name = kwargs.get("job_name")
+        provided_name = check.opt_str_elem(kwargs, "job_name")
         yield get_external_job_from_external_repo(external_repo, provided_name)
 
 
-def _sorted_quoted(strings):
-    return "[" + ", ".join(["'{}'".format(s) for s in sorted(list(strings))]) + "]"
+def _sorted_quoted(strings: Iterable[str]) -> str:
+    return "[" + ", ".join([f"'{s}'" for s in sorted(list(strings))]) + "]"
