@@ -56,9 +56,7 @@ class ConfigTypeKind(PythonEnum):
 
 
 class ConfigType:
-    """
-    The class backing DagsterTypes as they are used processing configuration data.
-    """
+    """The class backing DagsterTypes as they are used processing configuration data."""
 
     def __init__(
         self,
@@ -91,8 +89,7 @@ class ConfigType:
         return _CONFIG_MAP[builtin_enum]
 
     def post_process(self, value):
-        """
-        Implement this in order to take a value provided by the user
+        """Implement this in order to take a value provided by the user
         and perform computation on it. This can be done to coerce data types,
         fetch things from the environment (e.g. environment variables), or
         to do custom validation. If the value is not valid, throw a
@@ -203,7 +200,7 @@ class Noneable(ConfigType):
 
         self.inner_type = cast(ConfigType, resolve_to_config_type(inner_type))
         super(Noneable, self).__init__(
-            key="Noneable.{inner_type}".format(inner_type=self.inner_type.key),
+            key=f"Noneable.{self.inner_type.key}",
             kind=ConfigTypeKind.NONEABLE,
             type_params=[self.inner_type],
         )
@@ -226,7 +223,7 @@ class Array(ConfigType):
 
         self.inner_type = cast(ConfigType, resolve_to_config_type(inner_type))
         super(Array, self).__init__(
-            key="Array.{inner_type}".format(inner_type=self.inner_type.key),
+            key=f"Array.{self.inner_type.key}",
             type_params=[self.inner_type],
             kind=ConfigTypeKind.ARRAY,
         )
@@ -234,7 +231,7 @@ class Array(ConfigType):
     @public
     @property
     def description(self):
-        return "List of {inner_type}".format(inner_type=self.key)
+        return f"List of {self.key}"
 
     def type_iterator(self) -> Iterator["ConfigType"]:
         yield from self.inner_type.type_iterator()
@@ -326,8 +323,7 @@ class Enum(ConfigType):
 
     @classmethod
     def from_python_enum(cls, enum, name=None):
-        """
-        Create a Dagster enum corresponding to an existing Python enum.
+        """Create a Dagster enum corresponding to an existing Python enum.
 
         Args:
             enum (enum.EnumMeta):
@@ -418,7 +414,7 @@ class ScalarUnion(ConfigType):
 
         # https://github.com/dagster-io/dagster/issues/2133
         key = check.opt_str_param(
-            _key, "_key", "ScalarUnion.{}-{}".format(self.scalar_type.key, self.non_scalar_type.key)
+            _key, "_key", f"ScalarUnion.{self.scalar_type.key}-{self.non_scalar_type.key}"
         )
 
         super(ScalarUnion, self).__init__(
@@ -461,5 +457,5 @@ ALL_CONFIG_BUILTINS = set(_CONFIG_MAP.values())
 
 def get_builtin_scalar_by_name(type_name: str):
     if type_name not in _CONFIG_MAP_BY_NAME:
-        check.failed("Scalar {} is not supported".format(type_name))
+        check.failed(f"Scalar {type_name} is not supported")
     return _CONFIG_MAP_BY_NAME[type_name]
