@@ -1,5 +1,5 @@
 # pylint disable is for bug: https://github.com/PyCQA/pylint/issues/3299
-from typing import (  # pylint: disable=unused-import
+from typing import (
     Any,
     Generator,
     Generic,
