@@ -1,9 +1,11 @@
 import inspect
+from enum import Enum
 from typing import (
     AbstractSet,
     Any,
     Dict,
     Generic,
+    Iterable,
     Mapping,
     NamedTuple,
     Optional,
@@ -14,10 +16,12 @@ from typing import (
     cast,
 )
 
+from pydantic import ConstrainedFloat, ConstrainedInt, ConstrainedStr
 from typing_extensions import TypeAlias
 
+from dagster import Enum as DagsterEnum
 from dagster._annotations import experimental
-from dagster._config.config_type import Array, ConfigFloatInstance, ConfigType
+from dagster._config.config_type import Array, ConfigFloatInstance, ConfigType, EnumValue, Noneable
 from dagster._config.field_utils import config_dictionary_from_values
 from dagster._config.post_process import resolve_defaults
 from dagster._config.source import BoolSource, IntSource, StringSource
@@ -63,7 +67,7 @@ from dagster._core.storage.io_manager import IOManager, IOManagerDefinition
 from .typing_utils import BaseResourceMeta, LateBoundTypesForResourceTypeChecking
 from .utils import safe_is_subclass
 
-Self = TypeVar("Self", bound="ConfigurableResource")
+Self = TypeVar("Self", bound="ConfigurableResourceFactory")
 
 
 class MakeConfigCacheable(BaseModel):
@@ -97,13 +101,10 @@ class MakeConfigCacheable(BaseModel):
 
 @experimental
 class Config(MakeConfigCacheable):
-    """
-    Base class for Dagster configuration models.
-    """
+    """Base class for Dagster configuration models."""
 
-    def __init__(self, **config_dict):
-        """
-        This constructor is overridden to handle any remapping of raw config dicts to
+    def __init__(self, **config_dict) -> None:
+        """This constructor is overridden to handle any remapping of raw config dicts to
         the appropriate config classes. For example, discriminated unions are represented
         in Dagster config as dicts with a single key, which is the discriminator value.
         """
@@ -126,11 +127,21 @@ class Config(MakeConfigCacheable):
         super().__init__(**modified_data)
 
     def _as_config_dict(self) -> Mapping[str, Any]:
-        """
-        Returns a dictionary representation of this config object,
+        """Returns a dictionary representation of this config object,
         ignoring any private fields.
         """
-        return {k: v for k, v in self.__dict__.items() if not k.startswith("_")}
+        output = {}
+        for key, value in self.__dict__.items():
+            if key.startswith("_"):
+                continue
+            field = self.__fields__.get(key)
+            if field and value is None and not _is_pydantic_field_required(field):
+                continue
+            if field:
+                output[field.alias] = value
+            else:
+                output[key] = value
+        return output
 
 
 @experimental
@@ -196,7 +207,7 @@ def _process_config_values(
 
     if not post_processed_config.success:
         raise DagsterInvalidConfigError(
-            "Error while processing {} config ".format(config_obj_name),
+            f"Error while processing {config_obj_name} config ",
             post_processed_config.errors,
             data,
         )
@@ -206,7 +217,7 @@ def _process_config_values(
 
 
 def _curry_config_schema(schema_field: Field, data: Any) -> DefinitionConfigSchema:
-    """Return a new config schema configured with the passed in data"""
+    """Return a new config schema configured with the passed in data."""
     return DefinitionConfigSchema(_apply_defaults_to_schema_field(schema_field, data))
 
 
@@ -219,13 +230,12 @@ ResourceId: TypeAlias = int
 def _resolve_required_resource_keys_for_resource(
     resource: ResourceDefinition, resource_id_to_key_mapping: Mapping[ResourceId, str]
 ) -> AbstractSet[str]:
-    """
-    Gets the required resource keys for the provided resource, with the assistance of the passed
+    """Gets the required resource keys for the provided resource, with the assistance of the passed
     resource-id-to-key mapping. For resources which may hold nested partial resources,
     this mapping is used to obtain the top-level resource keys to depend on.
     """
     if isinstance(resource, AllowDelayedDependencies):
-        return resource._resolve_required_resource_keys(resource_id_to_key_mapping)
+        return resource._resolve_required_resource_keys(resource_id_to_key_mapping)  # noqa: SLF001
     return resource.required_resource_keys
 
 
@@ -257,7 +267,7 @@ class AllowDelayedDependencies:
                 _resolve_required_resource_keys_for_resource(v, resource_mapping)
             )
 
-        resources, _ = _separate_resource_params(self.__dict__)
+        resources, _ = separate_resource_params(self.__dict__)
         for v in resources.values():
             nested_resource_required_keys.update(
                 _resolve_required_resource_keys_for_resource(v, resource_mapping)
@@ -270,8 +280,7 @@ class AllowDelayedDependencies:
 
 
 class InitResourceContextWithKeyMapping(InitResourceContext):
-    """
-    Passes along a mapping from ResourceDefinition id to resource key alongside the
+    """Passes along a mapping from ResourceDefinition id to resource key alongside the
     InitResourceContext. This is used to resolve the required resource keys for
     resources which may hold nested partial resources.
     """
@@ -306,8 +315,7 @@ class InitResourceContextWithKeyMapping(InitResourceContext):
 
 
 class ResourceWithKeyMapping(ResourceDefinition):
-    """
-    Wrapper around a ResourceDefinition which helps the inner resource resolve its required
+    """Wrapper around a ResourceDefinition which helps the inner resource resolve its required
     resource keys. This is useful for resources which may hold nested resources. At construction
     time, they are unaware of the resource keys of their nested resources - the resource id to
     key mapping is used to resolve this.
@@ -328,8 +336,7 @@ class ResourceWithKeyMapping(ResourceDefinition):
         )
 
     def setup_context_resources_and_call(self, context: InitResourceContext):
-        """
-        Wrapper around the wrapped resource's resource_fn which attaches its
+        """Wrapper around the wrapped resource's resource_fn which attaches its
         resource id to key mapping to the context, and then calls the nested resource's resource_fn.
         """
         context_with_key_mapping = InitResourceContextWithKeyMapping(
@@ -347,11 +354,17 @@ class ResourceWithKeyMapping(ResourceDefinition):
             self._resource, self._resource_id_to_key_mapping
         )
 
+    @property
+    def wrapped_resource(self) -> ResourceDefinition:
+        return self._resource
+
+    @property
+    def inner_resource(self):
+        return self._resource
+
 
 class IOManagerWithKeyMapping(ResourceWithKeyMapping, IOManagerDefinition):
-    """
-    Version of ResourceWithKeyMapping wrapper that also implements IOManagerDefinition.
-    """
+    """Version of ResourceWithKeyMapping wrapper that also implements IOManagerDefinition."""
 
     def __init__(
         self, resource: ResourceDefinition, resource_id_to_key_mapping: Dict[ResourceId, str]
@@ -365,7 +378,7 @@ class IOManagerWithKeyMapping(ResourceWithKeyMapping, IOManagerDefinition):
 def attach_resource_id_to_key_mapping(
     resource_def: Any, resource_id_to_key_mapping: Dict[ResourceId, str]
 ) -> Any:
-    if isinstance(resource_def, (ConfigurableResource, PartialResource)):
+    if isinstance(resource_def, (ConfigurableResourceFactory, PartialResource)):
         return (
             IOManagerWithKeyMapping(resource_def, resource_id_to_key_mapping)
             if isinstance(resource_def, IOManagerDefinition)
@@ -375,7 +388,7 @@ def attach_resource_id_to_key_mapping(
 
 
 @experimental
-class ConfigurableResource(
+class ConfigurableResourceFactory(
     Generic[TResValue],
     ResourceDefinition,
     Config,
@@ -383,36 +396,40 @@ class ConfigurableResource(
     AllowDelayedDependencies,
     metaclass=BaseResourceMeta,
 ):
-    """
-    Base class for Dagster resources that utilize structured config.
+    """Base class for Dagster resources that utilize structured config.
 
     This class is a subclass of both :py:class:`ResourceDefinition` and :py:class:`Config`, and
-    provides a default implementation of the resource_fn that returns the resource itself.
+    must implement ``create_resource``, which defines the resource value to pass to user code.
 
     Example:
     .. code-block:: python
 
-        class WriterResource(ConfigurableResource):
-            prefix: str
+        class DatabseResource(ConfigurableResourceFactory[Database]):
+            connection_uri: str
 
-            def output(self, text: str) -> None:
-                print(f"{self.prefix}{text}")
+            def create_resource(self, _init_context) -> Database:
+                return Database(self.connection_uri)
 
     """
 
     def __init__(self, **data: Any):
-        resource_pointers, data_without_resources = _separate_resource_params(data)
+        resource_pointers, data_without_resources = separate_resource_params(data)
 
         schema = infer_schema_from_config_class(
             self.__class__, fields_to_omit=set(resource_pointers.keys())
         )
 
-        # Resolve e.g. EnvVar values
-        resolved_config_dict = config_dictionary_from_values(data_without_resources, schema)
-        curried_schema = _curry_config_schema(schema, resolved_config_dict)
-
+        # Populate config values
         Config.__init__(self, **{**data_without_resources, **resource_pointers})
 
+        # We pull the values from the Pydantic config object, which may cast values
+        # to the correct type under the hood - useful in particular for enums
+        casted_data_without_resources = {
+            k: v for k, v in self._as_config_dict().items() if k in data_without_resources
+        }
+        resolved_config_dict = config_dictionary_from_values(casted_data_without_resources, schema)
+        curried_schema = _curry_config_schema(schema, resolved_config_dict)
+
         # We keep track of any resources we depend on which are not fully configured
         # so that we can retrieve them at runtime
         self._nested_partial_resources: Mapping[str, ResourceDefinition] = {
@@ -428,17 +445,23 @@ class ConfigurableResource(
         self._resolved_config_dict = resolved_config_dict
         self._schema = schema
 
+        self._nested_resources = {k: v for k, v in resource_pointers.items()}
+
+    @property
+    def nested_resources(self) -> Mapping[str, ResourceDefinition]:
+        return self._nested_resources
+
     @classmethod
     def configure_at_launch(cls: "Type[Self]", **kwargs) -> "PartialResource[Self]":
-        """
-        Returns a partially initialized copy of the resource, with remaining config fields
+        """Returns a partially initialized copy of the resource, with remaining config fields
         set at runtime.
         """
         return PartialResource(cls, data=kwargs)
 
-    def _with_updated_values(self, values: Mapping[str, Any]) -> "ConfigurableResource[TResValue]":
-        """
-        Returns a new instance of the resource with the given values.
+    def _with_updated_values(
+        self, values: Mapping[str, Any]
+    ) -> "ConfigurableResourceFactory[TResValue]":
+        """Returns a new instance of the resource with the given values.
         Used when initializing a resource at runtime.
         """
         # Since Resource extends BaseModel and is a dataclass, we know that the
@@ -446,9 +469,8 @@ class ConfigurableResource(
         # of this class. We can therefore safely pass in the values as kwargs.
         return self.__class__(**{**self._as_config_dict(), **values})
 
-    def _resolve_and_update_env_vars(self) -> "ConfigurableResource[TResValue]":
-        """
-        Processes the config dictionary to resolve any EnvVar values. This is called at runtime
+    def _resolve_and_update_env_vars(self) -> "ConfigurableResourceFactory[TResValue]":
+        """Processes the config dictionary to resolve any EnvVar values. This is called at runtime
         when the resource is initialized, so the user is only shown the error if they attempt to
         kick off a run relying on this resource.
 
@@ -461,9 +483,8 @@ class ConfigurableResource(
 
     def _resolve_and_update_nested_resources(
         self, context: InitResourceContext
-    ) -> "ConfigurableResource[TResValue]":
-        """
-        Updates any nested resources with the resource values from the context.
+    ) -> "ConfigurableResourceFactory[TResValue]":
+        """Updates any nested resources with the resource values from the context.
         In this case, populating partially configured resources or
         resources that return plain Python types.
 
@@ -489,7 +510,7 @@ class ConfigurableResource(
             }
 
         # Also evaluate any resources that are not partial
-        resources_to_update, _ = _separate_resource_params(self.__dict__)
+        resources_to_update, _ = separate_resource_params(self.__dict__)
         resources_to_update = {
             attr_name: _call_resource_fn_with_default(resource_def, context)
             for attr_name, resource_def in resources_to_update.items()
@@ -501,23 +522,46 @@ class ConfigurableResource(
 
     def initialize_and_run(self, context: InitResourceContext) -> TResValue:
         with_nested_resources = self._resolve_and_update_nested_resources(context)
-        with_env_vars = with_nested_resources._resolve_and_update_env_vars()
+        with_env_vars = with_nested_resources._resolve_and_update_env_vars()  # noqa: SLF001
 
-        return with_env_vars._create_object_fn(context)
+        return with_env_vars._create_object_fn(context)  # noqa: SLF001
 
     def _create_object_fn(self, context: InitResourceContext) -> TResValue:
         return self.create_resource(context)
 
-    def create_resource(
-        self, context: InitResourceContext
-    ) -> TResValue:  # pylint: disable=unused-argument
+    def create_resource(self, context: InitResourceContext) -> TResValue:
+        """Returns the object that this resource hands to user code, accessible by ops or assets
+        through the context or resource parameters. This works like the function decorated
+        with @resource when using function-based resources.
         """
-        Returns the object that this resource hands to user code, accessible by ops or assets
+        raise NotImplementedError()
+
+
+@experimental
+class ConfigurableResource(ConfigurableResourceFactory[TResValue]):
+    """Base class for Dagster resources that utilize structured config.
+
+    This class is a subclass of both :py:class:`ResourceDefinition` and :py:class:`Config`, and
+    provides a default implementation of the resource_fn that returns the resource itself.
+
+    Example:
+    .. code-block:: python
+
+        class WriterResource(ConfigurableResource):
+            prefix: str
+
+            def output(self, text: str) -> None:
+                print(f"{self.prefix}{text}")
+
+    """
+
+    def create_resource(self, context: InitResourceContext) -> TResValue:
+        """Returns the object that this resource hands to user code, accessible by ops or assets
         through the context or resource parameters. This works like the function decorated
         with @resource when using function-based resources.
 
-        Default behavior for new class-based resources is to return itself, passing
-        the actual resource object to user code.
+        For ConfigurableResource, this function will return itself, passing
+        the actual ConfigurableResource object to user code.
         """
         return cast(TResValue, self)
 
@@ -538,10 +582,12 @@ class PartialResource(
     Generic[TResValue], ResourceDefinition, AllowDelayedDependencies, MakeConfigCacheable
 ):
     data: Dict[str, Any]
-    resource_cls: Type[ConfigurableResource[TResValue]]
+    resource_cls: Type[ConfigurableResourceFactory[TResValue]]
 
-    def __init__(self, resource_cls: Type[ConfigurableResource[TResValue]], data: Dict[str, Any]):
-        resource_pointers, data_without_resources = _separate_resource_params(data)
+    def __init__(
+        self, resource_cls: Type[ConfigurableResourceFactory[TResValue]], data: Dict[str, Any]
+    ):
+        resource_pointers, data_without_resources = separate_resource_params(data)
 
         MakeConfigCacheable.__init__(self, data=data, resource_cls=resource_cls)  # type: ignore  # extends BaseModel, takes kwargs
 
@@ -566,10 +612,21 @@ class PartialResource(
             description=resource_cls.__doc__,
         )
 
+        self._nested_resources = {k: v for k, v in resource_pointers.items()}
 
-ResourceOrPartial: TypeAlias = Union[ConfigurableResource[TResValue], PartialResource[TResValue]]
+    @property
+    def nested_resources(self) -> Mapping[str, ResourceDefinition]:
+        return self._nested_resources
+
+
+ResourceOrPartial: TypeAlias = Union[
+    ConfigurableResourceFactory[TResValue], PartialResource[TResValue]
+]
 ResourceOrPartialOrValue: TypeAlias = Union[
-    ConfigurableResource[TResValue], PartialResource[TResValue], ResourceDefinition, TResValue
+    ConfigurableResourceFactory[TResValue],
+    PartialResource[TResValue],
+    ResourceDefinition,
+    TResValue,
 ]
 
 
@@ -581,7 +638,7 @@ class ResourceDependency(Generic[V]):
     def __set_name__(self, _owner, name):
         self._name = name
 
-    def __get__(self, obj: "ConfigurableResource", __owner: Any) -> V:
+    def __get__(self, obj: "ConfigurableResourceFactory", __owner: Any) -> V:
         return getattr(obj, self._name)
 
     def __set__(self, obj: Optional[object], value: ResourceOrPartialOrValue[V]) -> None:
@@ -590,8 +647,7 @@ class ResourceDependency(Generic[V]):
 
 @experimental
 class ConfigurableLegacyResourceAdapter(ConfigurableResource, ABC):
-    """
-    Adapter base class for wrapping a decorated, function-style resource
+    """Adapter base class for wrapping a decorated, function-style resource
     with structured config.
 
     To use this class, subclass it, define config schema fields using Pydantic,
@@ -631,9 +687,10 @@ class ConfigurableLegacyResourceAdapter(ConfigurableResource, ABC):
 
 
 @experimental
-class ConfigurableIOManagerFactory(ConfigurableResource[TIOManagerValue], IOManagerDefinition):
-    """
-    Base class for Dagster IO managers that utilize structured config. This base class
+class ConfigurableIOManagerFactory(
+    ConfigurableResourceFactory[TIOManagerValue], IOManagerDefinition
+):
+    """Base class for Dagster IO managers that utilize structured config. This base class
     is useful for cases in which the returned IO manager is not the same as the class itself
     (e.g. when it is a wrapper around the actual IO manager implementation).
 
@@ -643,7 +700,7 @@ class ConfigurableIOManagerFactory(ConfigurableResource[TIOManagerValue], IOMana
     """
 
     def __init__(self, **data: Any):
-        ConfigurableResource.__init__(self, **data)
+        ConfigurableResourceFactory.__init__(self, **data)
         IOManagerDefinition.__init__(
             self,
             resource_fn=self.initialize_and_run,
@@ -656,20 +713,21 @@ class ConfigurableIOManagerFactory(ConfigurableResource[TIOManagerValue], IOMana
 
     @abstractmethod
     def create_io_manager(self, context) -> TIOManagerValue:
-        """Implement as one would implement a @io_manager decorator function"""
+        """Implement as one would implement a @io_manager decorator function."""
         raise NotImplementedError()
 
     @classmethod
     def configure_at_launch(cls: "Type[Self]", **kwargs) -> "PartialIOManager[Self]":
-        """
-        Returns a partially initialized copy of the IO manager, with remaining config fields
+        """Returns a partially initialized copy of the IO manager, with remaining config fields
         set at runtime.
         """
         return PartialIOManager(cls, data=kwargs)
 
 
 class PartialIOManager(Generic[TResValue], PartialResource[TResValue], IOManagerDefinition):
-    def __init__(self, resource_cls: Type[ConfigurableResource[TResValue]], data: Dict[str, Any]):
+    def __init__(
+        self, resource_cls: Type[ConfigurableResourceFactory[TResValue]], data: Dict[str, Any]
+    ):
         PartialResource.__init__(self, resource_cls, data)
         IOManagerDefinition.__init__(
             self,
@@ -681,8 +739,7 @@ class PartialIOManager(Generic[TResValue], PartialResource[TResValue], IOManager
 
 @experimental
 class ConfigurableIOManager(ConfigurableIOManagerFactory, IOManager):
-    """
-    Base class for Dagster IO managers that utilize structured config.
+    """Base class for Dagster IO managers that utilize structured config.
 
     This class is a subclass of both :py:class:`IOManagerDefinition`, :py:class:`Config`,
     and :py:class:`IOManager`. Implementers must provide an implementation of the
@@ -707,8 +764,7 @@ MAPPING_KEY_TYPE_TO_SCALAR = {
 def _wrap_config_type(
     shape_type: PydanticShapeType, key_type: Optional[ConfigType], config_type: ConfigType
 ) -> ConfigType:
-    """
-    Based on a Pydantic shape type, wraps a config type in the appropriate Dagster config wrapper.
+    """Based on a Pydantic shape type, wraps a config type in the appropriate Dagster config wrapper.
     For example, if the shape type is a Pydantic list, the config type will be wrapped in an Array.
     """
     if shape_type == SHAPE_SINGLETON:
@@ -728,9 +784,7 @@ def _wrap_config_type(
 
 
 def _convert_pydantic_field(pydantic_field: ModelField) -> Field:
-    """
-    Transforms a Pydantic field into a corresponding Dagster config field.
-    """
+    """Transforms a Pydantic field into a corresponding Dagster config field."""
     key_type = (
         _config_type_for_pydantic_field(pydantic_field.key_field)
         if pydantic_field.key_field
@@ -756,6 +810,8 @@ def _convert_pydantic_field(pydantic_field: ModelField) -> Field:
         wrapped_config_type = _wrap_config_type(
             shape_type=pydantic_field.shape, config_type=config_type, key_type=key_type
         )
+        if pydantic_field.allow_none:
+            wrapped_config_type = Noneable(wrapped_config_type)
         return Field(
             config=wrapped_config_type,
             description=pydantic_field.field_info.description,
@@ -771,6 +827,24 @@ def _config_type_for_pydantic_field(pydantic_field: ModelField) -> ConfigType:
 
 
 def _config_type_for_type_on_pydantic_field(potential_dagster_type: Any) -> ConfigType:
+    # special case pydantic constrained types to their source equivalents
+    if safe_is_subclass(potential_dagster_type, ConstrainedStr):
+        return StringSource
+    # no FloatSource, so we just return float
+    elif safe_is_subclass(potential_dagster_type, ConstrainedFloat):
+        potential_dagster_type = float
+    elif safe_is_subclass(potential_dagster_type, ConstrainedInt):
+        return IntSource
+
+    if safe_is_subclass(potential_dagster_type, Enum):
+        return DagsterEnum(
+            potential_dagster_type.__name__,
+            [
+                EnumValue(v.name, python_value=v.value)
+                for v in cast(Iterable[Enum], potential_dagster_type)
+            ],
+        )
+
     # special case raw python literals to their source equivalents
     if potential_dagster_type is str:
         return StringSource
@@ -796,8 +870,7 @@ def _is_pydantic_field_required(pydantic_field: ModelField) -> bool:
 
 @experimental
 class ConfigurableLegacyIOManagerAdapter(ConfigurableIOManagerFactory):
-    """
-    Adapter base class for wrapping a decorated, function-style I/O manager
+    """Adapter base class for wrapping a decorated, function-style I/O manager
     with structured config.
 
     To use this class, subclass it, define config schema fields using Pydantic,
@@ -840,8 +913,7 @@ class ConfigurableLegacyIOManagerAdapter(ConfigurableIOManagerFactory):
 
 
 def _convert_pydantic_descriminated_union_field(pydantic_field: ModelField) -> Field:
-    """
-    Builds a Selector config field from a Pydantic field which is a descriminated union.
+    """Builds a Selector config field from a Pydantic field which is a descriminated union.
 
     For example:
 
@@ -892,8 +964,7 @@ def _convert_pydantic_descriminated_union_field(pydantic_field: ModelField) -> F
 
 
 def infer_schema_from_config_annotation(model_cls: Any, config_arg_default: Any) -> Field:
-    """
-    Parses a structured config class or primitive type and returns a corresponding Dagster config Field.
+    """Parses a structured config class or primitive type and returns a corresponding Dagster config Field.
     """
     if safe_is_subclass(model_cls, Config):
         check.invariant(
@@ -918,9 +989,7 @@ def infer_schema_from_config_class(
     description: Optional[str] = None,
     fields_to_omit: Optional[Set[str]] = None,
 ) -> Field:
-    """
-    Parses a structured config class and returns a corresponding Dagster config Field.
-    """
+    """Parses a structured config class and returns a corresponding Dagster config Field."""
     fields_to_omit = fields_to_omit or set()
 
     check.param_invariant(
@@ -944,9 +1013,8 @@ class SeparatedResourceParams(NamedTuple):
     non_resources: Dict[str, Any]
 
 
-def _separate_resource_params(data: Dict[str, Any]) -> SeparatedResourceParams:
-    """
-    Separates out the key/value inputs of fields in a structured config Resource class which
+def separate_resource_params(data: Dict[str, Any]) -> SeparatedResourceParams:
+    """Separates out the key/value inputs of fields in a structured config Resource class which
     are themselves Resources and those which are not.
     """
     return SeparatedResourceParams(
@@ -969,6 +1037,6 @@ def _call_resource_fn_with_default(obj: ResourceDefinition, context: InitResourc
 
 LateBoundTypesForResourceTypeChecking.set_actual_types_for_type_checking(
     resource_dep_type=ResourceDependency,
-    resource_type=ConfigurableResource,
+    resource_type=ConfigurableResourceFactory,
     partial_resource_type=PartialResource,
 )
