@@ -49,6 +49,7 @@ class AssetGraph:
         freshness_policies_by_key: Mapping[AssetKey, Optional[FreshnessPolicy]],
         required_multi_asset_sets_by_key: Optional[Mapping[AssetKey, AbstractSet[AssetKey]]],
         code_versions_by_key: Mapping[AssetKey, Optional[str]],
+        is_observable_by_key: Mapping[AssetKey, bool],
     ):
         self._asset_dep_graph = asset_dep_graph
         self._source_asset_keys = source_asset_keys
@@ -58,6 +59,7 @@ class AssetGraph:
         self._freshness_policies_by_key = freshness_policies_by_key
         self._required_multi_asset_sets_by_key = required_multi_asset_sets_by_key
         self._code_versions_by_key = code_versions_by_key
+        self._is_observable_by_key = is_observable_by_key
 
     @property
     def asset_dep_graph(self) -> DependencyGraph[AssetKey]:
@@ -76,7 +78,7 @@ class AssetGraph:
         """Non-source asset keys that have no non-source parents."""
         from .asset_selection import AssetSelection
 
-        return AssetSelection.keys(*self.all_asset_keys).sources().resolve(self)
+        return AssetSelection.keys(*self.non_source_asset_keys).roots().resolve(self)
 
     @property
     def freshness_policies_by_key(self) -> Mapping[AssetKey, Optional[FreshnessPolicy]]:
@@ -84,7 +86,7 @@ class AssetGraph:
 
     @staticmethod
     def from_assets(
-        all_assets: Sequence[Union[AssetsDefinition, SourceAsset]]
+        all_assets: Iterable[Union[AssetsDefinition, SourceAsset]]
     ) -> "InternalAssetGraph":
         assets_defs: List[AssetsDefinition] = []
         source_assets: List[SourceAsset] = []
@@ -96,12 +98,14 @@ class AssetGraph:
         freshness_policies_by_key: Dict[AssetKey, Optional[FreshnessPolicy]] = {}
         required_multi_asset_sets_by_key: Dict[AssetKey, AbstractSet[AssetKey]] = {}
         code_versions_by_key: Dict[AssetKey, Optional[str]] = {}
+        is_observable_by_key: Dict[AssetKey, bool] = {}
 
         for asset in all_assets:
             if isinstance(asset, SourceAsset):
                 source_assets.append(asset)
                 partitions_defs_by_key[asset.key] = asset.partitions_def
                 group_names_by_key[asset.key] = asset.group_name
+                is_observable_by_key[asset.key] = asset.is_observable
             else:  # AssetsDefinition
                 assets_defs.append(asset)
                 partition_mappings_by_key.update(
@@ -126,12 +130,17 @@ class AssetGraph:
             assets=assets_defs,
             source_assets=source_assets,
             code_versions_by_key=code_versions_by_key,
+            is_observable_by_key=is_observable_by_key,
         )
 
     @property
     def all_asset_keys(self) -> AbstractSet[AssetKey]:
         return self._asset_dep_graph["upstream"].keys()
 
+    @property
+    def non_source_asset_keys(self) -> AbstractSet[AssetKey]:
+        return self._asset_dep_graph["upstream"].keys() - self._source_asset_keys
+
     def get_partitions_def(self, asset_key: AssetKey) -> Optional[PartitionsDefinition]:
         return self._partitions_defs_by_key.get(asset_key)
 
@@ -163,6 +172,9 @@ class AssetGraph:
             partitions_defs[i] == partitions_defs[0] for i in range(1, len(partitions_defs))
         )
 
+    def is_observable(self, asset_key: AssetKey) -> bool:
+        return self._is_observable_by_key.get(asset_key, False)
+
     def get_children(self, asset_key: AssetKey) -> AbstractSet[AssetKey]:
         """Returns all assets that depend on the given asset."""
         return self._asset_dep_graph["downstream"][asset_key]
@@ -177,8 +189,7 @@ class AssetGraph:
         asset_key: AssetKey,
         partition_key: Optional[str] = None,
     ) -> AbstractSet[AssetKeyPartitionKey]:
-        """
-        Returns every partition in every of the given asset's children that depends on the given
+        """Returns every partition in every of the given asset's children that depends on the given
         partition of that asset.
         """
         result: Set[AssetKeyPartitionKey] = set()
@@ -199,9 +210,9 @@ class AssetGraph:
         parent_asset_key: AssetKey,
         child_asset_key: AssetKey,
     ) -> Sequence[str]:
-        """
-        Converts a partition key from one asset to the corresponding partition keys in a downstream
+        """Converts a partition key from one asset to the corresponding partition keys in a downstream
         asset. Uses the existing partition mapping between the child asset and the parent asset.
+
         Args:
             parent_partition_key (Optional[str]): The partition key to convert.
             parent_asset_key (AssetKey): The asset key of the upstream asset, which the provided
@@ -245,8 +256,7 @@ class AssetGraph:
         asset_key: AssetKey,
         partition_key: Optional[str] = None,
     ) -> AbstractSet[AssetKeyPartitionKey]:
-        """
-        Returns every partition in every of the given asset's parents that the given partition of
+        """Returns every partition in every of the given asset's parents that the given partition of
         that asset depends on.
         """
         result: Set[AssetKeyPartitionKey] = set()
@@ -270,10 +280,10 @@ class AssetGraph:
         child_asset_key: AssetKey,
         dynamic_partitions_store: Optional[DynamicPartitionsStore] = None,
     ) -> Sequence[str]:
-        """
-        Converts a partition key from one asset to the corresponding partition keys in one of its
+        """Converts a partition key from one asset to the corresponding partition keys in one of its
         parent assets. Uses the existing partition mapping between the child asset and the parent
         asset.
+
         Args:
             partition_key (Optional[str]): The partition key to convert.
             child_asset_key (AssetKey): The asset key of the child asset, which the provided
@@ -311,7 +321,7 @@ class AssetGraph:
         return asset_key in self.source_asset_keys or asset_key not in self.all_asset_keys
 
     def has_non_source_parents(self, asset_key: AssetKey) -> bool:
-        """Determines if an asset has any parents which are not source assets"""
+        """Determines if an asset has any parents which are not source assets."""
         if self.is_source(asset_key):
             return False
         return any(
@@ -365,6 +375,23 @@ class AssetGraph:
             {key for key in level} for level in toposort.toposort(self._asset_dep_graph["upstream"])
         ]
 
+    @cached_method
+    def get_downstream_freshness_policies(
+        self, *, asset_key: AssetKey
+    ) -> AbstractSet[FreshnessPolicy]:
+        downstream_policies = set().union(
+            *(
+                self.get_downstream_freshness_policies(asset_key=child_key)
+                for child_key in self.get_children(asset_key)
+                if child_key != asset_key
+            )
+        )
+        current_policy = self.freshness_policies_by_key.get(asset_key)
+        if self.get_partitions_def(asset_key) is None and current_policy is not None:
+            downstream_policies.add(current_policy)
+
+        return downstream_policies
+
     def has_self_dependency(self, asset_key: AssetKey) -> bool:
         return asset_key in self.get_parents(asset_key)
 
@@ -374,11 +401,11 @@ class AssetGraph:
         condition_fn: Callable[[AssetKey, Optional[PartitionsSubset]], bool],
         initial_subset: "AssetGraphSubset",
     ) -> "AssetGraphSubset":
-        """
-        Returns asset partitions within the graph that
+        """Returns asset partitions within the graph that satisfy supplied criteria.
+
         - Are >= initial_asset_partitions
         - Asset matches the condition_fn
-        - Any of their ancestors >= initial_asset_partitions match the condition_fn
+        - Any of their ancestors >= initial_asset_partitions match the condition_fn.
 
         Visits parents before children.
         """
@@ -452,8 +479,8 @@ class AssetGraph:
         ],
         initial_asset_partitions: Iterable[AssetKeyPartitionKey],
     ) -> AbstractSet[AssetKeyPartitionKey]:
-        """
-        Returns asset partitions within the graph that
+        """Returns asset partitions within the graph that satisfy supplied criteria.
+
         - Are >= initial_asset_partitions
         - Match the condition_fn
         - Any of their ancestors >= initial_asset_partitions match the condition_fn
@@ -506,6 +533,7 @@ class InternalAssetGraph(AssetGraph):
         assets: Sequence[AssetsDefinition],
         source_assets: Sequence[SourceAsset],
         code_versions_by_key: Mapping[AssetKey, Optional[str]],
+        is_observable_by_key: Mapping[AssetKey, bool],
     ):
         super().__init__(
             asset_dep_graph=asset_dep_graph,
@@ -516,6 +544,7 @@ class InternalAssetGraph(AssetGraph):
             freshness_policies_by_key=freshness_policies_by_key,
             required_multi_asset_sets_by_key=required_multi_asset_sets_by_key,
             code_versions_by_key=code_versions_by_key,
+            is_observable_by_key=is_observable_by_key,
         )
         self._assets = assets
         self._source_assets = source_assets
@@ -530,7 +559,7 @@ class InternalAssetGraph(AssetGraph):
 
 
 class ToposortedPriorityQueue:
-    """Queue that returns parents before their children"""
+    """Queue that returns parents before their children."""
 
     @functools.total_ordering
     class QueueItem(NamedTuple):
