@@ -18,7 +18,9 @@ from typing import (
 )
 
 import dagster._check as check
-from dagster._core.definitions.metadata import MetadataUserInput, RawMetadataValue
+from dagster._core.definitions.metadata import (
+    ArbitraryMetadataMapping,
+)
 from dagster._core.selector.subset_selector import AssetSelectionData
 
 from ..errors import DagsterInvalidSubsetError
@@ -127,8 +129,7 @@ def _build_graph_dependencies(
     non_asset_inputs_by_node_handle: Dict[NodeHandle, Sequence[NodeOutputHandle]],
     assets_defs_by_node_handle: Mapping[NodeHandle, "AssetsDefinition"],
 ) -> None:
-    """
-    Scans through every node in the graph, making a recursive call when a node is a graph.
+    """Scans through every node in the graph, making a recursive call when a node is a graph.
 
     Builds two dictionaries:
 
@@ -176,8 +177,7 @@ def _get_dependency_node_output_handles(
     ],
     node_output_handle: NodeOutputHandle,
 ) -> Sequence[NodeOutputHandle]:
-    """
-    Given a node output handle, return all upstream op node output handles. All node output handles
+    """Given a node output handle, return all upstream op node output handles. All node output handles
     belong in the same graph-backed asset node.
 
     Arguments:
@@ -236,8 +236,7 @@ def _get_dependency_node_output_handles(
 def get_dep_node_handles_of_graph_backed_asset(
     graph_def: GraphDefinition, assets_def: "AssetsDefinition"
 ) -> Mapping[AssetKey, Set[NodeHandle]]:
-    """
-    Given a graph-backed asset with graph_def, return a mapping of asset keys outputted by the graph
+    """Given a graph-backed asset with graph_def, return a mapping of asset keys outputted by the graph
     to a list of node handles within graph_def that are the dependencies of the asset.
 
     Arguments:
@@ -260,8 +259,7 @@ def asset_key_to_dep_node_handles(
     graph_def: GraphDefinition,
     assets_defs_by_node_handle: Mapping[NodeHandle, "AssetsDefinition"],
 ) -> Tuple[Mapping[AssetKey, Set[NodeHandle]], Mapping[AssetKey, Sequence[NodeOutputHandle]]]:
-    """
-    For each asset in assets_defs_by_node_handle, determines all the op handles and output handles
+    """For each asset in assets_defs_by_node_handle, determines all the op handles and output handles
     within the asset's node that are upstream dependencies of the asset.
 
     Returns a tuple with two objects:
@@ -297,9 +295,6 @@ def asset_key_to_dep_node_handles(
             {}
         )  # memoized map of node output handles to all node output handle dependencies that are from ops
         for output_name, asset_key in assets_defs.keys_by_output_name.items():
-            output_def = assets_defs.node_def.output_def_named(output_name)
-            output_name = output_def.name
-
             dep_nodes_by_asset_key[
                 asset_key
             ] = []  # first element in list is node that outputs asset
@@ -359,8 +354,7 @@ def asset_key_to_dep_node_handles(
 
 
 class AssetLayer:
-    """
-    Stores all of the asset-related information for a Dagster job / pipeline. Maps each
+    """Stores all of the asset-related information for a Dagster job / pipeline. Maps each
     input / output in the underlying graph to the asset it represents (if any), and records the
     dependencies between each asset.
 
@@ -467,8 +461,7 @@ class AssetLayer:
         source_assets: Sequence["SourceAsset"],
         resolved_asset_deps: "ResolvedAssetDependencies",
     ) -> "AssetLayer":
-        """
-        Generate asset info from a GraphDefinition and a mapping from nodes in that graph to the
+        """Generate asset info from a GraphDefinition and a mapping from nodes in that graph to the
         corresponding AssetsDefinition objects.
 
         Args:
@@ -652,14 +645,12 @@ class AssetLayer:
         else:
             return None
 
-    def metadata_for_asset(self, asset_key: AssetKey) -> Optional[MetadataUserInput]:
+    def metadata_for_asset(
+        self, asset_key: AssetKey
+    ) -> Optional[Mapping[str, ArbitraryMetadataMapping]]:
         if asset_key in self._source_assets_by_key:
-            metadata = self._source_assets_by_key[asset_key].metadata
-            return (
-                {key: cast(RawMetadataValue, value.value) for key, value in metadata.items()}
-                if metadata
-                else None
-            )
+            raw_metadata = self._source_assets_by_key[asset_key].raw_metadata
+            return raw_metadata or None
         elif asset_key in self._assets_defs_by_key:
             return self._assets_defs_by_key[asset_key].metadata_by_key[asset_key]
         else:
@@ -706,8 +697,7 @@ class AssetLayer:
         return self._partition_mappings_by_asset_dep.get((node_handle.root, upstream_asset_key))
 
     def downstream_dep_assets(self, node_handle: NodeHandle, output_name: str) -> Set[AssetKey]:
-        """
-        Given the node handle of an op within a graph-backed asset and an output name,
+        """Given the node handle of an op within a graph-backed asset and an output name,
         returns the asset keys dependent on that output.
 
         For example, for the following asset:
@@ -779,9 +769,6 @@ def build_asset_selection_job(
                 ),
             )
 
-    # We should disallow simultaneous selection of assets and source assets (but for
-    # backcompat we will simply ignore the source asset selection if any regular assets are
-    # selected).
     if len(included_assets) > 0:
         asset_job = build_assets_job(
             name=name,
