@@ -22,7 +22,13 @@ from dagster._utils.merger import merge_dicts
 from .asset_layer import AssetLayer
 from .assets import AssetsDefinition
 from .config import ConfigMapping
-from .dependency import DependencyDefinition, IDependencyDefinition, NodeHandle, NodeInvocation
+from .dependency import (
+    DependencyDefinition,
+    DependencyMapping,
+    IDependencyDefinition,
+    NodeHandle,
+    NodeInvocation,
+)
 from .events import AssetKey
 from .executor_definition import ExecutorDefinition
 from .graph_definition import GraphDefinition
@@ -38,7 +44,7 @@ from .utils import DEFAULT_IO_MANAGER_KEY
 ASSET_BASE_JOB_PREFIX = "__ASSET_JOB"
 
 
-def is_base_asset_job_name(name) -> bool:
+def is_base_asset_job_name(name: str) -> bool:
     return name.startswith(ASSET_BASE_JOB_PREFIX)
 
 
@@ -195,10 +201,10 @@ def build_assets_job(
             partitions_def=partitions_def,
             asset_layer=asset_layer,
             _asset_selection_data=_asset_selection_data,
-            _metadata_entries=original_job._metadata_entries,  # pylint: disable=protected-access
+            metadata=original_job.metadata,
             logger_defs=original_job.get_mode_definition().loggers,
             hooks=original_job.hook_defs,
-            op_retry_policy=original_job._solid_retry_policy,  # pylint: disable=protected-access
+            op_retry_policy=original_job._op_retry_policy,  # noqa: SLF001
             version_strategy=original_job.version_strategy,
         )
 
@@ -294,10 +300,10 @@ def build_source_asset_observation_job(
             partitions_def=partitions_def,
             asset_layer=asset_layer,
             _asset_selection_data=_asset_selection_data,
-            _metadata_entries=original_job._metadata_entries,  # pylint: disable=protected-access
+            metadata=original_job.metadata,
             logger_defs=original_job.get_mode_definition().loggers,
             hooks=original_job.hook_defs,
-            op_retry_policy=original_job._solid_retry_policy,  # pylint: disable=protected-access
+            op_retry_policy=original_job._op_retry_policy,  # noqa: SLF001
             version_strategy=original_job.version_strategy,
         )
 
@@ -346,10 +352,7 @@ def _key_for_asset(asset: Union[AssetsDefinition, SourceAsset]) -> AssetKey:
 def build_node_deps(
     assets_defs: Iterable[AssetsDefinition],
     resolved_asset_deps: ResolvedAssetDependencies,
-) -> Tuple[
-    Mapping[Union[str, NodeInvocation], Mapping[str, IDependencyDefinition]],
-    Mapping[NodeHandle, AssetsDefinition],
-]:
+) -> Tuple[DependencyMapping[NodeInvocation], Mapping[NodeHandle, AssetsDefinition],]:
     # sort so that nodes get a consistent name
     assets_defs = sorted(assets_defs, key=lambda ad: (sorted((ak for ak in ad.keys))))
 
@@ -373,15 +376,12 @@ def build_node_deps(
         for output_name, key in assets_def.keys_by_output_name.items():
             node_alias_and_output_by_asset_key[key] = (node_alias, output_name)
 
-    deps: Dict[Union[str, NodeInvocation], Dict[str, IDependencyDefinition]] = {}
+    deps: Dict[NodeInvocation, Dict[str, IDependencyDefinition]] = {}
     for node_handle, assets_def in assets_defs_by_node_handle.items():
         # the key that we'll use to reference the node inside this AssetsDefinition
         node_def_name = assets_def.node_def.name
-        node_key: Union[NodeInvocation, str]
-        if node_handle.name != node_def_name:
-            node_key = NodeInvocation(node_def_name, alias=node_handle.name)
-        else:
-            node_key = node_def_name
+        alias = node_handle.name if node_handle.name != node_def_name else None
+        node_key = NodeInvocation(node_def_name, alias=alias)
         deps[node_key] = {}
 
         # connect each input of this AssetsDefinition to the proper upstream node
@@ -406,18 +406,14 @@ def build_node_deps(
 
 
 def _has_cycles(
-    deps: Mapping[Union[str, NodeInvocation], Mapping[str, IDependencyDefinition]]
+    deps: DependencyMapping[NodeInvocation],
 ) -> bool:
     """Detect if there are cycles in a dependency dictionary."""
     try:
         node_deps: Dict[str, Set[str]] = {}
         for upstream_node, downstream_deps in deps.items():
             # handle either NodeInvocation or str
-            node_name = (
-                upstream_node.alias or upstream_node.name
-                if isinstance(upstream_node, NodeInvocation)
-                else upstream_node
-            )
+            node_name = upstream_node.alias or upstream_node.name
             node_deps[node_name] = set()
             for dep in downstream_deps.values():
                 if isinstance(dep, DependencyDefinition):
@@ -436,8 +432,7 @@ def _attempt_resolve_cycles(
     assets_defs: Iterable["AssetsDefinition"],
     source_assets: Iterable["SourceAsset"],
 ) -> Sequence["AssetsDefinition"]:
-    """
-    DFS starting at root nodes to color the asset dependency graph. Each time you leave your
+    """DFS starting at root nodes to color the asset dependency graph. Each time you leave your
     current AssetsDefinition, the color increments.
 
     At the end of this process, we'll have a coloring for the asset graph such that any asset which
