@@ -11,7 +11,6 @@ from ..output import GraphOut, OutputDefinition
 
 
 class _Graph:
-
     name: Optional[str]
     description: Optional[str]
     input_defs: Sequence[InputDefinition]
@@ -76,6 +75,7 @@ class _Graph:
             solid_defs,
             config_mapping,
             positional_inputs,
+            node_input_source_assets,
         ) = do_composition(
             decorator_name="@graph",
             graph_name=self.name,
@@ -96,6 +96,7 @@ class _Graph:
             config=config_mapping,
             positional_inputs=positional_inputs,
             tags=self.tags,
+            node_input_source_assets=node_input_source_assets,
         )
         update_wrapper(graph_def, fn)
         return graph_def
