@@ -1,7 +1,7 @@
 from functools import update_wrapper
 from typing import (
-    Any,
     Callable,
+    Dict,
     Iterable,
     Iterator,
     List,
@@ -13,6 +13,8 @@ from typing import (
     overload,
 )
 
+from typing_extensions import TypeAlias
+
 import dagster._check as check
 from dagster._core.decorator_utils import get_function_params
 from dagster._core.definitions.resource_definition import ResourceDefinition
@@ -21,15 +23,16 @@ from dagster._core.errors import DagsterInvalidDefinitionError
 from ..executor_definition import ExecutorDefinition
 from ..graph_definition import GraphDefinition
 from ..logger_definition import LoggerDefinition
-from ..partition import PartitionSetDefinition
 from ..partitioned_schedule import UnresolvedPartitionedAssetScheduleDefinition
 from ..pipeline_definition import PipelineDefinition
 from ..repository_definition import (
     VALID_REPOSITORY_DATA_DICT_KEYS,
     CachingRepositoryData,
     PendingRepositoryDefinition,
+    PendingRepositoryListDefinition,
     RepositoryData,
     RepositoryDefinition,
+    RepositoryListDefinition,
 )
 from ..schedule_definition import ScheduleDefinition
 from ..sensor_definition import SensorDefinition
@@ -37,6 +40,8 @@ from ..unresolved_asset_job_definition import UnresolvedAssetJobDefinition
 
 T = TypeVar("T")
 
+RepositoryDictSpec: TypeAlias = Dict[str, Dict[str, RepositoryListDefinition]]
+
 
 def _flatten(items: Iterable[Union[T, List[T]]]) -> Iterator[T]:
     for x in items:
@@ -55,6 +60,7 @@ class _Repository:
         default_executor_def: Optional[ExecutorDefinition] = None,
         default_logger_defs: Optional[Mapping[str, LoggerDefinition]] = None,
         top_level_resources: Optional[Mapping[str, ResourceDefinition]] = None,
+        resource_key_mapping: Optional[Mapping[int, str]] = None,
     ):
         self.name = check.opt_str_param(name, "name")
         self.description = check.opt_str_param(description, "description")
@@ -67,9 +73,32 @@ class _Repository:
         self.top_level_resources = check.opt_mapping_param(
             top_level_resources, "top_level_resources", key_type=str, value_type=ResourceDefinition
         )
+        self.resource_key_mapping = check.opt_mapping_param(
+            resource_key_mapping, "resource_key_mapping", key_type=int, value_type=str
+        )
+
+    @overload
+    def __call__(
+        self,
+        fn: Union[
+            Callable[[], Sequence[RepositoryListDefinition]],
+            Callable[[], RepositoryDictSpec],
+        ],
+    ) -> RepositoryDefinition:
+        ...
+
+    @overload
+    def __call__(
+        self, fn: Callable[[], Sequence[PendingRepositoryListDefinition]]
+    ) -> PendingRepositoryDefinition:
+        ...
 
     def __call__(
-        self, fn: Callable[[], Sequence[Any]]
+        self,
+        fn: Union[
+            Callable[[], Sequence[PendingRepositoryListDefinition]],
+            Callable[[], RepositoryDictSpec],
+        ],
     ) -> Union[RepositoryDefinition, PendingRepositoryDefinition]:
         from dagster._core.definitions import AssetGroup, AssetsDefinition, SourceAsset
         from dagster._core.definitions.cacheable_assets import CacheableAssetsDefinition
@@ -93,7 +122,6 @@ class _Repository:
                     definition,
                     (
                         PipelineDefinition,
-                        PartitionSetDefinition,
                         ScheduleDefinition,
                         UnresolvedPartitionedAssetScheduleDefinition,
                         SensorDefinition,
@@ -110,15 +138,12 @@ class _Repository:
 
             if bad_defns:
                 bad_definitions_str = ", ".join(
-                    [
-                        "value of type {type_} at index {i}".format(type_=type_, i=i)
-                        for i, type_ in bad_defns
-                    ]
+                    [f"value of type {type_} at index {i}" for i, type_ in bad_defns]
                 )
                 raise DagsterInvalidDefinitionError(
                     "Bad return value from repository construction function: all elements of list "
                     "must be of type JobDefinition, GraphDefinition, PipelineDefinition, "
-                    "PartitionSetDefinition, ScheduleDefinition, SensorDefinition, "
+                    "ScheduleDefinition, SensorDefinition, "
                     "AssetsDefinition, or SourceAsset."
                     f"Got {bad_definitions_str}."
                 )
@@ -131,6 +156,7 @@ class _Repository:
                     default_executor_def=self.default_executor_def,
                     default_logger_defs=self.default_logger_defs,
                     top_level_resources=self.top_level_resources,
+                    resource_key_mapping=self.resource_key_mapping,
                 )
             )
 
@@ -138,11 +164,11 @@ class _Repository:
             if not set(repository_definitions.keys()).issubset(VALID_REPOSITORY_DATA_DICT_KEYS):
                 raise DagsterInvalidDefinitionError(
                     "Bad return value from repository construction function: dict must not contain "
-                    "keys other than {{'pipelines', 'partition_sets', 'schedules', 'jobs'}}: found "
+                    "keys other than {{'pipelines', 'schedules', 'jobs'}}: found "
                     "{bad_keys}".format(
                         bad_keys=", ".join(
                             [
-                                "'{key}'".format(key=key)
+                                f"'{key}'"
                                 for key in repository_definitions.keys()
                                 if key not in VALID_REPOSITORY_DATA_DICT_KEYS
                             ]
@@ -159,7 +185,7 @@ class _Repository:
                 "details and examples".format(type_=type(repository_definitions)),
             )
 
-        if repository_data is None:
+        if isinstance(repository_definitions, list) and repository_data is None:
             return PendingRepositoryDefinition(
                 self.name,
                 repository_definitions=list(_flatten(repository_definitions)),
@@ -180,7 +206,18 @@ class _Repository:
 
 
 @overload
-def repository(definitions_fn: Callable[..., Sequence[Any]]) -> RepositoryDefinition:
+def repository(
+    definitions_fn: Union[
+        Callable[[], Sequence[RepositoryListDefinition]], Callable[[], RepositoryDictSpec]
+    ],
+) -> RepositoryDefinition:
+    ...
+
+
+@overload
+def repository(
+    definitions_fn: Callable[..., Sequence[PendingRepositoryListDefinition]]
+) -> PendingRepositoryDefinition:
     ...
 
 
@@ -192,18 +229,25 @@ def repository(
     default_executor_def: Optional[ExecutorDefinition] = ...,
     default_logger_defs: Optional[Mapping[str, LoggerDefinition]] = ...,
     _top_level_resources: Optional[Mapping[str, ResourceDefinition]] = ...,
+    _resource_key_mapping: Optional[Mapping[int, str]] = ...,
 ) -> _Repository:
     ...
 
 
 def repository(
-    definitions_fn: Optional[Callable[..., Sequence[Any]]] = None,
+    definitions_fn: Optional[
+        Union[
+            Callable[[], Sequence[PendingRepositoryListDefinition]],
+            Callable[[], RepositoryDictSpec],
+        ]
+    ] = None,
     *,
     name: Optional[str] = None,
     description: Optional[str] = None,
     default_executor_def: Optional[ExecutorDefinition] = None,
     default_logger_defs: Optional[Mapping[str, LoggerDefinition]] = None,
     _top_level_resources: Optional[Mapping[str, ResourceDefinition]] = None,
+    _resource_key_mapping: Optional[Mapping[int, str]] = None,
 ) -> Union[RepositoryDefinition, PendingRepositoryDefinition, _Repository]:
     """Create a repository from the decorated function.
 
@@ -341,4 +385,5 @@ def repository(
         default_executor_def=default_executor_def,
         default_logger_defs=default_logger_defs,
         top_level_resources=_top_level_resources,
+        resource_key_mapping=_resource_key_mapping,
     )
