@@ -21,19 +21,16 @@ from typing import (
     cast,
 )
 
-from typing_extensions import TypeAlias
+from typing_extensions import TypeAlias, TypeVar
 
 import dagster._check as check
 from dagster._annotations import PublicAttr, public
 from dagster._core.definitions.policy import RetryPolicy
 from dagster._core.errors import DagsterInvalidDefinitionError
 from dagster._serdes.serdes import (
-    DefaultNamedTupleSerializer,
-    WhitelistMap,
-    register_serdes_tuple_fallbacks,
     whitelist_for_serdes,
 )
-from dagster._utils import frozentags
+from dagster._utils import hash_collection
 
 from .hook_definition import HookDefinition
 from .input import FanInInputPointer, InputDefinition, InputMapping, InputPointer
@@ -49,6 +46,9 @@ if TYPE_CHECKING:
     from .node_definition import NodeDefinition
     from .resource_requirement import ResourceRequirement
 
+T_DependencyKey = TypeVar("T_DependencyKey", str, "NodeInvocation")
+DependencyMapping: TypeAlias = Mapping[T_DependencyKey, Mapping[str, "IDependencyDefinition"]]
+
 
 class NodeInvocation(
     NamedTuple(
@@ -101,18 +101,20 @@ class NodeInvocation(
             cls,
             name=check.str_param(name, "name"),
             alias=check.opt_str_param(alias, "alias"),
-            tags=frozentags(check.opt_mapping_param(tags, "tags", value_type=str, key_type=str)),
-            hook_defs=frozenset(
-                check.opt_set_param(hook_defs, "hook_defs", of_type=HookDefinition)
-            ),
+            tags=check.opt_mapping_param(tags, "tags", value_type=str, key_type=str),
+            hook_defs=check.opt_set_param(hook_defs, "hook_defs", of_type=HookDefinition),
             retry_policy=check.opt_inst_param(retry_policy, "retry_policy", RetryPolicy),
         )
 
+    # Needs to be hashable because this class is used as a key in dependencies dicts
+    def __hash__(self) -> int:
+        if not hasattr(self, "_hash"):
+            self._hash = hash_collection(self)
+        return self._hash
+
 
 class Node(ABC):
-    """
-    Node invocation within a graph. Identified by its name inside the graph.
-    """
+    """Node invocation within a graph. Identified by its name inside the graph."""
 
     name: str
     definition: "NodeDefinition"
@@ -190,9 +192,8 @@ class Node(ABC):
         return self.definition.output_dict
 
     @property
-    def tags(self) -> frozentags:
-        # Type-ignore temporarily pending assessment of right data structure for `tags`
-        return self.definition.tags.updated_with(self._additional_tags)  # type: ignore
+    def tags(self) -> Mapping[str, str]:
+        return {**self.definition.tags, **self._additional_tags}
 
     def container_maps_input(self, input_name: str) -> bool:
         return (
@@ -335,29 +336,9 @@ class OpNode(Node):
         return f"op '{self.name}'"
 
 
-class NodeHandleSerializer(DefaultNamedTupleSerializer):
-    @classmethod
-    def value_to_storage_dict(
-        cls,
-        value: NamedTuple,
-        whitelist_map: WhitelistMap,
-        descent_path: str,
-    ) -> Dict[str, Any]:
-        storage = super().value_to_storage_dict(
-            value,
-            whitelist_map,
-            descent_path,
-        )
-        # persist using legacy name SolidHandle
-        storage["__class__"] = "SolidHandle"
-        return storage
-
-
-@whitelist_for_serdes(serializer=NodeHandleSerializer)
+@whitelist_for_serdes(storage_name="SolidHandle")
 class NodeHandle(NamedTuple("_NodeHandle", [("name", str), ("parent", Optional["NodeHandle"])])):
-    """
-    A structured object to identify nodes in the potentially recursive graph structure.
-    """
+    """A structured object to identify nodes in the potentially recursive graph structure."""
 
     def __new__(cls, name: str, parent: Optional["NodeHandle"]):
         return super(NodeHandle, cls).__new__(
@@ -428,7 +409,6 @@ class NodeHandle(NamedTuple("_NodeHandle", [("name", str), ("parent", Optional["
             NodeHandle:
 
         Example:
-
         .. code-block:: python
 
             handle = NodeHandle('baz', NodeHandle('bar', NodeHandle('foo', None)))
@@ -455,7 +435,6 @@ class NodeHandle(NamedTuple("_NodeHandle", [("name", str), ("parent", Optional["
             NodeHandle:
 
         Example:
-
         .. code-block:: python
 
             handle = NodeHandle('baz', NodeHandle('bar', NodeHandle('foo', None)))
@@ -518,21 +497,15 @@ class NodeHandle(NamedTuple("_NodeHandle", [("name", str), ("parent", Optional["
 class NodeInputHandle(
     NamedTuple("_NodeInputHandle", [("node_handle", NodeHandle), ("input_name", str)])
 ):
-    """
-    A structured object to uniquely identify inputs in the potentially recursive graph structure.
+    """A structured object to uniquely identify inputs in the potentially recursive graph structure.
     """
 
 
 class NodeOutputHandle(
     NamedTuple("_NodeOutputHandle", [("node_handle", NodeHandle), ("output_name", str)])
 ):
+    """A structured object to uniquely identify outputs in the potentially recursive graph structure.
     """
-    A structured object to uniquely identify outputs in the potentially recursive graph structure.
-    """
-
-
-# previous name for NodeHandle was SolidHandle
-register_serdes_tuple_fallbacks({"SolidHandle": NodeHandle})
 
 
 class NodeInput(NamedTuple("_NodeInput", [("node", Node), ("input_def", InputDefinition)])):
@@ -620,7 +593,7 @@ class DependencyType(Enum):
     DYNAMIC_COLLECT = "DYNAMIC_COLLECT"
 
 
-class IDependencyDefinition(ABC):  # pylint: disable=no-init
+class IDependencyDefinition(ABC):
     @abstractmethod
     def get_node_dependencies(self) -> Sequence["DependencyDefinition"]:
         pass
@@ -770,7 +743,7 @@ class MultiDependencyDefinition(
             elif dep is MappedInputPlaceholder:
                 pass
             else:
-                check.failed("Unexpected dependencies entry {}".format(dep))
+                check.failed(f"Unexpected dependencies entry {dep}")
 
         return super(MultiDependencyDefinition, cls).__new__(cls, deps)
 
@@ -810,7 +783,7 @@ InputToOutputMap: TypeAlias = Dict[NodeInput, DepTypeAndOutputs]
 
 def _create_handle_dict(
     node_dict: Mapping[str, Node],
-    dep_dict: Mapping[str, Mapping[str, IDependencyDefinition]],
+    dep_dict: DependencyMapping[str],
 ) -> InputToOutputMap:
     from .composition import MappedInputPlaceholder
 
@@ -857,7 +830,9 @@ def _create_handle_dict(
 
 class DependencyStructure:
     @staticmethod
-    def from_definitions(nodes: Mapping[str, Node], dep_dict: Mapping[str, Any]):
+    def from_definitions(
+        nodes: Mapping[str, Node], dep_dict: DependencyMapping[str]
+    ) -> "DependencyStructure":
         return DependencyStructure(list(dep_dict.keys()), _create_handle_dict(nodes, dep_dict))
 
     _node_input_index: DefaultDict[str, Dict[NodeInput, List[NodeOutput]]]
@@ -1006,8 +981,7 @@ class DependencyStructure:
     def input_to_upstream_outputs_for_node(
         self, node_name: str
     ) -> Mapping[NodeInput, Sequence[NodeOutput]]:
-        """
-        Returns a Dict[NodeInput, List[NodeOutput]] that encodes
+        """Returns a Dict[NodeInput, List[NodeOutput]] that encodes
         where all the the inputs are sourced from upstream. Usually the
         List[NodeOutput] will be a list of one, except for the
         multi-dependency case.
@@ -1018,8 +992,7 @@ class DependencyStructure:
     def output_to_downstream_inputs_for_node(
         self, node_name: str
     ) -> Mapping[NodeOutput, Sequence[NodeInput]]:
-        """
-        Returns a Dict[NodeOutput, List[NodeInput]] that
+        """Returns a Dict[NodeOutput, List[NodeInput]] that
         represents all the downstream inputs for each output in the
         dictionary.
         """
