@@ -1,5 +1,4 @@
 import re
-import warnings
 from enum import Enum
 from typing import (
     TYPE_CHECKING,
@@ -22,12 +21,13 @@ import dagster._seven as seven
 from dagster._annotations import PublicAttr, public
 from dagster._core.definitions.data_version import DataVersion
 from dagster._core.storage.tags import MULTIDIMENSIONAL_PARTITION_PREFIX, SYSTEM_TAG_PREFIX
-from dagster._serdes import DefaultNamedTupleSerializer, whitelist_for_serdes
-from dagster._utils import last_file_comp
+from dagster._serdes import whitelist_for_serdes
+from dagster._serdes.serdes import NamedTupleSerializer
 from dagster._utils.backcompat import experimental_class_param_warning
 
 from .metadata import (
     MetadataEntry,
+    MetadataFieldSerializer,
     MetadataMapping,
     MetadataValue,
     RawMetadataValue,
@@ -92,10 +92,10 @@ class AssetKey(NamedTuple("_AssetKey", [("path", PublicAttr[Sequence[str]])])):
         return super(AssetKey, cls).__new__(cls, path=path)
 
     def __str__(self):
-        return "AssetKey({})".format(self.path)
+        return f"AssetKey({self.path})"
 
     def __repr__(self):
-        return "AssetKey({})".format(self.path)
+        return f"AssetKey({self.path})"
 
     def __hash__(self):
         return hash(tuple(self.path))
@@ -111,15 +111,11 @@ class AssetKey(NamedTuple("_AssetKey", [("path", PublicAttr[Sequence[str]])])):
         return True
 
     def to_string(self) -> str:
-        """
-        E.g. '["first_component", "second_component"]'.
-        """
+        """E.g. '["first_component", "second_component"]'."""
         return seven.json.dumps(self.path)
 
     def to_user_string(self) -> str:
-        """
-        E.g. "first_component/second_component".
-        """
+        """E.g. "first_component/second_component"."""
         return ASSET_KEY_DELIMITER.join(self.path)
 
     def to_python_identifier(self, suffix: Optional[str] = None) -> str:
@@ -180,8 +176,7 @@ class AssetKey(NamedTuple("_AssetKey", [("path", PublicAttr[Sequence[str]])])):
 
 
 class AssetKeyPartitionKey(NamedTuple):
-    """
-    An AssetKey with an (optional) partition key. Refers either to a non-partitioned asset or a
+    """An AssetKey with an (optional) partition key. Refers either to a non-partitioned asset or a
     partition of a partitioned asset.
     """
 
@@ -193,6 +188,21 @@ CoercibleToAssetKey = Union[AssetKey, str, Sequence[str]]
 CoercibleToAssetKeyPrefix = Union[str, Sequence[str]]
 
 
+def check_opt_coercible_to_asset_key_prefix_param(
+    prefix: Optional[CoercibleToAssetKeyPrefix], param_name: str
+) -> Optional[Sequence[str]]:
+    if prefix is None:
+        return prefix
+    elif isinstance(prefix, str):
+        return [prefix]
+    elif isinstance(prefix, list):
+        return prefix
+    else:
+        raise check.ParameterCheckError(
+            f'Param "{param_name}" is not a string or a sequence of strings'
+        )
+
+
 DynamicAssetKey = Callable[["OutputContext"], Optional[AssetKey]]
 
 
@@ -225,7 +235,8 @@ class Output(Generic[T]):
         output_name (Optional[str]): Name of the corresponding out. (default:
             "result")
         metadata_entries (Optional[MetadataEntry]):
-            (Experimental) A set of metadata entries to attach to events related to this Output.
+            (Deprecated) A set of metadata entries to attach to events related to this Output. Use
+            `metadata` instead.
         metadata (Optional[Dict[str, Union[str, float, int, MetadataValue]]]):
             Arbitrary metadata about the failure.  Keys are displayed string labels, and values are
             one of the following: string, float, int, JSON-serializable dict, JSON-serializable
@@ -242,22 +253,19 @@ class Output(Generic[T]):
         metadata: Optional[Mapping[str, RawMetadataValue]] = None,
         data_version: Optional[DataVersion] = None,
     ):
-        metadata = check.opt_mapping_param(metadata, "metadata", key_type=str)
-        metadata_entries = check.opt_sequence_param(
-            metadata_entries,
-            "metadata_entries",
-            of_type=MetadataEntry,
-        )
         self._value = value
         self._output_name = check.str_param(output_name, "output_name")
-        self._metadata_entries = normalize_metadata(metadata, metadata_entries)
         if data_version is not None:
             experimental_class_param_warning("data_version", "Output")
         self._data_version = check.opt_inst_param(data_version, "data_version", DataVersion)
+        self._metadata = normalize_metadata(
+            check.opt_mapping_param(metadata, "metadata", key_type=str),
+            check.opt_sequence_param(metadata_entries, "metadata_entries", of_type=MetadataEntry),
+        )
 
     @property
-    def metadata_entries(self) -> Sequence[MetadataEntry]:
-        return self._metadata_entries
+    def metadata(self) -> MetadataMapping:
+        return self._metadata
 
     @public
     @property
@@ -279,13 +287,12 @@ class Output(Generic[T]):
             isinstance(other, Output)
             and self.value == other.value
             and self.output_name == other.output_name
-            and self.metadata_entries == other.metadata_entries
+            and self.metadata == other.metadata
         )
 
 
 class DynamicOutput(Generic[T]):
-    """
-    Variant of :py:class:`Output <dagster.Output>` used to support
+    """Variant of :py:class:`Output <dagster.Output>` used to support
     dynamic mapping & collect. Each ``DynamicOutput`` produced by an op represents
     one item in a set that can be processed individually with ``map`` or gathered
     with ``collect``.
@@ -303,7 +310,8 @@ class DynamicOutput(Generic[T]):
             Name of the corresponding :py:class:`DynamicOut` defined on the op.
             (default: "result")
         metadata_entries (Optional[MetadataEntry]):
-            (Experimental) A set of metadata entries to attach to events related to this output.
+            (Deprecated) A set of metadata entries to attach to events related to this Output. Use
+            `metadata` instead.
         metadata (Optional[Dict[str, Union[str, float, int, MetadataValue]]]):
             Arbitrary metadata about the failure.  Keys are displayed string labels, and values are
             one of the following: string, float, int, JSON-serializable dict, JSON-serializable
@@ -318,18 +326,17 @@ class DynamicOutput(Generic[T]):
         metadata_entries: Optional[Sequence[MetadataEntry]] = None,
         metadata: Optional[Mapping[str, RawMetadataValue]] = None,
     ):
-        metadata = check.opt_mapping_param(metadata, "metadata", key_type=str)
-        metadata_entries = check.opt_sequence_param(
-            metadata_entries, "metadata_entries", of_type=MetadataEntry
-        )
         self._mapping_key = check_valid_name(check.str_param(mapping_key, "mapping_key"))
         self._output_name = check.str_param(output_name, "output_name")
-        self._metadata_entries = normalize_metadata(metadata, metadata_entries)
         self._value = value
+        self._metadata = normalize_metadata(
+            check.opt_mapping_param(metadata, "metadata", key_type=str),
+            check.opt_sequence_param(metadata_entries, "metadata_entries", of_type=MetadataEntry),
+        )
 
     @property
-    def metadata_entries(self) -> Sequence[MetadataEntry]:
-        return self._metadata_entries
+    def metadata(self) -> Mapping[str, MetadataValue]:
+        return self._metadata
 
     @public
     @property
@@ -352,18 +359,36 @@ class DynamicOutput(Generic[T]):
             and self.value == other.value
             and self.output_name == other.output_name
             and self.mapping_key == other.mapping_key
-            and self.metadata_entries == other.metadata_entries
+            and self.metadata == other.metadata
         )
 
 
-@whitelist_for_serdes
+# This is a temporary function to consolidate metadata normalization logic on event classes where
+# we've had to mangle the signature for backcompat reasons. Can be deleted when we remove
+# `metadata_entries`/`MetadataEntry`.
+def _normalize_event_metadata(
+    metadata: Optional[Union[Mapping[str, RawMetadataValue], Sequence[MetadataEntry]]] = None,
+    metadata_entries: Optional[Sequence[MetadataEntry]] = None,
+) -> MetadataMapping:
+    metadata_dict = metadata if isinstance(metadata, dict) else {}
+    metadata_entries = metadata if isinstance(metadata, list) else metadata_entries
+    return normalize_metadata(
+        check.opt_mapping_param(metadata_dict, "metadata_dict", key_type=str),
+        check.opt_sequence_param(metadata_entries, "metadata_entries", of_type=MetadataEntry),
+    )
+
+
+@whitelist_for_serdes(
+    storage_field_names={"metadata": "metadata_entries"},
+    field_serializers={"metadata": MetadataFieldSerializer},
+)
 class AssetObservation(
     NamedTuple(
         "_AssetObservation",
         [
             ("asset_key", PublicAttr[AssetKey]),
             ("description", PublicAttr[Optional[str]]),
-            ("metadata_entries", Sequence[MetadataEntry]),
+            ("metadata", PublicAttr[Mapping[str, MetadataValue]]),
             ("partition", PublicAttr[Optional[str]]),
             ("tags", PublicAttr[Mapping[str, str]]),
         ],
@@ -373,7 +398,8 @@ class AssetObservation(
 
     Args:
         asset_key (Union[str, List[str], AssetKey]): A key to identify the asset.
-        metadata_entries (Optional[List[MetadataEntry]]): Arbitrary metadata about the asset.
+        metadata_entries (Optional[List[MetadataEntry]]): (Deprecated) Arbitrary metadata about the
+            asset. Use `metadata` instead.
         partition (Optional[str]): The name of a partition of the asset that the metadata
             corresponds to.
         tags (Optional[Mapping[str, str]]): A mapping containing system-populated tags for the
@@ -386,18 +412,18 @@ class AssetObservation(
 
     def __new__(
         cls,
-        asset_key: Union[Sequence[str], AssetKey, str],
+        asset_key: CoercibleToAssetKey,
         description: Optional[str] = None,
-        metadata_entries: Optional[Sequence[MetadataEntry]] = None,
+        metadata: Optional[Union[Mapping[str, RawMetadataValue], Sequence[MetadataEntry]]] = None,
         partition: Optional[str] = None,
         tags: Optional[Mapping[str, str]] = None,
-        metadata: Optional[Mapping[str, RawMetadataValue]] = None,
+        metadata_entries: Optional[Sequence[MetadataEntry]] = None,
     ):
         if isinstance(asset_key, AssetKey):
             check.inst_param(asset_key, "asset_key", AssetKey)
         elif isinstance(asset_key, str):
             asset_key = AssetKey(parse_asset_key_string(asset_key))
-        elif isinstance(asset_key, Sequence):
+        else:
             check.sequence_param(asset_key, "asset_key", of_type=str)
             asset_key = AssetKey(asset_key)
 
@@ -408,18 +434,13 @@ class AssetObservation(
                 "The tags argument is reserved for system-populated tags."
             )
 
-        metadata = check.opt_mapping_param(metadata, "metadata", key_type=str)
-        metadata_entries = check.opt_sequence_param(
-            metadata_entries, "metadata_entries", of_type=MetadataEntry
-        )
+        metadata = _normalize_event_metadata(metadata, metadata_entries)
 
         return super(AssetObservation, cls).__new__(
             cls,
             asset_key=asset_key,
             description=check.opt_str_param(description, "description"),
-            metadata_entries=cast(
-                List[MetadataEntry], normalize_metadata(metadata, metadata_entries)
-            ),
+            metadata=metadata,
             tags=tags,
             partition=check.opt_str_param(partition, "partition"),
         )
@@ -429,14 +450,33 @@ class AssetObservation(
         return " ".join(self.asset_key.path)
 
 
-@whitelist_for_serdes
+UNDEFINED_ASSET_KEY_PATH = ["__undefined__"]
+
+
+class AssetMaterializationSerializer(NamedTupleSerializer):
+    # There are old `Materialization` objects in storage. We set the default value for asset key to
+    # be `AssetKey(["__undefined__"])` to ensure that we can load these objects, without needing to
+    # allow for the construction of new `AssetMaterialization` objects with no defined AssetKey.
+    def before_unpack(self, **raw_dict: Any) -> Any:
+        # cover both the case where "asset_key" is not present at all and where it is None
+        if raw_dict.get("asset_key") is None:
+            raw_dict["asset_key"] = {"__class__": "AssetKey", "path": UNDEFINED_ASSET_KEY_PATH}
+        return raw_dict
+
+
+@whitelist_for_serdes(
+    old_storage_names={"Materialization"},
+    serializer=AssetMaterializationSerializer,
+    storage_field_names={"metadata": "metadata_entries"},
+    field_serializers={"metadata": MetadataFieldSerializer},
+)
 class AssetMaterialization(
     NamedTuple(
         "_AssetMaterialization",
         [
             ("asset_key", PublicAttr[AssetKey]),
             ("description", PublicAttr[Optional[str]]),
-            ("metadata_entries", Sequence[MetadataEntry]),
+            ("metadata", PublicAttr[Mapping[str, MetadataValue]]),
             ("partition", PublicAttr[Optional[str]]),
             ("tags", Optional[Mapping[str, str]]),
         ],
@@ -457,8 +497,8 @@ class AssetMaterialization(
         asset_key (Union[str, List[str], AssetKey]): A key to identify the materialized asset across
             job runs
         description (Optional[str]): A longer human-readable description of the materialized value.
-        metadata_entries (Optional[List[MetadataEntry]]): Arbitrary
-            metadata about the materialized value.
+        metadata_entries (Optional[List[MetadataEntry]]): (Deprecated) Arbitrary
+            metadata about the materialized value. Use `metadata` instead.
         partition (Optional[str]): The name of the partition
             that was materialized.
         tags (Optional[Mapping[str, str]]): A mapping containing system-populated tags for the
@@ -473,10 +513,10 @@ class AssetMaterialization(
         cls,
         asset_key: CoercibleToAssetKey,
         description: Optional[str] = None,
-        metadata_entries: Optional[Sequence[MetadataEntry]] = None,
+        metadata: Optional[Union[Mapping[str, RawMetadataValue], Sequence[MetadataEntry]]] = None,
         partition: Optional[str] = None,
         tags: Optional[Mapping[str, str]] = None,
-        metadata: Optional[Mapping[str, RawMetadataValue]] = None,
+        metadata_entries: Optional[Sequence[MetadataEntry]] = None,
     ):
         from dagster._core.definitions.multi_dimensional_partitions import MultiPartitionKey
 
@@ -484,7 +524,7 @@ class AssetMaterialization(
             check.inst_param(asset_key, "asset_key", AssetKey)
         elif isinstance(asset_key, str):
             asset_key = AssetKey(parse_asset_key_string(asset_key))
-        elif isinstance(asset_key, Sequence):
+        else:
             check.sequence_param(asset_key, "asset_key", of_type=str)
             asset_key = AssetKey(asset_key)
 
@@ -496,10 +536,7 @@ class AssetMaterialization(
                 " AssetMaterializations. The tags argument is reserved for system-populated tags."
             )
 
-        metadata = check.opt_mapping_param(metadata, "metadata", key_type=str)
-        metadata_entries = check.opt_sequence_param(
-            metadata_entries, "metadata_entries", of_type=MetadataEntry
-        )
+        metadata = _normalize_event_metadata(metadata, metadata_entries)
 
         partition = check.opt_str_param(partition, "partition")
 
@@ -518,7 +555,7 @@ class AssetMaterialization(
             cls,
             asset_key=asset_key,
             description=check.opt_str_param(description, "description"),
-            metadata_entries=normalize_metadata(metadata, metadata_entries),
+            metadata=metadata,
             tags=tags,
             partition=partition,
         )
@@ -549,119 +586,11 @@ class AssetMaterialization(
             metadata={"path": MetadataValue.path(path)},
         )
 
-    @public
-    @property
-    def metadata(self) -> MetadataMapping:
-        return {entry.label: entry.value for entry in self.metadata_entries}
 
-
-class MaterializationSerializer(DefaultNamedTupleSerializer):
-    @classmethod
-    def value_from_unpacked(cls, unpacked_dict, klass):
-        # override the default `from_storage_dict` implementation in order to skip the deprecation
-        # warning for historical Materialization events, loaded from event_log storage
-        return Materialization(skip_deprecation_warning=True, **unpacked_dict)
-
-
-@whitelist_for_serdes(serializer=MaterializationSerializer)
-class Materialization(
-    NamedTuple(
-        "_Materialization",
-        [
-            ("label", str),
-            ("description", Optional[str]),
-            ("metadata_entries", Sequence[MetadataEntry]),
-            ("asset_key", AssetKey),
-            ("partition", Optional[str]),
-        ],
-    )
-):
-    """Event indicating that an op has materialized a value.
-
-    Solid compute functions may yield events of this type whenever they wish to indicate to the
-    Dagster framework (and the end user) that they have produced a materialized value as a
-    side effect of computation. Unlike outputs, materializations can not be passed to other ops,
-    and their persistence is controlled by op logic, rather than by the Dagster framework.
-
-    Solid authors should use these events to organize metadata about the side effects of their
-    computations to enable downstream tooling like artifact catalogues and diff tools.
-
-    Args:
-        label (str): A short display name for the materialized value.
-        description (Optional[str]): A longer human-radable description of the materialized value.
-        metadata_entries (Optional[List[MetadataEntry]]): Arbitrary metadata about the
-            materialized value.
-        asset_key (Optional[Union[str, AssetKey]]): An optional parameter to identify the materialized asset
-            across runs
-        partition (Optional[str]): The name of the partition that was materialized.
-    """
-
-    def __new__(
-        cls,
-        label: Optional[str] = None,
-        description: Optional[str] = None,
-        metadata_entries: Optional[Sequence[MetadataEntry]] = None,
-        asset_key: Optional[Union[str, AssetKey]] = None,
-        partition: Optional[str] = None,
-        skip_deprecation_warning: Optional[bool] = False,
-    ):
-        if asset_key and isinstance(asset_key, str):
-            asset_key = AssetKey(parse_asset_key_string(asset_key))
-        else:
-            check.opt_inst_param(asset_key, "asset_key", AssetKey)
-
-        asset_key = cast(AssetKey, asset_key)
-        if not label:
-            check.param_invariant(
-                asset_key and asset_key.path,
-                "label",
-                "Either label or asset_key with a path must be provided",
-            )
-            label = asset_key.to_string()
-
-        if not skip_deprecation_warning:
-            warnings.warn("`Materialization` is deprecated; use `AssetMaterialization` instead.")
-
-        metadata_entries = check.opt_sequence_param(
-            metadata_entries, "metadata_entries", of_type=MetadataEntry
-        )
-
-        return super(Materialization, cls).__new__(
-            cls,
-            label=check.str_param(label, "label"),
-            description=check.opt_str_param(description, "description"),
-            metadata_entries=check.opt_sequence_param(
-                metadata_entries, "metadata_entries", of_type=MetadataEntry
-            ),
-            asset_key=asset_key,
-            partition=check.opt_str_param(partition, "partition"),
-        )
-
-    @staticmethod
-    def file(
-        path: str,
-        description: Optional[str] = None,
-        asset_key: Optional[Union[str, AssetKey]] = None,
-    ) -> "Materialization":
-        """Static constructor for standard materializations corresponding to files on disk.
-
-        Args:
-            path (str): The path to the file.
-            description (Optional[str]): A human-readable description of the materialization.
-        """
-        return Materialization(
-            label=last_file_comp(path),
-            description=description,
-            metadata_entries=[MetadataEntry("path", value=MetadataValue.path(path))],
-            asset_key=asset_key,
-        )
-
-    @property
-    def tags(self) -> Mapping[str, str]:
-        return {}
-
-
-@whitelist_for_serdes
+@whitelist_for_serdes(
+    storage_field_names={"metadata": "metadata_entries"},
+    field_serializers={"metadata": MetadataFieldSerializer},
+)
 class ExpectationResult(
     NamedTuple(
         "_ExpectationResult",
@@ -669,7 +598,7 @@ class ExpectationResult(
             ("success", PublicAttr[bool]),
             ("label", PublicAttr[Optional[str]]),
             ("description", PublicAttr[Optional[str]]),
-            ("metadata_entries", Sequence[MetadataEntry]),
+            ("metadata", PublicAttr[Mapping[str, MetadataValue]]),
         ],
     )
 ):
@@ -683,8 +612,8 @@ class ExpectationResult(
         success (bool): Whether the expectation passed or not.
         label (Optional[str]): Short display name for expectation. Defaults to "result".
         description (Optional[str]): A longer human-readable description of the expectation.
-        metadata_entries (Optional[List[MetadataEntry]]): Arbitrary metadata about the
-            expectation.
+        metadata_entries (Optional[List[MetadataEntry]]): (Deprecated) Arbitrary metadata about the
+            expectation. Use `metadata` instead.
         metadata (Optional[Dict[str, RawMetadataValue]]):
             Arbitrary metadata about the failure.  Keys are displayed string labels, and values are
             one of the following: string, float, int, JSON-serializable dict, JSON-serializable
@@ -696,25 +625,24 @@ class ExpectationResult(
         success: bool,
         label: Optional[str] = None,
         description: Optional[str] = None,
+        metadata: Optional[Union[Mapping[str, RawMetadataValue], Sequence[MetadataEntry]]] = None,
         metadata_entries: Optional[Sequence[MetadataEntry]] = None,
-        metadata: Optional[Mapping[str, RawMetadataValue]] = None,
     ):
-        metadata_entries = check.opt_sequence_param(
-            metadata_entries, "metadata_entries", of_type=MetadataEntry
-        )
-        metadata = check.opt_mapping_param(metadata, "metadata", key_type=str)
+        metadata = _normalize_event_metadata(metadata, metadata_entries)
 
         return super(ExpectationResult, cls).__new__(
             cls,
             success=check.bool_param(success, "success"),
             label=check.opt_str_param(label, "label", "result"),
             description=check.opt_str_param(description, "description"),
-            metadata_entries=cast(
-                List[MetadataEntry], normalize_metadata(metadata, metadata_entries)
-            ),
+            metadata=metadata,
         )
 
 
+@whitelist_for_serdes(
+    storage_field_names={"metadata": "metadata_entries"},
+    field_serializers={"metadata": MetadataFieldSerializer},
+)
 @whitelist_for_serdes
 class TypeCheck(
     NamedTuple(
@@ -722,7 +650,7 @@ class TypeCheck(
         [
             ("success", PublicAttr[bool]),
             ("description", PublicAttr[Optional[str]]),
-            ("metadata_entries", PublicAttr[Sequence[MetadataEntry]]),
+            ("metadata", PublicAttr[Mapping[str, MetadataValue]]),
         ],
     )
 ):
@@ -738,8 +666,8 @@ class TypeCheck(
     Args:
         success (bool): ``True`` if the type check succeeded, ``False`` otherwise.
         description (Optional[str]): A human-readable description of the type check.
-        metadata_entries (Optional[List[MetadataEntry]]): Arbitrary metadata about the
-            type check.
+        metadata_entries (Optional[List[MetadataEntry]]): (Deprecated) Arbitrary metadata about the
+            type check. Use `metadata` instead.
         metadata (Optional[Dict[str, RawMetadataValue]]):
             Arbitrary metadata about the failure.  Keys are displayed string labels, and values are
             one of the following: string, float, int, JSON-serializable dict, JSON-serializable
@@ -750,21 +678,16 @@ class TypeCheck(
         cls,
         success: bool,
         description: Optional[str] = None,
+        metadata: Optional[Union[Mapping[str, RawMetadataValue], Sequence[MetadataEntry]]] = None,
         metadata_entries: Optional[Sequence[MetadataEntry]] = None,
-        metadata: Optional[Mapping[str, RawMetadataValue]] = None,
     ):
-        metadata_entries = check.opt_sequence_param(
-            metadata_entries, "metadata_entries", of_type=MetadataEntry
-        )
-        metadata = check.opt_mapping_param(metadata, "metadata", key_type=str)
+        metadata = _normalize_event_metadata(metadata, metadata_entries)
 
         return super(TypeCheck, cls).__new__(
             cls,
             success=check.bool_param(success, "success"),
             description=check.opt_str_param(description, "description"),
-            metadata_entries=cast(
-                List[MetadataEntry], normalize_metadata(metadata, metadata_entries)
-            ),
+            metadata=metadata,
         )
 
 
@@ -777,8 +700,8 @@ class Failure(Exception):
 
     Args:
         description (Optional[str]): A human-readable description of the failure.
-        metadata_entries (Optional[List[MetadataEntry]]): Arbitrary metadata about the
-            failure.
+        metadata_entries (Optional[List[MetadataEntry]]): (Deprecated) Arbitrary metadata about the
+            failure. Use `metadata` instead.
         metadata (Optional[Dict[str, RawMetadataValue]]):
             Arbitrary metadata about the failure.  Keys are displayed string labels, and values are
             one of the following: string, float, int, JSON-serializable dict, JSON-serializable
@@ -795,20 +718,17 @@ class Failure(Exception):
         metadata: Optional[Mapping[str, RawMetadataValue]] = None,
         allow_retries: Optional[bool] = None,
     ):
-        metadata_entries = check.opt_sequence_param(
-            metadata_entries, "metadata_entries", of_type=MetadataEntry
-        )
-        metadata = check.opt_mapping_param(metadata, "metadata", key_type=str)
-
         super(Failure, self).__init__(description)
         self.description = check.opt_str_param(description, "description")
-        self.metadata_entries = normalize_metadata(metadata, metadata_entries)
+        self.metadata = normalize_metadata(
+            check.opt_mapping_param(metadata, "metadata", key_type=str),
+            check.opt_sequence_param(metadata_entries, "metadata_entries", of_type=MetadataEntry),
+        )
         self.allow_retries = check.opt_bool_param(allow_retries, "allow_retries", True)
 
 
 class RetryRequested(Exception):
-    """
-    An exception to raise from an op to indicate that it should be retried.
+    """An exception to raise from an op to indicate that it should be retried.
 
     Args:
         max_retries (Optional[int]):
@@ -943,4 +863,4 @@ class HookExecutionResult(
         )
 
 
-UserEvent = Union[Materialization, AssetMaterialization, AssetObservation, ExpectationResult]
+UserEvent = Union[AssetMaterialization, AssetObservation, ExpectationResult]
