@@ -39,6 +39,7 @@ class ExternalAssetGraph(AssetGraph):
         repo_handles_by_key: Mapping[AssetKey, RepositoryHandle],
         job_names_by_key: Mapping[AssetKey, Sequence[str]],
         code_versions_by_key: Mapping[AssetKey, Optional[str]],
+        is_observable_by_key: Mapping[AssetKey, bool],
     ):
         super().__init__(
             asset_dep_graph=asset_dep_graph,
@@ -49,21 +50,22 @@ class ExternalAssetGraph(AssetGraph):
             freshness_policies_by_key=freshness_policies_by_key,
             required_multi_asset_sets_by_key=required_multi_asset_sets_by_key,
             code_versions_by_key=code_versions_by_key,
+            is_observable_by_key=is_observable_by_key,
         )
         self._repo_handles_by_key = repo_handles_by_key
-        self._job_names_by_key = job_names_by_key
+        self._materialization_job_names_by_key = job_names_by_key
 
     @classmethod
     def from_workspace(cls, context: IWorkspace) -> "ExternalAssetGraph":
-        repo_locations = (
-            location_entry.repository_location
+        code_locations = (
+            location_entry.code_location
             for location_entry in context.get_workspace_snapshot().values()
-            if location_entry.repository_location
+            if location_entry.code_location
         )
         repos = (
             repo
-            for repo_location in repo_locations
-            for repo in repo_location.get_repositories().values()
+            for code_location in code_locations
+            for repo in code_location.get_repositories().values()
         )
         repo_handle_external_asset_nodes: Sequence[Tuple[RepositoryHandle, "ExternalAssetNode"]] = [
             (repo.handle, external_asset_node)
@@ -120,10 +122,15 @@ class ExternalAssetGraph(AssetGraph):
             node.asset_key for _, node in repo_handle_external_asset_nodes if not node.is_source
         }
 
+        is_observable_by_key = {key: False for key in all_non_source_keys}
+
         for repo_handle, node in repo_handle_external_asset_nodes:
             if node.is_source:
+                # We need to set this even if the node is a regular asset in another code location.
+                # `is_observable` will only ever be consulted in the source asset context.
+                is_observable_by_key[node.asset_key] = node.is_observable
                 if node.asset_key in all_non_source_keys:
-                    # one repo's source is another repo's non-source
+                    # one location's source is another location's non-source
                     continue
 
                 source_asset_keys.add(node.asset_key)
@@ -169,6 +176,7 @@ class ExternalAssetGraph(AssetGraph):
             repo_handles_by_key=repo_handles_by_key,
             job_names_by_key=job_names_by_key,
             code_versions_by_key=code_versions_by_key,
+            is_observable_by_key=is_observable_by_key,
         )
 
     @property
@@ -178,8 +186,14 @@ class ExternalAssetGraph(AssetGraph):
     def get_repository_handle(self, asset_key: AssetKey) -> RepositoryHandle:
         return self._repo_handles_by_key[asset_key]
 
-    def get_job_names(self, asset_key: AssetKey) -> Iterable[str]:
-        return self._job_names_by_key[asset_key]
+    def get_materialization_job_names(self, asset_key: AssetKey) -> Iterable[str]:
+        """Returns the names of jobs that materialize this asset."""
+        return self._materialization_job_names_by_key[asset_key]
 
-    def get_asset_keys_for_job(self, job_name: str) -> Sequence[AssetKey]:
-        return [k for k in self.all_asset_keys if job_name in self.get_job_names(k)]
+    def get_materialization_asset_keys_for_job(self, job_name: str) -> Sequence[AssetKey]:
+        """Returns asset keys that are targeted for materialization in the given job."""
+        return [
+            k
+            for k in self.non_source_asset_keys
+            if job_name in self.get_materialization_job_names(k)
+        ]
