@@ -4,7 +4,9 @@ import pendulum
 
 import dagster._check as check
 from dagster._annotations import PublicAttr, experimental
+from dagster._core.decorator_utils import has_at_least_one_parameter
 from dagster._core.definitions.asset_selection import AssetSelection
+from dagster._core.definitions.data_time import CachingDataTimeResolver
 from dagster._core.definitions.events import AssetKey
 from dagster._core.definitions.freshness_policy import FreshnessPolicy
 from dagster._core.errors import (
@@ -15,11 +17,11 @@ from dagster._core.errors import (
 )
 from dagster._core.instance import DagsterInstance
 from dagster._serdes import (
-    deserialize_json_to_dagster_namedtuple,
-    serialize_dagster_namedtuple,
+    serialize_value,
     whitelist_for_serdes,
 )
 from dagster._serdes.errors import DeserializationError
+from dagster._serdes.serdes import deserialize_value
 from dagster._seven import JSONDecodeError
 
 from ..decorator_utils import get_function_params
@@ -29,7 +31,6 @@ from .sensor_definition import (
     SensorEvaluationContext,
     SensorType,
     SkipReason,
-    has_at_least_one_parameter,
 )
 
 if TYPE_CHECKING:
@@ -54,8 +55,8 @@ class FreshnessPolicySensorCursor(
     @staticmethod
     def is_valid(json_str: str) -> bool:
         try:
-            obj = deserialize_json_to_dagster_namedtuple(json_str)
-            return isinstance(obj, FreshnessPolicySensorCursor)
+            deserialize_value(json_str, FreshnessPolicySensorCursor)
+            return True
         except (JSONDecodeError, DeserializationError):
             return False
 
@@ -72,11 +73,11 @@ class FreshnessPolicySensorCursor(
         return {AssetKey.from_user_string(k): v for k, v in self.minutes_late_by_key_str.items()}
 
     def to_json(self) -> str:
-        return serialize_dagster_namedtuple(cast(NamedTuple, self))
+        return serialize_value(cast(NamedTuple, self))
 
     @staticmethod
     def from_json(json_str: str) -> "FreshnessPolicySensorCursor":
-        return cast(FreshnessPolicySensorCursor, deserialize_json_to_dagster_namedtuple(json_str))
+        return deserialize_value(json_str, FreshnessPolicySensorCursor)
 
 
 class FreshnessPolicySensorContext(
@@ -139,8 +140,7 @@ def build_freshness_policy_sensor_context(
     previous_minutes_late: Optional[float] = None,
     instance: Optional[DagsterInstance] = None,
 ) -> FreshnessPolicySensorContext:
-    """
-    Builds freshness policy sensor context from provided parameters.
+    """Builds freshness policy sensor context from provided parameters.
 
     This function can be used to provide the context argument when directly invoking a function
     decorated with `@freshness_policy_sensor`, such as when writing unit tests.
@@ -176,8 +176,7 @@ def build_freshness_policy_sensor_context(
 
 
 class FreshnessPolicySensorDefinition(SensorDefinition):
-    """
-    Define a sensor that reacts to the status of a given set of asset freshness policies,
+    """Define a sensor that reacts to the status of a given set of asset freshness policies,
     where the decorated function will be evaluated on every sensor tick.
 
     Args:
@@ -229,8 +228,11 @@ class FreshnessPolicySensorDefinition(SensorDefinition):
                 return
 
             evaluation_time = pendulum.now("UTC")
-            instance_queryer = CachingInstanceQueryer(context.instance)
             asset_graph = context.repository_def.asset_graph
+            instance_queryer = CachingInstanceQueryer(context.instance)
+            data_time_resolver = CachingDataTimeResolver(
+                instance_queryer=instance_queryer, asset_graph=asset_graph
+            )
             monitored_keys = asset_selection.resolve(asset_graph)
 
             # get the previous status from the cursor
@@ -245,9 +247,8 @@ class FreshnessPolicySensorDefinition(SensorDefinition):
                     continue
 
                 # get the current minutes_late value for this asset
-                minutes_late_by_key[asset_key] = instance_queryer.get_current_minutes_late_for_key(
+                minutes_late_by_key[asset_key] = data_time_resolver.get_current_minutes_late(
                     evaluation_time=evaluation_time,
-                    asset_graph=asset_graph,
                     asset_key=asset_key,
                 )
 
@@ -341,8 +342,7 @@ def freshness_policy_sensor(
     description: Optional[str] = None,
     default_status: DefaultSensorStatus = DefaultSensorStatus.STOPPED,
 ) -> Callable[[Callable[[FreshnessPolicySensorContext], None]], FreshnessPolicySensorDefinition,]:
-    """
-    Define a sensor that reacts to the status of a given set of asset freshness policies, where the
+    """Define a sensor that reacts to the status of a given set of asset freshness policies, where the
     decorated function will be evaluated on every tick for each asset in the selection that has a
     FreshnessPolicy defined.
 
