@@ -18,12 +18,12 @@ from typing import (
 )
 
 from toposort import CircularDependencyError, toposort_flatten
+from typing_extensions import Self
 
 import dagster._check as check
 from dagster._annotations import public
 from dagster._core.definitions.config import ConfigMapping
 from dagster._core.definitions.definition_config_schema import IDefinitionConfigSchema
-from dagster._core.definitions.metadata import MetadataEntry
 from dagster._core.definitions.policy import RetryPolicy
 from dagster._core.definitions.resource_definition import ResourceDefinition
 from dagster._core.errors import DagsterInvalidDefinitionError
@@ -35,9 +35,9 @@ from dagster._core.types.dagster_type import (
 )
 
 from .dependency import (
+    DependencyMapping,
     DependencyStructure,
     GraphNode,
-    IDependencyDefinition,
     Node,
     NodeHandle,
     NodeInput,
@@ -48,7 +48,7 @@ from .hook_definition import HookDefinition
 from .input import FanInInputPointer, InputDefinition, InputMapping, InputPointer
 from .logger_definition import LoggerDefinition
 from .metadata import RawMetadataValue
-from .node_container import create_execution_structure, validate_dependency_dict
+from .node_container import create_execution_structure, normalize_dependency_dict
 from .node_definition import NodeDefinition
 from .output import OutputDefinition, OutputMapping
 from .resource_requirement import ResourceRequirement
@@ -88,9 +88,7 @@ def _check_node_defs_arg(
                 )
             )
         else:
-            raise DagsterInvalidDefinitionError(
-                "Invalid item in node list: {item}".format(item=repr(node_def))
-            )
+            raise DagsterInvalidDefinitionError(f"Invalid item in node list: {repr(node_def)}")
 
     return node_defs
 
@@ -178,7 +176,7 @@ class GraphDefinition(NodeDefinition):
 
     _node_defs: Sequence[NodeDefinition]
     _dagster_type_dict: Mapping[str, DagsterType]
-    _dependencies: Mapping[Union[str, NodeInvocation], Mapping[str, IDependencyDefinition]]
+    _dependencies: DependencyMapping[NodeInvocation]
     _dependency_structure: DependencyStructure
     _node_dict: Mapping[str, Node]
     _input_mappings: Sequence[InputMapping]
@@ -199,7 +197,7 @@ class GraphDefinition(NodeDefinition):
         description: Optional[str] = None,
         node_defs: Optional[Sequence[NodeDefinition]] = None,
         dependencies: Optional[
-            Mapping[Union[str, NodeInvocation], Mapping[str, IDependencyDefinition]]
+            Union[DependencyMapping[str], DependencyMapping[NodeInvocation]]
         ] = None,
         input_mappings: Optional[Sequence[InputMapping]] = None,
         output_mappings: Optional[Sequence[OutputMapping]] = None,
@@ -209,7 +207,10 @@ class GraphDefinition(NodeDefinition):
         **kwargs: object,
     ):
         self._node_defs = _check_node_defs_arg(name, node_defs)
-        self._dependencies = validate_dependency_dict(dependencies)
+
+        # `dependencies` will be converted to `dependency_structure` and `node_dict`, which may
+        # alternatively be passed directly (useful when copying)
+        self._dependencies = normalize_dependency_dict(dependencies)
         self._dependency_structure, self._node_dict = create_execution_structure(
             self._node_defs, self._dependencies, graph_definition=self
         )
@@ -324,7 +325,7 @@ class GraphDefinition(NodeDefinition):
         check.str_param(name, "name")
         check.invariant(
             name in self._node_dict,
-            "{graph_name} has no op named {name}.".format(graph_name=self._name, name=name),
+            f"{self._name} has no op named {name}.",
         )
 
         return self._node_dict[name]
@@ -362,8 +363,7 @@ class GraphDefinition(NodeDefinition):
         for node in self.node_dict.values():
             cur_node_handle = NodeHandle(node.name, parent_node_handle)
             if isinstance(node, GraphNode):
-                graph_def = node.definition.ensure_graph_def()
-                yield from graph_def.iterate_node_handles(cur_node_handle)
+                yield from node.definition.iterate_node_handles(cur_node_handle)
             yield cur_node_handle
 
     @public
@@ -470,9 +470,7 @@ class GraphDefinition(NodeDefinition):
         return mapped_node.definition.input_has_default(mapping.maps_to.input_name)
 
     @property
-    def dependencies(
-        self,
-    ) -> Mapping[Union[str, NodeInvocation], Mapping[str, IDependencyDefinition]]:
+    def dependencies(self) -> DependencyMapping[NodeInvocation]:
         return self._dependencies
 
     @property
@@ -498,6 +496,28 @@ class GraphDefinition(NodeDefinition):
             mapping.maps_to.input_name
         )
 
+    def copy(
+        self,
+        name: Optional[str] = None,
+        description: Optional[str] = None,
+        input_mappings: Optional[Sequence[InputMapping]] = None,
+        output_mappings: Optional[Sequence[OutputMapping]] = None,
+        config: Optional[ConfigMapping] = None,
+        tags: Optional[Mapping[str, str]] = None,
+        node_input_source_assets: Optional[Mapping[str, Mapping[str, "SourceAsset"]]] = None,
+    ) -> Self:
+        return GraphDefinition(
+            node_defs=self.node_defs,
+            dependencies=self.dependencies,
+            name=name or self.name,
+            description=description or self.description,
+            input_mappings=input_mappings or self._input_mappings,
+            output_mappings=output_mappings or self._output_mappings,
+            config=config or self.config_mapping,
+            tags=tags or self.tags,
+            node_input_source_assets=node_input_source_assets or self.node_input_source_assets,
+        )
+
     def copy_for_configured(
         self,
         name: str,
@@ -511,13 +531,9 @@ class GraphDefinition(NodeDefinition):
                 "configured.".format(graph_name=self.name)
             )
         config_mapping = cast(ConfigMapping, self.config_mapping)
-        return GraphDefinition(
+        return self.copy(
             name=name,
             description=check.opt_str_param(description, "description", default=self.description),
-            node_defs=self._node_defs,
-            dependencies=self._dependencies,
-            input_mappings=self._input_mappings,
-            output_mappings=self._output_mappings,
             config=ConfigMapping(
                 config_mapping.config_fn,
                 config_schema=config_schema,
@@ -547,10 +563,8 @@ class GraphDefinition(NodeDefinition):
         asset_layer: Optional["AssetLayer"] = None,
         input_values: Optional[Mapping[str, object]] = None,
         _asset_selection_data: Optional[AssetSelectionData] = None,
-        _metadata_entries: Optional[Sequence[MetadataEntry]] = None,
     ) -> "JobDefinition":
-        """
-        Make this graph in to an executable Job by providing remaining components required for execution.
+        """Make this graph in to an executable Job by providing remaining components required for execution.
 
         Args:
             name (Optional[str]):
@@ -627,7 +641,6 @@ class GraphDefinition(NodeDefinition):
             asset_layer=asset_layer,
             input_values=input_values,
             _subset_selection_data=_asset_selection_data,
-            _metadata_entries=_metadata_entries,
         ).get_job_def_for_subset_selection(op_selection)
 
     def coerce_to_job(self) -> "JobDefinition":
@@ -651,8 +664,7 @@ class GraphDefinition(NodeDefinition):
         run_id: Optional[str] = None,
         input_values: Optional[Mapping[str, object]] = None,
     ) -> "ExecuteInProcessResult":
-        """
-        Execute this graph in-process, collecting results in-memory.
+        """Execute this graph in-process, collecting results in-memory.
 
         Args:
             run_config (Optional[Mapping[str, Any]]):
@@ -799,7 +811,10 @@ class SubselectedGraphDefinition(GraphDefinition):
         parent_graph_def: GraphDefinition,
         node_defs: Optional[Sequence[NodeDefinition]],
         dependencies: Optional[
-            Mapping[Union[str, NodeInvocation], Mapping[str, IDependencyDefinition]]
+            Union[
+                DependencyMapping[str],
+                DependencyMapping[NodeInvocation],
+            ]
         ],
         input_mappings: Optional[Sequence[InputMapping]],
         output_mappings: Optional[Sequence[OutputMapping]],
