@@ -1,7 +1,7 @@
-import inspect
+from inspect import Parameter, Signature, isgeneratorfunction, signature
 from typing import Any, Callable, Mapping, NamedTuple, Optional, Sequence
 
-from dagster._seven import funcsigs, is_module_available
+from dagster._seven import is_module_available
 
 from .utils import NoValueSentinel
 
@@ -55,11 +55,11 @@ def _infer_output_description_from_docstring(fn: Callable) -> Optional[str]:
 
 
 def infer_output_props(fn: Callable) -> InferredOutputProps:
-    signature = funcsigs.signature(fn)
+    sig = signature(fn)
 
-    annotation = inspect.Parameter.empty
-    if not inspect.isgeneratorfunction(fn):
-        annotation = signature.return_annotation
+    annotation = Parameter.empty
+    if not isgeneratorfunction(fn):
+        annotation = sig.return_annotation
 
     return InferredOutputProps(
         annotation=annotation,
@@ -68,18 +68,18 @@ def infer_output_props(fn: Callable) -> InferredOutputProps:
 
 
 def has_explicit_return_type(fn: Callable) -> bool:
-    signature = funcsigs.signature(fn)
-    return signature.return_annotation is not funcsigs.Signature.empty
+    sig = signature(fn)
+    return sig.return_annotation is not Signature.empty
 
 
 def _infer_inputs_from_params(
-    params: Sequence[funcsigs.Parameter],
+    params: Sequence[Parameter],
     descriptions: Optional[Mapping[str, Optional[str]]] = None,
 ) -> Sequence[InferredInputProps]:
     _descriptions: Mapping[str, Optional[str]] = descriptions or {}
     input_defs = []
     for param in params:
-        if param.default is not funcsigs.Parameter.empty:
+        if param.default is not Parameter.empty:
             input_def = InferredInputProps(
                 param.name,
                 param.annotation,
@@ -99,8 +99,8 @@ def _infer_inputs_from_params(
 
 
 def infer_input_props(fn: Callable, context_arg_provided: bool) -> Sequence[InferredInputProps]:
-    signature = funcsigs.signature(fn)
-    params = list(signature.parameters.values())
+    sig = signature(fn)
+    params = list(sig.parameters.values())
     descriptions = _infer_input_description_from_docstring(fn)
     params_to_infer = params[1:] if context_arg_provided else params
     defs = _infer_inputs_from_params(params_to_infer, descriptions=descriptions)
