@@ -13,8 +13,7 @@ from dagster._utils.log import create_console_logger
 
 
 class DispatchingLogHandler(logging.Handler):
-    """
-    Proxies logging records to a set of downstream loggers which themselves will route to their own
+    """Proxies logging records to a set of downstream loggers which themselves will route to their own
     set of handlers.  Needed to bridge to the set of dagster logging utilities which were
     implemented as loggers rather than log handlers.
     """
@@ -38,9 +37,7 @@ class DispatchingLogHandler(logging.Handler):
 
 
 class CapturedLogHandler(logging.Handler):
-    """
-    Persist logging records to an IO stream controlled by the CapturedLogManager.
-    """
+    """Persist logging records to an IO stream controlled by the CapturedLogManager."""
 
     def __init__(self, write_stream: IO):
         self._write_stream = write_stream
@@ -57,8 +54,7 @@ class CapturedLogHandler(logging.Handler):
 
 
 class InstigationLogger(logging.Logger):
-    """
-    Logger exposed on the evaluation context of sensor/schedule evaluation functions.  This is tied
+    """Logger exposed on the evaluation context of sensor/schedule evaluation functions.  This is tied
     to the Python logging system by setting up a custom logging handler that writes JSON payloads
     representing the log events to the dagster-managed captured log manager.  These log events are
     persisted, using the given log_key, which is stored on the sensor/schedule tick. These logs can
@@ -119,9 +115,7 @@ class InstigationLogger(logging.Logger):
             record.msg = " - ".join([self._repository_name, self._name, message])
         return record
 
-    def makeRecord(  # pylint: disable=signature-differs
-        self, name, level, fn, lno, msg, args, exc_info, func, extra, sinfo
-    ):
+    def makeRecord(self, name, level, fn, lno, msg, args, exc_info, func, extra, sinfo):
         record = super().makeRecord(name, level, fn, lno, msg, args, exc_info, func, extra, sinfo)
         return self._annotate_record(record)
 
