@@ -6,19 +6,23 @@ from types import ModuleType
 from typing import Dict, Generator, Iterable, List, Optional, Sequence, Set, Tuple, Union
 
 import dagster._check as check
+from dagster._core.definitions.freshness_policy import FreshnessPolicy
 from dagster._core.errors import DagsterInvalidDefinitionError
 
 from .assets import AssetsDefinition
 from .cacheable_assets import CacheableAssetsDefinition
-from .events import AssetKey, CoercibleToAssetKeyPrefix
+from .events import (
+    AssetKey,
+    CoercibleToAssetKeyPrefix,
+    check_opt_coercible_to_asset_key_prefix_param,
+)
 from .source_asset import SourceAsset
 
 
 def _find_assets_in_module(
     module: ModuleType,
 ) -> Generator[Union[AssetsDefinition, SourceAsset, CacheableAssetsDefinition], None, None]:
-    """
-    Finds assets in the given module and adds them to the given sets of assets and source assets.
+    """Finds assets in the given module and adds them to the given sets of assets and source assets.
     """
     for attr in dir(module):
         value = getattr(module, attr)
@@ -34,8 +38,7 @@ def _find_assets_in_module(
 def assets_from_modules(
     modules: Iterable[ModuleType], extra_source_assets: Optional[Sequence[SourceAsset]] = None
 ) -> Tuple[Sequence[AssetsDefinition], Sequence[SourceAsset], Sequence[CacheableAssetsDefinition]]:
-    """
-    Constructs three lists, a list of assets, a list of source assets, and a list of cacheable
+    """Constructs three lists, a list of assets, a list of source assets, and a list of cacheable
     assets from the given modules.
 
     Args:
@@ -68,13 +71,17 @@ def assets_from_modules(
                             modules_str = ", ".join(
                                 set([asset_keys[key].__name__, module.__name__])
                             )
-                            error_str = f"Asset key {key} is defined multiple times. Definitions found in modules: {modules_str}. "
+                            error_str = (
+                                f"Asset key {key} is defined multiple times. Definitions found in"
+                                f" modules: {modules_str}. "
+                            )
 
                             if key in assets and isinstance(asset, AssetsDefinition):
                                 if assets[key].node_def == asset.node_def:
                                     error_str += (
-                                        "One possible cause of this bug is a call to with_resources outside of "
-                                        "a repository definition, causing a duplicate asset definition."
+                                        "One possible cause of this bug is a call to with_resources"
+                                        " outside of a repository definition, causing a duplicate"
+                                        " asset definition."
                                     )
 
                             raise DagsterInvalidDefinitionError(error_str)
@@ -91,9 +98,10 @@ def load_assets_from_modules(
     modules: Iterable[ModuleType],
     group_name: Optional[str] = None,
     key_prefix: Optional[CoercibleToAssetKeyPrefix] = None,
+    *,
+    freshness_policy: Optional[FreshnessPolicy] = None,
 ) -> Sequence[Union[AssetsDefinition, SourceAsset, CacheableAssetsDefinition]]:
-    """
-    Constructs a list of assets and source assets from the given modules.
+    """Constructs a list of assets and source assets from the given modules.
 
     Args:
         modules (Iterable[ModuleType]): The Python modules to look for assets inside.
@@ -103,49 +111,40 @@ def load_assets_from_modules(
         key_prefix (Optional[Union[str, Sequence[str]]]):
             Prefix to prepend to the keys of the loaded assets. The returned assets will be copies
             of the loaded objects, with the prefix prepended.
+        freshness_policy (Optional[FreshnessPolicy]): FreshnessPolicy to apply to all the loaded
+            assets.
 
     Returns:
         Sequence[Union[AssetsDefinition, SourceAsset]]:
             A list containing assets and source assets defined in the given modules.
     """
     group_name = check.opt_str_param(group_name, "group_name")
-    key_prefix = check.opt_inst_param(key_prefix, "key_prefix", (str, list))
+    key_prefix = check_opt_coercible_to_asset_key_prefix_param(key_prefix, "key_prefix")
+    freshness_policy = check.opt_inst_param(freshness_policy, "freshness_policy", FreshnessPolicy)
 
-    # There is a tricky edge case here where if a non-cacheable asset depends on a cacheable asset,
-    # and the assets are prefixed, the non-cacheable asset's dependency will not be prefixed since
-    # at prefix-time it is not known that its dependency is one of the cacheable assets.
-    # https://github.com/dagster-io/dagster/pull/10389#pullrequestreview-1170913271
     (
         assets,
         source_assets,
         cacheable_assets,
     ) = assets_from_modules(modules)
-    if key_prefix:
-        assets = prefix_assets(assets, key_prefix)
-        cacheable_assets = [
-            cached_asset.with_prefix_for_all(key_prefix) for cached_asset in cacheable_assets
-        ]
-    if group_name:
-        assets = [
-            asset.with_prefix_or_group(
-                group_names_by_key={asset_key: group_name for asset_key in asset.keys}
-            )
-            for asset in assets
-        ]
-        source_assets = [source_asset.with_group_name(group_name) for source_asset in source_assets]
-        cacheable_assets = [
-            cached_asset.with_group_for_all(group_name) for cached_asset in cacheable_assets
-        ]
 
-    return [*assets, *source_assets, *cacheable_assets]
+    return assets_with_attributes(
+        assets,
+        source_assets,
+        cacheable_assets,
+        key_prefix=key_prefix,
+        group_name=group_name,
+        freshness_policy=freshness_policy,
+    )
 
 
 def load_assets_from_current_module(
     group_name: Optional[str] = None,
     key_prefix: Optional[CoercibleToAssetKeyPrefix] = None,
+    *,
+    freshness_policy: Optional[FreshnessPolicy] = None,
 ) -> Sequence[Union[AssetsDefinition, SourceAsset, CacheableAssetsDefinition]]:
-    """
-    Constructs a list of assets, source assets, and cacheable assets from the module where
+    """Constructs a list of assets, source assets, and cacheable assets from the module where
     this function is called.
 
     Args:
@@ -155,6 +154,8 @@ def load_assets_from_current_module(
         key_prefix (Optional[Union[str, Sequence[str]]]):
             Prefix to prepend to the keys of the loaded assets. The returned assets will be copies
             of the loaded objects, with the prefix prepended.
+        freshness_policy (Optional[FreshnessPolicy]): FreshnessPolicy to apply to all the loaded
+            assets.
 
     Returns:
         Sequence[Union[AssetsDefinition, SourceAsset, CachableAssetsDefinition]]:
@@ -169,6 +170,7 @@ def load_assets_from_current_module(
         [module],
         group_name=group_name,
         key_prefix=key_prefix,
+        freshness_policy=freshness_policy,
     )
 
 
@@ -176,8 +178,7 @@ def assets_from_package_module(
     package_module: ModuleType,
     extra_source_assets: Optional[Sequence[SourceAsset]] = None,
 ) -> Tuple[Sequence[AssetsDefinition], Sequence[SourceAsset], Sequence[CacheableAssetsDefinition]]:
-    """
-    Constructs three lists, a list of assets, a list of source assets, and a list of cacheable assets
+    """Constructs three lists, a list of assets, a list of source assets, and a list of cacheable assets
     from the given package module.
 
     Args:
@@ -199,9 +200,10 @@ def load_assets_from_package_module(
     package_module: ModuleType,
     group_name: Optional[str] = None,
     key_prefix: Optional[CoercibleToAssetKeyPrefix] = None,
+    *,
+    freshness_policy: Optional[FreshnessPolicy] = None,
 ) -> Sequence[Union[AssetsDefinition, SourceAsset, CacheableAssetsDefinition]]:
-    """
-    Constructs a list of assets and source assets that includes all asset
+    """Constructs a list of assets and source assets that includes all asset
     definitions, source assets, and cacheable assets in all sub-modules of the given package module.
 
     A package module is the result of importing a package.
@@ -214,45 +216,40 @@ def load_assets_from_package_module(
         key_prefix (Optional[Union[str, Sequence[str]]]):
             Prefix to prepend to the keys of the loaded assets. The returned assets will be copies
             of the loaded objects, with the prefix prepended.
+        freshness_policy (Optional[FreshnessPolicy]): FreshnessPolicy to apply to all the loaded
+            assets.
 
     Returns:
         Sequence[Union[AssetsDefinition, SourceAsset, CacheableAssetsDefinition]]:
             A list containing assets, source assets, and cacheable assets defined in the module.
     """
     group_name = check.opt_str_param(group_name, "group_name")
-    key_prefix = check.opt_inst_param(key_prefix, "key_prefix", (str, list))
+    key_prefix = check_opt_coercible_to_asset_key_prefix_param(key_prefix, "key_prefix")
+    freshness_policy = check.opt_inst_param(freshness_policy, "freshness_policy", FreshnessPolicy)
 
-    # There is a tricky edge case here where if a non-cacheable asset depends on a cacheable asset,
-    # and the assets are prefixed, the non-cacheable asset's dependency will not be prefixed since
-    # at prefix-time it is not known that its dependency is one of the cacheable assets.
-    # https://github.com/dagster-io/dagster/pull/10389#pullrequestreview-1170913271
     (
         assets,
         source_assets,
         cacheable_assets,
     ) = assets_from_package_module(package_module)
-    if key_prefix:
-        assets = prefix_assets(assets, key_prefix)
-        cacheable_assets = [
-            cached_asset.with_prefix_for_all(key_prefix) for cached_asset in cacheable_assets
-        ]
-    if group_name:
-        assets = list(with_group(assets, group_name))
-        source_assets = [asset.with_group_name(group_name) for asset in source_assets]
-        cacheable_assets = [
-            cached_asset.with_group_for_all(group_name) for cached_asset in cacheable_assets
-        ]
-
-    return [*assets, *source_assets, *cacheable_assets]
+    return assets_with_attributes(
+        assets,
+        source_assets,
+        cacheable_assets,
+        key_prefix=key_prefix,
+        group_name=group_name,
+        freshness_policy=freshness_policy,
+    )
 
 
 def load_assets_from_package_name(
     package_name: str,
     group_name: Optional[str] = None,
     key_prefix: Optional[CoercibleToAssetKeyPrefix] = None,
+    *,
+    freshness_policy: Optional[FreshnessPolicy] = None,
 ) -> Sequence[Union[AssetsDefinition, SourceAsset, CacheableAssetsDefinition]]:
-    """
-    Constructs a list of assets, source assets, and cacheable assets that includes all asset
+    """Constructs a list of assets, source assets, and cacheable assets that includes all asset
     definitions and source assets in all sub-modules of the given package.
 
     Args:
@@ -263,6 +260,8 @@ def load_assets_from_package_name(
         key_prefix (Optional[Union[str, Sequence[str]]]):
             Prefix to prepend to the keys of the loaded assets. The returned assets will be copies
             of the loaded objects, with the prefix prepended.
+        freshness_policy (Optional[FreshnessPolicy]): FreshnessPolicy to apply to all the loaded
+            assets.
 
     Returns:
         Sequence[Union[AssetsDefinition, SourceAsset, CacheableAssetsDefinition]]:
@@ -273,6 +272,7 @@ def load_assets_from_package_name(
         package_module,
         group_name=group_name,
         key_prefix=key_prefix,
+        freshness_policy=freshness_policy,
     )
 
 
@@ -295,8 +295,7 @@ def _find_modules_in_package(package_module: ModuleType) -> Iterable[ModuleType]
 def prefix_assets(
     assets_defs: Sequence[AssetsDefinition], key_prefix: CoercibleToAssetKeyPrefix
 ) -> Sequence[AssetsDefinition]:
-    """
-    Given a list of assets, prefix the input and output asset keys with key_prefix.
+    """Given a list of assets, prefix the input and output asset keys with key_prefix.
     The prefix is not added to source assets.
 
     Input asset keys that reference other assets within assets_defs are "brought along" -
@@ -350,7 +349,7 @@ def prefix_assets(
                 )
 
         result_assets.append(
-            assets_def.with_prefix_or_group(
+            assets_def.with_attributes(
                 output_asset_key_replacements=output_asset_key_replacements,
                 input_asset_key_replacements=input_asset_key_replacements,
             )
@@ -358,19 +357,41 @@ def prefix_assets(
     return result_assets
 
 
-def with_group(
-    assets_defs: Sequence[AssetsDefinition], group_name: Optional[str]
-) -> Sequence[AssetsDefinition]:
-    """
-    Given a list of assets, groups them under the group_name.
-    """
-    group_name = check.opt_str_param(group_name, "group_name")
-    if not group_name:
-        return assets_defs
+def assets_with_attributes(
+    assets_defs: Sequence[AssetsDefinition],
+    source_assets: Sequence[SourceAsset],
+    cacheable_assets: Sequence[CacheableAssetsDefinition],
+    key_prefix: Optional[Sequence[str]],
+    group_name: Optional[str],
+    freshness_policy: Optional[FreshnessPolicy],
+) -> Sequence[Union[AssetsDefinition, SourceAsset, CacheableAssetsDefinition]]:
+    # There is a tricky edge case here where if a non-cacheable asset depends on a cacheable asset,
+    # and the assets are prefixed, the non-cacheable asset's dependency will not be prefixed since
+    # at prefix-time it is not known that its dependency is one of the cacheable assets.
+    # https://github.com/dagster-io/dagster/pull/10389#pullrequestreview-1170913271
+    if key_prefix:
+        assets_defs = prefix_assets(assets_defs, key_prefix)
+        cacheable_assets = [
+            cached_asset.with_prefix_for_all(key_prefix) for cached_asset in cacheable_assets
+        ]
 
-    return [
-        assets_def.with_prefix_or_group(
-            group_names_by_key={asset_key: group_name for asset_key in assets_def.keys}
-        )
-        for assets_def in assets_defs
-    ]
+    if group_name or freshness_policy:
+        assets_defs = [
+            asset.with_attributes(
+                group_names_by_key={asset_key: group_name for asset_key in asset.keys}
+                if group_name
+                else None,
+                freshness_policy=freshness_policy,
+            )
+            for asset in assets_defs
+        ]
+        if group_name:
+            source_assets = [
+                source_asset.with_group_name(group_name) for source_asset in source_assets
+            ]
+        cacheable_assets = [
+            cached_asset.with_attributes_for_all(group_name, freshness_policy=freshness_policy)
+            for cached_asset in cacheable_assets
+        ]
+
+    return [*assets_defs, *source_assets, *cacheable_assets]
