@@ -11,13 +11,11 @@ from typing import (
     NamedTuple,
     Optional,
     Sequence,
-    TypeVar,
     Union,
     cast,
-    overload,
 )
 
-from typing_extensions import Self, TypeAlias
+from typing_extensions import Self, TypeAlias, TypeVar
 
 import dagster._check as check
 import dagster._seven as seven
@@ -25,7 +23,11 @@ from dagster._annotations import PublicAttr, experimental, public
 from dagster._core.errors import DagsterInvalidMetadata
 from dagster._serdes import whitelist_for_serdes
 from dagster._serdes.serdes import (
+    FieldSerializer,
     PackableValue,
+    WhitelistMap,
+    pack_value,
+    unpack_value,
 )
 from dagster._utils.backcompat import (
     canonicalize_backcompat_args,
@@ -43,8 +45,13 @@ from .table import (  # re-exported
 if TYPE_CHECKING:
     from dagster._core.definitions.events import AssetKey
 
+ArbitraryMetadataMapping: TypeAlias = Mapping[str, Any]
+
 RawMetadataValue = Union[
     "MetadataValue",
+    TableSchema,
+    "AssetKey",
+    os.PathLike,
     Dict[Any, Any],
     float,
     int,
@@ -56,43 +63,24 @@ RawMetadataValue = Union[
 MetadataMapping: TypeAlias = Mapping[str, "MetadataValue"]
 MetadataUserInput: TypeAlias = Mapping[str, RawMetadataValue]
 
-T_Packable = TypeVar("T_Packable", bound=PackableValue)
+T_Packable = TypeVar("T_Packable", bound=PackableValue, default=PackableValue, covariant=True)
 
 # ########################
 # ##### NORMALIZATION
 # ########################
 
 
-@overload
 def normalize_metadata(
     metadata: Mapping[str, RawMetadataValue],
-    metadata_entries: Sequence["MetadataEntry"],
+    metadata_entries: Optional[Sequence["MetadataEntry"]] = None,
     allow_invalid: bool = False,
-) -> Sequence["MetadataEntry"]:
-    ...
-
-
-@overload
-def normalize_metadata(
-    metadata: Mapping[str, RawMetadataValue],
-    metadata_entries: Sequence["MetadataEntry"],
-    allow_invalid: bool = False,
-) -> Sequence["MetadataEntry"]:
-    ...
-
-
-def normalize_metadata(
-    metadata: Mapping[str, RawMetadataValue],
-    metadata_entries: Sequence["MetadataEntry"],
-    allow_invalid: bool = False,
-) -> Sequence["MetadataEntry"]:
+) -> Mapping[str, "MetadataValue"]:
     if metadata and metadata_entries:
         raise DagsterInvalidMetadata(
             "Attempted to provide both `metadata` and `metadata_entries` arguments to an event. "
             "Must provide only one of the two."
         )
-
-    if metadata_entries:
+    elif metadata_entries:
         deprecation_warning(
             'Argument "metadata_entries"',
             "1.0.0",
@@ -102,42 +90,38 @@ def normalize_metadata(
             ),
             stacklevel=4,  # to get the caller of `normalize_metadata`
         )
-        return check.sequence_param(metadata_entries, "metadata_entries", MetadataEntry)
+        return {entry.label: entry.value for entry in metadata_entries}
 
     # This is a stopgap measure to deal with unsupported metadata values, which occur when we try
     # to convert arbitrary metadata (on e.g. OutputDefinition) to a MetadataValue, which is required
     # for serialization. This will cause unsupported values to be silently replaced with a
     # string placeholder.
-    elif allow_invalid:
-        metadata_entries = []
-        for k, v in metadata.items():
-            try:
-                metadata_entries.append(package_metadata_value(k, v))
-            except DagsterInvalidMetadata:
+    normalized_metadata: Dict[str, MetadataValue] = {}
+    for k, v in metadata.items():
+        try:
+            normalized_value = normalize_metadata_value(v)
+        except DagsterInvalidMetadata as e:
+            if allow_invalid:
                 deprecation_warning(
                     "Support for arbitrary metadata values",
-                    "1.0.0",
+                    "1.4.0",
                     additional_warn_txt=(
                         "In the future, all user-supplied metadata values must be one of"
                         f" {RawMetadataValue}"
                     ),
                     stacklevel=4,  # to get the caller of `normalize_metadata`
                 )
-                metadata_entries.append(
-                    MetadataEntry(
-                        value=TextMetadataValue(f"[{v.__class__.__name__}] (unserializable)"),
-                        label=k,
-                    )
-                )
-        return metadata_entries
+                normalized_value = TextMetadataValue(f"[{v.__class__.__name__}] (unserializable)")
+            else:
+                raise DagsterInvalidMetadata(
+                    f'Could not resolve the metadata value for "{k}" to a known type. {e}'
+                ) from None
+        normalized_metadata[k] = normalized_value
 
-    return [
-        package_metadata_value(k, v)
-        for k, v in check.opt_mapping_param(metadata, "metadata", key_type=str).items()
-    ]
+    return normalized_metadata
 
 
-def normalize_metadata_value(raw_value: RawMetadataValue) -> "MetadataValue":
+def normalize_metadata_value(raw_value: RawMetadataValue) -> "MetadataValue[Any]":
     from dagster._core.definitions.events import AssetKey
 
     if isinstance(raw_value, MetadataValue):
@@ -167,21 +151,8 @@ def normalize_metadata_value(raw_value: RawMetadataValue) -> "MetadataValue":
     )
 
 
-def package_metadata_value(label: str, raw_value: RawMetadataValue) -> "MetadataEntry":
-    check.str_param(label, "label")
-
-    if isinstance(raw_value, MetadataEntry):
-        raise DagsterInvalidMetadata(
-            f"Expected a metadata value, found an instance of {raw_value.__class__.__name__}."
-            " Consider instead using a MetadataValue wrapper for the value."
-        )
-    try:
-        value = normalize_metadata_value(raw_value)
-    except DagsterInvalidMetadata as e:
-        raise DagsterInvalidMetadata(
-            f'Could not resolve the metadata value for "{label}" to a known type. {e}'
-        ) from None
-    return MetadataEntry(label=label, value=value)
+def to_metadata_entries(metadata: Mapping[str, "MetadataValue"]) -> Sequence["MetadataEntry"]:
+    return [MetadataEntry(k, value=v) for k, v in metadata.items()]
 
 
 # ########################
@@ -218,18 +189,19 @@ class MetadataValue(ABC, Generic[T_Packable]):
     def text(text: str) -> "TextMetadataValue":
         """Static constructor for a metadata value wrapping text as
         :py:class:`TextMetadataValue`. Can be used as the value type for the `metadata`
-        parameter for supported events. For example:
+        parameter for supported events.
 
-        .. code-block:: python
+        Example:
+            .. code-block:: python
 
-            @op
-            def emit_metadata(context, df):
-                yield AssetMaterialization(
-                    asset_key="my_dataset",
-                    metadata={
-                        "my_text_label": MetadataValue.text("hello")
-                    },
-                )
+                @op
+                def emit_metadata(context, df):
+                    yield AssetMaterialization(
+                        asset_key="my_dataset",
+                        metadata={
+                            "my_text_label": MetadataValue.text("hello")
+                        },
+                    )
 
         Args:
             text (str): The text string for a metadata entry.
@@ -241,19 +213,19 @@ class MetadataValue(ABC, Generic[T_Packable]):
     def url(url: str) -> "UrlMetadataValue":
         """Static constructor for a metadata value wrapping a URL as
         :py:class:`UrlMetadataValue`. Can be used as the value type for the `metadata`
-        parameter for supported events. For example:
+        parameter for supported events.
 
-        .. code-block:: python
-
-            @op
-            def emit_metadata(context):
-                yield AssetMaterialization(
-                    asset_key="my_dashboard",
-                    metadata={
-                        "dashboard_url": MetadataValue.url("http://mycoolsite.com/my_dashboard"),
-                    }
-                )
+        Example:
+            .. code-block:: python
 
+                @op
+                def emit_metadata(context):
+                    yield AssetMaterialization(
+                        asset_key="my_dashboard",
+                        metadata={
+                            "dashboard_url": MetadataValue.url("http://mycoolsite.com/my_dashboard"),
+                        }
+                    )
 
         Args:
             url (str): The URL for a metadata entry.
@@ -264,18 +236,19 @@ class MetadataValue(ABC, Generic[T_Packable]):
     @staticmethod
     def path(path: Union[str, os.PathLike]) -> "PathMetadataValue":
         """Static constructor for a metadata value wrapping a path as
-        :py:class:`PathMetadataValue`. For example:
+        :py:class:`PathMetadataValue`.
 
-        .. code-block:: python
+        Example:
+            .. code-block:: python
 
-            @op
-            def emit_metadata(context):
-                yield AssetMaterialization(
-                    asset_key="my_dataset",
-                    metadata={
-                        "filepath": MetadataValue.path("path/to/file"),
-                    }
-                )
+                @op
+                def emit_metadata(context):
+                    yield AssetMaterialization(
+                        asset_key="my_dataset",
+                        metadata={
+                            "filepath": MetadataValue.path("path/to/file"),
+                        }
+                    )
 
         Args:
             path (str): The path for a metadata entry.
@@ -286,18 +259,19 @@ class MetadataValue(ABC, Generic[T_Packable]):
     @staticmethod
     def notebook(path: Union[str, os.PathLike]) -> "NotebookMetadataValue":
         """Static constructor for a metadata value wrapping a notebook path as
-        :py:class:`NotebookMetadataValue`. For example:
+        :py:class:`NotebookMetadataValue`.
 
-        .. code-block:: python
+        Example:
+            .. code-block:: python
 
-            @op
-            def emit_metadata(context):
-                yield AssetMaterialization(
-                    asset_key="my_dataset",
-                    metadata={
-                        "notebook_path": MetadataValue.notebook("path/to/notebook.ipynb"),
-                    }
-                )
+                @op
+                def emit_metadata(context):
+                    yield AssetMaterialization(
+                        asset_key="my_dataset",
+                        metadata={
+                            "notebook_path": MetadataValue.notebook("path/to/notebook.ipynb"),
+                        }
+                    )
 
         Args:
             path (str): The path to a notebook for a metadata entry.
@@ -309,19 +283,20 @@ class MetadataValue(ABC, Generic[T_Packable]):
     def json(data: Union[Sequence[Any], Mapping[str, Any]]) -> "JsonMetadataValue":
         """Static constructor for a metadata value wrapping a json-serializable list or dict
         as :py:class:`JsonMetadataValue`. Can be used as the value type for the `metadata`
-        parameter for supported events. For example:
+        parameter for supported events.
 
-        .. code-block:: python
+        Example:
+            .. code-block:: python
 
-            @op
-            def emit_metadata(context):
-                yield ExpectationResult(
-                    success=not missing_things,
-                    label="is_present",
-                    metadata={
-                        "about my dataset": MetadataValue.json({"missing_columns": missing_things})
-                    },
-                )
+                @op
+                def emit_metadata(context):
+                    yield ExpectationResult(
+                        success=not missing_things,
+                        label="is_present",
+                        metadata={
+                            "about my dataset": MetadataValue.json({"missing_columns": missing_things})
+                        },
+                    )
 
         Args:
             data (Union[Sequence[Any], Mapping[str, Any]]): The JSON data for a metadata entry.
@@ -333,19 +308,20 @@ class MetadataValue(ABC, Generic[T_Packable]):
     def md(data: str) -> "MarkdownMetadataValue":
         """Static constructor for a metadata value wrapping markdown data as
         :py:class:`MarkdownMetadataValue`. Can be used as the value type for the `metadata`
-        parameter for supported events. For example:
+        parameter for supported events.
 
 
-        .. code-block:: python
+        Example:
+            .. code-block:: python
 
-            @op
-            def emit_metadata(context, md_str):
-                yield AssetMaterialization(
-                    asset_key="info",
-                    metadata={
-                        'Details': MetadataValue.md(md_str)
-                    },
-                )
+                @op
+                def emit_metadata(context, md_str):
+                    yield AssetMaterialization(
+                        asset_key="info",
+                        metadata={
+                            'Details': MetadataValue.md(md_str)
+                        },
+                    )
 
         Args:
             md_str (str): The markdown for a metadata entry.
@@ -357,19 +333,20 @@ class MetadataValue(ABC, Generic[T_Packable]):
     def python_artifact(python_artifact: Callable) -> "PythonArtifactMetadataValue":
         """Static constructor for a metadata value wrapping a python artifact as
         :py:class:`PythonArtifactMetadataValue`. Can be used as the value type for the
-        `metadata` parameter for supported events. For example:
+        `metadata` parameter for supported events.
 
-        .. code-block:: python
+        Example:
+            .. code-block:: python
 
-            @op
-            def emit_metadata(context, df):
-                yield AssetMaterialization(
-                    asset_key="my_dataset",
-                    metadata={
-                        "class": MetadataValue.python_artifact(MyClass),
-                        "function": MetadataValue.python_artifact(my_function),
-                    }
-                )
+                @op
+                def emit_metadata(context, df):
+                    yield AssetMaterialization(
+                        asset_key="my_dataset",
+                        metadata={
+                            "class": MetadataValue.python_artifact(MyClass),
+                            "function": MetadataValue.python_artifact(my_function),
+                        }
+                    )
 
         Args:
             value (Callable): The python class or function for a metadata entry.
@@ -382,18 +359,19 @@ class MetadataValue(ABC, Generic[T_Packable]):
     def float(value: float) -> "FloatMetadataValue":
         """Static constructor for a metadata value wrapping a float as
         :py:class:`FloatMetadataValue`. Can be used as the value type for the `metadata`
-        parameter for supported events. For example:
+        parameter for supported events.
 
-        .. code-block:: python
+        Example:
+            .. code-block:: python
 
-            @op
-            def emit_metadata(context, df):
-                yield AssetMaterialization(
-                    asset_key="my_dataset",
-                    metadata={
-                        "size (bytes)": MetadataValue.float(calculate_bytes(df)),
-                    }
-                )
+                @op
+                def emit_metadata(context, df):
+                    yield AssetMaterialization(
+                        asset_key="my_dataset",
+                        metadata={
+                            "size (bytes)": MetadataValue.float(calculate_bytes(df)),
+                        }
+                    )
 
         Args:
             value (float): The float value for a metadata entry.
@@ -405,18 +383,19 @@ class MetadataValue(ABC, Generic[T_Packable]):
     def int(value: int) -> "IntMetadataValue":
         """Static constructor for a metadata value wrapping an int as
         :py:class:`IntMetadataValue`. Can be used as the value type for the `metadata`
-        parameter for supported events. For example:
+        parameter for supported events.
 
-        .. code-block:: python
+        Example:
+            .. code-block:: python
 
-            @op
-            def emit_metadata(context, df):
-                yield AssetMaterialization(
-                    asset_key="my_dataset",
-                    metadata={
-                        "number of rows": MetadataValue.int(len(df)),
-                    },
-                )
+                @op
+                def emit_metadata(context, df):
+                    yield AssetMaterialization(
+                        asset_key="my_dataset",
+                        metadata={
+                            "number of rows": MetadataValue.int(len(df)),
+                        },
+                    )
 
         Args:
             value (int): The int value for a metadata entry.
@@ -428,18 +407,19 @@ class MetadataValue(ABC, Generic[T_Packable]):
     def bool(value: bool) -> "BoolMetadataValue":
         """Static constructor for a metadata value wrapping a bool as
         :py:class:`BoolMetadataValuye`. Can be used as the value type for the `metadata`
-        parameter for supported events. For example:
+        parameter for supported events.
 
-        .. code-block:: python
+        Example:
+            .. code-block:: python
 
-            @op
-            def emit_metadata(context, df):
-                yield AssetMaterialization(
-                    asset_key="my_dataset",
-                    metadata={
-                        "num rows > 1000": MetadataValue.bool(len(df) > 1000),
-                    },
-                )
+                @op
+                def emit_metadata(context, df):
+                    yield AssetMaterialization(
+                        asset_key="my_dataset",
+                        metadata={
+                            "num rows > 1000": MetadataValue.bool(len(df) > 1000),
+                        },
+                    )
 
         Args:
             value (bool): The bool value for a metadata entry.
@@ -490,30 +470,31 @@ class MetadataValue(ABC, Generic[T_Packable]):
     ) -> "TableMetadataValue":
         """Static constructor for a metadata value wrapping arbitrary tabular data as
         :py:class:`TableMetadataValue`. Can be used as the value type for the `metadata`
-        parameter for supported events. For example:
+        parameter for supported events.
 
-        .. code-block:: python
+        Example:
+            .. code-block:: python
 
-            @op
-            def emit_metadata(context):
-                yield ExpectationResult(
-                    success=not has_errors,
-                    label="is_valid",
-                    metadata={
-                        "errors": MetadataValue.table(
-                            records=[
-                                TableRecord(code="invalid-data-type", row=2, col="name"),
-                            ],
-                            schema=TableSchema(
-                                columns=[
-                                    TableColumn(name="code", type="string"),
-                                    TableColumn(name="row", type="int"),
-                                    TableColumn(name="col", type="string"),
-                                ]
-                            )
-                        ),
-                    },
-                )
+                @op
+                def emit_metadata(context):
+                    yield ExpectationResult(
+                        success=not has_errors,
+                        label="is_valid",
+                        metadata={
+                            "errors": MetadataValue.table(
+                                records=[
+                                    TableRecord(code="invalid-data-type", row=2, col="name"),
+                                ],
+                                schema=TableSchema(
+                                    columns=[
+                                        TableColumn(name="code", type="string"),
+                                        TableColumn(name="row", type="int"),
+                                        TableColumn(name="col", type="string"),
+                                    ]
+                                )
+                            ),
+                        },
+                    )
         """
         return TableMetadataValue(records, schema)
 
@@ -524,24 +505,25 @@ class MetadataValue(ABC, Generic[T_Packable]):
     ) -> "TableSchemaMetadataValue":
         """Static constructor for a metadata value wrapping a table schema as
         :py:class:`TableSchemaMetadataValue`. Can be used as the value type
-        for the `metadata` parameter for supported events. For example:
+        for the `metadata` parameter for supported events.
 
-        .. code-block:: python
+        Example:
+            .. code-block:: python
 
-            schema = TableSchema(
-                columns = [
-                    TableColumn(name="id", type="int"),
-                    TableColumn(name="status", type="bool"),
-                ]
-            )
+                schema = TableSchema(
+                    columns = [
+                        TableColumn(name="id", type="int"),
+                        TableColumn(name="status", type="bool"),
+                    ]
+                )
 
-            DagsterType(
-                type_check_fn=some_validation_fn,
-                name='MyTable',
-                metadata={
-                    'my_table_schema': MetadataValue.table_schema(schema),
-                }
-            )
+                DagsterType(
+                    type_check_fn=some_validation_fn,
+                    name='MyTable',
+                    metadata={
+                        'my_table_schema': MetadataValue.table_schema(schema),
+                    }
+                )
 
         Args:
             schema (TableSchema): The table schema for a metadata entry.
@@ -952,6 +934,50 @@ class NullMetadataValue(NamedTuple("_NullMetadataValue", []), MetadataValue[None
 # ##### METADATA ENTRY
 # ########################
 
+
+class MetadataFieldSerializer(FieldSerializer):
+    """Converts between metadata dict (new) and metadata entries list (old)."""
+
+    storage_name = "metadata_entries"
+    loaded_name = "metadata"
+
+    def pack(
+        self,
+        metadata_dict: Mapping[str, MetadataValue],
+        whitelist_map: WhitelistMap,
+        descent_path: str,
+    ) -> Sequence[Mapping[str, Any]]:
+        return [
+            {
+                "__class__": "EventMetadataEntry",
+                "label": k,
+                # MetadataValue itself can't inherit from NamedTuple and so isn't a PackableValue,
+                # but one of its subclasses will always be returned here.
+                "entry_data": pack_value(v, whitelist_map, descent_path),  # type: ignore
+                "description": None,
+            }
+            for k, v in metadata_dict.items()
+        ]
+
+    def unpack(
+        self,
+        metadata_entries: List[Mapping[str, Any]],
+        whitelist_map: WhitelistMap,
+        descent_path: str,
+    ) -> Mapping[str, MetadataValue]:
+        return {
+            e["label"]: unpack_value(
+                e["entry_data"],
+                # MetadataValue itself can't inherit from NamedTuple and so isn't a PackableValue,
+                # but one of its subclasses will always be returned here.
+                as_type=MetadataValue,  # type: ignore
+                whitelist_map=whitelist_map,
+                descent_path=descent_path,
+            )
+            for e in metadata_entries
+        }
+
+
 T_MetadataValue = TypeVar("T_MetadataValue", bound=MetadataValue, covariant=True)
 
 
