@@ -19,6 +19,7 @@ from typing import (
 
 import dagster._check as check
 from dagster._annotations import experimental, public
+from dagster._core.decorator_utils import has_at_least_one_parameter
 from dagster._core.definitions.asset_selection import AssetSelection
 from dagster._core.definitions.assets import AssetsDefinition
 from dagster._core.definitions.partition import PartitionsDefinition
@@ -29,21 +30,22 @@ from dagster._core.errors import (
 )
 from dagster._core.instance import DagsterInstance
 from dagster._core.instance.ref import InstanceRef
+from dagster._utils import normalize_to_repository
 
 from ..decorator_utils import get_function_params
 from .events import AssetKey
-from .run_request import RunRequest, SkipReason
+from .run_request import RunRequest, SensorResult, SkipReason
 from .sensor_definition import (
     DefaultSensorStatus,
     SensorDefinition,
     SensorEvaluationContext,
     SensorType,
-    has_at_least_one_parameter,
 )
 from .target import ExecutableDefinition
 from .utils import check_valid_name
 
 if TYPE_CHECKING:
+    from dagster._core.definitions.definitions_class import Definitions
     from dagster._core.definitions.repository_definition import RepositoryDefinition
     from dagster._core.events.log import EventLogEntry
     from dagster._core.storage.event_log.base import EventLogRecord
@@ -182,7 +184,9 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
         monitored_assets (Union[Sequence[AssetKey], AssetSelection]): The assets monitored
             by the sensor. If an AssetSelection object is provided, it will only apply to assets
             within the Definitions that this sensor is part of.
-        repository_def (RepositoryDefinition): The repository that the sensor belongs to.
+        repository_def (Optional[RepositoryDefinition]): The repository that the sensor belongs to.
+            If needed by the sensor top-level resource definitions will be pulled from this repository.
+            You can provide either this or `definitions`.
         instance_ref (Optional[InstanceRef]): The serialized instance configured to run the schedule
         cursor (Optional[str]): The cursor, passed back from the last sensor evaluation via
             the cursor attribute of SkipReason and RunRequest. Must be a dictionary of asset key
@@ -194,17 +198,18 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
         repository_name (Optional[str]): The name of the repository that the sensor belongs to.
         instance (Optional[DagsterInstance]): The deserialized instance can also be passed in
             directly (primarily useful in testing contexts).
+        definitions (Optional[Definitions]): `Definitions` object that the sensor is defined in.
+            If needed by the sensor, top-level resource definitions will be pulled from these
+            definitions. You can provide either this or `repository_def`.
 
     Example:
+        .. code-block:: python
 
-    .. code-block:: python
-
-        from dagster import multi_asset_sensor, MultiAssetSensorEvaluationContext
-
-        @multi_asset_sensor(monitored_assets=[AssetKey("asset_1), AssetKey("asset_2)])
-        def the_sensor(context: MultiAssetSensorEvaluationContext):
-            ...
+            from dagster import multi_asset_sensor, MultiAssetSensorEvaluationContext
 
+            @multi_asset_sensor(monitored_assets=[AssetKey("asset_1), AssetKey("asset_2)])
+            def the_sensor(context: MultiAssetSensorEvaluationContext):
+                ...
     """
 
     def __init__(
@@ -214,16 +219,22 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
         last_run_key: Optional[str],
         cursor: Optional[str],
         repository_name: Optional[str],
-        repository_def: "RepositoryDefinition",
+        repository_def: Optional["RepositoryDefinition"],
         monitored_assets: Union[Sequence[AssetKey], AssetSelection],
         instance: Optional[DagsterInstance] = None,
+        definitions: Optional["Definitions"] = None,
     ):
+        from dagster._core.definitions.definitions_class import Definitions
+        from dagster._core.definitions.repository_definition import RepositoryDefinition
         from dagster._core.storage.event_log.base import EventLogRecord
 
-        self._repository_def = repository_def
+        self._repository_def = normalize_to_repository(
+            check.opt_inst_param(definitions, "definitions", Definitions),
+            check.opt_inst_param(repository_def, "repository_def", RepositoryDefinition),
+        )
         self._monitored_asset_keys: Sequence[AssetKey]
         if isinstance(monitored_assets, AssetSelection):
-            repo_assets = self._repository_def._assets_defs_by_key.values()
+            repo_assets = self._repository_def.assets_defs_by_key.values()
             repo_source_assets = self._repository_def.source_assets_by_key.values()
             self._monitored_asset_keys = list(
                 monitored_assets.resolve([*repo_assets, *repo_source_assets])
@@ -233,11 +244,7 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
 
         self._assets_by_key: Dict[AssetKey, Optional[AssetsDefinition]] = {}
         for asset_key in self._monitored_asset_keys:
-            assets_def = (
-                self._repository_def._assets_defs_by_key.get(  # pylint:disable=protected-access
-                    asset_key
-                )
-            )
+            assets_def = self._repository_def.assets_defs_by_key.get(asset_key)
             self._assets_by_key[asset_key] = assets_def
 
         self._partitions_def_by_asset_key = {
@@ -250,7 +257,6 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
         # At the end of each tick, must call update_cursor_after_evaluation to update the serialized
         # cursor.
         self._unpacked_cursor = MultiAssetSensorContextCursor(cursor, self)
-        self._cursor_has_been_updated = False
         self._cursor_advance_state_mutation = MultiAssetSensorCursorAdvances()
 
         self._initial_unconsumed_events_by_id: Dict[int, EventLogRecord] = {}
@@ -404,8 +410,7 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
     def materialization_records_for_key(
         self, asset_key: AssetKey, limit: int
     ) -> Iterable["EventLogRecord"]:
-        """
-        Fetches asset materialization event records for asset_key, with the earliest event first.
+        """Fetches asset materialization event records for asset_key, with the earliest event first.
 
         Only fetches events after the latest consumed event ID for the given asset key.
 
@@ -435,8 +440,7 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
         return events
 
     def _get_cursor(self, asset_key: AssetKey) -> MultiAssetSensorAssetCursorComponent:
-        """
-        Returns the MultiAssetSensorAssetCursorComponent for the asset key.
+        """Returns the MultiAssetSensorAssetCursorComponent for the asset key.
 
         For more information, view the docstring for the MultiAssetSensorAssetCursorComponent class.
         """
@@ -450,8 +454,7 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
         asset_key: AssetKey,
         after_cursor_partition: Optional[bool] = False,
     ) -> Mapping[str, "EventLogRecord"]:
-        """
-        Given an asset, returns a mapping of partition key to the latest materialization event
+        """Given an asset, returns a mapping of partition key to the latest materialization event
         for that partition. Fetches only materializations that have not been marked as "consumed"
         via a call to `advance_cursor`.
 
@@ -548,8 +551,7 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
     def latest_materialization_records_by_partition_and_asset(
         self,
     ) -> Mapping[str, Mapping[AssetKey, "EventLogRecord"]]:
-        """
-        Finds the most recent unconsumed materialization for each partition for each asset
+        """Finds the most recent unconsumed materialization for each partition for each asset
         monitored by the sensor. Aggregates all materializations into a mapping of partition key
         to a mapping of asset key to the materialization event for that partition.
 
@@ -609,8 +611,7 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
     def all_partitions_materialized(
         self, asset_key: AssetKey, partitions: Optional[Sequence[str]] = None
     ) -> bool:
-        """
-        A utility method to check if a provided list of partitions have been materialized
+        """A utility method to check if a provided list of partitions have been materialized
         for a particular asset. This method ignores the cursor and checks all materializations
         for the asset.
 
@@ -654,7 +655,7 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
         from dagster._core.definitions.repository_definition import RepositoryDefinition
 
         repo_def = cast(RepositoryDefinition, self._repository_def)
-        repository_assets = repo_def._assets_defs_by_key  # pylint:disable=protected-access
+        repository_assets = repo_def.assets_defs_by_key
         if asset_key in self._assets_by_key:
             asset_def = self._assets_by_key[asset_key]
             if asset_def is None:
@@ -676,8 +677,7 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
     def get_downstream_partition_keys(
         self, partition_key: str, from_asset_key: AssetKey, to_asset_key: AssetKey
     ) -> Sequence[str]:
-        """
-        Converts a partition key from one asset to the corresponding partition key in a downstream
+        """Converts a partition key from one asset to the corresponding partition key in a downstream
         asset. Uses the existing partition mapping between the upstream asset and the downstream
         asset if it exists, otherwise, uses the default partition mapping.
 
@@ -725,8 +725,7 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
     def advance_cursor(
         self, materialization_records_by_key: Mapping[AssetKey, Optional["EventLogRecord"]]
     ):
-        """
-        Marks the provided materialization records as having been consumed by the sensor.
+        """Marks the provided materialization records as having been consumed by the sensor.
 
         At the end of the tick, the cursor will be updated to advance past all materializations
         records provided via `advance_cursor`. In the next tick, records that have been consumed
@@ -743,12 +742,11 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
                 will not be updated.
         """
         self._cursor_advance_state_mutation.add_advanced_records(materialization_records_by_key)
-        self._cursor_has_been_updated = True
+        self._cursor_updated = True
 
     @public
     def advance_all_cursors(self):
-        """
-        Updates the cursor to the most recent materialization event for all assets monitored by
+        """Updates the cursor to the most recent materialization event for all assets monitored by
         the multi_asset_sensor.
 
         Marks all materialization events as consumed by the sensor, including unconsumed events.
@@ -757,7 +755,7 @@ class MultiAssetSensorEvaluationContext(SensorEvaluationContext):
 
         self._cursor_advance_state_mutation.add_advanced_records(materializations_by_key)
         self._cursor_advance_state_mutation.advance_all_cursors_called = True
-        self._cursor_has_been_updated = True
+        self._cursor_updated = True
 
     @public
     @property
@@ -796,8 +794,7 @@ class MultiAssetSensorCursorAdvances:
         context: MultiAssetSensorEvaluationContext,
         initial_cursor: MultiAssetSensorContextCursor,
     ) -> Optional[str]:
-        """
-        Given the multi asset sensor context and the cursor at the start of the tick,
+        """Given the multi asset sensor context and the cursor at the start of the tick,
         returns the cursor that should be used in the next tick.
 
         If the cursor has not been updated, returns None
@@ -931,12 +928,14 @@ def get_cursor_from_latest_materializations(
 
 @experimental
 def build_multi_asset_sensor_context(
-    repository_def: "RepositoryDefinition",
+    *,
     monitored_assets: Union[Sequence[AssetKey], AssetSelection],
+    repository_def: Optional["RepositoryDefinition"] = None,
     instance: Optional[DagsterInstance] = None,
     cursor: Optional[str] = None,
     repository_name: Optional[str] = None,
     cursor_from_latest_materializations: bool = False,
+    definitions: Optional["Definitions"] = None,
 ) -> MultiAssetSensorEvaluationContext:
     """Builds multi asset sensor execution context for testing purposes using the provided parameters.
 
@@ -945,16 +944,19 @@ def build_multi_asset_sensor_context(
     error.
 
     Args:
-        repository_def (RepositoryDefinition): The repository definition that the sensor belongs to.
         monitored_assets (Union[Sequence[AssetKey], AssetSelection]): The assets monitored
             by the sensor. If an AssetSelection object is provided, it will only apply to assets
             within the Definitions that this sensor is part of.
+        repository_def (RepositoryDefinition): `RepositoryDefinition` object that
+            the sensor is defined in. Must provide `definitions` if this is not provided.
         instance (Optional[DagsterInstance]): The dagster instance configured to run the sensor.
         cursor (Optional[str]): A string cursor to provide to the evaluation of the sensor. Must be
             a dictionary of asset key strings to ints that has been converted to a json string
         repository_name (Optional[str]): The name of the repository that the sensor belongs to.
         cursor_from_latest_materializations (bool): If True, the cursor will be set to the latest
             materialization for each monitored asset. By default, set to False.
+        definitions (Optional[Definitions]): `Definitions` object that the sensor is defined in.
+            Must provide `repository_def` if this is not provided.
 
     Examples:
         .. code-block:: python
@@ -968,11 +970,15 @@ def build_multi_asset_sensor_context(
 
     """
     from dagster._core.definitions import RepositoryDefinition
+    from dagster._core.definitions.definitions_class import Definitions
 
     check.opt_inst_param(instance, "instance", DagsterInstance)
     check.opt_str_param(cursor, "cursor")
     check.opt_str_param(repository_name, "repository_name")
-    check.inst_param(repository_def, "repository_def", RepositoryDefinition)
+    repository_def = normalize_to_repository(
+        check.opt_inst_param(definitions, "definitions", Definitions),
+        check.opt_inst_param(repository_def, "repository_def", RepositoryDefinition),
+    )
 
     check.bool_param(cursor_from_latest_materializations, "cursor_from_latest_materializations")
 
@@ -994,13 +1000,7 @@ def build_multi_asset_sensor_context(
             asset_keys = cast(
                 List[AssetKey],
                 list(
-                    monitored_assets.resolve(
-                        list(
-                            set(
-                                repository_def._assets_defs_by_key.values()  # pylint: disable=protected-access
-                            )
-                        )
-                    )
+                    monitored_assets.resolve(list(set(repository_def.assets_defs_by_key.values())))
                 ),
             )
         else:
@@ -1021,7 +1021,12 @@ def build_multi_asset_sensor_context(
 
 
 AssetMaterializationFunctionReturn = Union[
-    Iterator[Union[RunRequest, SkipReason]], Sequence[RunRequest], RunRequest, SkipReason, None
+    Iterator[Union[RunRequest, SkipReason, SensorResult]],
+    Sequence[RunRequest],
+    RunRequest,
+    SkipReason,
+    None,
+    SensorResult,
 ]
 AssetMaterializationFunction = Callable[
     ["SensorEvaluationContext", "EventLogEntry"],
@@ -1080,6 +1085,13 @@ class MultiAssetSensorDefinition(SensorDefinition):
     ):
         def _wrap_asset_fn(materialization_fn):
             def _fn(context):
+                def _check_cursor_not_set(sensor_result: SensorResult):
+                    if sensor_result.cursor:
+                        raise DagsterInvariantViolationError(
+                            "Cannot set cursor in a multi_asset_sensor. Cursor is set automatically"
+                            " based on the latest materialization for each monitored asset."
+                        )
+
                 multi_asset_sensor_context = MultiAssetSensorEvaluationContext(
                     instance_ref=context.instance_ref,
                     last_completion_time=context.last_completion_time,
@@ -1103,6 +1115,11 @@ class MultiAssetSensorDefinition(SensorDefinition):
                     for item in result:
                         if isinstance(item, RunRequest):
                             runs_yielded = True
+                        if isinstance(item, SensorResult):
+                            raise DagsterInvariantViolationError(
+                                "Cannot yield a SensorResult from a multi_asset_sensor. Instead"
+                                " return the SensorResult."
+                            )
                         yield item
                 elif isinstance(result, RunRequest):
                     runs_yielded = True
@@ -1110,11 +1127,13 @@ class MultiAssetSensorDefinition(SensorDefinition):
                 elif isinstance(result, SkipReason):
                     # if result is a SkipReason, we don't update the cursor, so don't set runs_yielded = True
                     yield result
+                elif isinstance(result, SensorResult):
+                    _check_cursor_not_set(result)
+                    if result.run_requests:
+                        runs_yielded = True
+                    yield result
 
-                if (
-                    runs_yielded
-                    and not multi_asset_sensor_context._cursor_has_been_updated  # pylint: disable=protected-access
-                ):
+                if runs_yielded and not multi_asset_sensor_context.cursor_updated:
                     raise DagsterInvalidDefinitionError(
                         "Asset materializations have been handled in this sensor, but the cursor"
                         " was not updated. This means the same materialization events will be"
@@ -1133,7 +1152,7 @@ class MultiAssetSensorDefinition(SensorDefinition):
             name=check_valid_name(name),
             job_name=job_name,
             evaluation_fn=_wrap_asset_fn(
-                check.callable_param(asset_materialization_fn, "asset_materialization_fn"),
+                check.callable_param(asset_materialization_fn, "asset_materialization_fn")
             ),
             minimum_interval_seconds=minimum_interval_seconds,
             description=description,
