@@ -13,7 +13,6 @@ from typing import (
 import dagster._check as check
 from dagster._core.definitions.configurable import NamedConfigurableDefinition
 from dagster._core.definitions.policy import RetryPolicy
-from dagster._utils import frozendict, frozenlist
 
 from .hook_definition import HookDefinition
 from .utils import check_valid_name, validate_tags
@@ -24,7 +23,6 @@ if TYPE_CHECKING:
     from .asset_layer import AssetLayer
     from .composition import PendingNodeInvocation
     from .dependency import NodeHandle, NodeInputHandle
-    from .graph_definition import GraphDefinition
     from .input import InputDefinition
     from .op_definition import OpDefinition
     from .output import OutputDefinition
@@ -54,11 +52,11 @@ class NodeDefinition(NamedConfigurableDefinition):
         self._name = check_valid_name(name)
         self._description = check.opt_str_param(description, "description")
         self._tags = validate_tags(tags)
-        self._input_defs = frozenlist(input_defs)
-        self._input_dict = frozendict({input_def.name: input_def for input_def in input_defs})
+        self._input_defs = input_defs
+        self._input_dict = {input_def.name: input_def for input_def in input_defs}
         check.invariant(len(self._input_defs) == len(self._input_dict), "Duplicate input def names")
-        self._output_defs = frozenlist(output_defs)
-        self._output_dict = frozendict({output_def.name: output_def for output_def in output_defs})
+        self._output_defs = output_defs
+        self._output_dict = {output_def.name: output_def for output_def in output_defs}
         check.invariant(
             len(self._output_defs) == len(self._output_dict), "Duplicate output def names"
         )
@@ -175,8 +173,7 @@ class NodeDefinition(NamedConfigurableDefinition):
     def resolve_input_to_destinations(
         self, input_handle: "NodeInputHandle"
     ) -> Sequence["NodeInputHandle"]:
-        """
-        Recursively follow input mappings to find all op inputs that correspond to the given input
+        """Recursively follow input mappings to find all op inputs that correspond to the given input
         to this graph.
         """
 
@@ -201,80 +198,38 @@ class NodeDefinition(NamedConfigurableDefinition):
             yield output_def.dagster_type
             yield from output_def.dagster_type.inner_types
 
-    def __call__(self, *args: object, **kwargs: object) -> object:
+    def get_pending_invocation(
+        self,
+        given_alias: Optional[str] = None,
+        tags: Optional[Mapping[str, str]] = None,
+        hook_defs: Optional[AbstractSet[HookDefinition]] = None,
+        retry_policy: Optional[RetryPolicy] = None,
+    ) -> "PendingNodeInvocation":
         from .composition import PendingNodeInvocation
 
         return PendingNodeInvocation(
             node_def=self,
-            given_alias=None,
-            tags=None,
-            hook_defs=None,
-            retry_policy=None,
-        )(*args, **kwargs)
-
-    def alias(self, name: str) -> "PendingNodeInvocation":
-        from .composition import PendingNodeInvocation
-
-        check.str_param(name, "name")
-
-        return PendingNodeInvocation(
-            node_def=self,
-            given_alias=name,
-            tags=None,
-            hook_defs=None,
-            retry_policy=None,
-        )
-
-    def tag(self, tags: Optional[Mapping[str, str]]) -> "PendingNodeInvocation":
-        from .composition import PendingNodeInvocation
-
-        return PendingNodeInvocation(
-            node_def=self,
-            given_alias=None,
-            tags=validate_tags(tags),
-            hook_defs=None,
-            retry_policy=None,
-        )
-
-    def with_hooks(self, hook_defs: AbstractSet[HookDefinition]) -> "PendingNodeInvocation":
-        from .composition import PendingNodeInvocation
-
-        hook_defs = frozenset(check.set_param(hook_defs, "hook_defs", of_type=HookDefinition))
-
-        return PendingNodeInvocation(
-            node_def=self,
-            given_alias=None,
-            tags=None,
+            given_alias=given_alias,
+            tags=validate_tags(tags) if tags else None,
             hook_defs=hook_defs,
-            retry_policy=None,
-        )
-
-    def with_retry_policy(self, retry_policy: RetryPolicy) -> "PendingNodeInvocation":
-        from .composition import PendingNodeInvocation
-
-        return PendingNodeInvocation(
-            node_def=self,
-            given_alias=None,
-            tags=None,
-            hook_defs=None,
             retry_policy=retry_policy,
         )
 
-    def ensure_graph_def(self) -> "GraphDefinition":
-        from .graph_definition import GraphDefinition
+    def __call__(self, *args: object, **kwargs: object) -> object:
+        return self.get_pending_invocation()(*args, **kwargs)
 
-        if isinstance(self, GraphDefinition):
-            return self
+    def alias(self, name: str) -> "PendingNodeInvocation":
+        return self.get_pending_invocation(given_alias=name)
 
-        check.failed(f"{self.name} is not a GraphDefinition")
+    def tag(self, tags: Optional[Mapping[str, str]]) -> "PendingNodeInvocation":
+        return self.get_pending_invocation(tags=tags)
 
-    def ensure_op_def(self) -> "OpDefinition":
-        from .op_definition import OpDefinition
+    def with_hooks(self, hook_defs: AbstractSet[HookDefinition]) -> "PendingNodeInvocation":
+        hook_defs = frozenset(check.set_param(hook_defs, "hook_defs", of_type=HookDefinition))
+        return self.get_pending_invocation(hook_defs=hook_defs)
 
-        if isinstance(self, OpDefinition):
-            return self
-
-        check.failed(f"{self.name} is not an OpDefinition")
+    def with_retry_policy(self, retry_policy: RetryPolicy) -> "PendingNodeInvocation":
+        return self.get_pending_invocation(retry_policy=retry_policy)
 
     @abstractmethod
     def get_inputs_must_be_resolved_top_level(
