@@ -3,7 +3,6 @@ from typing import (
     Any,
     NamedTuple,
     Optional,
-    Sequence,
     Type,
     TypeVar,
     Union,
@@ -11,7 +10,11 @@ from typing import (
 
 import dagster._check as check
 from dagster._annotations import PublicAttr
-from dagster._core.definitions.metadata import MetadataEntry, MetadataUserInput, normalize_metadata
+from dagster._core.definitions.metadata import (
+    ArbitraryMetadataMapping,
+    MetadataUserInput,
+    normalize_metadata,
+)
 from dagster._core.errors import DagsterError, DagsterInvalidDefinitionError
 from dagster._core.types.dagster_type import (
     DagsterType,
@@ -64,7 +67,7 @@ class OutputDefinition:
         description: Optional[str] = None,
         is_required: bool = True,
         io_manager_key: Optional[str] = None,
-        metadata: Optional[MetadataUserInput] = None,
+        metadata: Optional[ArbitraryMetadataMapping] = None,
         code_version: Optional[str] = None,
         # make sure new parameters are updated in combine_with_inferred below
     ):
@@ -79,10 +82,8 @@ class OutputDefinition:
             default=DEFAULT_IO_MANAGER_KEY,
         )
         self._code_version = check.opt_str_param(code_version, "code_version")
-        self._metadata = check.opt_mapping_param(metadata, "metadata", key_type=str)
-        self._metadata_entries = check.is_list(
-            normalize_metadata(self._metadata, [], allow_invalid=True), MetadataEntry
-        )
+        self._raw_metadata = check.opt_mapping_param(metadata, "metadata", key_type=str)
+        self._metadata = normalize_metadata(self._raw_metadata, allow_invalid=True)
 
     @property
     def name(self) -> str:
@@ -113,12 +114,8 @@ class OutputDefinition:
         return not self.is_required
 
     @property
-    def metadata(self) -> MetadataUserInput:
-        return self._metadata
-
-    @property
-    def metadata_entries(self) -> Sequence[MetadataEntry]:
-        return self._metadata_entries
+    def metadata(self) -> ArbitraryMetadataMapping:
+        return self._raw_metadata
 
     @property
     def is_dynamic(self) -> bool:
@@ -209,8 +206,7 @@ def _checked_inferred_type(inferred: Any) -> DagsterType:
 
 
 class DynamicOutputDefinition(OutputDefinition):
-    """
-    Variant of :py:class:`OutputDefinition <dagster.OutputDefinition>` for an
+    """Variant of :py:class:`OutputDefinition <dagster.OutputDefinition>` for an
     output that will dynamically alter the graph at runtime.
 
     When using in a composition function such as :py:func:`@pipeline <dagster.pipeline>`,
@@ -333,8 +329,7 @@ class Out(
         ],
     )
 ):
-    """
-    Defines an output from an op's compute function.
+    """Defines an output from an op's compute function.
 
     Ops can have multiple outputs, in which case outputs cannot be anonymous.
 
@@ -366,7 +361,7 @@ class Out(
         description: Optional[str] = None,
         is_required: bool = True,
         io_manager_key: Optional[str] = None,
-        metadata: Optional[MetadataUserInput] = None,
+        metadata: Optional[ArbitraryMetadataMapping] = None,
         code_version: Optional[str] = None,
         # make sure new parameters are updated in combine_with_inferred below
     ):
@@ -427,8 +422,7 @@ class Out(
 
 
 class DynamicOut(Out):
-    """
-    Variant of :py:class:`Out <dagster.Out>` for an output that will dynamically alter the graph at
+    """Variant of :py:class:`Out <dagster.Out>` for an output that will dynamically alter the graph at
     runtime.
 
     When using in a composition function such as :py:func:`@graph <dagster.graph>`,
@@ -493,8 +487,7 @@ class DynamicOut(Out):
 
 
 class GraphOut(NamedTuple("_GraphOut", [("description", PublicAttr[Optional[str]])])):
-    """
-    Represents information about the outputs that a graph maps.
+    """Represents information about the outputs that a graph maps.
 
     Args:
         description (Optional[str]): Human-readable description of the output.
