@@ -1,6 +1,5 @@
 import copy
 import hashlib
-import inspect
 import json
 from abc import ABC, abstractmethod
 from datetime import (
@@ -28,64 +27,33 @@ from typing import (
 
 import pendulum
 from dateutil.relativedelta import relativedelta
-from typing_extensions import TypeAlias
 
 import dagster._check as check
 from dagster._annotations import PublicAttr, public
 from dagster._core.definitions.partition_key_range import PartitionKeyRange
-from dagster._core.definitions.target import ExecutableDefinition
-from dagster._core.instance import DynamicPartitionsStore
-from dagster._core.storage.tags import PARTITION_NAME_TAG
+from dagster._core.instance import DagsterInstance, DynamicPartitionsStore
+from dagster._core.storage.tags import PARTITION_NAME_TAG, PARTITION_SET_TAG
 from dagster._serdes import whitelist_for_serdes
 from dagster._seven.compat.pendulum import PendulumDateTime, to_timezone
-from dagster._utils import frozenlist
 from dagster._utils.backcompat import deprecation_warning, experimental_arg_warning
-from dagster._utils.merger import merge_dicts
+from dagster._utils.cached_method import cached_method
 from dagster._utils.schedules import schedule_execution_time_iterator
 
-from ..decorator_utils import get_function_params
 from ..errors import (
     DagsterInvalidDefinitionError,
     DagsterInvalidDeserializationVersionError,
     DagsterInvalidInvocationError,
     DagsterInvariantViolationError,
     DagsterUnknownPartitionError,
-    ScheduleExecutionError,
-    user_code_error_boundary,
 )
-from ..storage.pipeline_run import DagsterRun
 from .config import ConfigMapping
-from .mode import DEFAULT_MODE_NAME
-from .run_request import RunRequest, SkipReason
-from .schedule_definition import (
-    DefaultScheduleStatus,
-    ScheduleDefinition,
-    ScheduleEvaluationContext,
-    ScheduleExecutionFunction,
-    ScheduleRunConfigFunction,
-    ScheduleShouldExecuteFunction,
-    ScheduleTagsFunction,
-)
-from .utils import check_valid_name, validate_tags
+from .utils import validate_tags
 
 DEFAULT_DATE_FORMAT = "%Y-%m-%d"
 
 T_cov = TypeVar("T_cov", covariant=True)
 
 
-RawPartitionFunction: TypeAlias = Union[
-    Callable[[Optional[datetime]], Sequence[Union[str, "Partition[T_cov]"]]],
-    Callable[[], Sequence[Union[str, "Partition[T_cov]"]]],
-]
-
-PartitionFunction: TypeAlias = Callable[[Optional[datetime]], Sequence["Partition[Any]"]]
-PartitionTagsFunction: TypeAlias = Callable[["Partition[object]"], Mapping[str, str]]
-PartitionScheduleFunction: TypeAlias = Callable[[datetime], Mapping[str, Any]]
-PartitionSelectorFunction: TypeAlias = Callable[
-    [ScheduleEvaluationContext, "PartitionSetDefinition[T_cov]"],
-    Union["Partition[T_cov]", Sequence["Partition[T_cov]"], SkipReason],
-]
-
 # Dagit selects partition ranges following the format '2022-01-13...2022-01-14'
 # "..." is an invalid substring in partition keys
 # The other escape characters are characters that may not display in Dagit
@@ -93,8 +61,7 @@ INVALID_PARTITION_SUBSTRINGS = ["...", "\a", "\b", "\f", "\n", "\r", "\t", "\v",
 
 
 class Partition(Generic[T_cov]):
-    """
-    A Partition represents a single slice of the entire set of a job's possible work. It consists
+    """A Partition represents a single slice of the entire set of a job's possible work. It consists
     of a value, which is an object that represents that partition, and an optional name, which is
     used to label the partition in a human-readable way.
 
@@ -223,8 +190,7 @@ class ScheduleType(Enum):
 
 
 class PartitionsDefinition(ABC, Generic[T_cov]):
-    """
-    Defines a set of partitions, which can be attached to a software-defined asset or job.
+    """Defines a set of partitions, which can be attached to a software-defined asset or job.
 
     Abstract class with implementations for different kinds of partitions.
     """
@@ -349,9 +315,14 @@ class PartitionsDefinition(ABC, Generic[T_cov]):
             serialized_partitions_def_class_name,
         )
 
-    @property
-    def serializable_unique_identifier(self) -> str:
-        return hashlib.sha1(json.dumps(self.get_partition_keys()).encode("utf-8")).hexdigest()
+    def get_serializable_unique_identifier(
+        self, dynamic_partitions_store: Optional[DynamicPartitionsStore] = None
+    ) -> str:
+        return hashlib.sha1(
+            json.dumps(
+                self.get_partition_keys(dynamic_partitions_store=dynamic_partitions_store)
+            ).encode("utf-8")
+        ).hexdigest()
 
     def get_tags_for_partition_key(self, partition_key: str) -> Mapping[str, str]:
         tags = {PARTITION_NAME_TAG: partition_key}
@@ -364,6 +335,17 @@ class PartitionsDefinition(ABC, Generic[T_cov]):
     ) -> int:
         return len(self.get_partition_keys(current_time, dynamic_partitions_store))
 
+    def has_partition_key(
+        self,
+        partition_key: str,
+        current_time: Optional[datetime] = None,
+        dynamic_partitions_store: Optional[DynamicPartitionsStore] = None,
+    ) -> bool:
+        return partition_key in self.get_partition_keys(
+            current_time=current_time,
+            dynamic_partitions_store=dynamic_partitions_store,
+        )
+
 
 def raise_error_on_invalid_partition_key_substring(partition_keys: Sequence[str]) -> None:
     for partition_key in partition_keys:
@@ -378,11 +360,8 @@ def raise_error_on_invalid_partition_key_substring(partition_keys: Sequence[str]
             )
 
 
-class StaticPartitionsDefinition(
-    PartitionsDefinition[str]
-):  # pylint: disable=unsubscriptable-object
-    """
-    A statically-defined set of partitions.
+class StaticPartitionsDefinition(PartitionsDefinition[str]):
+    """A statically-defined set of partitions.
 
     Example:
         .. code-block:: python
@@ -401,13 +380,14 @@ class StaticPartitionsDefinition(
     def __init__(self, partition_keys: Sequence[str]):
         check.sequence_param(partition_keys, "partition_keys", of_type=str)
 
+        # TODO 1.3.0 enforce that partition keys are unique
         raise_error_on_invalid_partition_key_substring(partition_keys)
 
         self._partitions = [Partition(key) for key in partition_keys]
 
     def get_partitions(
         self,
-        current_time: Optional[datetime] = None,  # pylint: disable=unused-argument
+        current_time: Optional[datetime] = None,
         dynamic_partitions_store: Optional[DynamicPartitionsStore] = None,
     ) -> Sequence[Partition[str]]:
         return self._partitions
@@ -423,9 +403,19 @@ class StaticPartitionsDefinition(
     def __repr__(self) -> str:
         return f"{type(self).__name__}(partition_keys={[p.name for p in self._partitions]})"
 
+    def get_num_partitions(
+        self,
+        current_time: Optional[datetime] = None,
+        dynamic_partitions_store: Optional[DynamicPartitionsStore] = None,
+    ) -> int:
+        # We don't currently throw an error when a duplicate partition key is defined
+        # in a static partitions definition, though we will at 1.3.0.
+        # This ensures that partition counts are correct in Dagit.
+        return len(set(self.get_partition_keys(current_time, dynamic_partitions_store)))
+
 
 class ScheduleTimeBasedPartitionsDefinition(
-    PartitionsDefinition[datetime],  # pylint: disable=unsubscriptable-object
+    PartitionsDefinition[datetime],
     NamedTuple(
         "_ScheduleTimeBasedPartitionsDefinition",
         [
@@ -442,7 +432,7 @@ class ScheduleTimeBasedPartitionsDefinition(
 ):
     """Computes the partitions backwards from the scheduled execution times."""
 
-    def __new__(  # pylint: disable=arguments-differ
+    def __new__(
         cls,
         schedule_type: ScheduleType,
         start: datetime,
@@ -562,6 +552,19 @@ class ScheduleTimeBasedPartitionsDefinition(
             check.assert_never(self.schedule_type)
 
 
+class CachingDynamicPartitionsLoader(DynamicPartitionsStore):
+    """A batch loader that caches the partition keys for a given dynamic partitions definition,
+    to avoid repeated calls to the database for the same partitions definition.
+    """
+
+    def __init__(self, instance: DagsterInstance):
+        self._instance = instance
+
+    @cached_method
+    def get_dynamic_partitions(self, partitions_def_name: str) -> Sequence[str]:
+        return self._instance.get_dynamic_partitions(partitions_def_name)
+
+
 class DynamicPartitionsDefinition(
     PartitionsDefinition,
     NamedTuple(
@@ -579,8 +582,7 @@ class DynamicPartitionsDefinition(
         ],
     ),
 ):
-    """
-    A partitions definition whose partition keys can be dynamically added and removed.
+    """A partitions definition whose partition keys can be dynamically added and removed.
 
     This is useful for cases where the set of partitions is not known at definition time,
     but is instead determined at runtime.
@@ -606,7 +608,7 @@ class DynamicPartitionsDefinition(
 
     """
 
-    def __new__(  # pylint: disable=arguments-differ
+    def __new__(
         cls,
         partition_fn: Optional[
             Callable[[Optional[datetime]], Union[Sequence[Partition], Sequence[str]]]
@@ -659,17 +661,10 @@ class DynamicPartitionsDefinition(
 
     def __str__(self) -> str:
         if self.name:
-            return f"Dynamic partitions definition {self._validated_name()}"
+            return f'Dynamic partitions: "{self._validated_name()}"'
         else:
             return super().__str__()
 
-    @property
-    def serializable_unique_identifier(self) -> str:
-        if not self.name:
-            return super().serializable_unique_identifier
-
-        return hashlib.sha1(self.__repr__().encode("utf-8")).hexdigest()
-
     def get_partitions(
         self,
         current_time: Optional[datetime] = None,
@@ -698,400 +693,22 @@ class DynamicPartitionsDefinition(
             )
             return [Partition(key) for key in partitions]
 
-
-class PartitionSetDefinition(Generic[T_cov]):
-    """
-    Defines a partition set, representing the set of slices making up an axis of a pipeline.
-
-    Args:
-        name (str): Name for this partition set
-        pipeline_name (str): The name of the pipeline definition
-        partition_fn (Optional[Callable[void, Sequence[Partition]]]): User-provided function to define
-            the set of valid partition objects.
-        solid_selection (Optional[Sequence[str]]): A list of solid subselection (including single
-            solid names) to execute with this partition. e.g. ``['*some_solid+', 'other_solid']``
-        mode (Optional[str]): The mode to apply when executing this partition. (default: 'default')
-        run_config_fn_for_partition (Callable[[Partition], Any]): A
-            function that takes a :py:class:`~dagster.Partition` and returns the run
-            configuration that parameterizes the execution for this partition.
-        tags_fn_for_partition (Callable[[Partition], Optional[dict[str, str]]]): A function that
-            takes a :py:class:`~dagster.Partition` and returns a list of key value pairs that will
-            be added to the generated run for this partition.
-        partitions_def (Optional[PartitionsDefinition]): A set of parameters used to construct the set
-            of valid partition objects.
-    """
-
-    _name: str
-    _pipeline_name: Optional[str]
-    _job_name: Optional[str]
-    _solid_selection: Optional[Sequence[str]]
-    _mode: Optional[str]
-    _user_defined_run_config_fn_for_partition: Callable[[Partition], Mapping[str, Any]]
-    _user_defined_tags_fn_for_partition: Callable[[Partition], Optional[Mapping[str, str]]]
-    _partitions_def: PartitionsDefinition
-
-    def __init__(
+    def has_partition_key(
         self,
-        name: str,
-        pipeline_name: Optional[str] = None,
-        partition_fn: Optional[RawPartitionFunction] = None,
-        solid_selection: Optional[Sequence[str]] = None,
-        mode: Optional[str] = None,
-        run_config_fn_for_partition: Callable[
-            [Partition[T_cov]], Mapping[str, Any]
-        ] = lambda _partition: {},
-        tags_fn_for_partition: Callable[
-            [Partition[T_cov]], Optional[Mapping[str, str]]
-        ] = lambda _partition: {},
-        partitions_def: Optional[
-            PartitionsDefinition[T_cov]  # pylint: disable=unsubscriptable-object
-        ] = None,
-        job_name: Optional[str] = None,
-    ):
-        check.invariant(
-            not (partition_fn and partitions_def),
-            "Only one of `partition_fn` or `partitions_def` must be supplied.",
-        )
-        check.invariant(
-            (pipeline_name or job_name) and not (pipeline_name and job_name),
-            "Exactly one one of `job_name` and `pipeline_name` must be supplied.",
-        )
-
-        self._name = check_valid_name(name)
-        self._pipeline_name = check.opt_str_param(pipeline_name, "pipeline_name")
-        self._job_name = check.opt_str_param(job_name, "job_name")
-        self._solid_selection = check.opt_nullable_sequence_param(
-            solid_selection, "solid_selection", of_type=str
-        )
-        self._mode = check.opt_str_param(mode, "mode", DEFAULT_MODE_NAME)
-        # Type ignores workaround for mypy bug "cannot assign to a method"
-        self._user_defined_run_config_fn_for_partition = check.callable_param(
-            run_config_fn_for_partition, "run_config_fn_for_partition"
-        )
-        self._user_defined_tags_fn_for_partition = check.callable_param(
-            tags_fn_for_partition, "tags_fn_for_partition"
-        )
-        if partitions_def is not None:
-            self._partitions_def = check.inst_param(
-                partitions_def, "partitions_def", PartitionsDefinition
-            )
-        elif partition_fn is not None:
-            _wrapped = self._wrap_partition_fn(partition_fn)
-            self._partitions_def = DynamicPartitionsDefinition(partition_fn=_wrapped)
-        else:
+        partition_key: str,
+        current_time: Optional[datetime] = None,
+        dynamic_partitions_store: Optional[DynamicPartitionsStore] = None,
+    ) -> bool:
+        if dynamic_partitions_store is None:
             check.failed(
-                "One of `partition_fn` or `partitions_def` must be supplied.",
+                "The instance is not available to load partitions. You may be seeing this error"
+                " when using dynamic partitions with a version of dagit or dagster-cloud that"
+                " is older than 1.1.18."
             )
 
-    def _wrap_partition_fn(self, partition_fn: RawPartitionFunction) -> PartitionFunction:
-        partition_fn_param_count = len(inspect.signature(partition_fn).parameters)
-
-        def wrap_partition(x: Union[str, Partition]) -> Partition:
-            if isinstance(x, Partition):
-                return x
-            if isinstance(x, str):
-                return Partition(x)
-            raise DagsterInvalidDefinitionError(
-                "Expected <Partition> | <str>, received {type}".format(type=type(x))
-            )
-
-        def wrapper(current_time: Optional[datetime] = None) -> Sequence[Partition]:
-            if not current_time:
-                current_time = pendulum.now("UTC")
-
-            check.callable_param(partition_fn, "partition_fn")
-
-            if partition_fn_param_count == 1:
-                obj_list = cast(
-                    Callable[..., Sequence[Union[Partition[T_cov], str]]],
-                    partition_fn,
-                )(current_time)
-            else:
-                obj_list = partition_fn()  # type: ignore
-
-            return [wrap_partition(obj) for obj in obj_list]
-
-        return wrapper
-
-    @property
-    def name(self) -> str:
-        return self._name
-
-    @property
-    def pipeline_name(self) -> Optional[str]:
-        return self._pipeline_name
-
-    @property
-    def job_name(self) -> Optional[str]:
-        return self._job_name
-
-    @property
-    def pipeline_or_job_name(self) -> str:
-        # one is guaranteed to be set
-        return cast(str, self._pipeline_name or self._job_name)
-
-    @property
-    def solid_selection(self) -> Optional[Sequence[str]]:
-        return self._solid_selection
-
-    @property
-    def mode(self) -> Optional[str]:
-        return self._mode
-
-    @property
-    def partitions_def(self) -> PartitionsDefinition:
-        return self._partitions_def
-
-    def run_config_for_partition(self, partition: Partition[T_cov]) -> Mapping[str, Any]:
-        return copy.deepcopy(self._user_defined_run_config_fn_for_partition(partition))
-
-    def tags_for_partition(self, partition: Partition[T_cov]) -> Mapping[str, str]:
-        user_tags = validate_tags(
-            self._user_defined_tags_fn_for_partition(partition), allow_reserved_tags=False
+        return dynamic_partitions_store.has_dynamic_partition(
+            partitions_def_name=self._validated_name(), partition_key=partition_key
         )
-        tags = merge_dicts(user_tags, DagsterRun.tags_for_partition_set(self, partition))
-
-        return tags
-
-    def get_partitions(
-        self,
-        current_time: Optional[datetime] = None,
-        dynamic_partitions_store: Optional[DynamicPartitionsStore] = None,
-    ) -> Sequence[Partition[T_cov]]:
-        """Return the set of known partitions.
-
-        Arguments:
-            current_time (Optional[datetime]): The evaluation time for the partition function, which
-                is passed through to the ``partition_fn`` (if it accepts a parameter).  Defaults to
-                the current time in UTC.
-
-        """
-        return self._partitions_def.get_partitions(
-            current_time, dynamic_partitions_store=dynamic_partitions_store
-        )
-
-    def get_partition(
-        self,
-        name: str,
-        dynamic_partitions_store: Optional[DynamicPartitionsStore] = None,
-        current_time: Optional[datetime] = None,
-    ) -> Partition[T_cov]:
-        return self._partitions_def.get_partition(
-            name, current_time=current_time, dynamic_partitions_store=dynamic_partitions_store
-        )
-
-    def get_partition_names(
-        self,
-        current_time: Optional[datetime] = None,
-        dynamic_partitions_store: Optional[DynamicPartitionsStore] = None,
-    ) -> Sequence[str]:
-        return [part.name for part in self.get_partitions(current_time, dynamic_partitions_store)]
-
-    def create_schedule_definition(
-        self,
-        schedule_name: str,
-        cron_schedule: str,
-        partition_selector: PartitionSelectorFunction,
-        should_execute: Optional[Callable[..., bool]] = None,
-        environment_vars: Optional[Mapping[str, str]] = None,
-        execution_timezone: Optional[str] = None,
-        description: Optional[str] = None,
-        decorated_fn: Optional[PartitionScheduleFunction] = None,
-        job: Optional[ExecutableDefinition] = None,
-        default_status=DefaultScheduleStatus.STOPPED,
-    ) -> "PartitionScheduleDefinition":
-        """Create a ScheduleDefinition from a PartitionSetDefinition.
-
-        Arguments:
-            schedule_name (str): The name of the schedule.
-            cron_schedule (str): A valid cron string for the schedule
-            partition_selector (Callable[[ScheduleEvaluationContext, PartitionSetDefinition], Union[Partition, Sequence[Partition]]]):
-                Function that determines the partition to use at a given execution time. Can return
-                either a single Partition or a list of Partitions. For time-based partition sets,
-                will likely be either `identity_partition_selector` or a selector returned by
-                `create_offset_partition_selector`.
-            should_execute (Optional[function]): Function that runs at schedule execution time that
-                determines whether a schedule should execute. Defaults to a function that always returns
-                ``True``.
-            environment_vars (Optional[dict]): The environment variables to set for the schedule.
-            execution_timezone (Optional[str]): Timezone in which the schedule should run.
-                Supported strings for timezones are the ones provided by the
-                `IANA time zone database <https://www.iana.org/time-zones>` - e.g. "America/Los_Angeles".
-            description (Optional[str]): A human-readable description of the schedule.
-            default_status (DefaultScheduleStatus): Whether the schedule starts as running or not. The default
-                status can be overridden from Dagit or via the GraphQL API.
-
-        Returns:
-            PartitionScheduleDefinition: The generated PartitionScheduleDefinition for the partition
-                selector
-        """
-        check.str_param(schedule_name, "schedule_name")
-        check.str_param(cron_schedule, "cron_schedule")
-        check.opt_callable_param(should_execute, "should_execute")
-        check.opt_mapping_param(environment_vars, "environment_vars", key_type=str, value_type=str)
-        check.callable_param(partition_selector, "partition_selector")
-        check.opt_str_param(execution_timezone, "execution_timezone")
-        check.opt_str_param(description, "description")
-        check.inst_param(default_status, "default_status", DefaultScheduleStatus)
-
-        def _execution_fn(context):
-            check.inst_param(context, "context", ScheduleEvaluationContext)
-            with user_code_error_boundary(
-                ScheduleExecutionError,
-                lambda: f"Error occurred during the execution of partition_selector for schedule {schedule_name}",
-            ):
-                selector_result = partition_selector(context, self)
-
-            if isinstance(selector_result, SkipReason):
-                yield selector_result
-                return
-
-            selected_partitions = (
-                selector_result
-                if isinstance(selector_result, (frozenlist, list))
-                else [selector_result]
-            )
-
-            check.is_list(selected_partitions, of_type=Partition)
-
-            if not selected_partitions:
-                yield SkipReason("Partition selector returned an empty list of partitions.")
-                return
-
-            partition_names = self.get_partition_names(context.scheduled_execution_time)
-
-            missing_partition_names = [
-                partition.name
-                for partition in selected_partitions
-                if partition.name not in partition_names
-            ]
-
-            if missing_partition_names:
-                yield SkipReason(
-                    "Partition selector returned partition"
-                    + ("s" if len(missing_partition_names) > 1 else "")
-                    + f" not in the partition set: {', '.join(missing_partition_names)}."
-                )
-                return
-
-            with user_code_error_boundary(
-                ScheduleExecutionError,
-                lambda: f"Error occurred during the execution of should_execute for schedule {schedule_name}",
-            ):
-                if should_execute and not should_execute(context):
-                    yield SkipReason(
-                        "should_execute function for {schedule_name} returned false.".format(
-                            schedule_name=schedule_name
-                        )
-                    )
-                    return
-
-            for selected_partition in selected_partitions:
-                with user_code_error_boundary(
-                    ScheduleExecutionError,
-                    lambda: f"Error occurred during the execution of run_config_fn for schedule {schedule_name}",
-                ):
-                    run_config = self.run_config_for_partition(selected_partition)
-
-                with user_code_error_boundary(
-                    ScheduleExecutionError,
-                    lambda: f"Error occurred during the execution of tags_fn for schedule {schedule_name}",
-                ):
-                    tags = self.tags_for_partition(selected_partition)
-                yield RunRequest(
-                    run_key=selected_partition.name if len(selected_partitions) > 0 else None,
-                    run_config=run_config,
-                    tags=tags,
-                )
-
-        return PartitionScheduleDefinition(
-            name=schedule_name,
-            cron_schedule=cron_schedule,
-            pipeline_name=self._pipeline_name,
-            tags_fn=None,
-            should_execute=None,
-            environment_vars=environment_vars,
-            partition_set=self,
-            execution_timezone=execution_timezone,
-            execution_fn=_execution_fn,
-            description=description,
-            decorated_fn=decorated_fn,
-            job=job,
-            default_status=default_status,
-        )
-
-
-class PartitionScheduleDefinition(ScheduleDefinition):
-    __slots__ = ["_partition_set"]
-
-    def __init__(
-        self,
-        name: str,
-        cron_schedule: str,
-        pipeline_name: Optional[str],
-        tags_fn: Optional[ScheduleTagsFunction],
-        should_execute: Optional[ScheduleShouldExecuteFunction],
-        partition_set: PartitionSetDefinition,
-        environment_vars: Optional[Mapping[str, str]] = None,
-        run_config_fn: Optional[ScheduleRunConfigFunction] = None,
-        execution_timezone: Optional[str] = None,
-        execution_fn: Optional[ScheduleExecutionFunction] = None,
-        description: Optional[str] = None,
-        decorated_fn: Optional[PartitionScheduleFunction] = None,
-        job: Optional[ExecutableDefinition] = None,
-        default_status: DefaultScheduleStatus = DefaultScheduleStatus.STOPPED,
-    ):
-        super(PartitionScheduleDefinition, self).__init__(
-            name=check_valid_name(name),
-            cron_schedule=cron_schedule,
-            job_name=pipeline_name,
-            run_config_fn=run_config_fn,
-            tags_fn=tags_fn,
-            should_execute=should_execute,
-            environment_vars=environment_vars,
-            execution_timezone=execution_timezone,
-            execution_fn=execution_fn,
-            description=description,
-            job=job,
-            default_status=default_status,
-        )
-        self._partition_set = check.inst_param(
-            partition_set, "partition_set", PartitionSetDefinition
-        )
-        self._decorated_fn = check.opt_callable_param(decorated_fn, "decorated_fn")
-
-    def __call__(self, *args, **kwargs) -> Mapping[str, Any]:
-        if not self._decorated_fn:
-            raise DagsterInvalidInvocationError(
-                "Only partition schedules created using one of the partition schedule decorators "
-                "can be directly invoked."
-            )
-        if len(args) == 0 and len(kwargs) == 0:
-            raise DagsterInvalidInvocationError(
-                "Schedule decorated function has date argument, but no date argument was "
-                "provided when invoking."
-            )
-        if len(args) + len(kwargs) > 1:
-            raise DagsterInvalidInvocationError(
-                "Schedule invocation received multiple arguments. Only a first "
-                "positional date parameter should be provided when invoking."
-            )
-
-        date_param_name = get_function_params(self._decorated_fn)[0].name
-
-        if args:
-            date = check.opt_inst_param(args[0], date_param_name, datetime)
-        else:
-            if date_param_name not in kwargs:
-                raise DagsterInvalidInvocationError(
-                    f"Schedule invocation expected argument '{date_param_name}'."
-                )
-            date = check.opt_inst_param(kwargs[date_param_name], date_param_name, datetime)
-
-        return self._decorated_fn(date)  # type: ignore
-
-    def get_partition_set(self) -> PartitionSetDefinition:
-        return self._partition_set
 
 
 class PartitionedConfig(Generic[T_cov]):
@@ -1104,7 +721,7 @@ class PartitionedConfig(Generic[T_cov]):
 
     def __init__(
         self,
-        partitions_def: PartitionsDefinition[T_cov],  # pylint: disable=unsubscriptable-object
+        partitions_def: PartitionsDefinition[T_cov],
         run_config_for_partition_fn: Callable[[Partition[T_cov]], Mapping[str, Any]],
         decorated_fn: Optional[Callable[..., Mapping[str, Any]]] = None,
         tags_for_partition_fn: Optional[Callable[[Partition[T_cov]], Mapping[str, str]]] = None,
@@ -1122,7 +739,7 @@ class PartitionedConfig(Generic[T_cov]):
     @property
     def partitions_def(
         self,
-    ) -> PartitionsDefinition[T_cov]:  # pylint: disable=unsubscriptable-object
+    ) -> PartitionsDefinition[T_cov]:
         return self._partitions
 
     @public
@@ -1138,17 +755,65 @@ class PartitionedConfig(Generic[T_cov]):
     def get_partition_keys(self, current_time: Optional[datetime] = None) -> Sequence[str]:
         return [partition.name for partition in self.partitions_def.get_partitions(current_time)]
 
-    def get_run_config_for_partition_key(self, partition_key: str) -> Mapping[str, Any]:
+    def get_run_config_for_partition_key(
+        self,
+        partition_key: str,
+        dynamic_partitions_store: Optional[DynamicPartitionsStore] = None,
+        current_time: Optional[datetime] = None,
+    ) -> Mapping[str, Any]:
         """Generates the run config corresponding to a partition key.
 
         Args:
             partition_key (str): the key for a partition that should be used to generate a run config.
         """
-        partitions = self.partitions_def.get_partitions()
-        partition = [p for p in partitions if p.name == partition_key]
-        if len(partition) == 0:
-            raise DagsterInvalidInvocationError(f"No partition for partition key {partition_key}.")
-        return self.run_config_for_partition_fn(partition[0])
+        partition = self._key_to_partition(partition_key, current_time, dynamic_partitions_store)
+        return copy.deepcopy(self.run_config_for_partition_fn(partition))
+
+    def get_tags_for_partition_key(
+        self,
+        partition_key: str,
+        dynamic_partitions_store: Optional[DynamicPartitionsStore] = None,
+        current_time: Optional[datetime] = None,
+        job_name: Optional[str] = None,
+    ) -> Mapping[str, str]:
+        from dagster._core.host_representation.external_data import (
+            external_partition_set_name_for_job_name,
+        )
+
+        partition = self._key_to_partition(partition_key, current_time, dynamic_partitions_store)
+        user_tags = (
+            validate_tags(self._tags_for_partition_fn(partition), allow_reserved_tags=False)
+            if self._tags_for_partition_fn
+            else {}
+        )
+        system_tags = {
+            **self.partitions_def.get_tags_for_partition_key(partition.name),
+            **(
+                {PARTITION_SET_TAG: external_partition_set_name_for_job_name(job_name)}
+                if job_name
+                else {}
+            ),
+        }
+        # `PartitionSetDefinition` has been deleted but we still need to attach this special tag in
+        # order for reexecution against partitions to work properly.
+        return {**user_tags, **system_tags}
+
+    def _key_to_partition(
+        self,
+        partition_key: str,
+        current_time: Optional[datetime],
+        dynamic_partitions_store: Optional[DynamicPartitionsStore],
+    ) -> Partition[T_cov]:
+        matches = [
+            p
+            for p in self.partitions_def.get_partitions(
+                current_time=current_time, dynamic_partitions_store=dynamic_partitions_store
+            )
+            if p.name == partition_key
+        ]
+        if len(matches) == 0:
+            raise DagsterUnknownPartitionError(f"No partition for partition key `{partition_key}`.")
+        return matches[0]
 
     @classmethod
     def from_flexible_config(
@@ -1279,7 +944,7 @@ def cron_schedule_from_schedule_type_and_offsets(
     minute_offset: int,
     hour_offset: int,
     day_offset: Optional[int],
-):
+) -> str:
     if schedule_type is ScheduleType.HOURLY:
         return f"{minute_offset} * * * *"
     elif schedule_type is ScheduleType.DAILY:
@@ -1485,8 +1150,8 @@ class DefaultPartitionsSubset(PartitionsSubset[T_cov]):
     def __eq__(self, other: object) -> bool:
         return (
             isinstance(other, DefaultPartitionsSubset)
-            and self._partitions_def == other._partitions_def
-            and self._subset == other._subset
+            and self._partitions_def == other._partitions_def  # noqa: SLF001
+            and self._subset == other._subset  # noqa: SLF001
         )
 
     def __len__(self) -> int:
