@@ -13,8 +13,7 @@ if TYPE_CHECKING:
 
 
 class IPipeline(ABC):
-    """
-    IPipeline is a wrapper interface for PipelineDefinitions to be used as parameters to Dagster's
+    """IPipeline is a wrapper interface for PipelineDefinitions to be used as parameters to Dagster's
     core execution APIs.  This enables these execution APIs to operate on both in memory pipeline
     definitions to be executed in the current process (InMemoryPipeline) as well as definitions that
     can be reconstructed and executed in a different process (ReconstructablePipeline).
