@@ -294,7 +294,8 @@ class _ResourceDecoratorCallable:
             required_resource_keys=self.required_resource_keys,
         )
 
-        update_wrapper(resource_def, wrapped=resource_fn)
+        # `update_wrapper` typing cannot currently handle a Union of Callables correctly
+        update_wrapper(resource_def, wrapped=resource_fn)  # type: ignore
 
         return resource_def
 
