@@ -18,10 +18,12 @@ from typing_extensions import TypeAlias
 
 import dagster._check as check
 from dagster._annotations import public
+from dagster._core.decorator_utils import has_at_least_one_parameter
 from dagster._core.definitions.instigation_logger import InstigationLogger
 from dagster._core.errors import (
     DagsterInvalidDefinitionError,
     DagsterInvalidInvocationError,
+    DagsterInvariantViolationError,
     RunStatusSensorExecutionError,
     user_code_error_boundary,
 )
@@ -29,12 +31,11 @@ from dagster._core.events import PIPELINE_RUN_STATUS_TO_EVENT_TYPE, DagsterEvent
 from dagster._core.instance import DagsterInstance
 from dagster._core.storage.pipeline_run import DagsterRun, DagsterRunStatus, RunsFilter
 from dagster._serdes import (
-    deserialize_json_to_dagster_namedtuple,
-    serialize_dagster_namedtuple,
+    serialize_value,
     whitelist_for_serdes,
 )
 from dagster._serdes.errors import DeserializationError
-from dagster._serdes.serdes import register_serdes_tuple_fallbacks
+from dagster._serdes.serdes import deserialize_value
 from dagster._seven import JSONDecodeError
 from dagster._utils import utc_datetime_from_timestamp
 from dagster._utils.backcompat import deprecation_warning
@@ -50,9 +51,9 @@ from .sensor_definition import (
     RunRequest,
     SensorDefinition,
     SensorEvaluationContext,
+    SensorResult,
     SensorType,
     SkipReason,
-    has_at_least_one_parameter,
 )
 from .target import ExecutableDefinition
 from .unresolved_asset_job_definition import UnresolvedAssetJobDefinition
@@ -74,7 +75,7 @@ RunFailureSensorEvaluationFn: TypeAlias = Union[
 ]
 
 
-@whitelist_for_serdes
+@whitelist_for_serdes(old_storage_names={"PipelineSensorCursor"})
 class RunStatusSensorCursor(
     NamedTuple(
         "_RunStatusSensorCursor",
@@ -91,21 +92,17 @@ class RunStatusSensorCursor(
     @staticmethod
     def is_valid(json_str: str) -> bool:
         try:
-            obj = deserialize_json_to_dagster_namedtuple(json_str)
+            obj = deserialize_value(json_str, RunStatusSensorCursor)
             return isinstance(obj, RunStatusSensorCursor)
         except (JSONDecodeError, DeserializationError):
             return False
 
     def to_json(self) -> str:
-        return serialize_dagster_namedtuple(cast(NamedTuple, self))
+        return serialize_value(cast(NamedTuple, self))
 
     @staticmethod
-    def from_json(json_str: str) -> tuple:
-        return deserialize_json_to_dagster_namedtuple(json_str)
-
-
-# handle backcompat
-register_serdes_tuple_fallbacks({"PipelineSensorCursor": RunStatusSensorCursor})
+    def from_json(json_str: str) -> "RunStatusSensorCursor":
+        return deserialize_value(json_str, RunStatusSensorCursor)
 
 
 class RunStatusSensorContext:
@@ -198,8 +195,7 @@ def build_run_status_sensor_context(
     dagster_run: DagsterRun,
     context: Optional[SensorEvaluationContext] = None,
 ) -> RunStatusSensorContext:
-    """
-    Builds run status sensor context from provided parameters.
+    """Builds run status sensor context from provided parameters.
 
     This function can be used to provide the context argument when directly invoking a function
     decorated with `@run_status_sensor` or `@run_failure_sensor`, such as when writing unit tests.
@@ -314,8 +310,7 @@ def run_failure_sensor(
     request_job: Optional[ExecutableDefinition] = None,
     request_jobs: Optional[Sequence[ExecutableDefinition]] = None,
 ) -> Union[SensorDefinition, Callable[[RunFailureSensorEvaluationFn], SensorDefinition,]]:
-    """
-    Creates a sensor that reacts to job failure events, where the decorated function will be
+    """Creates a sensor that reacts to job failure events, where the decorated function will be
     run when a run fails.
 
     Takes a :py:class:`~dagster.RunFailureSensorContext`.
@@ -382,8 +377,7 @@ def run_failure_sensor(
 
 
 class RunStatusSensorDefinition(SensorDefinition):
-    """
-    Define a sensor that reacts to a given status of pipeline execution, where the decorated
+    """Define a sensor that reacts to a given status of pipeline execution, where the decorated
     function will be evaluated when a run is at the given status.
 
     Args:
@@ -490,7 +484,7 @@ class RunStatusSensorDefinition(SensorDefinition):
 
         def _wrapped_fn(
             context: SensorEvaluationContext,
-        ) -> Iterator[Union[RunRequest, SkipReason, PipelineRunReaction]]:
+        ) -> Iterator[Union[RunRequest, SkipReason, PipelineRunReaction, SensorResult]]:
             # initiate the cursor to (most recent event id, current timestamp) when:
             # * it's the first time starting the sensor
             # * or, the cursor isn't in valid format (backcompt)
@@ -590,7 +584,7 @@ class RunStatusSensorDefinition(SensorDefinition):
                         pipeline_run.external_pipeline_origin
                     ).external_repository_origin
                     run_job_selector = JobSelector(
-                        location_name=external_repository_origin.repository_location_origin.location_name,
+                        location_name=external_repository_origin.code_location_origin.location_name,
                         repository_name=external_repository_origin.repository_name,
                         job_name=pipeline_run.pipeline_name,
                     )
@@ -599,7 +593,7 @@ class RunStatusSensorDefinition(SensorDefinition):
 
                     # make a RepositorySelector for the run in question
                     run_repo_selector = RepositorySelector(
-                        location_name=external_repository_origin.repository_location_origin.location_name,
+                        location_name=external_repository_origin.code_location_origin.location_name,
                         repository_name=external_repository_origin.repository_name,
                     )
                     if run_repo_selector in other_repos:
@@ -621,16 +615,20 @@ class RunStatusSensorDefinition(SensorDefinition):
                         RunStatusSensorExecutionError,
                         lambda: f'Error occurred during the execution sensor "{name}".',
                     ):
-                        # one user code invocation maps to one failure event
-                        sensor_return = run_status_sensor_fn(
-                            RunStatusSensorContext(  # type: ignore
-                                sensor_name=name,
-                                dagster_run=pipeline_run,
-                                dagster_event=event_log_entry.dagster_event,
-                                instance=context.instance,
-                                context=context,
+                        if has_at_least_one_parameter(run_status_sensor_fn):
+                            # one user code invocation maps to one failure event
+                            sensor_return = run_status_sensor_fn(
+                                RunStatusSensorContext(
+                                    sensor_name=name,
+                                    dagster_run=pipeline_run,
+                                    dagster_event=event_log_entry.dagster_event,
+                                    instance=context.instance,
+                                    context=context,
+                                )
                             )
-                        )
+                        else:
+                            sensor_return = run_status_sensor_fn()  # type: ignore
+
                         if sensor_return is not None:
                             context.update_cursor(
                                 RunStatusSensorCursor(
@@ -639,8 +637,17 @@ class RunStatusSensorDefinition(SensorDefinition):
                                 ).to_json()
                             )
 
-                            if isinstance(
-                                sensor_return, (RunRequest, SkipReason, PipelineRunReaction)
+                            if isinstance(sensor_return, SensorResult):
+                                if sensor_return.cursor:
+                                    raise DagsterInvariantViolationError(
+                                        f"Error in run status sensor {name}: Sensor returned a"
+                                        " SensorResult with a cursor value. The cursor is managed"
+                                        " by the sensor and should not be modified by a user."
+                                    )
+                                yield sensor_return
+                            elif isinstance(
+                                sensor_return,
+                                (RunRequest, SkipReason, PipelineRunReaction),
                             ):
                                 yield sensor_return
                             else:
@@ -759,8 +766,7 @@ def run_status_sensor(
     request_job: Optional[ExecutableDefinition] = None,
     request_jobs: Optional[Sequence[ExecutableDefinition]] = None,
 ) -> Callable[[RunStatusSensorEvaluationFunction], RunStatusSensorDefinition,]:
-    """
-    Creates a sensor that reacts to a given status of pipeline execution, where the decorated
+    """Creates a sensor that reacts to a given status of pipeline execution, where the decorated
     function will be run when a pipeline is at the given status.
 
     Takes a :py:class:`~dagster.RunStatusSensorContext`.
