@@ -3,7 +3,6 @@ import logging
 import os
 import sys
 from enum import Enum
-from inspect import Parameter
 from typing import (
     TYPE_CHECKING,
     AbstractSet,
@@ -13,7 +12,7 @@ from typing import (
     NamedTuple,
     Optional,
     Sequence,
-    Type,
+    Tuple,
     Union,
     cast,
 )
@@ -26,12 +25,15 @@ from dagster._core.definitions import (
     AssetObservation,
     ExpectationResult,
     HookDefinition,
-    Materialization,
-    MetadataEntry,
     NodeHandle,
 )
 from dagster._core.definitions.events import AssetLineageInfo, ObjectStoreOperationType
-from dagster._core.definitions.metadata import MetadataValue
+from dagster._core.definitions.metadata import (
+    MetadataFieldSerializer,
+    MetadataValue,
+    RawMetadataValue,
+    normalize_metadata,
+)
 from dagster._core.errors import HookExecutionError
 from dagster._core.execution.context.system import IPlanContext, IStepContext, StepExecutionContext
 from dagster._core.execution.plan.handle import ResolvedFromDynamicStepHandle, StepHandle
@@ -42,12 +44,9 @@ from dagster._core.log_manager import DagsterLogManager
 from dagster._core.storage.captured_log_manager import CapturedLogContext
 from dagster._core.storage.pipeline_run import DagsterRunStatus
 from dagster._serdes import (
-    DefaultNamedTupleSerializer,
-    WhitelistMap,
-    register_serdes_tuple_fallbacks,
+    NamedTupleSerializer,
     whitelist_for_serdes,
 )
-from dagster._serdes.serdes import replace_storage_keys
 from dagster._utils.error import SerializableErrorInfo, serializable_error_info_from_exc_info
 from dagster._utils.timing import format_duration
 
@@ -314,50 +313,41 @@ def log_resource_event(log_manager: DagsterLogManager, event: "DagsterEvent") ->
     log_manager.log_dagster_event(level=log_level, msg=event.message or "", dagster_event=event)
 
 
-class DagsterEventSerializer(DefaultNamedTupleSerializer):
-    @classmethod
-    def value_from_storage_dict(
-        cls,
-        storage_dict: Dict[str, Any],
-        klass: Type,
-        args_for_class: Mapping[str, Parameter],
-        whitelist_map: WhitelistMap,
-        descent_path: str,
-    ) -> NamedTuple:
-        event_type_value = storage_dict["event_type_value"]
-        pipeline_name = storage_dict["pipeline_name"]
-        message = storage_dict.get("message")
-        step_key = storage_dict.get("step_key")
-        event_specific_data = storage_dict.get("event_specific_data")
-
+class DagsterEventSerializer(NamedTupleSerializer["DagsterEvent"]):
+    def before_unpack(self, **storage_dict: Any) -> Dict[str, Any]:
         event_type_value, event_specific_data = _handle_back_compat(
-            event_type_value, event_specific_data
+            storage_dict["event_type_value"], storage_dict.get("event_specific_data")
         )
         storage_dict["event_type_value"] = event_type_value
         storage_dict["event_specific_data"] = event_specific_data
 
-        try:
-            return super().value_from_storage_dict(
-                storage_dict, klass, args_for_class, whitelist_map, descent_path
-            )
-        except Exception:
-            new_message = (
-                f"Could not deserialize event of type {event_type_value}. This event may have been"
-                " written by a newer version of Dagster."
-                + (f' Original message: "{message}"' if message else "")
-            )
-            return DagsterEvent(
-                event_type_value=DagsterEventType.ENGINE_EVENT.value,
-                pipeline_name=pipeline_name,
-                message=new_message,
-                step_key=step_key,
-                event_specific_data=EngineEventData(
-                    error=serializable_error_info_from_exc_info(sys.exc_info())
-                ),
-            )
+        return storage_dict
+
+    def handle_unpack_error(self, exc: Exception, **storage_dict: Any) -> "DagsterEvent":
+        event_type_value, _ = _handle_back_compat(
+            storage_dict["event_type_value"], storage_dict.get("event_specific_data")
+        )
+        step_key = storage_dict.get("step_key")
+        orig_message = storage_dict.get("message")
+        new_message = (
+            f"Could not deserialize event of type {event_type_value}. This event may have been"
+            " written by a newer version of Dagster."
+            + (f' Original message: "{orig_message}"' if orig_message else "")
+        )
+        return DagsterEvent(
+            event_type_value=DagsterEventType.ENGINE_EVENT.value,
+            pipeline_name=storage_dict["pipeline_name"],
+            message=new_message,
+            step_key=step_key,
+            event_specific_data=EngineEventData(
+                error=serializable_error_info_from_exc_info(sys.exc_info())
+            ),
+        )
 
 
-@whitelist_for_serdes(serializer=DagsterEventSerializer)
+@whitelist_for_serdes(
+    serializer=DagsterEventSerializer, storage_field_names={"node_handle": "solid_handle"}
+)
 class DagsterEvent(
     NamedTuple(
         "_DagsterEvent",
@@ -365,7 +355,7 @@ class DagsterEvent(
             ("event_type_value", str),
             ("pipeline_name", str),
             ("step_handle", Optional[Union[StepHandle, ResolvedFromDynamicStepHandle]]),
-            ("solid_handle", Optional[NodeHandle]),
+            ("node_handle", Optional[NodeHandle]),
             ("step_kind_value", Optional[str]),
             ("logging_tags", Optional[Mapping[str, str]]),
             ("event_specific_data", Optional["EventSpecificData"]),
@@ -382,7 +372,7 @@ class DagsterEvent(
     Attributes:
         event_type_value (str): Value for a DagsterEventType.
         pipeline_name (str)
-        solid_handle (NodeHandle)
+        node_handle (NodeHandle)
         step_kind_value (str): Value for a StepKind.
         logging_tags (Dict[str, str])
         event_specific_data (Any): Type must correspond to event_type_value.
@@ -402,7 +392,7 @@ class DagsterEvent(
             event_type_value=check.inst_param(event_type, "event_type", DagsterEventType).value,
             pipeline_name=step_context.pipeline_name,
             step_handle=step_context.step.handle,
-            solid_handle=step_context.step.node_handle,
+            node_handle=step_context.step.node_handle,
             step_kind_value=step_context.step.kind.value,
             logging_tags=step_context.logging_tags,
             event_specific_data=_validate_event_specific_data(event_type, event_specific_data),
@@ -466,7 +456,7 @@ class DagsterEvent(
         event_type_value: str,
         pipeline_name: str,
         step_handle: Optional[Union[StepHandle, ResolvedFromDynamicStepHandle]] = None,
-        solid_handle: Optional[NodeHandle] = None,
+        node_handle: Optional[NodeHandle] = None,
         step_kind_value: Optional[str] = None,
         logging_tags: Optional[Mapping[str, str]] = None,
         event_specific_data: Optional["EventSpecificData"] = None,
@@ -475,9 +465,9 @@ class DagsterEvent(
         # legacy
         step_key: Optional[str] = None,
     ):
-        # old events may contain solid_handle but not step_handle
-        if solid_handle is not None and step_handle is None:
-            step_handle = StepHandle(solid_handle)
+        # old events may contain node_handle but not step_handle
+        if node_handle is not None and step_handle is None:
+            step_handle = StepHandle(node_handle)
 
         # Legacy events may have step_key set directly, preserve those to stay in sync
         # with legacy execution plan snapshots.
@@ -491,7 +481,7 @@ class DagsterEvent(
             check.opt_inst_param(
                 step_handle, "step_handle", (StepHandle, ResolvedFromDynamicStepHandle)
             ),
-            check.opt_inst_param(solid_handle, "solid_handle", NodeHandle),
+            check.opt_inst_param(node_handle, "node_handle", NodeHandle),
             check.opt_str_param(step_kind_value, "step_kind_value"),
             check.opt_mapping_param(logging_tags, "logging_tags"),
             _validate_event_specific_data(DagsterEventType(event_type_value), event_specific_data),
@@ -500,15 +490,11 @@ class DagsterEvent(
             check.opt_str_param(step_key, "step_key"),
         )
 
-    @property
-    def node_handle(self) -> Optional[NodeHandle]:
-        return self.solid_handle
-
     @property
     def solid_name(self) -> str:
-        check.invariant(self.solid_handle is not None)
-        solid_handle = cast(NodeHandle, self.solid_handle)
-        return solid_handle.name
+        check.invariant(self.node_handle is not None)
+        node_handle = cast(NodeHandle, self.node_handle)
+        return node_handle.name
 
     @public
     @property
@@ -713,7 +699,7 @@ class DagsterEvent(
         return cast(StepExpectationResultData, self.event_specific_data)
 
     @property
-    def materialization(self) -> Union[Materialization, AssetMaterialization]:
+    def materialization(self) -> AssetMaterialization:
         _assert_type(
             "step_materialization_data", DagsterEventType.ASSET_MATERIALIZATION, self.event_type
         )
@@ -803,10 +789,7 @@ class DagsterEvent(
             event_type=DagsterEventType.STEP_FAILURE,
             step_context=step_context,
             event_specific_data=step_failure_data,
-            message=(
-                message
-                or 'Execution of step "{step_key}" failed.'.format(step_key=step_context.step.key)
-            ),
+            message=(message or f'Execution of step "{step_context.step.key}" failed.'),
         )
 
     @staticmethod
@@ -820,7 +803,7 @@ class DagsterEvent(
             message=(
                 'Execution of step "{step_key}" failed and has requested a retry{wait_str}.'.format(
                     step_key=step_context.step.key,
-                    wait_str=" in {n} seconds".format(n=step_retry_data.seconds_to_wait)
+                    wait_str=f" in {step_retry_data.seconds_to_wait} seconds"
                     if step_retry_data.seconds_to_wait
                     else "",
                 )
@@ -897,7 +880,7 @@ class DagsterEvent(
     @staticmethod
     def asset_materialization(
         step_context: IStepContext,
-        materialization: Union[AssetMaterialization, Materialization],
+        materialization: AssetMaterialization,
     ) -> "DagsterEvent":
         return DagsterEvent.from_step(
             event_type=DagsterEventType.ASSET_MATERIALIZATION,
@@ -906,9 +889,7 @@ class DagsterEvent(
             message=materialization.description
             if materialization.description
             else "Materialized value{label_clause}.".format(
-                label_clause=" {label}".format(label=materialization.label)
-                if materialization.label
-                else ""
+                label_clause=f" {materialization.label}" if materialization.label else ""
             ),
         )
 
@@ -1014,14 +995,14 @@ class DagsterEvent(
     def step_worker_starting(
         step_context: IStepContext,
         message: str,
-        metadata_entries: Sequence[MetadataEntry],
+        metadata: Mapping[str, MetadataValue],
     ) -> "DagsterEvent":
         return DagsterEvent.from_step(
             DagsterEventType.STEP_WORKER_STARTING,
             step_context,
             message=message,
             event_specific_data=EngineEventData(
-                metadata_entries=metadata_entries, marker_start="step_process_start"
+                metadata=metadata, marker_start="step_process_start"
             ),
         )
 
@@ -1030,16 +1011,14 @@ class DagsterEvent(
         log_manager: DagsterLogManager,
         pipeline_name: str,
         message: str,
-        metadata_entries: Sequence[MetadataEntry],
+        metadata: Mapping[str, MetadataValue],
         step_key: Optional[str],
     ) -> "DagsterEvent":
         event = DagsterEvent(
             DagsterEventType.STEP_WORKER_STARTED.value,
             pipeline_name=pipeline_name,
             message=message,
-            event_specific_data=EngineEventData(
-                metadata_entries=metadata_entries, marker_end="step_process_start"
-            ),
+            event_specific_data=EngineEventData(metadata=metadata, marker_end="step_process_start"),
             pid=os.getpid(),
             step_key=step_key,
         )
@@ -1065,7 +1044,7 @@ class DagsterEvent(
             message="Starting initialization of resources [{}].".format(
                 ", ".join(sorted(resource_keys))
             ),
-            event_specific_data=EngineEventData(metadata_entries=[], marker_start="resources"),
+            event_specific_data=EngineEventData(metadata={}, marker_start="resources"),
         )
 
     @staticmethod
@@ -1076,17 +1055,10 @@ class DagsterEvent(
         resource_instances: Mapping[str, Any],
         resource_init_times: Mapping[str, str],
     ) -> "DagsterEvent":
-        metadata_entries = []
+        metadata = {}
         for key in resource_instances.keys():
-            metadata_entries.extend(
-                [
-                    MetadataEntry(
-                        key,
-                        value=MetadataValue.python_artifact(resource_instances[key].__class__),
-                    ),
-                    MetadataEntry(f"{key}:init_time", value=resource_init_times[key]),
-                ]
-            )
+            metadata[key] = MetadataValue.python_artifact(resource_instances[key].__class__)
+            metadata[f"{key}:init_time"] = resource_init_times[key]
 
         return DagsterEvent.from_resource(
             DagsterEventType.RESOURCE_INIT_SUCCESS,
@@ -1097,7 +1069,7 @@ class DagsterEvent(
                 ", ".join(sorted(resource_init_times.keys()))
             ),
             event_specific_data=EngineEventData(
-                metadata_entries=metadata_entries,
+                metadata=metadata,
                 marker_end="resources",
             ),
         )
@@ -1117,7 +1089,7 @@ class DagsterEvent(
             log_manager=log_manager,
             message="Initialization of resources [{}] failed.".format(", ".join(resource_keys)),
             event_specific_data=EngineEventData(
-                metadata_entries=[],
+                metadata={},
                 marker_end="resources",
                 error=error,
             ),
@@ -1138,7 +1110,7 @@ class DagsterEvent(
             log_manager=log_manager,
             message="Teardown of resources [{}] failed.".format(", ".join(resource_keys)),
             event_specific_data=EngineEventData(
-                metadata_entries=[],
+                metadata={},
                 marker_start=None,
                 marker_end=None,
                 error=error,
@@ -1233,11 +1205,7 @@ class DagsterEvent(
                 op=object_store_operation_result.op,
                 value_name=value_name,
                 address=object_store_operation_result.key,
-                metadata_entries=[
-                    MetadataEntry(
-                        "key", value=MetadataValue.path(object_store_operation_result.key)
-                    ),
-                ],
+                metadata={"key": MetadataValue.path(object_store_operation_result.key)},
                 version=object_store_operation_result.version,
                 mapping_key=object_store_operation_result.mapping_key,
             ),
@@ -1250,7 +1218,7 @@ class DagsterEvent(
         output_name: str,
         manager_key: str,
         message_override: Optional[str] = None,
-        metadata_entries: Optional[Sequence[MetadataEntry]] = None,
+        metadata: Optional[Mapping[str, MetadataValue]] = None,
     ) -> "DagsterEvent":
         message = f'Handled output "{output_name}" using IO manager "{manager_key}"'
         return DagsterEvent.from_step(
@@ -1259,7 +1227,7 @@ class DagsterEvent(
             event_specific_data=HandledOutputData(
                 output_name=output_name,
                 manager_key=manager_key,
-                metadata_entries=metadata_entries if metadata_entries else [],
+                metadata=metadata if metadata else {},
             ),
             message=message_override or message,
         )
@@ -1272,7 +1240,7 @@ class DagsterEvent(
         upstream_output_name: Optional[str] = None,
         upstream_step_key: Optional[str] = None,
         message_override: Optional[str] = None,
-        metadata_entries: Optional[Sequence[MetadataEntry]] = None,
+        metadata: Optional[Mapping[str, MetadataValue]] = None,
     ) -> "DagsterEvent":
         message = f'Loaded input "{input_name}" using input manager "{manager_key}"'
         if upstream_output_name:
@@ -1286,7 +1254,7 @@ class DagsterEvent(
                 manager_key=manager_key,
                 upstream_output_name=upstream_output_name,
                 upstream_step_key=upstream_step_key,
-                metadata_entries=metadata_entries if metadata_entries else [],
+                metadata=metadata if metadata else {},
             ),
             message=message_override or message,
         )
@@ -1301,7 +1269,7 @@ class DagsterEvent(
             event_type_value=event_type.value,
             pipeline_name=step_context.pipeline_name,
             step_handle=step_context.step.handle,
-            solid_handle=step_context.step.node_handle,
+            node_handle=step_context.step.node_handle,
             step_kind_value=step_context.step.kind.value,
             logging_tags=step_context.logging_tags,
             message=(
@@ -1325,7 +1293,7 @@ class DagsterEvent(
             event_type_value=event_type.value,
             pipeline_name=step_context.pipeline_name,
             step_handle=step_context.step.handle,
-            solid_handle=step_context.step.node_handle,
+            node_handle=step_context.step.node_handle,
             step_kind_value=step_context.step.kind.value,
             logging_tags=step_context.logging_tags,
             event_specific_data=_validate_event_specific_data(
@@ -1350,7 +1318,7 @@ class DagsterEvent(
             event_type_value=event_type.value,
             pipeline_name=step_context.pipeline_name,
             step_handle=step_context.step.handle,
-            solid_handle=step_context.step.node_handle,
+            node_handle=step_context.step.node_handle,
             step_kind_value=step_context.step.kind.value,
             logging_tags=step_context.logging_tags,
             message=(
@@ -1430,20 +1398,20 @@ class StepMaterializationData(
     NamedTuple(
         "_StepMaterializationData",
         [
-            ("materialization", Union[Materialization, AssetMaterialization]),
+            ("materialization", AssetMaterialization),
             ("asset_lineage", Sequence[AssetLineageInfo]),
         ],
     )
 ):
     def __new__(
         cls,
-        materialization: Union[Materialization, AssetMaterialization],
+        materialization: AssetMaterialization,
         asset_lineage: Optional[Sequence[AssetLineageInfo]] = None,
     ):
         return super(StepMaterializationData, cls).__new__(
             cls,
             materialization=check.inst_param(
-                materialization, "materialization", (Materialization, AssetMaterialization)
+                materialization, "materialization", AssetMaterialization
             ),
             asset_lineage=check.opt_sequence_param(
                 asset_lineage, "asset_lineage", of_type=AssetLineageInfo
@@ -1484,14 +1452,17 @@ class StepExpectationResultData(
         )
 
 
-@whitelist_for_serdes
+@whitelist_for_serdes(
+    storage_field_names={"metadata": "metadata_entries"},
+    field_serializers={"metadata": MetadataFieldSerializer},
+)
 class ObjectStoreOperationResultData(
     NamedTuple(
         "_ObjectStoreOperationResultData",
         [
             ("op", ObjectStoreOperationType),
             ("value_name", Optional[str]),
-            ("metadata_entries", Sequence[MetadataEntry]),
+            ("metadata", Mapping[str, MetadataValue]),
             ("address", Optional[str]),
             ("version", Optional[str]),
             ("mapping_key", Optional[str]),
@@ -1502,7 +1473,7 @@ class ObjectStoreOperationResultData(
         cls,
         op: ObjectStoreOperationType,
         value_name: Optional[str] = None,
-        metadata_entries: Optional[Sequence[MetadataEntry]] = None,
+        metadata: Optional[Mapping[str, MetadataValue]] = None,
         address: Optional[str] = None,
         version: Optional[str] = None,
         mapping_key: Optional[str] = None,
@@ -1511,8 +1482,8 @@ class ObjectStoreOperationResultData(
             cls,
             op=cast(ObjectStoreOperationType, check.str_param(op, "op")),
             value_name=check.opt_str_param(value_name, "value_name"),
-            metadata_entries=check.opt_sequence_param(
-                metadata_entries, "metadata_entries", of_type=MetadataEntry
+            metadata=normalize_metadata(
+                check.opt_mapping_param(metadata, "metadata", key_type=str)
             ),
             address=check.opt_str_param(address, "address"),
             version=check.opt_str_param(version, "version"),
@@ -1520,12 +1491,15 @@ class ObjectStoreOperationResultData(
         )
 
 
-@whitelist_for_serdes
+@whitelist_for_serdes(
+    storage_field_names={"metadata": "metadata_entries"},
+    field_serializers={"metadata": MetadataFieldSerializer},
+)
 class EngineEventData(
     NamedTuple(
         "_EngineEventData",
         [
-            ("metadata_entries", Sequence[MetadataEntry]),
+            ("metadata", Mapping[str, MetadataValue]),
             ("error", Optional[SerializableErrorInfo]),
             ("marker_start", Optional[str]),
             ("marker_end", Optional[str]),
@@ -1538,15 +1512,15 @@ class EngineEventData(
     #
     def __new__(
         cls,
-        metadata_entries: Optional[Sequence[MetadataEntry]] = None,
+        metadata: Optional[Mapping[str, RawMetadataValue]] = None,
         error: Optional[SerializableErrorInfo] = None,
         marker_start: Optional[str] = None,
         marker_end: Optional[str] = None,
     ):
         return super(EngineEventData, cls).__new__(
             cls,
-            metadata_entries=check.opt_sequence_param(
-                metadata_entries, "metadata_entries", of_type=MetadataEntry
+            metadata=normalize_metadata(
+                check.opt_mapping_param(metadata, "metadata", key_type=str)
             ),
             error=check.opt_inst_param(error, "error", SerializableErrorInfo),
             marker_start=check.opt_str_param(marker_start, "marker_start"),
@@ -1558,12 +1532,14 @@ class EngineEventData(
         pid: int, step_keys_to_execute: Optional[Sequence[str]] = None
     ) -> "EngineEventData":
         return EngineEventData(
-            metadata_entries=[MetadataEntry("pid", value=str(pid))]
-            + (
-                [MetadataEntry("step_keys", value=str(step_keys_to_execute))]
-                if step_keys_to_execute
-                else []
-            ),
+            metadata={
+                "pid": MetadataValue.text(str(pid)),
+                **(
+                    {"step_keys": MetadataValue.text(str(step_keys_to_execute))}
+                    if step_keys_to_execute
+                    else {}
+                ),
+            }
         )
 
     @staticmethod
@@ -1571,23 +1547,25 @@ class EngineEventData(
         pid: int, step_keys_to_execute: Optional[Sequence[str]] = None
     ) -> "EngineEventData":
         return EngineEventData(
-            metadata_entries=[MetadataEntry("pid", value=str(pid))]
-            + (
-                [MetadataEntry("step_keys", value=str(step_keys_to_execute))]
-                if step_keys_to_execute
-                else []
-            )
+            metadata={
+                "pid": MetadataValue.text(str(pid)),
+                **(
+                    {"step_keys": MetadataValue.text(str(step_keys_to_execute))}
+                    if step_keys_to_execute
+                    else {}
+                ),
+            }
         )
 
     @staticmethod
     def interrupted(steps_interrupted: Sequence[str]) -> "EngineEventData":
         return EngineEventData(
-            metadata_entries=[MetadataEntry("steps_interrupted", value=str(steps_interrupted))]
+            metadata={"steps_interrupted": MetadataValue.text(str(steps_interrupted))}
         )
 
     @staticmethod
     def engine_error(error: SerializableErrorInfo) -> "EngineEventData":
-        return EngineEventData(metadata_entries=[], error=error)
+        return EngineEventData(metadata={}, error=error)
 
 
 @whitelist_for_serdes
@@ -1635,14 +1613,17 @@ class HookErroredData(
         )
 
 
-@whitelist_for_serdes
+@whitelist_for_serdes(
+    storage_field_names={"metadata": "metadata_entries"},
+    field_serializers={"metadata": MetadataFieldSerializer},
+)
 class HandledOutputData(
     NamedTuple(
         "_HandledOutputData",
         [
             ("output_name", str),
             ("manager_key", str),
-            ("metadata_entries", Sequence[MetadataEntry]),
+            ("metadata", Mapping[str, MetadataValue]),
         ],
     )
 ):
@@ -1650,19 +1631,22 @@ class HandledOutputData(
         cls,
         output_name: str,
         manager_key: str,
-        metadata_entries: Optional[Sequence[MetadataEntry]] = None,
+        metadata: Optional[Mapping[str, MetadataValue]] = None,
     ):
         return super(HandledOutputData, cls).__new__(
             cls,
             output_name=check.str_param(output_name, "output_name"),
             manager_key=check.str_param(manager_key, "manager_key"),
-            metadata_entries=check.opt_sequence_param(
-                metadata_entries, "metadata_entries", of_type=MetadataEntry
+            metadata=normalize_metadata(
+                check.opt_mapping_param(metadata, "metadata", key_type=str)
             ),
         )
 
 
-@whitelist_for_serdes
+@whitelist_for_serdes(
+    storage_field_names={"metadata": "metadata_entries"},
+    field_serializers={"metadata": MetadataFieldSerializer},
+)
 class LoadedInputData(
     NamedTuple(
         "_LoadedInputData",
@@ -1671,7 +1655,7 @@ class LoadedInputData(
             ("manager_key", str),
             ("upstream_output_name", Optional[str]),
             ("upstream_step_key", Optional[str]),
-            ("metadata_entries", Optional[Sequence[MetadataEntry]]),
+            ("metadata", Mapping[str, MetadataValue]),
         ],
     )
 ):
@@ -1681,7 +1665,7 @@ class LoadedInputData(
         manager_key: str,
         upstream_output_name: Optional[str] = None,
         upstream_step_key: Optional[str] = None,
-        metadata_entries: Optional[Sequence[MetadataEntry]] = None,
+        metadata: Optional[Mapping[str, MetadataValue]] = None,
     ):
         return super(LoadedInputData, cls).__new__(
             cls,
@@ -1689,50 +1673,13 @@ class LoadedInputData(
             manager_key=check.str_param(manager_key, "manager_key"),
             upstream_output_name=check.opt_str_param(upstream_output_name, "upstream_output_name"),
             upstream_step_key=check.opt_str_param(upstream_step_key, "upstream_step_key"),
-            metadata_entries=check.opt_sequence_param(
-                metadata_entries, "metadata_entries", of_type=MetadataEntry
+            metadata=normalize_metadata(
+                check.opt_mapping_param(metadata, "metadata", key_type=str)
             ),
         )
 
 
-class ComputeLogsDataSerializer(DefaultNamedTupleSerializer):
-    @classmethod
-    def value_from_storage_dict(
-        cls,
-        storage_dict,
-        klass,
-        args_for_class,
-        whitelist_map,
-        descent_path,
-    ):
-        storage_dict = replace_storage_keys(storage_dict, {"log_key": "file_key"})
-        return super().value_from_storage_dict(
-            storage_dict, klass, args_for_class, whitelist_map, descent_path
-        )
-
-    @classmethod
-    def value_to_storage_dict(
-        cls,
-        value: NamedTuple,
-        whitelist_map: WhitelistMap,
-        descent_path: str,
-    ) -> Dict[str, Any]:
-        storage = super().value_to_storage_dict(
-            value,
-            whitelist_map,
-            descent_path,
-        )
-        # For backcompat, we store:
-        # file_key as log_key
-        return replace_storage_keys(
-            storage,
-            {
-                "file_key": "log_key",
-            },
-        )
-
-
-@whitelist_for_serdes(serializer=ComputeLogsDataSerializer)
+@whitelist_for_serdes(storage_field_names={"file_key": "log_key"})
 class ComputeLogsCaptureData(
     NamedTuple(
         "_ComputeLogsCaptureData",
@@ -1768,69 +1715,77 @@ class ComputeLogsCaptureData(
 ###################################################################################################
 
 
-# Keep these around to prevent issues like https://github.com/dagster-io/dagster/issues/3533
-@whitelist_for_serdes
-class AssetStoreOperationData(NamedTuple):
-    op: str
-    step_key: str
-    output_name: str
-    asset_store_key: str
+# Old data structures referenced below
+# class AssetStoreOperationData(NamedTuple):
+#     op: str
+#     step_key: str
+#     output_name: str
+#     asset_store_key: str
+#
+#
+# class AssetStoreOperationType(Enum):
+#     SET_ASSET = "SET_ASSET"
+#     GET_ASSET = "GET_ASSET"
+#
+#
+# class PipelineInitFailureData(NamedTuple):
+#     error: SerializableErrorInfo
 
 
-@whitelist_for_serdes
-class AssetStoreOperationType(Enum):
-    SET_ASSET = "SET_ASSET"
-    GET_ASSET = "GET_ASSET"
-
-
-@whitelist_for_serdes
-class PipelineInitFailureData(NamedTuple):
-    error: SerializableErrorInfo
-
-
-def _handle_back_compat(event_type_value, event_specific_data):
+def _handle_back_compat(
+    event_type_value: str, event_specific_data: Optional[Dict[str, Any]]
+) -> Tuple[str, Optional[Dict[str, Any]]]:
     # transform old specific process events in to engine events
-    if event_type_value == "PIPELINE_PROCESS_START":
-        return DagsterEventType.ENGINE_EVENT.value, EngineEventData([])
-    elif event_type_value == "PIPELINE_PROCESS_STARTED":
-        return DagsterEventType.ENGINE_EVENT.value, EngineEventData([])
-    elif event_type_value == "PIPELINE_PROCESS_EXITED":
-        return DagsterEventType.ENGINE_EVENT.value, EngineEventData([])
+    if event_type_value in [
+        "PIPELINE_PROCESS_START",
+        "PIPELINE_PROCESS_STARTED",
+        "PIPELINE_PROCESS_EXITED",
+    ]:
+        return "ENGINE_EVENT", {"__class__": "EngineEventData"}
 
     # changes asset store ops in to get/set asset
     elif event_type_value == "ASSET_STORE_OPERATION":
-        if event_specific_data.op in ("GET_ASSET", AssetStoreOperationType.GET_ASSET):
+        assert (
+            event_specific_data is not None
+        ), "ASSET_STORE_OPERATION event must have specific data"
+        if event_specific_data["op"] in (
+            "GET_ASSET",
+            '{"__enum__": "AssetStoreOperationType.GET_ASSET"}',
+        ):
             return (
-                DagsterEventType.LOADED_INPUT.value,
-                LoadedInputData(
-                    event_specific_data.output_name, event_specific_data.asset_store_key
-                ),
+                "LOADED_INPUT",
+                {
+                    "__class__": "LoadedInputData",
+                    "input_name": event_specific_data["output_name"],
+                    "manager_key": event_specific_data["asset_store_key"],
+                },
             )
-        if event_specific_data.op in ("SET_ASSET", AssetStoreOperationType.SET_ASSET):
+        if event_specific_data["op"] in (
+            "SET_ASSET",
+            '{"__enum__": "AssetStoreOperationType.SET_ASSET"}',
+        ):
             return (
-                DagsterEventType.HANDLED_OUTPUT.value,
-                HandledOutputData(
-                    event_specific_data.output_name, event_specific_data.asset_store_key, []
-                ),
+                "HANDLED_OUTPUT",
+                {
+                    "__class__": "HandledOutputData",
+                    "output_name": event_specific_data["output_name"],
+                    "manager_key": event_specific_data["asset_store_key"],
+                },
             )
 
     # previous name for ASSET_MATERIALIZATION was STEP_MATERIALIZATION
     if event_type_value == "STEP_MATERIALIZATION":
-        return DagsterEventType.ASSET_MATERIALIZATION.value, event_specific_data
+        assert event_specific_data is not None, "STEP_MATERIALIZATION event must have specific data"
+        return "ASSET_MATERIALIZATION", event_specific_data
 
     # transform PIPELINE_INIT_FAILURE to PIPELINE_FAILURE
     if event_type_value == "PIPELINE_INIT_FAILURE":
-        return DagsterEventType.PIPELINE_FAILURE.value, PipelineFailureData(
-            event_specific_data.error
-        )
+        assert (
+            event_specific_data is not None
+        ), "PIPELINE_INIT_FAILURE event must have specific data"
+        return "PIPELINE_FAILURE", {
+            "__class__": "PipelineFailureData",
+            "error": event_specific_data.get("error"),
+        }
 
     return event_type_value, event_specific_data
-
-
-register_serdes_tuple_fallbacks(
-    {
-        "PipelineProcessStartedData": None,
-        "PipelineProcessExitedData": None,
-        "PipelineProcessStartData": None,
-    }
-)
