@@ -1,18 +1,19 @@
 from json import JSONDecodeError
 
 import dagster._check as check
-from dagster._serdes import deserialize_json_to_dagster_namedtuple
+from dagster._core.events import DagsterEvent
+from dagster._serdes.errors import DeserializationError
+from dagster._serdes.serdes import deserialize_value
 
 
 def filter_dagster_events_from_cli_logs(log_lines):
-    """
-    Filters the raw log lines from a dagster-cli invocation to return only the lines containing json.
+    """Filters the raw log lines from a dagster-cli invocation to return only the lines containing json.
 
-     - Log lines don't necessarily come back in order
-     - Something else might log JSON
-     - Docker appears to silently split very long log lines -- this is undocumented behavior
+    - Log lines don't necessarily come back in order
+    - Something else might log JSON
+    - Docker appears to silently split very long log lines -- this is undocumented behavior
 
-     TODO: replace with reading event logs from the DB
+    TODO: replace with reading event logs from the DB
 
     """
     check.list_param(log_lines, "log_lines", str)
@@ -20,8 +21,8 @@ def filter_dagster_events_from_cli_logs(log_lines):
     coalesced_lines = []
     buffer = []
     in_split_line = False
-    for line in log_lines:
-        line = line.strip()
+    for raw_line in log_lines:
+        line = raw_line.strip()
         if not in_split_line and line.startswith("{"):
             if line.endswith("}"):
                 coalesced_lines.append(line)
@@ -38,10 +39,12 @@ def filter_dagster_events_from_cli_logs(log_lines):
     events = []
     for line in coalesced_lines:
         try:
-            events.append(deserialize_json_to_dagster_namedtuple(line))
+            events.append(deserialize_value(line, DagsterEvent))
         except JSONDecodeError:
             pass
         except check.CheckError:
             pass
+        except DeserializationError:
+            pass
 
     return events
