@@ -92,7 +92,16 @@ def execute_run_iterator(
 
     if not resume_from_failure:
         if dagster_run.status not in (DagsterRunStatus.NOT_STARTED, DagsterRunStatus.STARTING):
-            if instance.run_monitoring_enabled:
+            if dagster_run.is_finished:
+
+                def gen_ignore_duplicate_run_worker():
+                    yield instance.report_engine_event(
+                        "Ignoring a run worker that started after the run had already finished.",
+                        dagster_run,
+                    )
+
+                return gen_ignore_duplicate_run_worker()
+            elif instance.run_monitoring_enabled:
                 # This can happen if the pod was unexpectedly restarted by the cluster - ignore it since
                 # the run monitoring daemon will also spin up a new pod
                 def gen_ignore_duplicate_run_worker():
@@ -104,15 +113,6 @@ def execute_run_iterator(
                         dagster_run,
                     )
 
-                return gen_ignore_duplicate_run_worker()
-            elif dagster_run.is_finished:
-
-                def gen_ignore_duplicate_run_worker():
-                    yield instance.report_engine_event(
-                        "Ignoring a run worker that started after the run had already finished.",
-                        dagster_run,
-                    )
-
                 return gen_ignore_duplicate_run_worker()
             else:
 
@@ -149,8 +149,8 @@ def execute_run_iterator(
                 dagster_run.solids_to_execute == pipeline.solids_to_execute,
                 "Cannot execute DagsterRun with solids_to_execute {solids_to_execute} that"
                 " conflicts with pipeline subset {pipeline_solids_to_execute}.".format(
-                    pipeline_solids_to_execute=str_format_set(pipeline.solids_to_execute),
-                    solids_to_execute=str_format_set(dagster_run.solids_to_execute),
+                    pipeline_solids_to_execute=str_format_set(pipeline.solids_to_execute),  # type: ignore  # (possible none)
+                    solids_to_execute=str_format_set(dagster_run.solids_to_execute),  # type: ignore  # (possible none)
                 ),
             )
         else:
@@ -249,8 +249,8 @@ def execute_run(
                 dagster_run.solids_to_execute == pipeline.solids_to_execute,
                 "Cannot execute DagsterRun with solids_to_execute {solids_to_execute} that "
                 "conflicts with pipeline subset {pipeline_solids_to_execute}.".format(
-                    pipeline_solids_to_execute=str_format_set(pipeline.solids_to_execute),
-                    solids_to_execute=str_format_set(dagster_run.solids_to_execute),
+                    pipeline_solids_to_execute=str_format_set(pipeline.solids_to_execute),  # type: ignore  # (possible none)
+                    solids_to_execute=str_format_set(dagster_run.solids_to_execute),  # type: ignore  # (possible none)
                 ),
             )
         else:
@@ -885,29 +885,28 @@ def _check_pipeline(pipeline: Union[PipelineDefinition, IPipeline]) -> IPipeline
 
 
 def _get_execution_plan_from_run(
-    pipeline: IPipeline, dagster_run: DagsterRun, instance: DagsterInstance
+    pipeline: IPipeline,
+    dagster_run: DagsterRun,
+    instance: DagsterInstance,
 ) -> ExecutionPlan:
-    execution_plan_snapshot = None
-    if (
-        pipeline.solids_to_execute is None
-        and pipeline.asset_selection is None
-        and dagster_run.execution_plan_snapshot_id
-    ):
-        execution_plan_snapshot = instance.get_execution_plan_snapshot(
-            dagster_run.execution_plan_snapshot_id
-        )
-        if execution_plan_snapshot.can_reconstruct_plan:
-            return ExecutionPlan.rebuild_from_snapshot(
-                dagster_run.pipeline_name,
-                execution_plan_snapshot,
-            )
+    execution_plan_snapshot = (
+        instance.get_execution_plan_snapshot(dagster_run.execution_plan_snapshot_id)
+        if dagster_run.execution_plan_snapshot_id
+        else None
+    )
 
-    if dagster_run.has_repository_load_data:
-        # if you haven't fetched it already, get the snapshot now
-        execution_plan_snapshot = execution_plan_snapshot or instance.get_execution_plan_snapshot(
-            check.not_none(dagster_run.execution_plan_snapshot_id)
+    # Rebuild from snapshot if able and selection has not changed
+    if (
+        execution_plan_snapshot is not None
+        and execution_plan_snapshot.can_reconstruct_plan
+        and pipeline.solids_to_execute == dagster_run.solids_to_execute
+        and pipeline.asset_selection == dagster_run.asset_selection
+    ):
+        return ExecutionPlan.rebuild_from_snapshot(
+            dagster_run.pipeline_name,
+            execution_plan_snapshot,
         )
-    # need to rebuild execution plan so it matches the subsetted graph
+
     return create_execution_plan(
         pipeline,
         run_config=dagster_run.run_config,
@@ -917,6 +916,9 @@ def _get_execution_plan_from_run(
         repository_load_data=execution_plan_snapshot.repository_load_data
         if execution_plan_snapshot
         else None,
+        known_state=execution_plan_snapshot.initial_known_state
+        if execution_plan_snapshot
+        else None,
     )
 
 
@@ -1053,7 +1055,7 @@ def pipeline_execution_iterator(
         elif failed_steps:
             event = DagsterEvent.pipeline_failure(
                 pipeline_context,
-                "Steps failed: {}.".format(failed_steps),
+                f"Steps failed: {failed_steps}.",
             )
         else:
             event = DagsterEvent.pipeline_success(pipeline_context)
