@@ -1,13 +1,17 @@
 import json
+from enum import Enum
 from typing import (
     TYPE_CHECKING,
     AbstractSet,
+    Dict,
     Iterable,
     List,
+    Mapping,
     NamedTuple,
     Optional,
     Sequence,
     Set,
+    Tuple,
     cast,
 )
 
@@ -27,16 +31,30 @@ from dagster._core.definitions.run_request import RunRequest
 from dagster._core.definitions.selector import PipelineSelector
 from dagster._core.errors import DagsterBackfillFailedError
 from dagster._core.events import DagsterEventType
+from dagster._core.host_representation import (
+    ExternalExecutionPlan,
+    ExternalPipeline,
+)
 from dagster._core.instance import DagsterInstance, DynamicPartitionsStore
 from dagster._core.storage.pipeline_run import DagsterRunStatus, RunsFilter
 from dagster._core.storage.tags import BACKFILL_ID_TAG, PARTITION_NAME_TAG
-from dagster._core.workspace.context import BaseWorkspaceRequestContext
+from dagster._core.workspace.context import (
+    BaseWorkspaceRequestContext,
+    IWorkspaceProcessContext,
+)
+from dagster._utils import hash_collection
 from dagster._utils.caching_instance_queryer import CachingInstanceQueryer
 
 if TYPE_CHECKING:
     from .backfill import PartitionBackfill
 
 
+class BackfillPartitionsStatus(Enum):
+    REQUESTED = "REQUESTED"
+    COMPLETED = "COMPLETED"
+    FAILED = "FAILED"
+
+
 class AssetBackfillData(NamedTuple):
     """Has custom serialization instead of standard Dagster NamedTuple serialization because the
     asset graph is required to build the AssetGraphSubset objects.
@@ -50,9 +68,14 @@ class AssetBackfillData(NamedTuple):
     failed_and_downstream_subset: AssetGraphSubset
 
     def is_complete(self) -> bool:
+        """The asset backfill is complete when all runs to be requested have finished (success,
+        failure, or cancellation). Since the AssetBackfillData object stores materialization states
+        per asset partition, the daemon continues to update the backfill data until all runs have
+        finished in order to display the final partition statuses in the UI.
+        """
         return (
             (
-                self.requested_subset | self.failed_and_downstream_subset
+                self.materialized_subset | self.failed_and_downstream_subset
             ).num_partitions_and_non_partitioned_assets
             == self.target_subset.num_partitions_and_non_partitioned_assets
         )
@@ -68,8 +91,7 @@ class AssetBackfillData(NamedTuple):
         )
 
     def get_num_partitions(self) -> Optional[int]:
-        """
-        Only valid when the same number of partitions are targeted in every asset.
+        """Only valid when the same number of partitions are targeted in every asset.
 
         When not valid, returns None.
         """
@@ -83,9 +105,42 @@ class AssetBackfillData(NamedTuple):
         else:
             return None
 
-    def get_partition_names(self) -> Optional[Sequence[str]]:
+    def get_num_targeted_partitions_by_asset_key(self) -> Mapping[AssetKey, int]:
+        return {
+            asset_key: len(subset)
+            for asset_key, subset in self.target_subset.partitions_subsets_by_asset_key.items()
+        }
+
+    def get_partitions_status_counts_by_asset_key(
+        self,
+    ) -> Mapping[AssetKey, Mapping[BackfillPartitionsStatus, int]]:
+        """Returns a mapping from asset key to a mapping from status to count of partitions in that
+        status.
+
+        Only includes assets that are partitioned.
         """
-        Only valid when the same number of partitions are targeted in every asset.
+        return {
+            asset_key: {
+                BackfillPartitionsStatus.COMPLETED: len(
+                    self.materialized_subset.get_partitions_subset(asset_key)
+                ),
+                BackfillPartitionsStatus.FAILED: len(
+                    self.failed_and_downstream_subset.get_partitions_subset(asset_key)
+                ),
+                BackfillPartitionsStatus.REQUESTED: len(
+                    self.requested_subset.get_partitions_subset(asset_key)
+                )
+                - (
+                    len(self.failed_and_downstream_subset.get_partitions_subset(asset_key))
+                    + len(self.materialized_subset.get_partitions_subset(asset_key))
+                ),
+            }
+            for asset_key in self.target_subset.asset_keys
+            if self.target_subset.asset_graph.get_partitions_def(asset_key) is not None
+        }
+
+    def get_partition_names(self) -> Optional[Sequence[str]]:
+        """Only valid when the same number of partitions are targeted in every asset.
 
         When not valid, returns None.
         """
@@ -187,23 +242,32 @@ class AssetBackfillData(NamedTuple):
 
         return cls.empty(target_subset)
 
-    def serialize(self) -> str:
+    def serialize(self, dynamic_partitions_store: DynamicPartitionsStore) -> str:
         storage_dict = {
             "requested_runs_for_target_roots": self.requested_runs_for_target_roots,
-            "serialized_target_subset": self.target_subset.to_storage_dict(),
+            "serialized_target_subset": self.target_subset.to_storage_dict(
+                dynamic_partitions_store=dynamic_partitions_store
+            ),
             "latest_storage_id": self.latest_storage_id,
-            "serialized_requested_subset": self.requested_subset.to_storage_dict(),
-            "serialized_materialized_subset": self.materialized_subset.to_storage_dict(),
-            "serialized_failed_subset": self.failed_and_downstream_subset.to_storage_dict(),
+            "serialized_requested_subset": self.requested_subset.to_storage_dict(
+                dynamic_partitions_store=dynamic_partitions_store
+            ),
+            "serialized_materialized_subset": self.materialized_subset.to_storage_dict(
+                dynamic_partitions_store=dynamic_partitions_store
+            ),
+            "serialized_failed_subset": self.failed_and_downstream_subset.to_storage_dict(
+                dynamic_partitions_store=dynamic_partitions_store
+            ),
         }
         return json.dumps(storage_dict)
 
 
 def execute_asset_backfill_iteration(
-    backfill: "PartitionBackfill", workspace: BaseWorkspaceRequestContext, instance: DagsterInstance
+    backfill: "PartitionBackfill",
+    workspace_process_context: IWorkspaceProcessContext,
+    instance: DagsterInstance,
 ) -> Iterable[None]:
-    """
-    Runs an iteration of the backfill, including submitting runs and updating the backfill object
+    """Runs an iteration of the backfill, including submitting runs and updating the backfill object
     in the DB.
 
     This is a generator so that we can return control to the daemon and let it heartbeat during
@@ -211,7 +275,9 @@ def execute_asset_backfill_iteration(
     """
     from dagster._core.execution.backfill import BulkActionStatus
 
-    asset_graph = ExternalAssetGraph.from_workspace(workspace)
+    asset_graph = ExternalAssetGraph.from_workspace(
+        workspace_process_context.create_request_context()
+    )
     if backfill.serialized_asset_backfill_data is None:
         check.failed("Asset backfill missing serialized_asset_backfill_data")
 
@@ -225,6 +291,7 @@ def execute_asset_backfill_iteration(
         asset_backfill_data=asset_backfill_data,
         instance=instance,
         asset_graph=asset_graph,
+        run_tags=backfill.tags,
     ):
         yield None
 
@@ -234,14 +301,30 @@ def execute_asset_backfill_iteration(
             " AssetBackfillIterationResult"
         )
 
-    updated_backfill = backfill.with_asset_backfill_data(result.backfill_data)
+    updated_backfill = backfill.with_asset_backfill_data(
+        result.backfill_data, dynamic_partitions_store=instance
+    )
     if result.backfill_data.is_complete():
+        # The asset backfill is complete when all runs to be requested have finished (success,
+        # failure, or cancellation). Since the AssetBackfillData object stores materialization states
+        # per asset partition, the daemon continues to update the backfill data until all runs have
+        # finished in order to display the final partition statuses in the UI.
         updated_backfill = updated_backfill.with_status(BulkActionStatus.COMPLETED)
 
+    pipeline_and_execution_plan_cache: Dict[
+        int, Tuple[ExternalPipeline, ExternalExecutionPlan]
+    ] = {}
+
     for run_request in result.run_requests:
         yield None
         submit_run_request(
-            run_request=run_request, asset_graph=asset_graph, workspace=workspace, instance=instance
+            run_request=run_request,
+            asset_graph=asset_graph,
+            # create a new request context for each run in case the code location server
+            # is swapped out in the middle of the backfill
+            workspace=workspace_process_context.create_request_context(),
+            instance=instance,
+            pipeline_and_execution_plan_cache=pipeline_and_execution_plan_cache,
         )
 
     instance.update_backfill(updated_backfill)
@@ -252,17 +335,13 @@ def submit_run_request(
     run_request: RunRequest,
     instance: DagsterInstance,
     workspace: BaseWorkspaceRequestContext,
+    pipeline_and_execution_plan_cache: Dict[int, Tuple[ExternalPipeline, ExternalExecutionPlan]],
 ) -> None:
-    """
-    Creates and submits a run for the given run request
-    """
+    """Creates and submits a run for the given run request."""
     repo_handle = asset_graph.get_repository_handle(
         cast(Sequence[AssetKey], run_request.asset_selection)[0]
     )
-    location_name = repo_handle.repository_location_origin.location_name
-    repo_location = workspace.get_repository_location(
-        repo_handle.repository_location_origin.location_name
-    )
+    location_name = repo_handle.code_location_origin.location_name
     job_name = _get_implicit_job_name_for_assets(
         asset_graph, cast(Sequence[AssetKey], run_request.asset_selection)
     )
@@ -271,6 +350,10 @@ def submit_run_request(
             "Could not find an implicit asset job for the given assets:"
             f" {run_request.asset_selection}"
         )
+
+    if not run_request.asset_selection:
+        check.failed("Expected RunRequest to have an asset selection")
+
     pipeline_selector = PipelineSelector(
         location_name=location_name,
         repository_name=repo_handle.repository_name,
@@ -278,19 +361,28 @@ def submit_run_request(
         asset_selection=run_request.asset_selection,
         solid_selection=None,
     )
-    external_pipeline = repo_location.get_external_pipeline(pipeline_selector)
 
-    external_execution_plan = repo_location.get_external_execution_plan(
-        external_pipeline,
-        {},
-        DEFAULT_MODE_NAME,
-        step_keys_to_execute=None,
-        known_state=None,
-        instance=instance,
-    )
+    selector_id = hash_collection(pipeline_selector)
 
-    if not run_request.asset_selection:
-        check.failed("Expected RunRequest to have an asset selection")
+    if selector_id not in pipeline_and_execution_plan_cache:
+        code_location = workspace.get_code_location(repo_handle.code_location_origin.location_name)
+
+        external_pipeline = code_location.get_external_pipeline(pipeline_selector)
+
+        external_execution_plan = code_location.get_external_execution_plan(
+            external_pipeline,
+            {},
+            DEFAULT_MODE_NAME,
+            step_keys_to_execute=None,
+            known_state=None,
+            instance=instance,
+        )
+        pipeline_and_execution_plan_cache[selector_id] = (
+            external_pipeline,
+            external_execution_plan,
+        )
+
+    external_pipeline, external_execution_plan = pipeline_and_execution_plan_cache[selector_id]
 
     run = instance.create_run(
         pipeline_snapshot=external_pipeline.pipeline_snapshot,
@@ -318,9 +410,9 @@ def submit_run_request(
 def _get_implicit_job_name_for_assets(
     asset_graph: ExternalAssetGraph, asset_keys: Sequence[AssetKey]
 ) -> Optional[str]:
-    job_names = set(asset_graph.get_job_names(asset_keys[0]))
+    job_names = set(asset_graph.get_materialization_job_names(asset_keys[0]))
     for asset_key in asset_keys[1:]:
-        job_names &= set(asset_graph.get_job_names(asset_key))
+        job_names &= set(asset_graph.get_materialization_job_names(asset_key))
 
     return next(job_name for job_name in job_names if is_base_asset_job_name(job_name))
 
@@ -335,9 +427,9 @@ def execute_asset_backfill_iteration_inner(
     asset_backfill_data: AssetBackfillData,
     asset_graph: ExternalAssetGraph,
     instance: DagsterInstance,
+    run_tags: Mapping[str, str],
 ) -> Iterable[Optional[AssetBackfillIterationResult]]:
-    """
-    Core logic of a backfill iteration. Has no side effects.
+    """Core logic of a backfill iteration. Has no side effects.
 
     Computes which runs should be requested, if any, as well as updated bookkeeping about the status
     of asset partitions targeted by the backfill.
@@ -425,7 +517,7 @@ def execute_asset_backfill_iteration_inner(
     )
 
     run_requests = build_run_requests(
-        asset_partitions_to_request, asset_graph, {BACKFILL_ID_TAG: backfill_id}
+        asset_partitions_to_request, asset_graph, {**run_tags, BACKFILL_ID_TAG: backfill_id}
     )
 
     if request_roots:
@@ -455,10 +547,9 @@ def should_backfill_atomic_asset_partitions_unit(
     failed_and_downstream_subset: AssetGraphSubset,
     dynamic_partitions_store: DynamicPartitionsStore,
 ) -> bool:
-    """
-    Args:
-        candidates_unit: A set of asset partitions that must all be materialized if any is
-            materialized.
+    """Args:
+    candidates_unit: A set of asset partitions that must all be materialized if any is
+        materialized.
     """
     for candidate in candidates_unit:
         if (
@@ -490,8 +581,7 @@ def should_backfill_atomic_asset_partitions_unit(
 def _get_failed_asset_partitions(
     instance_queryer: CachingInstanceQueryer, backfill_id: str
 ) -> Sequence[AssetKeyPartitionKey]:
-    """
-    Returns asset partitions that materializations were requested for as part of the backfill, but
+    """Returns asset partitions that materializations were requested for as part of the backfill, but
     will not be materialized.
 
     Includes canceled asset partitions. Implementation assumes that successful runs won't have any
