@@ -5,6 +5,7 @@ from typing_extensions import TypeAlias
 
 import dagster._check as check
 from dagster._annotations import experimental, public
+from dagster._core.definitions.assets import AssetsDefinition
 from dagster._core.definitions.data_version import (
     DataProvenance,
     extract_data_provenance_from_entry,
@@ -15,7 +16,6 @@ from dagster._core.definitions.events import (
     AssetMaterialization,
     AssetObservation,
     ExpectationResult,
-    Materialization,
     UserEvent,
 )
 from dagster._core.definitions.job_definition import JobDefinition
@@ -26,7 +26,10 @@ from dagster._core.definitions.partition_key_range import PartitionKeyRange
 from dagster._core.definitions.pipeline_definition import PipelineDefinition
 from dagster._core.definitions.step_launcher import StepLauncher
 from dagster._core.definitions.time_window_partitions import TimeWindow
-from dagster._core.errors import DagsterInvalidPropertyError, DagsterInvariantViolationError
+from dagster._core.errors import (
+    DagsterInvalidPropertyError,
+    DagsterInvariantViolationError,
+)
 from dagster._core.events import DagsterEvent
 from dagster._core.instance import DagsterInstance
 from dagster._core.log_manager import DagsterLogManager
@@ -37,7 +40,7 @@ from dagster._utils.forked_pdb import ForkedPdb
 from .system import StepExecutionContext
 
 
-class AbstractComputeExecutionContext(ABC):  # pylint: disable=no-init
+class AbstractComputeExecutionContext(ABC):
     """Base class for solid context implemented by SolidExecutionContext and DagstermillExecutionContext.
     """
 
@@ -229,7 +232,7 @@ class OpExecutionContext(AbstractComputeExecutionContext):
         return self._step_execution_context.log
 
     @property
-    def solid_handle(self) -> NodeHandle:
+    def node_handle(self) -> NodeHandle:
         """NodeHandle: The current solid's handle.
 
         :meta private:
@@ -242,7 +245,7 @@ class OpExecutionContext(AbstractComputeExecutionContext):
 
         :meta private:
         """
-        return self.solid_handle
+        return self.node_handle
 
     @property
     def solid(self) -> Node:
@@ -251,7 +254,7 @@ class OpExecutionContext(AbstractComputeExecutionContext):
         :meta private:
 
         """
-        return self._step_execution_context.pipeline_def.get_solid(self.solid_handle)
+        return self._step_execution_context.pipeline_def.get_node(self.node_handle)
 
     @property
     def op(self) -> Node:
@@ -268,6 +271,16 @@ class OpExecutionContext(AbstractComputeExecutionContext):
         """OpDefinition: The current op definition."""
         return cast(OpDefinition, self.op.definition)
 
+    @public
+    @property
+    def assets_def(self) -> AssetsDefinition:
+        assets_def = self.job_def.asset_layer.assets_def_for_node(self.node_handle)
+        if assets_def is None:
+            raise DagsterInvalidPropertyError(
+                f"Op '{self.op.name}' does not have an assets definition."
+            )
+        return assets_def
+
     @public
     @property
     def has_partition_key(self) -> bool:
@@ -294,7 +307,7 @@ class OpExecutionContext(AbstractComputeExecutionContext):
 
     @public
     @property
-    def partition_time_window(self) -> str:
+    def partition_time_window(self) -> TimeWindow:
         """The partition time window for the current run.
 
         Raises an error if the current run is not a partitioned run, or if the job's partition
@@ -305,7 +318,7 @@ class OpExecutionContext(AbstractComputeExecutionContext):
     @public
     @property
     def selected_asset_keys(self) -> AbstractSet[AssetKey]:
-        assets_def = self.job_def.asset_layer.assets_def_for_node(self.solid_handle)
+        assets_def = self.job_def.asset_layer.assets_def_for_node(self.node_handle)
         if assets_def is None:
             return set()
         return assets_def.keys
@@ -318,13 +331,13 @@ class OpExecutionContext(AbstractComputeExecutionContext):
         selected_outputs: Set[str] = set()
         for output_name in self.op.output_dict.keys():
             asset_info = self.job_def.asset_layer.asset_info_for_output(
-                self.solid_handle, output_name
+                self.node_handle, output_name
             )
             if any(  #  For graph-backed assets, check if a downstream asset is selected
                 [
                     asset_key in selected_asset_keys
                     for asset_key in self.job_def.asset_layer.downstream_dep_assets(
-                        self.solid_handle, output_name
+                        self.node_handle, output_name
                     )
                 ]
             ) or (asset_info and asset_info.key in selected_asset_keys):
@@ -385,7 +398,8 @@ class OpExecutionContext(AbstractComputeExecutionContext):
 
         Raises an error if either of the following are true:
         - The output asset has no partitioning.
-        - The output asset is not partitioned with a TimeWindowPartitionsDefinition.
+        - The output asset is not partitioned with a TimeWindowPartitionsDefinition or a
+        MultiPartitionsDefinition with one time-partitioned dimension.
         """
         return self._step_execution_context.asset_partitions_time_window_for_output(output_name)
 
@@ -496,7 +510,7 @@ class OpExecutionContext(AbstractComputeExecutionContext):
         Events logged with this method will appear in the list of DagsterEvents, as well as the event log.
 
         Args:
-            event (Union[AssetMaterialization, Materialization, AssetObservation, ExpectationResult]): The event to log.
+            event (Union[AssetMaterialization, AssetObservation, ExpectationResult]): The event to log.
 
         **Examples:**
 
@@ -508,7 +522,7 @@ class OpExecutionContext(AbstractComputeExecutionContext):
             def log_materialization(context):
                 context.log_event(AssetMaterialization("foo"))
         """
-        if isinstance(event, (AssetMaterialization, Materialization)):
+        if isinstance(event, AssetMaterialization):
             self._events.append(
                 DagsterEvent.asset_materialization(self._step_execution_context, event)
             )
@@ -519,7 +533,7 @@ class OpExecutionContext(AbstractComputeExecutionContext):
                 DagsterEvent.step_expectation_result(self._step_execution_context, event)
             )
         else:
-            check.failed("Unexpected event {event}".format(event=event))
+            check.failed(f"Unexpected event {event}")
 
     def add_output_metadata(
         self,
@@ -584,8 +598,7 @@ class OpExecutionContext(AbstractComputeExecutionContext):
     @public
     @property
     def retry_number(self) -> int:
-        """
-        Which retry attempt is currently executing i.e. 0 for initial attempt, 1 for first retry, etc.
+        """Which retry attempt is currently executing i.e. 0 for initial attempt, 1 for first retry, etc.
         """
         return self._step_execution_context.previous_attempt_count
 
@@ -594,16 +607,14 @@ class OpExecutionContext(AbstractComputeExecutionContext):
 
     @public
     def get_mapping_key(self) -> Optional[str]:
-        """
-        Which mapping_key this execution is for if downstream of a DynamicOutput, otherwise None.
+        """Which mapping_key this execution is for if downstream of a DynamicOutput, otherwise None.
         """
         return self._step_execution_context.step.get_mapping_key()
 
     @public
     @experimental
     def get_asset_provenance(self, asset_key: AssetKey) -> Optional[DataProvenance]:
-        """
-        Return the provenance information for the most recent materialization of an asset.
+        """Return the provenance information for the most recent materialization of an asset.
 
         Args:
             asset_key (AssetKey): Key of the asset for which to retrieve provenance.
