@@ -15,7 +15,8 @@ import dagster._check as check
 from dagster._annotations import public
 from dagster._core.definitions.events import AssetKey, AssetObservation
 from dagster._core.definitions.metadata import (
-    MetadataEntry,
+    ArbitraryMetadataMapping,
+    MetadataValue,
 )
 from dagster._core.definitions.partition import PartitionsSubset
 from dagster._core.definitions.partition_key_range import PartitionKeyRange
@@ -36,8 +37,7 @@ if TYPE_CHECKING:
 
 
 class InputContext:
-    """
-    The ``context`` object available to the load_input method of :py:class:`RootInputManager`.
+    """The ``context`` object available to the load_input method of :py:class:`RootInputManager`.
 
     Users should not instantiate this object directly. In order to construct
     an `InputContext` for testing an IO Manager's `load_input` method, use
@@ -68,7 +68,6 @@ class InputContext:
         op_def (Optional[OpDefinition]): The definition of the op that's loading the input.
 
     Example:
-
     .. code-block:: python
 
         from dagster import IOManager, InputContext
@@ -86,7 +85,7 @@ class InputContext:
         job_name: Optional[str] = None,
         op_def: Optional["OpDefinition"] = None,
         config: Optional[Any] = None,
-        metadata: Optional[Mapping[str, Any]] = None,
+        metadata: Optional[ArbitraryMetadataMapping] = None,
         upstream_output: Optional["OutputContext"] = None,
         dagster_type: Optional["DagsterType"] = None,
         log_manager: Optional["DagsterLogManager"] = None,
@@ -106,7 +105,7 @@ class InputContext:
         self._job_name = job_name
         self._op_def = op_def
         self._config = config
-        self._metadata = metadata
+        self._metadata = metadata or {}
         self._upstream_output = upstream_output
         self._dagster_type = dagster_type
         self._log = log_manager
@@ -128,13 +127,12 @@ class InputContext:
             self._resources_cm = build_resources(
                 check.opt_mapping_param(resources, "resources", key_type=str)
             )
-            self._resources = self._resources_cm.__enter__()  # pylint: disable=no-member
+            self._resources = self._resources_cm.__enter__()
             self._resources_contain_cm = isinstance(self._resources, IContainsGenerator)
             self._cm_scope_entered = False
 
         self._events: List["DagsterEvent"] = []
         self._observations: List[AssetObservation] = []
-        self._metadata_entries: List[MetadataEntry] = []
         self._instance = instance
 
     def __enter__(self):
@@ -144,11 +142,11 @@ class InputContext:
 
     def __exit__(self, *exc):
         if self._resources_cm:
-            self._resources_cm.__exit__(*exc)  # pylint: disable=no-member
+            self._resources_cm.__exit__(*exc)
 
     def __del__(self):
         if self._resources_cm and self._resources_contain_cm and not self._cm_scope_entered:
-            self._resources_cm.__exit__(None, None, None)  # pylint: disable=no-member
+            self._resources_cm.__exit__(None, None, None)
 
     @property
     def instance(self) -> DagsterInstance:
@@ -209,7 +207,7 @@ class InputContext:
 
     @public
     @property
-    def metadata(self) -> Optional[Mapping[str, Any]]:
+    def metadata(self) -> Optional[ArbitraryMetadataMapping]:
         return self._metadata
 
     @public
@@ -488,7 +486,7 @@ class InputContext:
         from dagster._core.events import DagsterEvent
 
         metadata = check.mapping_param(metadata, "metadata", key_type=str)
-        self._metadata_entries.extend(normalize_metadata(metadata, []))
+        self._metadata = {**self._metadata, **normalize_metadata(metadata)}
         if self.has_asset_key:
             check.opt_str_param(description, "description")
 
@@ -528,16 +526,16 @@ class InputContext:
         """
         return self._observations
 
-    def consume_metadata_entries(self) -> Sequence[MetadataEntry]:
-        result = self._metadata_entries
-        self._metadata_entries = []
+    def consume_metadata(self) -> Mapping[str, MetadataValue]:
+        result = self._metadata
+        self._metadata = {}
         return result
 
 
 def build_input_context(
     name: Optional[str] = None,
     config: Optional[Any] = None,
-    metadata: Optional[Mapping[str, Any]] = None,
+    metadata: Optional[ArbitraryMetadataMapping] = None,
     upstream_output: Optional["OutputContext"] = None,
     dagster_type: Optional["DagsterType"] = None,
     resource_config: Optional[Mapping[str, Any]] = None,
@@ -638,7 +636,7 @@ def build_input_context(
 
 
 class KeyRangeNoPartitionsDefPartitionsSubset(PartitionsSubset):
-    """For build_input_context when no PartitionsDefinition has been provided"""
+    """For build_input_context when no PartitionsDefinition has been provided."""
 
     def __init__(self, key_range: PartitionKeyRange):
         self._key_range = key_range
