@@ -1,4 +1,3 @@
-# pylint: disable=super-init-not-called
 from typing import (
     AbstractSet,
     Any,
@@ -14,6 +13,7 @@ from typing import (
 )
 
 import dagster._check as check
+from dagster._core.definitions.assets import AssetsDefinition
 from dagster._core.definitions.composition import PendingNodeInvocation
 from dagster._core.definitions.decorators.op_decorator import DecoratedOpFunction
 from dagster._core.definitions.dependency import Node, NodeHandle
@@ -21,11 +21,11 @@ from dagster._core.definitions.events import (
     AssetMaterialization,
     AssetObservation,
     ExpectationResult,
-    Materialization,
     UserEvent,
 )
 from dagster._core.definitions.hook_definition import HookDefinition
 from dagster._core.definitions.mode import ModeDefinition
+from dagster._core.definitions.multi_dimensional_partitions import MultiPartitionsDefinition
 from dagster._core.definitions.op_definition import OpDefinition
 from dagster._core.definitions.pipeline_definition import PipelineDefinition
 from dagster._core.definitions.resource_definition import (
@@ -36,6 +36,11 @@ from dagster._core.definitions.resource_definition import (
 )
 from dagster._core.definitions.resource_requirement import ensure_requirements_satisfied
 from dagster._core.definitions.step_launcher import StepLauncher
+from dagster._core.definitions.time_window_partitions import (
+    TimeWindow,
+    TimeWindowPartitionsDefinition,
+    has_one_dimension_time_window_partitioning,
+)
 from dagster._core.errors import (
     DagsterInvalidInvocationError,
     DagsterInvalidPropertyError,
@@ -72,7 +77,8 @@ class UnboundOpExecutionContext(OpExecutionContext):
         instance: Optional[DagsterInstance],
         partition_key: Optional[str],
         mapping_key: Optional[str],
-    ):  # pylint: disable=super-init-not-called
+        assets_def: Optional[AssetsDefinition],
+    ):
         from dagster._core.execution.api import ephemeral_instance_if_missing
         from dagster._core.execution.context_creation_pipeline import initialize_console_manager
 
@@ -86,7 +92,7 @@ class UnboundOpExecutionContext(OpExecutionContext):
         self._instance_cm = ephemeral_instance_if_missing(instance)
         # Pylint can't infer that the ephemeral_instance context manager has an __enter__ method,
         # so ignore lint error
-        self._instance = self._instance_cm.__enter__()  # pylint: disable=no-member
+        self._instance = self._instance_cm.__enter__()
 
         self._resources_config = resources_config
         # Open resource context manager
@@ -97,7 +103,7 @@ class UnboundOpExecutionContext(OpExecutionContext):
             instance=instance,
             resource_config=resources_config,
         )
-        self._resources = self._resources_cm.__enter__()  # pylint: disable=no-member
+        self._resources = self._resources_cm.__enter__()
         self._resources_contain_cm = isinstance(self._resources, IContainsGenerator)
 
         self._log = initialize_console_manager(None)
@@ -107,25 +113,31 @@ class UnboundOpExecutionContext(OpExecutionContext):
         self._user_events: List[UserEvent] = []
         self._output_metadata: Dict[str, Any] = {}
 
+        self._assets_def = check.opt_inst_param(assets_def, "assets_def", AssetsDefinition)
+
     def __enter__(self):
         self._cm_scope_entered = True
         return self
 
     def __exit__(self, *exc):
-        self._resources_cm.__exit__(*exc)  # pylint: disable=no-member
+        self._resources_cm.__exit__(*exc)
         if self._instance_provided:
-            self._instance_cm.__exit__(*exc)  # pylint: disable=no-member
+            self._instance_cm.__exit__(*exc)
 
     def __del__(self):
         if self._resources_contain_cm and not self._cm_scope_entered:
-            self._resources_cm.__exit__(None, None, None)  # pylint: disable=no-member
+            self._resources_cm.__exit__(None, None, None)
         if self._instance_provided and not self._cm_scope_entered:
-            self._instance_cm.__exit__(None, None, None)  # pylint: disable=no-member
+            self._instance_cm.__exit__(None, None, None)
 
     @property
     def op_config(self) -> Any:
         return self._op_config
 
+    @property
+    def resource_keys(self) -> AbstractSet[str]:
+        return self._resource_defs.keys()
+
     @property
     def resources(self) -> Resources:
         if self._resources_contain_cm and not self._cm_scope_entered:
@@ -192,7 +204,7 @@ class UnboundOpExecutionContext(OpExecutionContext):
         return self._log
 
     @property
-    def solid_handle(self) -> NodeHandle:
+    def node_handle(self) -> NodeHandle:
         raise DagsterInvalidPropertyError(_property_msg("solid_handle", "property"))
 
     @property
@@ -203,6 +215,10 @@ class UnboundOpExecutionContext(OpExecutionContext):
     def op_def(self) -> OpDefinition:
         raise DagsterInvalidPropertyError(_property_msg("op_def", "property"))
 
+    @property
+    def assets_def(self) -> AssetsDefinition:
+        raise DagsterInvalidPropertyError(_property_msg("assets_def", "property"))
+
     @property
     def has_partition_key(self) -> bool:
         return self._partition_key is not None
@@ -227,12 +243,12 @@ class UnboundOpExecutionContext(OpExecutionContext):
 
     def bind(
         self,
-        op_def_or_invocation: Union[OpDefinition, PendingNodeInvocation],
+        op_def_or_invocation: Union[OpDefinition, PendingNodeInvocation[OpDefinition]],
     ) -> "BoundOpExecutionContext":
         op_def = (
             op_def_or_invocation
             if isinstance(op_def_or_invocation, OpDefinition)
-            else op_def_or_invocation.node_def.ensure_op_def()
+            else op_def_or_invocation.node_def
         )
 
         _validate_resource_requirements(self._resource_defs, op_def)
@@ -262,6 +278,7 @@ class UnboundOpExecutionContext(OpExecutionContext):
             output_metadata=self._output_metadata,
             mapping_key=self._mapping_key,
             partition_key=self._partition_key,
+            assets_def=self._assets_def,
         )
 
     def get_events(self) -> Sequence[UserEvent]:
@@ -309,6 +326,24 @@ class UnboundOpExecutionContext(OpExecutionContext):
     def get_mapping_key(self) -> Optional[str]:
         return self._mapping_key
 
+    def replace_resources(self, resources_dict: Mapping[str, Any]) -> "UnboundOpExecutionContext":
+        """Replace the resources of this context.
+
+        This method is intended to be used by the Dagster framework, and should not be called by user code.
+
+        Args:
+            resources (Mapping[str, Any]): The resources to add to the context.
+        """
+        return UnboundOpExecutionContext(
+            op_config=self._op_config,
+            resources_dict=resources_dict,
+            resources_config=self._resources_config,
+            instance=self._instance,
+            partition_key=self._partition_key,
+            mapping_key=self._mapping_key,
+            assets_def=self._assets_def,
+        )
+
 
 def _validate_resource_requirements(
     resource_defs: Mapping[str, ResourceDefinition], op_def: OpDefinition
@@ -342,6 +377,7 @@ class BoundOpExecutionContext(OpExecutionContext):
     _output_metadata: Dict[str, Any]
     _mapping_key: Optional[str]
     _partition_key: Optional[str]
+    _assets_def: Optional[AssetsDefinition]
 
     def __init__(
         self,
@@ -359,6 +395,7 @@ class BoundOpExecutionContext(OpExecutionContext):
         output_metadata: Dict[str, Any],
         mapping_key: Optional[str],
         partition_key: Optional[str],
+        assets_def: Optional[AssetsDefinition],
     ):
         self._op_def = op_def
         self._op_config = op_config
@@ -375,6 +412,7 @@ class BoundOpExecutionContext(OpExecutionContext):
         self._output_metadata = output_metadata
         self._mapping_key = mapping_key
         self._partition_key = partition_key
+        self._assets_def = assets_def
 
     @property
     def op_config(self) -> Any:
@@ -444,7 +482,7 @@ class BoundOpExecutionContext(OpExecutionContext):
         return self._log
 
     @property
-    def solid_handle(self) -> NodeHandle:
+    def node_handle(self) -> NodeHandle:
         raise DagsterInvalidPropertyError(_property_msg("solid_handle", "property"))
 
     @property
@@ -455,6 +493,14 @@ class BoundOpExecutionContext(OpExecutionContext):
     def op_def(self) -> OpDefinition:
         return self._op_def
 
+    @property
+    def assets_def(self) -> AssetsDefinition:
+        if self._assets_def is None:
+            raise DagsterInvalidPropertyError(
+                f"Op {self.op_def.name} does not have an assets definition."
+            )
+        return self._assets_def
+
     def has_tag(self, key: str) -> bool:
         return key in self._tags
 
@@ -471,7 +517,10 @@ class BoundOpExecutionContext(OpExecutionContext):
     def for_type(self, dagster_type: DagsterType) -> TypeCheckContext:
         resources = cast(NamedTuple, self.resources)
         return TypeCheckContext(
-            self.run_id, self.log, ScopedResourcesBuilder(resources._asdict()), dagster_type
+            self.run_id,
+            self.log,
+            ScopedResourcesBuilder(resources._asdict()),
+            dagster_type,
         )
 
     def get_mapping_key(self) -> Optional[str]:
@@ -487,7 +536,7 @@ class BoundOpExecutionContext(OpExecutionContext):
         check.inst_param(
             event,
             "event",
-            (AssetMaterialization, AssetObservation, ExpectationResult, Materialization),
+            (AssetMaterialization, AssetObservation, ExpectationResult),
         )
         self._user_events.append(event)
 
@@ -515,6 +564,21 @@ class BoundOpExecutionContext(OpExecutionContext):
     def asset_partition_key_for_output(self, output_name: str = "result") -> str:
         return self.partition_key
 
+    def asset_partitions_time_window_for_output(self, output_name: str = "result") -> TimeWindow:
+        partitions_def = self.assets_def.partitions_def
+        if partitions_def is None:
+            check.failed("Tried to access partition_key for a non-partitioned asset")
+
+        if not has_one_dimension_time_window_partitioning(partitions_def=partitions_def):
+            raise DagsterInvariantViolationError(
+                "Expected a TimeWindowPartitionsDefinition or MultiPartitionsDefinition with a"
+                f" single time dimension, but instead found {type(partitions_def)}"
+            )
+
+        return cast(
+            Union[MultiPartitionsDefinition, TimeWindowPartitionsDefinition], partitions_def
+        ).time_window_for_partition_key(self.partition_key)
+
     def add_output_metadata(
         self,
         metadata: Mapping[str, Any],
@@ -607,6 +671,7 @@ def build_op_context(
     config: Any = None,
     partition_key: Optional[str] = None,
     mapping_key: Optional[str] = None,
+    _assets_def: Optional[AssetsDefinition] = None,
 ) -> UnboundOpExecutionContext:
     """Builds op execution context from provided parameters.
 
@@ -624,6 +689,8 @@ def build_op_context(
         mapping_key (Optional[str]): A key representing the mapping key from an upstream dynamic
             output. Can be accessed using ``context.get_mapping_key()``.
         partition_key (Optional[str]): String value representing partition key to execute with.
+        _assets_def (Optional[AssetsDefinition]): Internal argument that populates the op's assets
+            definition, not meant to be populated by users.
 
     Examples:
         .. code-block:: python
@@ -650,4 +717,5 @@ def build_op_context(
         instance=check.opt_inst_param(instance, "instance", DagsterInstance),
         partition_key=check.opt_str_param(partition_key, "partition_key"),
         mapping_key=check.opt_str_param(mapping_key, "mapping_key"),
+        assets_def=check.opt_inst_param(_assets_def, "_assets_def", AssetsDefinition),
     )
