@@ -19,9 +19,12 @@ from dagster._core.definitions.events import (
     AssetKey,
     AssetMaterialization,
     AssetObservation,
-    Materialization,
 )
-from dagster._core.definitions.metadata import MetadataEntry, RawMetadataValue
+from dagster._core.definitions.metadata import (
+    ArbitraryMetadataMapping,
+    MetadataValue,
+    RawMetadataValue,
+)
 from dagster._core.definitions.partition_key_range import PartitionKeyRange
 from dagster._core.definitions.time_window_partitions import TimeWindow
 from dagster._core.errors import DagsterInvalidMetadata, DagsterInvariantViolationError
@@ -43,8 +46,7 @@ RUN_ID_PLACEHOLDER = "__EPHEMERAL_RUN_ID"
 
 
 class OutputContext:
-    """
-    The context object that is available to the `handle_output` method of an :py:class:`IOManager`.
+    """The context object that is available to the `handle_output` method of an :py:class:`IOManager`.
 
     Users should not instantiate this object directly. To construct an
     `OutputContext` for testing an IO Manager's `handle_output` method, use
@@ -70,7 +72,6 @@ class OutputContext:
             output.
 
     Example:
-
     .. code-block:: python
 
         from dagster import IOManager, OutputContext
@@ -85,7 +86,8 @@ class OutputContext:
     _name: Optional[str]
     _pipeline_name: Optional[str]
     _run_id: Optional[str]
-    _metadata: Optional[Mapping[str, RawMetadataValue]]
+    _metadata: ArbitraryMetadataMapping
+    _user_generated_metadata: Mapping[str, MetadataValue]
     _mapping_key: Optional[str]
     _config: object
     _op_def: Optional["OpDefinition"]
@@ -101,8 +103,7 @@ class OutputContext:
     _resources_contain_cm: Optional[bool]
     _cm_scope_entered: Optional[bool]
     _events: List["DagsterEvent"]
-    _user_events: List[Union[AssetMaterialization, AssetObservation, Materialization]]
-    _metadata_entries: Optional[Sequence[MetadataEntry]]
+    _user_events: List[Union[AssetMaterialization, AssetObservation]]
 
     def __init__(
         self,
@@ -110,7 +111,7 @@ class OutputContext:
         name: Optional[str] = None,
         pipeline_name: Optional[str] = None,
         run_id: Optional[str] = None,
-        metadata: Optional[Mapping[str, RawMetadataValue]] = None,
+        metadata: Optional[ArbitraryMetadataMapping] = None,
         mapping_key: Optional[str] = None,
         config: object = None,
         dagster_type: Optional["DagsterType"] = None,
@@ -131,7 +132,7 @@ class OutputContext:
         self._name = name
         self._pipeline_name = pipeline_name
         self._run_id = run_id
-        self._metadata = metadata
+        self._metadata = metadata or {}
         self._mapping_key = mapping_key
         self._config = config
         self._op_def = op_def
@@ -154,13 +155,13 @@ class OutputContext:
             self._resources_cm = build_resources(
                 check.opt_mapping_param(resources, "resources", key_type=str)
             )
-            self._resources = self._resources_cm.__enter__()  # pylint: disable=no-member
+            self._resources = self._resources_cm.__enter__()
             self._resources_contain_cm = isinstance(self._resources, IContainsGenerator)
             self._cm_scope_entered = False
 
         self._events = []
         self._user_events = []
-        self._metadata_entries = None
+        self._user_generated_metadata = {}
 
     def __enter__(self):
         if self._resources_cm:
@@ -169,7 +170,7 @@ class OutputContext:
 
     def __exit__(self, *exc):
         if self._resources_cm:
-            self._resources_cm.__exit__(*exc)  # pylint: disable=no-member
+            self._resources_cm.__exit__(*exc)
 
     def __del__(self):
         if (
@@ -178,7 +179,7 @@ class OutputContext:
             and self._resources_contain_cm
             and not self._cm_scope_entered
         ):
-            self._resources_cm.__exit__(None, None, None)  # pylint: disable=no-member
+            self._resources_cm.__exit__(None, None, None)
 
     @public
     @property
@@ -225,7 +226,7 @@ class OutputContext:
 
     @public
     @property
-    def metadata(self) -> Optional[Mapping[str, object]]:
+    def metadata(self) -> Optional[ArbitraryMetadataMapping]:
         return self._metadata
 
     @public
@@ -466,7 +467,8 @@ class OutputContext:
 
         Raises an error if either of the following are true:
         - The output asset has no partitioning.
-        - The output asset is not partitioned with a TimeWindowPartitionsDefinition.
+        - The output asset is not partitioned with a TimeWindowPartitionsDefinition or a
+        MultiPartitionsDefinition with one time-partitioned dimension.
         """
         if self._warn_on_step_context_use:
             warnings.warn(
@@ -587,15 +589,13 @@ class OutputContext:
         return self.get_asset_identifier()
 
     @public
-    def log_event(
-        self, event: Union[AssetObservation, AssetMaterialization, Materialization]
-    ) -> None:
+    def log_event(self, event: Union[AssetObservation, AssetMaterialization]) -> None:
         """Log an AssetMaterialization or AssetObservation from within the body of an io manager's `handle_output` method.
 
         Events logged with this method will appear in the event log.
 
         Args:
-            event (Union[AssetMaterialization, Materialization, AssetObservation]): The event to log.
+            event (Union[AssetMaterialization, AssetObservation]): The event to log.
 
         Examples:
         .. code-block:: python
@@ -608,7 +608,7 @@ class OutputContext:
         """
         from dagster._core.events import DagsterEvent
 
-        if isinstance(event, (AssetMaterialization, Materialization)):
+        if isinstance(event, (AssetMaterialization)):
             if self._step_context:
                 self._events.append(DagsterEvent.asset_materialization(self._step_context, event))
             self._user_events.append(event)
@@ -617,7 +617,7 @@ class OutputContext:
                 self._events.append(DagsterEvent.asset_observation(self._step_context, event))
             self._user_events.append(event)
         else:
-            check.failed("Unexpected event {event}".format(event=event))
+            check.failed(f"Unexpected event {event}")
 
     def consume_events(self) -> Iterator["DagsterEvent"]:
         """Pops and yields all user-generated events that have been recorded from this context.
@@ -630,7 +630,7 @@ class OutputContext:
 
     def get_logged_events(
         self,
-    ) -> Sequence[Union[AssetMaterialization, Materialization, AssetObservation]]:
+    ) -> Sequence[Union[AssetMaterialization, AssetObservation]]:
         """Retrieve the list of user-generated events that were logged via the context.
 
 
@@ -676,32 +676,36 @@ class OutputContext:
         """
         from dagster._core.definitions.metadata import normalize_metadata
 
-        prior_metadata_entries = self._metadata_entries or []
-
-        overlapping_labels = {entry.label for entry in prior_metadata_entries} & metadata.keys()
+        overlapping_labels = set(self._user_generated_metadata.keys()) & metadata.keys()
         if overlapping_labels:
             raise DagsterInvalidMetadata(
                 f"Tried to add metadata for key(s) that already have metadata: {overlapping_labels}"
             )
 
-        self._metadata_entries = [*prior_metadata_entries, *normalize_metadata(metadata, [])]
+        self._user_generated_metadata = {
+            **self._user_generated_metadata,
+            **normalize_metadata(metadata),
+        }
 
-    def get_logged_metadata_entries(
+    def get_logged_metadata(
         self,
-    ) -> Sequence[MetadataEntry]:
-        """Get the list of metadata entries that have been logged for use with this output."""
-        return self._metadata_entries or []
+    ) -> Mapping[str, MetadataValue]:
+        """Get the mapping of metadata entries that have been logged for use with this output."""
+        return self._user_generated_metadata
 
-    def consume_logged_metadata_entries(
+    def consume_logged_metadata(
         self,
-    ) -> Sequence[MetadataEntry]:
+    ) -> Mapping[str, MetadataValue]:
         """Pops and yields all user-generated metadata entries that have been recorded from this context.
 
-        If consume_logged_metadata_entries has not yet been called, this will yield all logged events since the call to `handle_output`. If consume_logged_metadata_entries has been called, it will yield all events since the last time consume_logged_metadata_entries was called. Designed for internal use. Users should never need to invoke this method.
+        If consume_logged_metadata has not yet been called, this will yield all logged events since
+        the call to `handle_output`. If consume_logged_metadata has been called, it will yield all
+        events since the last time consume_logged_metadata_entries was called. Designed for internal
+        use. Users should never need to invoke this method.
         """
-        result = self._metadata_entries
-        self._metadata_entries = []
-        return result or []
+        result = self._user_generated_metadata
+        self._user_generated_metadata = {}
+        return result or {}
 
 
 def get_output_context(
@@ -716,10 +720,9 @@ def get_output_context(
     version: Optional[str],
     warn_on_step_context_use: bool = False,
 ) -> "OutputContext":
-    """
-    Args:
-        run_id (str): The run ID of the run that produced the output, not necessarily the run that
-            the context will be used in.
+    """Args:
+    run_id (str): The run ID of the run that produced the output, not necessarily the run that
+        the context will be used in.
     """
     step = execution_plan.get_step_by_key(step_output_handle.step_key)
     # get config
@@ -732,7 +735,7 @@ def get_output_context(
         output_config = None
 
     step_output = execution_plan.get_step_output(step_output_handle)
-    output_def = pipeline_def.get_solid(step_output.solid_handle).output_def_named(step_output.name)
+    output_def = pipeline_def.get_node(step_output.node_handle).output_def_named(step_output.name)
 
     io_manager_key = output_def.io_manager_key
     resource_config = resolved_run_config.resources[io_manager_key].config
@@ -761,7 +764,7 @@ def get_output_context(
         metadata=output_def.metadata,
         mapping_key=step_output_handle.mapping_key,
         config=output_config,
-        op_def=pipeline_def.get_solid(step.node_handle).definition,  # type: ignore  # (should be OpDefinition not NodeDefinition)
+        op_def=pipeline_def.get_node(step.node_handle).definition,  # type: ignore  # (should be OpDefinition not NodeDefinition)
         dagster_type=output_def.dagster_type,
         log_manager=log_manager,
         version=version,
