@@ -1,5 +1,4 @@
-"""
-This module contains the execution context objects that are internal to the system.
+"""This module contains the execution context objects that are internal to the system.
 Not every property on these should be exposed to random Jane or Joe dagster user
 so we have a different layer of objects that encode the explicit public API
 in the user_context module.
@@ -22,9 +21,11 @@ from typing import (
 
 import dagster._check as check
 from dagster._annotations import public
+from dagster._core.definitions.dependency import OpNode
 from dagster._core.definitions.events import AssetKey, AssetLineageInfo
 from dagster._core.definitions.hook_definition import HookDefinition
 from dagster._core.definitions.mode import ModeDefinition
+from dagster._core.definitions.multi_dimensional_partitions import MultiPartitionsDefinition
 from dagster._core.definitions.op_definition import OpDefinition
 from dagster._core.definitions.partition import PartitionsDefinition, PartitionsSubset
 from dagster._core.definitions.partition_key_range import PartitionKeyRange
@@ -38,6 +39,7 @@ from dagster._core.definitions.step_launcher import StepLauncher
 from dagster._core.definitions.time_window_partitions import (
     TimeWindow,
     TimeWindowPartitionsDefinition,
+    has_one_dimension_time_window_partitioning,
 )
 from dagster._core.errors import DagsterInvariantViolationError
 from dagster._core.execution.plan.handle import ResolvedFromDynamicStepHandle, StepHandle
@@ -64,7 +66,7 @@ if TYPE_CHECKING:
     from dagster._core.definitions.data_version import (
         DataVersion,
     )
-    from dagster._core.definitions.dependency import Node, NodeHandle
+    from dagster._core.definitions.dependency import NodeHandle
     from dagster._core.definitions.job_definition import JobDefinition
     from dagster._core.definitions.resource_definition import Resources
     from dagster._core.event_api import EventLogRecord
@@ -429,19 +431,21 @@ class PlanExecutionContext(IPlanContext):
             return PartitionKeyRange(partition_key_range_start, tags[ASSET_PARTITION_RANGE_END_TAG])
 
     @property
-    def partition_time_window(self) -> str:
+    def partition_time_window(self) -> TimeWindow:
         partitions_def = self.partitions_def
 
-        if not isinstance(partitions_def, TimeWindowPartitionsDefinition):
-            check.failed(
-                (
-                    "Expected a TimeWindowPartitionsDefinition, but instead found"
-                    f" {type(partitions_def)}"
-                ),
+        if partitions_def is None:
+            raise DagsterInvariantViolationError("Partitions definition is not defined")
+
+        if not has_one_dimension_time_window_partitioning(partitions_def=partitions_def):
+            raise DagsterInvariantViolationError(
+                "Expected a TimeWindowPartitionsDefinition or MultiPartitionsDefinition with a"
+                f" single time dimension, but instead found {type(partitions_def)}"
             )
 
-        # mypy thinks partitions_def is <nothing> here because ????
-        return partitions_def.time_window_for_partition_key(self.partition_key)  # type: ignore
+        return cast(
+            Union[MultiPartitionsDefinition, TimeWindowPartitionsDefinition], partitions_def
+        ).time_window_for_partition_key(self.partition_key)
 
     @property
     def has_partition_key(self) -> bool:
@@ -547,7 +551,7 @@ class StepExecutionContext(PlanExecutionContext, IStepContext):
 
     @property
     def op_def(self) -> OpDefinition:
-        return self.solid.definition.ensure_op_def()
+        return self.solid.definition
 
     @property
     def pipeline_def(self) -> PipelineDefinition:
@@ -566,23 +570,20 @@ class StepExecutionContext(PlanExecutionContext, IStepContext):
         return self._execution_data.mode_def
 
     @property
-    def solid(self) -> "Node":
-        return self.pipeline_def.get_solid(self._step.node_handle)
+    def solid(self) -> OpNode:
+        return self.pipeline_def.get_op(self._step.node_handle)
 
     @property
     def solid_retry_policy(self) -> Optional[RetryPolicy]:
         return self.pipeline_def.get_retry_policy_for_handle(self.node_handle)
 
     def describe_op(self) -> str:
-        if isinstance(self.op_def, OpDefinition):
-            return f'op "{str(self.node_handle)}"'
-
-        return f'solid "{str(self.node_handle)}"'
+        return f'op "{str(self.node_handle)}"'
 
     def get_io_manager(self, step_output_handle: StepOutputHandle) -> IOManager:
         step_output = self.execution_plan.get_step_output(step_output_handle)
         io_manager_key = (
-            self.pipeline_def.get_solid(step_output.solid_handle)
+            self.pipeline_def.get_node(step_output.node_handle)
             .output_def_named(step_output.name)
             .io_manager_key
         )
@@ -881,7 +882,6 @@ class StepExecutionContext(PlanExecutionContext, IStepContext):
 
     # "external" refers to records for inputs generated outside of this step
     def fetch_external_input_asset_records(self) -> None:
-        # pylint: disable=protected-access
         output_keys: List[AssetKey] = []
         for step_output in self.step.step_outputs:
             asset_info = self.pipeline_def.asset_layer.asset_info_for_output(
@@ -893,7 +893,7 @@ class StepExecutionContext(PlanExecutionContext, IStepContext):
 
         all_dep_keys: List[AssetKey] = []
         for output_key in output_keys:
-            if output_key not in self.pipeline_def.asset_layer._asset_deps:
+            if output_key not in self.pipeline_def.asset_layer._asset_deps:  # noqa: SLF001
                 continue
             dep_keys = self.pipeline_def.asset_layer.upstream_assets_for_asset(output_key)
             for key in dep_keys:
@@ -1031,7 +1031,8 @@ class StepExecutionContext(PlanExecutionContext, IStepContext):
 
         Raises an error if either of the following are true:
         - The output asset has no partitioning.
-        - The output asset is not partitioned with a TimeWindowPartitionsDefinition.
+        - The output asset is not partitioned with a TimeWindowPartitionsDefinition or a
+          MultiPartitionsDefinition with one time-partitioned dimension.
         """
         partitions_def = self._partitions_def_for_output(output_name)
 
@@ -1041,11 +1042,15 @@ class StepExecutionContext(PlanExecutionContext, IStepContext):
                 "partitioned asset."
             )
 
-        if not isinstance(partitions_def, TimeWindowPartitionsDefinition):
+        if not has_one_dimension_time_window_partitioning(partitions_def):
             raise ValueError(
                 "Tried to get asset partitions for an output that correponds to a partitioned "
-                "asset that is not partitioned with a TimeWindowPartitionsDefinition."
+                "asset that is not time-partitioned."
             )
+
+        partitions_def = cast(
+            Union[TimeWindowPartitionsDefinition, MultiPartitionsDefinition], partitions_def
+        )
         partition_key_range = self.asset_partition_key_range_for_output(output_name)
         return TimeWindow(
             # mypy thinks partitions_def is <nothing> here because ????
