@@ -1,9 +1,9 @@
 from abc import ABC, abstractmethod
-from typing import AbstractSet, Callable, List, Optional, Sequence, Set, Union, cast
+from typing import AbstractSet, Callable, List, Optional, Sequence, Set, cast
 
 import dagster._check as check
 from dagster._core.definitions import JobDefinition, NodeHandle
-from dagster._core.definitions.events import AssetMaterialization, AssetObservation, Materialization
+from dagster._core.definitions.events import AssetMaterialization, AssetObservation
 from dagster._core.definitions.utils import DEFAULT_OUTPUT
 from dagster._core.errors import DagsterError, DagsterInvariantViolationError
 from dagster._core.events import (
@@ -65,7 +65,7 @@ class ExecutionResult(ABC):
         def _is_event_from_node(event: DagsterEvent) -> bool:
             if not event.is_step_event:
                 return False
-            node_handle = cast(NodeHandle, event.solid_handle)
+            node_handle = cast(NodeHandle, event.node_handle)
             return node_handle.is_or_descends_from(handle)
 
         return self.filter_events(_is_event_from_node)
@@ -147,9 +147,7 @@ class ExecutionResult(ABC):
 
         return events[0]
 
-    def asset_materializations_for_node(
-        self, node_name: str
-    ) -> Sequence[Union[Materialization, AssetMaterialization]]:
+    def asset_materializations_for_node(self, node_name: str) -> Sequence[AssetMaterialization]:
         return [
             cast(StepMaterializationData, event.event_specific_data).materialization
             for event in self.events_for_node(node_name)
@@ -163,6 +161,9 @@ class ExecutionResult(ABC):
             if event.event_type_value == DagsterEventType.ASSET_OBSERVATION.value
         ]
 
+    def get_asset_materialization_events(self) -> Sequence[DagsterEvent]:
+        return [event for event in self.all_events if event.is_step_materialization]
+
     def get_step_success_events(self) -> Sequence[DagsterEvent]:
         return [event for event in self.all_events if event.is_step_success]
 
