@@ -1,7 +1,7 @@
 import logging
 import os
 import time
-from typing import Iterable, Mapping, Optional, Sequence, Tuple, cast
+from typing import Callable, Iterable, Mapping, Optional, Sequence, Tuple, cast
 
 import dagster._check as check
 from dagster._core.definitions.selector import PipelineSelector
@@ -9,9 +9,9 @@ from dagster._core.errors import DagsterBackfillFailedError
 from dagster._core.execution.plan.resume_retry import ReexecutionStrategy
 from dagster._core.execution.plan.state import KnownExecutionState
 from dagster._core.host_representation import (
+    CodeLocation,
     ExternalPartitionSet,
     ExternalPipeline,
-    RepositoryLocation,
 )
 from dagster._core.host_representation.external_data import (
     ExternalPartitionExecutionParamData,
@@ -28,8 +28,10 @@ from dagster._core.storage.tags import (
 )
 from dagster._core.telemetry import BACKFILL_RUN_CREATED, hash_name, log_action
 from dagster._core.utils import make_new_run_id
-from dagster._core.workspace.context import BaseWorkspaceRequestContext
-from dagster._core.workspace.workspace import IWorkspace
+from dagster._core.workspace.context import (
+    BaseWorkspaceRequestContext,
+    IWorkspaceProcessContext,
+)
 from dagster._utils.error import SerializableErrorInfo
 from dagster._utils.merger import merge_dicts
 
@@ -44,7 +46,7 @@ CHECKPOINT_COUNT = 25
 def execute_job_backfill_iteration(
     backfill: PartitionBackfill,
     logger: logging.Logger,
-    workspace: BaseWorkspaceRequestContext,
+    workspace_process_context: IWorkspaceProcessContext,
     debug_crash_flags: Optional[Mapping[str, int]],
     instance: DagsterInstance,
 ) -> Iterable[Optional[SerializableErrorInfo]]:
@@ -56,13 +58,7 @@ def execute_job_backfill_iteration(
             f" {backfill.last_submitted_partition_name}"
         )
 
-    origin = cast(
-        ExternalPartitionSetOrigin, backfill.partition_set_origin
-    ).external_repository_origin.repository_location_origin
-
-    repo_location = workspace.get_repository_location(origin.location_name)
-
-    _check_repo_has_partition_set(repo_location, backfill)
+    _check_repo_has_partition_set(workspace_process_context, backfill)
 
     has_more = True
     while has_more:
@@ -76,7 +72,10 @@ def execute_job_backfill_iteration(
 
         if chunk:
             for _run_id in submit_backfill_runs(
-                instance, workspace, repo_location, backfill, chunk
+                instance,
+                lambda: workspace_process_context.create_request_context(),
+                backfill,
+                chunk,
             ):
                 yield None
                 # before submitting, refetch the backfill job to check for status changes
@@ -103,19 +102,24 @@ def execute_job_backfill_iteration(
 
 
 def _check_repo_has_partition_set(
-    repo_location: RepositoryLocation, backfill_job: PartitionBackfill
+    workspace_process_context: IWorkspaceProcessContext, backfill_job: PartitionBackfill
 ) -> None:
     origin = cast(ExternalPartitionSetOrigin, backfill_job.partition_set_origin)
 
+    location_name = origin.external_repository_origin.code_location_origin.location_name
+
+    workspace = workspace_process_context.create_request_context()
+    code_location = workspace.get_code_location(location_name)
+
     repo_name = origin.external_repository_origin.repository_name
-    if not repo_location.has_repository(repo_name):
+    if not code_location.has_repository(repo_name):
         raise DagsterBackfillFailedError(
-            f"Could not find repository {repo_name} in location {repo_location.name} to "
+            f"Could not find repository {repo_name} in location {code_location.name} to "
             f"run backfill {backfill_job.backfill_id}."
         )
 
     partition_set_name = origin.partition_set_name
-    external_repo = repo_location.get_repository(repo_name)
+    external_repo = code_location.get_repository(repo_name)
     if not external_repo.has_external_partition_set(partition_set_name):
         raise DagsterBackfillFailedError(
             f"Could not find partition set {partition_set_name} in repository {repo_name}. "
@@ -166,30 +170,31 @@ def _get_partitions_chunk(
 
 def submit_backfill_runs(
     instance: DagsterInstance,
-    workspace: IWorkspace,
-    repo_location: RepositoryLocation,
+    create_workspace: Callable[[], BaseWorkspaceRequestContext],
     backfill_job: PartitionBackfill,
     partition_names: Optional[Sequence[str]] = None,
 ) -> Iterable[Optional[str]]:
-    """Returns the run IDs of the submitted runs"""
+    """Returns the run IDs of the submitted runs."""
     origin = cast(ExternalPartitionSetOrigin, backfill_job.partition_set_origin)
 
     repository_origin = origin.external_repository_origin
     repo_name = repository_origin.repository_name
+    location_name = repository_origin.code_location_origin.location_name
 
     if not partition_names:
         partition_names = cast(Sequence[str], backfill_job.partition_names)
 
+    workspace = create_workspace()
+    code_location = workspace.get_code_location(location_name)
+
     check.invariant(
-        repo_location.has_repository(repo_name),
-        "Could not find repository {repo_name} in location {repo_location_name}".format(
-            repo_name=repo_name, repo_location_name=repo_location.name
-        ),
+        code_location.has_repository(repo_name),
+        f"Could not find repository {repo_name} in location {code_location.name}",
     )
-    external_repo = repo_location.get_repository(repo_name)
+    external_repo = code_location.get_repository(repo_name)
     partition_set_name = origin.partition_set_name
     external_partition_set = external_repo.get_external_partition_set(partition_set_name)
-    result = repo_location.get_external_partition_set_execution_param_data(
+    result = code_location.get_external_partition_set_execution_param_data(
         external_repo.handle, partition_set_name, partition_names, instance
     )
 
@@ -198,21 +203,25 @@ def submit_backfill_runs(
         # need to make another call to the user code location to properly subset
         # for an asset selection
         pipeline_selector = PipelineSelector(
-            location_name=repo_location.name,
+            location_name=code_location.name,
             repository_name=repo_name,
             pipeline_name=external_partition_set.pipeline_name,
             solid_selection=None,
             asset_selection=backfill_job.asset_selection,
         )
-        external_pipeline = repo_location.get_external_pipeline(pipeline_selector)
+        external_pipeline = code_location.get_external_pipeline(pipeline_selector)
     else:
         external_pipeline = external_repo.get_full_external_job(
             external_partition_set.pipeline_name
         )
     for partition_data in result.partition_data:
+        # Refresh the code location in case the workspace has reloaded mid-backfill
+        workspace = create_workspace()
+        code_location = workspace.get_code_location(location_name)
+
         dagster_run = create_backfill_run(
             instance,
-            repo_location,
+            code_location,
             external_pipeline,
             external_partition_set,
             backfill_job,
@@ -229,7 +238,7 @@ def submit_backfill_runs(
 
 def create_backfill_run(
     instance: DagsterInstance,
-    repo_location: RepositoryLocation,
+    code_location: CodeLocation,
     external_pipeline: ExternalPipeline,
     external_partition_set: ExternalPartitionSet,
     backfill_job: PartitionBackfill,
@@ -242,7 +251,7 @@ def create_backfill_run(
         BACKFILL_RUN_CREATED,
         metadata={
             "DAEMON_SESSION_ID": get_telemetry_daemon_session_id(),
-            "repo_hash": hash_name(repo_location.name),
+            "repo_hash": hash_name(code_location.name),
             "pipeline_name_hash": hash_name(external_pipeline.name),
         },
     )
@@ -271,7 +280,7 @@ def create_backfill_run(
             return None
         return instance.create_reexecuted_run(
             parent_run=last_run,
-            repo_location=repo_location,
+            code_location=code_location,
             external_pipeline=external_pipeline,
             strategy=ReexecutionStrategy.FROM_FAILURE,
             extra_tags=tags,
@@ -301,7 +310,7 @@ def create_backfill_run(
             solids_to_execute = frozenset(external_partition_set.solid_selection)
             solid_selection = external_partition_set.solid_selection
 
-    external_execution_plan = repo_location.get_external_execution_plan(
+    external_execution_plan = code_location.get_external_execution_plan(
         external_pipeline,
         partition_data.run_config,
         check.not_none(external_partition_set.mode),
