@@ -136,16 +136,14 @@ class ActiveExecution:
                 self._executable + self._pending_abandon + self._pending_retry + self._pending_skip
             )
             state_str = "{pending_str}{in_flight_str}{action_str}{retry_str}".format(
-                in_flight_str="\nSteps still in flight: {}".format(self._in_flight)
+                in_flight_str=f"\nSteps still in flight: {self._in_flight}"
                 if self._in_flight
                 else "",
-                pending_str="\nSteps pending processing: {}".format(self._pending.keys())
+                pending_str=f"\nSteps pending processing: {self._pending.keys()}"
                 if self._pending
                 else "",
-                action_str="\nSteps pending action: {}".format(pending_action)
-                if pending_action
-                else "",
-                retry_str="\nSteps waiting to retry: {}".format(self._waiting_to_retry.keys())
+                action_str=f"\nSteps pending action: {pending_action}" if pending_action else "",
+                retry_str=f"\nSteps waiting to retry: {self._waiting_to_retry.keys()}"
                 if self._waiting_to_retry
                 else "",
             )
@@ -388,8 +386,8 @@ class ActiveExecution:
                 step_context.log.error(
                     "Dependencies for step {step}{fail_str}{abandon_str}. Not executing.".format(
                         step=step.key,
-                        fail_str=" failed: {}".format(failed_inputs) if failed_inputs else "",
-                        abandon_str=" were not executed: {}".format(abandoned_inputs)
+                        fail_str=f" failed: {failed_inputs}" if failed_inputs else "",
+                        abandon_str=f" were not executed: {abandoned_inputs}"
                         if abandoned_inputs
                         else "",
                     )
@@ -425,7 +423,7 @@ class ActiveExecution:
     def mark_up_for_retry(self, step_key: str, at_time: Optional[float] = None) -> None:
         check.invariant(
             not self._retry_mode.disabled,
-            "Attempted to mark {} as up for retry but retries are disabled".format(step_key),
+            f"Attempted to mark {step_key} as up for retry but retries are disabled",
         )
         check.opt_float_param(at_time, "at_time")
 
@@ -473,7 +471,7 @@ class ActiveExecution:
         elif dagster_event.is_step_skipped:
             # Skip events are generated by this class. They should not be sent via handle_event
             raise DagsterInvariantViolationError(
-                "Step {} was reported as skipped from outside the ActiveExecution.".format(step_key)
+                f"Step {step_key} was reported as skipped from outside the ActiveExecution."
             )
         elif dagster_event.is_step_up_for_retry:
             self.mark_up_for_retry(
@@ -567,8 +565,7 @@ class ActiveExecution:
     def rebuild_from_events(
         self, dagster_events: Sequence[DagsterEvent]
     ) -> Sequence[ExecutionStep]:
-        """
-        Replay events to rebuild the execution state and continue after a failure.
+        """Replay events to rebuild the execution state and continue after a failure.
 
         Returns a list of steps that are possibly in flight. Current status of the event log implies
         that the previous run worker might have crashed before launching these steps, or it may have
