@@ -7,6 +7,7 @@ from typing import (
     Dict,
     Generator,
     Iterator,
+    Mapping,
     Optional,
     Sequence,
     Tuple,
@@ -22,7 +23,6 @@ from dagster._core.definitions import (
     AssetMaterialization,
     DynamicOutput,
     ExpectationResult,
-    Materialization,
     Output,
     OutputDefinition,
 )
@@ -97,12 +97,12 @@ async def _coerce_async_solid_to_async_gen(awaitable, context, output_defs):
 def invoke_compute_fn(
     fn: Callable,
     context: OpExecutionContext,
-    kwargs: Dict[str, Any],
+    kwargs: Mapping[str, Any],
     context_arg_provided: bool,
     config_arg_cls: Optional[Type[Config]],
     resource_args: Optional[Dict[str, str]] = None,
 ) -> Any:
-    args_to_pass = kwargs
+    args_to_pass = {**kwargs}
     if config_arg_cls:
         # config_arg_cls is either a Config class or a primitive type
         if issubclass(config_arg_cls, Config):
@@ -112,6 +112,7 @@ def invoke_compute_fn(
     if resource_args:
         for resource_name, arg_name in resource_args.items():
             args_to_pass[arg_name] = getattr(context.resources, resource_name)
+
     return fn(context, **args_to_pass) if context_arg_provided else fn(**args_to_pass)
 
 
@@ -192,7 +193,7 @@ def validate_and_coerce_op_result_to_iterator(
         # this happens when a user explicitly returns a generator in the solid
         for event in result:
             yield event
-    elif isinstance(result, (AssetMaterialization, Materialization, ExpectationResult)):
+    elif isinstance(result, (AssetMaterialization, ExpectationResult)):
         raise DagsterInvariantViolationError(
             f"Error in {context.describe_op()}: If you are "
             "returning an AssetMaterialization "
@@ -240,7 +241,7 @@ def validate_and_coerce_op_result_to_iterator(
                             output_name=output_def.name,
                             value=dynamic_output.value,
                             mapping_key=dynamic_output.mapping_key,
-                            metadata_entries=list(dynamic_output.metadata_entries),
+                            metadata=dynamic_output.metadata,
                         )
             elif isinstance(element, Output):
                 if annotation != inspect.Parameter.empty and not is_generic_output_annotation(
@@ -259,7 +260,7 @@ def validate_and_coerce_op_result_to_iterator(
                     yield Output(
                         output_name=output_def.name,
                         value=element.value,
-                        metadata_entries=element.metadata_entries,
+                        metadata=element.metadata,
                         data_version=element.data_version,
                     )
             else:
