@@ -5,10 +5,8 @@ from typing import (
     Any,
     Dict,
     Iterator,
-    List,
     Mapping,
     Optional,
-    Sequence,
     Tuple,
     Union,
     cast,
@@ -22,7 +20,6 @@ from dagster._core.definitions import (
     AssetMaterialization,
     AssetObservation,
     ExpectationResult,
-    Materialization,
     Output,
     OutputDefinition,
     TypeCheck,
@@ -41,6 +38,7 @@ from dagster._core.definitions.decorators.op_decorator import DecoratedOpFunctio
 from dagster._core.definitions.events import DynamicOutput
 from dagster._core.definitions.metadata import (
     MetadataEntry,
+    MetadataValue,
     normalize_metadata,
 )
 from dagster._core.definitions.multi_dimensional_partitions import (
@@ -77,8 +75,7 @@ from .utils import op_execution_error_boundary
 def _step_output_error_checked_user_event_sequence(
     step_context: StepExecutionContext, user_event_sequence: Iterator[OpOutputUnion]
 ) -> Iterator[OpOutputUnion]:
-    """
-    Process the event sequence to check for invariant violations in the event
+    """Process the event sequence to check for invariant violations in the event
     sequence related to Output events emitted from the compute_fn.
 
     This consumes and emits an event sequence.
@@ -104,7 +101,7 @@ def _step_output_error_checked_user_event_sequence(
             )
 
         step_output = step.step_output_named(cast(str, output.output_name))
-        output_def = step_context.pipeline_def.get_solid(step_output.solid_handle).output_def_named(
+        output_def = step_context.pipeline_def.get_node(step_output.node_handle).output_def_named(
             step_output.name
         )
 
@@ -130,10 +127,10 @@ def _step_output_error_checked_user_event_sequence(
                 output = Output(
                     value=output.value,
                     output_name=output.output_name,
-                    metadata_entries=[
-                        *output.metadata_entries,
-                        *normalize_metadata(cast(Dict[str, Any], metadata), []),
-                    ],
+                    metadata={
+                        **output.metadata,
+                        **normalize_metadata(metadata or {}),
+                    },
                     data_version=output.data_version,
                 )
         else:
@@ -154,10 +151,7 @@ def _step_output_error_checked_user_event_sequence(
             output = DynamicOutput(
                 value=output.value,
                 output_name=output.output_name,
-                metadata_entries=[
-                    *output.metadata_entries,
-                    *normalize_metadata(cast(Dict[str, Any], metadata), []),
-                ],
+                metadata={**output.metadata, **normalize_metadata(metadata or {})},
                 mapping_key=output.mapping_key,
             )
 
@@ -210,7 +204,7 @@ def _create_step_input_event(
                 success=success,
                 label=input_name,
                 description=type_check.description if type_check else None,
-                metadata_entries=type_check.metadata_entries if type_check else [],
+                metadata=type_check.metadata if type_check else {},
             ),
         ),
     )
@@ -255,7 +249,7 @@ def _type_checked_event_sequence_for_input(
                 f'expected type "{dagster_type.display_name}". '
                 f"Description: {type_check.description}"
             ),
-            metadata_entries=type_check.metadata_entries,
+            metadata=type_check.metadata,
             dagster_type=dagster_type,
         )
 
@@ -292,12 +286,10 @@ def _type_check_output(
                 success=type_check.success,
                 label=step_output_handle.output_name,
                 description=type_check.description if type_check else None,
-                metadata_entries=type_check.metadata_entries if type_check else [],
+                metadata=type_check.metadata if type_check else {},
             ),
             version=version,
-            metadata_entries=[
-                entry for entry in output.metadata_entries if isinstance(entry, MetadataEntry)
-            ],
+            metadata=output.metadata,
         ),
     )
 
@@ -308,7 +300,7 @@ def _type_check_output(
                 f'expected type "{dagster_type.display_name}". '
                 f"Description: {type_check.description}"
             ),
-            metadata_entries=type_check.metadata_entries,
+            metadata=type_check.metadata,
             dagster_type=dagster_type,
         )
 
@@ -316,8 +308,7 @@ def _type_check_output(
 def core_dagster_event_sequence_for_step(
     step_context: StepExecutionContext,
 ) -> Iterator[DagsterEvent]:
-    """
-    Execute the step within the step_context argument given the in-memory
+    """Execute the step within the step_context argument given the in-memory
     events. This function yields a sequence of DagsterEvents, but without
     catching any exceptions that have bubbled up during the computation
     of the step.
@@ -385,7 +376,7 @@ def core_dagster_event_sequence_for_step(
                     yield evt
             # for now, I'm ignoring AssetMaterializations yielded manually, but we might want
             # to do something with these in the above path eventually
-            elif isinstance(user_event, (AssetMaterialization, Materialization)):
+            elif isinstance(user_event, AssetMaterialization):
                 yield DagsterEvent.asset_materialization(step_context, user_event)
             elif isinstance(user_event, AssetObservation):
                 yield DagsterEvent.asset_observation(step_context, user_event)
@@ -459,10 +450,10 @@ def _get_output_asset_materializations(
     asset_partitions: AbstractSet[str],
     output: Union[Output, DynamicOutput],
     output_def: OutputDefinition,
-    io_manager_metadata_entries: Sequence[MetadataEntry],
+    io_manager_metadata: Mapping[str, MetadataValue],
     step_context: StepExecutionContext,
 ) -> Iterator[AssetMaterialization]:
-    all_metadata = [*output.metadata_entries, *io_manager_metadata_entries]
+    all_metadata = {**output.metadata, **io_manager_metadata}
 
     # Clear any cached record associated with this asset, since we are about to generate a new
     # materialization.
@@ -509,16 +500,14 @@ def _get_output_asset_materializations(
                 yield AssetMaterialization(
                     asset_key=asset_key,
                     partition=partition,
-                    metadata_entries=all_metadata,
+                    metadata=all_metadata,
                     tags=tags,
                 )
     else:
         with warnings.catch_warnings():
             warnings.simplefilter("ignore", category=DeprecationWarning)
 
-            yield AssetMaterialization(
-                asset_key=asset_key, metadata_entries=all_metadata, tags=tags
-            )
+            yield AssetMaterialization(asset_key=asset_key, metadata=all_metadata, tags=tags)
 
 
 def _get_code_version(asset_key: AssetKey, step_context: StepExecutionContext) -> str:
@@ -584,7 +573,7 @@ def _store_output(
     output_context = step_context.get_output_context(step_output_handle)
 
     manager_materializations = []
-    manager_metadata_entries: List[MetadataEntry] = []
+    manager_metadata: Dict[str, MetadataValue] = {}
 
     # output_manager.handle_output is either a generator function, or a normal function with or
     # without a return value. In the case that handle_output is a normal function, we need to
@@ -617,16 +606,21 @@ def _store_output(
         for event in output_context.consume_events():
             yield event
 
-        manager_metadata_entries.extend(output_context.consume_logged_metadata_entries())
+        manager_metadata = {**manager_metadata, **output_context.consume_logged_metadata()}
         if isinstance(elt, DagsterEvent):
             yield elt
         elif isinstance(elt, AssetMaterialization):
             manager_materializations.append(elt)
-        elif isinstance(elt, MetadataEntry):
+        elif isinstance(elt, MetadataEntry):  # should remove this?
             experimental_functionality_warning(
                 "Yielding metadata from an IOManager's handle_output() function"
             )
-            manager_metadata_entries.append(elt)
+            manager_metadata[elt.label] = elt.value
+        elif isinstance(elt, dict):  # should remove this?
+            experimental_functionality_warning(
+                "Yielding metadata from an IOManager's handle_output() function"
+            )
+            manager_metadata = {**manager_metadata, **normalize_metadata(elt)}
         else:
             raise DagsterInvariantViolationError(
                 f"IO manager on output {output_def.name} has returned "
@@ -637,10 +631,10 @@ def _store_output(
     for event in output_context.consume_events():
         yield event
 
-    manager_metadata_entries.extend(output_context.consume_logged_metadata_entries())
+    manager_metadata = {**manager_metadata, **output_context.consume_logged_metadata()}
     # do not alter explicitly created AssetMaterializations
-    for materialization in manager_materializations:
-        if materialization.metadata_entries and manager_metadata_entries:
+    for mgr_materialization in manager_materializations:
+        if mgr_materialization.metadata and manager_metadata:
             raise DagsterInvariantViolationError(
                 f"When handling output '{output_context.name}' of"
                 f" {output_context.op_def.node_type_str} '{output_context.op_def.name}', received a"
@@ -649,17 +643,19 @@ def _store_output(
                 " Please specify metadata in one place within the `handle_output` function."
             )
 
-        if manager_metadata_entries:
+        if manager_metadata:
             with warnings.catch_warnings():
                 warnings.simplefilter("ignore", category=ExperimentalWarning)
 
                 materialization = AssetMaterialization(
-                    asset_key=materialization.asset_key,
-                    description=materialization.description,
-                    metadata_entries=manager_metadata_entries,
-                    partition=materialization.partition,
-                    metadata=None,
+                    asset_key=mgr_materialization.asset_key,
+                    description=mgr_materialization.description,
+                    metadata=manager_metadata,
+                    partition=mgr_materialization.partition,
                 )
+        else:
+            materialization = mgr_materialization
+
         yield DagsterEvent.asset_materialization(step_context, materialization)
 
     asset_key, partitions = _asset_key_and_partitions_for_output(output_context)
@@ -669,7 +665,7 @@ def _store_output(
             partitions,
             output,
             output_def,
-            manager_metadata_entries,
+            manager_metadata,
             step_context,
         ):
             yield DagsterEvent.asset_materialization(step_context, materialization)
@@ -678,7 +674,5 @@ def _store_output(
         step_context,
         output_name=step_output_handle.output_name,
         manager_key=output_def.io_manager_key,
-        metadata_entries=[
-            entry for entry in manager_metadata_entries if isinstance(entry, MetadataEntry)
-        ],
+        metadata=manager_metadata,
     )
