@@ -6,16 +6,17 @@ from dagster._core.definitions.dependency import NodeHandle
 from dagster._serdes import whitelist_for_serdes
 
 
-@whitelist_for_serdes
-class StepHandle(NamedTuple("_StepHandle", [("solid_handle", NodeHandle), ("key", str)])):
+# Serialize node_handle -> solid_handle for backcompat
+@whitelist_for_serdes(storage_field_names={"node_handle": "solid_handle"})
+class StepHandle(NamedTuple("_StepHandle", [("node_handle", NodeHandle), ("key", str)])):
     """A reference to an ExecutionStep that was determined statically."""
 
-    def __new__(cls, solid_handle: NodeHandle, key: Optional[str] = None):
+    def __new__(cls, node_handle: NodeHandle, key: Optional[str] = None):
         return super(StepHandle, cls).__new__(
             cls,
-            solid_handle=check.inst_param(solid_handle, "solid_handle", NodeHandle),
+            node_handle=check.inst_param(node_handle, "node_handle", NodeHandle),
             # mypy can't tell that if default is set, this is guaranteed to be a str
-            key=cast(str, check.opt_str_param(key, "key", default=solid_handle.to_string())),
+            key=cast(str, check.opt_str_param(key, "key", default=node_handle.to_string())),
         )
 
     def to_key(self) -> str:
@@ -38,46 +39,47 @@ class StepHandle(NamedTuple("_StepHandle", [("solid_handle", NodeHandle), ("key"
         return StepHandle(NodeHandle.from_string(string))
 
 
-@whitelist_for_serdes
-class UnresolvedStepHandle(NamedTuple("_UnresolvedStepHandle", [("solid_handle", NodeHandle)])):
+# Serialize node_handle -> solid_handle for backcompat
+@whitelist_for_serdes(storage_field_names={"node_handle": "solid_handle"})
+class UnresolvedStepHandle(NamedTuple("_UnresolvedStepHandle", [("node_handle", NodeHandle)])):
     """A reference to an UnresolvedMappedExecutionStep in an execution."""
 
-    def __new__(cls, solid_handle: NodeHandle):
+    def __new__(cls, node_handle: NodeHandle):
         return super(UnresolvedStepHandle, cls).__new__(
             cls,
-            solid_handle=check.inst_param(solid_handle, "solid_handle", NodeHandle),
+            node_handle=check.inst_param(node_handle, "node_handle", NodeHandle),
         )
 
     def to_key(self):
-        return f"{self.solid_handle.to_string()}[?]"
+        return f"{self.node_handle.to_string()}[?]"
 
     def resolve(self, map_key) -> "ResolvedFromDynamicStepHandle":
-        return ResolvedFromDynamicStepHandle(self.solid_handle, map_key)
+        return ResolvedFromDynamicStepHandle(self.node_handle, map_key)
 
 
-@whitelist_for_serdes
+# Serialize node_handle -> solid_handle for backcompat
+@whitelist_for_serdes(storage_field_names={"node_handle": "solid_handle"})
 class ResolvedFromDynamicStepHandle(
     NamedTuple(
         "_ResolvedFromDynamicStepHandle",
-        [("solid_handle", NodeHandle), ("mapping_key", str), ("key", str)],
+        [("node_handle", NodeHandle), ("mapping_key", str), ("key", str)],
     )
 ):
-    """
-    A reference to an ExecutionStep that came from resolving an UnresolvedMappedExecutionStep
+    """A reference to an ExecutionStep that came from resolving an UnresolvedMappedExecutionStep
     (and associated UnresolvedStepHandle) downstream of a dynamic output after it has
     completed successfully.
     """
 
-    def __new__(cls, solid_handle: NodeHandle, mapping_key: str, key: Optional[str] = None):
+    def __new__(cls, node_handle: NodeHandle, mapping_key: str, key: Optional[str] = None):
         return super(ResolvedFromDynamicStepHandle, cls).__new__(
             cls,
-            solid_handle=check.inst_param(solid_handle, "solid_handle", NodeHandle),
+            node_handle=check.inst_param(node_handle, "node_handle", NodeHandle),
             mapping_key=check.str_param(mapping_key, "mapping_key"),
             # mypy can't tell that if default is set, this is guaranteed to be a str
             key=cast(
                 str,
                 check.opt_str_param(
-                    key, "key", default=f"{solid_handle.to_string()}[{mapping_key}]"
+                    key, "key", default=f"{node_handle.to_string()}[{mapping_key}]"
                 ),
             ),
         )
@@ -87,4 +89,4 @@ class ResolvedFromDynamicStepHandle(
 
     @property
     def unresolved_form(self) -> UnresolvedStepHandle:
-        return UnresolvedStepHandle(solid_handle=self.solid_handle)
+        return UnresolvedStepHandle(node_handle=self.node_handle)
