@@ -1,13 +1,16 @@
-from typing import NamedTuple, Optional, Sequence, Union
+from typing import Mapping, NamedTuple, Optional
 
 import dagster._check as check
 from dagster._core.definitions import (
     AssetMaterialization,
-    Materialization,
-    MetadataEntry,
     NodeHandle,
 )
 from dagster._core.definitions.events import AssetKey
+from dagster._core.definitions.metadata import (
+    MetadataFieldSerializer,
+    MetadataValue,
+    normalize_metadata,
+)
 from dagster._serdes import whitelist_for_serdes
 
 from .handle import UnresolvedStepHandle
@@ -48,11 +51,12 @@ class StepOutputProperties(
         )
 
 
+@whitelist_for_serdes(storage_field_names={"node_handle": "solid_handle"})
 class StepOutput(
     NamedTuple(
         "_StepOutput",
         [
-            ("solid_handle", NodeHandle),
+            ("node_handle", NodeHandle),
             ("name", str),
             ("dagster_type_key", str),
             ("properties", StepOutputProperties),
@@ -63,14 +67,14 @@ class StepOutput(
 
     def __new__(
         cls,
-        solid_handle: NodeHandle,
+        node_handle: NodeHandle,
         name: str,
         dagster_type_key: str,
         properties: StepOutputProperties,
     ):
         return super(StepOutput, cls).__new__(
             cls,
-            solid_handle=check.inst_param(solid_handle, "solid_handle", NodeHandle),
+            node_handle=check.inst_param(node_handle, "node_handle", NodeHandle),
             name=check.str_param(name, "name"),
             dagster_type_key=check.str_param(dagster_type_key, "dagster_type_key"),
             properties=check.inst_param(properties, "properties", StepOutputProperties),
@@ -99,7 +103,10 @@ class StepOutput(
         return self.properties.asset_key
 
 
-@whitelist_for_serdes
+@whitelist_for_serdes(
+    storage_field_names={"metadata": "metadata_entries"},
+    field_serializers={"metadata": MetadataFieldSerializer},
+)
 class StepOutputData(
     NamedTuple(
         "_StepOutputData",
@@ -107,7 +114,7 @@ class StepOutputData(
             ("step_output_handle", "StepOutputHandle"),
             ("type_check_data", Optional[TypeCheckData]),
             ("version", Optional[str]),
-            ("metadata_entries", Sequence[MetadataEntry]),
+            ("metadata", Mapping[str, MetadataValue]),
         ],
     )
 ):
@@ -118,10 +125,9 @@ class StepOutputData(
         step_output_handle: "StepOutputHandle",
         type_check_data: Optional[TypeCheckData] = None,
         version: Optional[str] = None,
-        metadata_entries: Optional[Sequence[MetadataEntry]] = None,
+        metadata: Optional[Mapping[str, MetadataValue]] = None,
         # graveyard
-        # pylint: disable=unused-argument
-        intermediate_materialization: Optional[Union[AssetMaterialization, Materialization]] = None,
+        intermediate_materialization: Optional[AssetMaterialization] = None,
     ):
         return super(StepOutputData, cls).__new__(
             cls,
@@ -130,8 +136,8 @@ class StepOutputData(
             ),
             type_check_data=check.opt_inst_param(type_check_data, "type_check_data", TypeCheckData),
             version=check.opt_str_param(version, "version"),
-            metadata_entries=check.opt_sequence_param(
-                metadata_entries, "metadata_entries", MetadataEntry
+            metadata=normalize_metadata(
+                check.opt_mapping_param(metadata, "metadata", key_type=str)
             ),
         )
 
@@ -174,8 +180,7 @@ class UnresolvedStepOutputHandle(
         ],
     )
 ):
-    """
-    Placeholding that will resolve to StepOutputHandle for each mapped output once the
+    """Placeholding that will resolve to StepOutputHandle for each mapped output once the
     upstream dynamic step has completed.
     """
 
