@@ -10,7 +10,7 @@ POLLING_INTERVAL = 0.1
 def current_process_is_orphaned(parent_pid):
     parent_pid = int(parent_pid)
     if sys.platform == "win32":
-        import psutil  # pylint: disable=import-error
+        import psutil
 
         try:
             parent = psutil.Process(parent_pid)
@@ -23,15 +23,14 @@ def current_process_is_orphaned(parent_pid):
 
 
 def tail_polling(filepath, stream=sys.stdout, parent_pid=None):
-    """
-    Tails a file and outputs the content to the specified stream via polling.
+    """Tails a file and outputs the content to the specified stream via polling.
     The pid of the parent process (if provided) is checked to see if the tail process should be
-    terminated, in case the parent is hard-killed / segfaults
+    terminated, in case the parent is hard-killed / segfaults.
     """
     with open(filepath, "r", encoding="utf8") as file:
         for block in iter(lambda: file.read(1024), None):
             if block:
-                print(block, end="", file=stream)  # pylint: disable=print-call
+                print(block, end="", file=stream)
             else:
                 if pop_captured_interrupt() or (
                     parent_pid and current_process_is_orphaned(parent_pid)
