@@ -342,9 +342,7 @@ def single_resource_event_generator(
                     resource, format_duration(timer_result.millis), is_gen
                 )
             except StopIteration:
-                check.failed(
-                    "Resource generator {name} must yield one item.".format(name=resource_name)
-                )
+                check.failed(f"Resource generator {resource_name} must yield one item.")
 
         yield resource
 
@@ -357,9 +355,7 @@ def single_resource_event_generator(
         except StopIteration:
             pass
         else:
-            check.failed(
-                "Resource generator {name} yielded more than one item.".format(name=resource_name)
-            )
+            check.failed(f"Resource generator {resource_name} yielded more than one item.")
 
 
 def get_required_resource_keys_to_init(
@@ -407,7 +403,7 @@ def get_required_resource_keys_for_step(
     resource_keys: Set[str] = set()
 
     # add all the solid compute resource keys
-    solid_def = pipeline_def.get_solid(execution_step.node_handle).definition
+    solid_def = pipeline_def.get_node(execution_step.node_handle).definition
     resource_keys = resource_keys.union(solid_def.required_resource_keys)  # type: ignore  # (should be OpDefinition)
 
     # add input type, input loader, and input io manager resource keys
