@@ -20,7 +20,6 @@ from dagster._core.definitions.dependency import GraphNode, OpNode
 from dagster._core.definitions.events import (
     AssetMaterialization,
     ExpectationResult,
-    Materialization,
 )
 from dagster._core.definitions.utils import DEFAULT_OUTPUT
 from dagster._core.errors import DagsterInvariantViolationError
@@ -138,9 +137,7 @@ class GraphExecutionResult:
         self, node: Node, handle: NodeHandle
     ) -> Union["CompositeSolidExecutionResult", "OpExecutionResult"]:
         if not node:
-            raise DagsterInvariantViolationError(
-                "Can not find solid handle {handle_str}.".format(handle_str=handle.to_string())
-            )
+            raise DagsterInvariantViolationError(f"Can not find solid handle {handle.to_string()}.")
 
         events_by_kind: DefaultDict[StepKind, List[DagsterEvent]] = defaultdict(list)
 
@@ -424,8 +421,8 @@ class OpExecutionResult:
     @property
     def materializations_during_compute(
         self,
-    ) -> Sequence[Union[Materialization, AssetMaterialization]]:
-        """List[Materialization]: All materializations yielded by the solid."""
+    ) -> Sequence[AssetMaterialization]:
+        """List[AssetMaterialization]: All materializations yielded by the solid."""
         return [
             cast(StepMaterializationData, mat_event.event_specific_data).materialization
             for mat_event in self.materialization_events_during_compute
@@ -460,7 +457,7 @@ class OpExecutionResult:
             if step_event.event_type == DagsterEventType.STEP_SUCCESS:
                 return step_event
 
-        check.failed("Step success not found for solid {}".format(self.node.name))
+        check.failed(f"Step success not found for solid {self.node.name}")
 
     @property
     def compute_step_failure_event(self) -> DagsterEvent:
