@@ -20,7 +20,7 @@ import dagster._check as check
 from dagster._config.snap import ConfigFieldSnap, ConfigSchemaSnapshot
 from dagster._core.definitions.events import AssetKey
 from dagster._core.definitions.metadata import (
-    MetadataEntry,
+    MetadataValue,
 )
 from dagster._core.definitions.run_request import InstigatorType
 from dagster._core.definitions.schedule_definition import DefaultScheduleStatus
@@ -42,16 +42,17 @@ from dagster._core.host_representation.origin import (
     ExternalPipelineOrigin,
     ExternalRepositoryOrigin,
 )
+from dagster._core.instance import DagsterInstance
 from dagster._core.origin import PipelinePythonOrigin, RepositoryPythonOrigin
 from dagster._core.snap import ExecutionPlanSnapshot
 from dagster._core.snap.execution_plan_snapshot import ExecutionStepSnap
 from dagster._core.utils import toposort
 from dagster._serdes import create_snapshot_id
-from dagster._utils import iter_to_list
 from dagster._utils.cached_method import cached_method
 from dagster._utils.schedules import schedule_execution_time_iterator
 
 from .external_data import (
+    EnvVarConsumer,
     ExternalAssetNode,
     ExternalJobRef,
     ExternalPartitionSetData,
@@ -59,10 +60,13 @@ from .external_data import (
     ExternalPresetData,
     ExternalRepositoryData,
     ExternalResourceData,
+    ExternalResourceValue,
     ExternalScheduleData,
     ExternalSensorData,
     ExternalSensorMetadata,
     ExternalTargetData,
+    NestedResource,
+    ResourceJobUsageEntry,
 )
 from .handle import InstigatorHandle, JobHandle, PartitionSetHandle, RepositoryHandle
 from .pipeline_index import PipelineIndex
@@ -73,8 +77,7 @@ if TYPE_CHECKING:
 
 
 class ExternalRepository:
-    """
-    ExternalRepository is a object that represents a loaded repository definition that
+    """ExternalRepository is a object that represents a loaded repository definition that
     is resident in another process or container. Host processes such as dagit use
     objects such as these to interact with user-defined artifacts.
     """
@@ -141,7 +144,7 @@ class ExternalRepository:
         return self._external_schedules[schedule_name]
 
     def get_external_schedules(self) -> Sequence[ExternalSchedule]:
-        return iter_to_list(self._external_schedules.values())
+        return list(self._external_schedules.values())
 
     @property
     @cached_method
@@ -162,6 +165,13 @@ class ExternalRepository:
     def get_external_resources(self) -> Iterable[ExternalResource]:
         return self._external_resources.values()
 
+    @property
+    def _utilized_env_vars(self) -> Mapping[str, Sequence[EnvVarConsumer]]:
+        return self.external_repository_data.utilized_env_vars or {}
+
+    def get_utilized_env_vars(self) -> Mapping[str, Sequence[EnvVarConsumer]]:
+        return self._utilized_env_vars
+
     @property
     @cached_method
     def _external_sensors(self) -> Dict[str, ExternalSensor]:
@@ -177,7 +187,7 @@ class ExternalRepository:
         return self._external_sensors[sensor_name]
 
     def get_external_sensors(self) -> Sequence[ExternalSensor]:
-        return iter_to_list(self._external_sensors.values())
+        return list(self._external_sensors.values())
 
     @property
     @cached_method
@@ -196,7 +206,7 @@ class ExternalRepository:
         return self._external_partition_sets[partition_set_name]
 
     def get_external_partition_sets(self) -> Sequence[ExternalPartitionSet]:
-        return iter_to_list(self._external_partition_sets.values())
+        return list(self._external_partition_sets.values())
 
     def has_external_job(self, job_name: str) -> bool:
         return job_name in self._job_map
@@ -249,8 +259,7 @@ class ExternalRepository:
         return self.handle.get_python_origin()
 
     def get_external_origin_id(self) -> str:
-        """
-        A means of identifying the repository this ExternalRepository represents based on
+        """A means of identifying the repository this ExternalRepository represents based on
         where it came from.
         """
         return self.get_external_origin().get_id()
@@ -277,8 +286,7 @@ class ExternalRepository:
 
 
 class ExternalPipeline(RepresentedPipeline):
-    """
-    ExternalPipeline is a object that represents a loaded job definition that
+    """ExternalPipeline is a object that represents a loaded job definition that
     is resident in another process or container. Host processes such as dagit use
     objects such as these to interact with user-defined artifacts.
     """
@@ -340,7 +348,7 @@ class ExternalPipeline(RepresentedPipeline):
 
     @property
     def solid_names_in_topological_order(self):
-        return self._pipeline_index.pipeline_snapshot.solid_names_in_topological_order
+        return self._pipeline_index.pipeline_snapshot.node_names_in_topological_order
 
     @property
     def external_pipeline_data(self):
@@ -359,7 +367,7 @@ class ExternalPipeline(RepresentedPipeline):
     @property
     def solid_selection(self) -> Optional[Sequence[str]]:
         return (
-            self._pipeline_index.pipeline_snapshot.lineage_snapshot.solid_selection
+            self._pipeline_index.pipeline_snapshot.lineage_snapshot.node_selection
             if self._pipeline_index.pipeline_snapshot.lineage_snapshot
             else None
         )
@@ -367,7 +375,7 @@ class ExternalPipeline(RepresentedPipeline):
     @property
     def solids_to_execute(self) -> Optional[AbstractSet[str]]:
         return (
-            self._pipeline_index.pipeline_snapshot.lineage_snapshot.solids_to_execute
+            self._pipeline_index.pipeline_snapshot.lineage_snapshot.nodes_to_execute
             if self._pipeline_index.pipeline_snapshot.lineage_snapshot
             else None
         )
@@ -386,7 +394,7 @@ class ExternalPipeline(RepresentedPipeline):
 
     @property
     def solid_names(self) -> Sequence[str]:
-        return self._pipeline_index.pipeline_snapshot.solid_names
+        return self._pipeline_index.pipeline_snapshot.node_names
 
     def has_solid_invocation(self, solid_name: str):
         check.str_param(solid_name, "solid_name")
@@ -422,7 +430,7 @@ class ExternalPipeline(RepresentedPipeline):
         return self._pipeline_index.pipeline_snapshot.tags
 
     @property
-    def metadata(self) -> Sequence[MetadataEntry]:
+    def metadata(self) -> Mapping[str, MetadataValue]:
         return self._pipeline_index.pipeline_snapshot.metadata
 
     @property
@@ -453,8 +461,7 @@ class ExternalPipeline(RepresentedPipeline):
 
 
 class ExternalExecutionPlan:
-    """
-    ExternalExecution is a object that represents an execution plan that
+    """ExternalExecution is a object that represents an execution plan that
     was compiled in another process or persisted in an instance.
     """
 
@@ -541,8 +548,7 @@ class ExternalExecutionPlan:
 
 
 class ExternalResource:
-    """
-    Represents a top-level resource in a repository, e.g. one passed through the Definitions API.
+    """Represents a top-level resource in a repository, e.g. one passed through the Definitions API.
     """
 
     def __init__(self, external_resource_data: ExternalResourceData, handle: RepositoryHandle):
@@ -566,13 +572,37 @@ class ExternalResource:
         return self._external_resource_data.config_field_snaps
 
     @property
-    def configured_values(self) -> Dict[str, str]:
+    def configured_values(self) -> Dict[str, ExternalResourceValue]:
         return self._external_resource_data.configured_values
 
     @property
     def config_schema_snap(self) -> ConfigSchemaSnapshot:
         return self._external_resource_data.config_schema_snap
 
+    @property
+    def nested_resources(self) -> Dict[str, NestedResource]:
+        return self._external_resource_data.nested_resources
+
+    @property
+    def parent_resources(self) -> Dict[str, str]:
+        return self._external_resource_data.parent_resources
+
+    @property
+    def resource_type(self) -> str:
+        return self._external_resource_data.resource_type
+
+    @property
+    def is_top_level(self) -> bool:
+        return self._external_resource_data.is_top_level
+
+    @property
+    def asset_keys_using(self) -> List[AssetKey]:
+        return self._external_resource_data.asset_keys_using
+
+    @property
+    def job_ops_using(self) -> List[ResourceJobUsageEntry]:
+        return self._external_resource_data.job_ops_using
+
 
 class ExternalSchedule:
     def __init__(self, external_schedule_data: ExternalScheduleData, handle: RepositoryHandle):
@@ -879,9 +909,9 @@ class ExternalPartitionSet:
         # names
         return self._external_partition_set_data.external_partitions_data is not None
 
-    def get_partition_names(self) -> Sequence[str]:
+    def get_partition_names(self, instance: DagsterInstance) -> Sequence[str]:
         check.invariant(self.has_partition_name_data())
         partitions = (
             self._external_partition_set_data.external_partitions_data.get_partitions_definition()  # type: ignore
         )
-        return partitions.get_partition_keys()
+        return partitions.get_partition_keys(dynamic_partitions_store=instance)
