@@ -6,31 +6,24 @@ from dagster._config import ConfigSchemaSnapshot
 from dagster._core.snap.dagster_types import DagsterTypeSnap
 from dagster._core.snap.dep_snapshot import DependencyStructureIndex
 from dagster._core.snap.mode import ModeDefSnap
+from dagster._core.snap.node import GraphDefSnap, OpDefSnap
 from dagster._core.snap.pipeline_snapshot import PipelineSnapshot
-from dagster._core.snap.solid import CompositeSolidDefSnap, SolidDefSnap
 
 from .pipeline_index import PipelineIndex
 
 
 class RepresentedPipeline(ABC):
-    """
-    RepresentedPipeline is a base class for ExternalPipeline or HistoricalPipeline.
+    """RepresentedPipeline is a base class for ExternalPipeline or HistoricalPipeline.
 
     The name is "represented" because this is an in-memory representation of a pipeline.
     The representation of a pipeline could be referring to a pipeline resident in
     another process *or* could be referring to a historical view of the pipeline.
     """
 
-    _pipeline_index: PipelineIndex
-
-    def __init__(self, pipeline_index):
-        self._pipeline_index = check.inst_param(pipeline_index, "pipeline_index", PipelineIndex)
-
-    # Temporary method to allow for incrementally
-    # replacing pipeline index with the representation hierarchy
-    # Chosen for grepability
-    def get_pipeline_index_for_compat(self) -> PipelineIndex:
-        return self._pipeline_index
+    @property
+    @abstractmethod
+    def _pipeline_index(self) -> PipelineIndex:
+        ...
 
     @property
     def name(self) -> str:
@@ -63,7 +56,7 @@ class RepresentedPipeline(ABC):
     @property
     def solid_selection(self) -> Optional[Sequence[str]]:
         return (
-            self._pipeline_index.pipeline_snapshot.lineage_snapshot.solid_selection
+            self._pipeline_index.pipeline_snapshot.lineage_snapshot.node_selection
             if self._pipeline_index.pipeline_snapshot.lineage_snapshot
             else None
         )
@@ -71,7 +64,7 @@ class RepresentedPipeline(ABC):
     @property
     def solids_to_execute(self) -> Optional[AbstractSet[str]]:
         return (
-            self._pipeline_index.pipeline_snapshot.lineage_snapshot.solids_to_execute
+            self._pipeline_index.pipeline_snapshot.lineage_snapshot.nodes_to_execute
             if self._pipeline_index.pipeline_snapshot.lineage_snapshot
             else None
         )
@@ -110,7 +103,7 @@ class RepresentedPipeline(ABC):
         return self._pipeline_index.dep_structure_index
 
     # Solids
-    def get_node_def_snap(self, solid_def_name: str) -> Union[SolidDefSnap, CompositeSolidDefSnap]:
+    def get_node_def_snap(self, solid_def_name: str) -> Union[OpDefSnap, GraphDefSnap]:
         check.str_param(solid_def_name, "solid_def_name")
         return self._pipeline_index.get_node_def_snap(solid_def_name)
 
