@@ -1,15 +1,13 @@
 from abc import ABC, abstractmethod
 from typing import NamedTuple, Optional
 
-from dagster._core.instance import MayHaveInstanceWeakref
+from dagster._core.instance import MayHaveInstanceWeakref, T_DagsterInstance
 from dagster._core.storage.pipeline_run import DagsterRun
 from dagster._core.workspace.context import IWorkspace, WorkspaceRequestContext
 
 
 class SubmitRunContext(NamedTuple):
-    """
-    Context available within a run coordinator's submit_run method
-    """
+    """Context available within a run coordinator's submit_run method."""
 
     pipeline_run: DagsterRun
     workspace: IWorkspace
@@ -27,11 +25,10 @@ class SubmitRunContext(NamedTuple):
         return None
 
 
-class RunCoordinator(ABC, MayHaveInstanceWeakref):
+class RunCoordinator(ABC, MayHaveInstanceWeakref[T_DagsterInstance]):
     @abstractmethod
     def submit_run(self, context: SubmitRunContext) -> DagsterRun:
-        """
-        Submit a run to the run coordinator for execution.
+        """Submit a run to the run coordinator for execution.
 
         Args:
             context (SubmitRunContext): information about the submission - every run coordinator
@@ -43,16 +40,14 @@ class RunCoordinator(ABC, MayHaveInstanceWeakref):
         """
 
     @abstractmethod
-    def cancel_run(self, run_id):
-        """
-        Cancels a run. The run may be queued in the coordinator, or it may have been launched.
+    def cancel_run(self, run_id: str) -> bool:
+        """Cancels a run. The run may be queued in the coordinator, or it may have been launched.
 
         Returns False is the process was already canceled. Returns true if the cancellation was
         successful.
         """
 
-    def dispose(self):
-        """
-        Do any resource cleanup that should happen when the DagsterInstance is
+    def dispose(self) -> None:
+        """Do any resource cleanup that should happen when the DagsterInstance is
         cleaning itself up.
         """
