@@ -1,6 +1,10 @@
 import logging
+from typing import Mapping, Optional
+
+from typing_extensions import Self
 
 import dagster._check as check
+from dagster._config.config_schema import UserConfigSchema
 from dagster._core.storage.pipeline_run import DagsterRun, DagsterRunStatus
 from dagster._serdes import ConfigurableClass, ConfigurableClassData
 
@@ -10,21 +14,23 @@ from .base import RunCoordinator, SubmitRunContext
 class DefaultRunCoordinator(RunCoordinator, ConfigurableClass):
     """Immediately send runs to the run launcher."""
 
-    def __init__(self, inst_data=None):
+    def __init__(self, inst_data: Optional[ConfigurableClassData] = None):
         self._inst_data = check.opt_inst_param(inst_data, "inst_data", ConfigurableClassData)
         self._logger = logging.getLogger("dagster.run_coordinator.default_run_coordinator")
         super().__init__()
 
     @property
-    def inst_data(self):
+    def inst_data(self) -> Optional[ConfigurableClassData]:
         return self._inst_data
 
     @classmethod
-    def config_type(cls):
+    def config_type(cls) -> UserConfigSchema:
         return {}
 
     @classmethod
-    def from_config_value(cls, inst_data, config_value):
+    def from_config_value(
+        cls, inst_data: Optional[ConfigurableClassData], config_value: Mapping[str, object]
+    ) -> Self:
         return cls(inst_data=inst_data, **config_value)
 
     def submit_run(self, context: SubmitRunContext) -> DagsterRun:
@@ -43,5 +49,5 @@ class DefaultRunCoordinator(RunCoordinator, ConfigurableClass):
             check.failed(f"Failed to reload run {pipeline_run.run_id}")
         return run
 
-    def cancel_run(self, run_id):
+    def cancel_run(self, run_id: str) -> bool:
         return self._instance.run_launcher.terminate(run_id)
