@@ -1,26 +1,28 @@
 from enum import Enum
-from inspect import Parameter
-from typing import Any, Dict, List, Mapping, NamedTuple, Optional, Sequence, Type, Union
+from typing import List, NamedTuple, Optional, Sequence, Union
+
+from typing_extensions import TypeAlias
 
 import dagster._check as check
-from dagster._core.definitions.run_request import InstigatorType
+
+# re-export
+from dagster._core.definitions.run_request import (
+    InstigatorType as InstigatorType,
+    SkipReason as SkipReason,
+)
+from dagster._core.definitions.selector import InstigatorSelector, RepositorySelector
 from dagster._core.host_representation.origin import ExternalInstigatorOrigin
-from dagster._core.host_representation.selector import InstigatorSelector, RepositorySelector
 from dagster._serdes import create_snapshot_id
 from dagster._serdes.serdes import (
-    DefaultNamedTupleSerializer,
-    WhitelistMap,
-    register_serdes_enum_fallbacks,
-    register_serdes_tuple_fallbacks,
-    replace_storage_keys,
-    unpack_inner_value,
     whitelist_for_serdes,
 )
 from dagster._utils.error import SerializableErrorInfo
 from dagster._utils.merger import merge_dicts
 
+InstigatorData: TypeAlias = Union["ScheduleInstigatorData", "SensorInstigatorData"]
 
-@whitelist_for_serdes
+
+@whitelist_for_serdes(old_storage_names={"JobStatus"})
 class InstigatorStatus(Enum):
     # User has taken some action to start the run instigator
     RUNNING = "RUNNING"
@@ -31,12 +33,7 @@ class InstigatorStatus(Enum):
     STOPPED = "STOPPED"
 
 
-register_serdes_enum_fallbacks({"JobStatus": InstigatorStatus})
-# for internal backcompat
-JobStatus = InstigatorStatus
-
-
-@whitelist_for_serdes
+@whitelist_for_serdes(old_storage_names={"SensorJobData"})
 class SensorInstigatorData(
     NamedTuple(
         "_SensorInstigatorData",
@@ -70,12 +67,7 @@ class SensorInstigatorData(
         )
 
 
-register_serdes_tuple_fallbacks({"SensorJobData": SensorInstigatorData})
-# for internal backcompat
-SensorJobData = SensorInstigatorData
-
-
-@whitelist_for_serdes
+@whitelist_for_serdes(old_storage_names={"ScheduleJobData"})
 class ScheduleInstigatorData(
     NamedTuple(
         "_ScheduleInstigatorData",
@@ -86,7 +78,7 @@ class ScheduleInstigatorData(
     def __new__(
         cls, cron_schedule: Union[str, Sequence[str]], start_timestamp: Optional[float] = None
     ):
-        cron_schedule = check.inst_param(cron_schedule, "cron_schedule", (str, Sequence))
+        cron_schedule = check.inst_param(cron_schedule, "cron_schedule", (str, list))
         if not isinstance(cron_schedule, str):
             cron_schedule = check.sequence_param(cron_schedule, "cron_schedule", of_type=str)
 
@@ -100,15 +92,10 @@ class ScheduleInstigatorData(
         )
 
 
-register_serdes_tuple_fallbacks({"ScheduleJobData": ScheduleInstigatorData})
-# for internal backcompat
-ScheduleJobData = ScheduleInstigatorData
-
-
 def check_instigator_data(
     instigator_type: InstigatorType,
-    instigator_data: Optional[Union[ScheduleInstigatorData, SensorInstigatorData]],
-):
+    instigator_data: Optional[InstigatorData],
+) -> Optional[InstigatorData]:
     if instigator_type == InstigatorType.SCHEDULE:
         check.inst_param(instigator_data, "instigator_data", ScheduleInstigatorData)
     elif instigator_type == InstigatorType.SENSOR:
@@ -122,55 +109,13 @@ def check_instigator_data(
     return instigator_data
 
 
-class InstigatorStateSerializer(DefaultNamedTupleSerializer):
-    @classmethod
-    def value_from_storage_dict(
-        cls,
-        storage_dict: Dict[str, Any],
-        klass: Type,
-        args_for_class: Mapping[str, Parameter],
-        whitelist_map: WhitelistMap,
-        descent_path: str,
-    ) -> NamedTuple:
-        klass_kwargs = {}
-        for key, value in storage_dict.items():
-            unpacked = unpack_inner_value(value, whitelist_map, f"{descent_path}.{key}")
-            if key in args_for_class:
-                klass_kwargs[key] = unpacked
-            elif key == "job_type":
-                # For backcompat, we store instigator_type as job_type
-                klass_kwargs["instigator_type"] = unpacked
-            elif key == "job_specific_data":
-                # For backcompat, we store instigator_data as job_specific_data
-                klass_kwargs["instigator_data"] = unpacked
-
-        return klass(**klass_kwargs)
-
-    @classmethod
-    def value_to_storage_dict(
-        cls,
-        value: NamedTuple,
-        whitelist_map: WhitelistMap,
-        descent_path: str,
-    ) -> Dict[str, Any]:
-        storage = super().value_to_storage_dict(
-            value,
-            whitelist_map,
-            descent_path,
-        )
-        # For backcompat, we store:
-        # instigator_type as job_type
-        # instigator_data as job_specific_data
-        return replace_storage_keys(
-            storage,
-            {
-                "instigator_type": "job_type",
-                "instigator_data": "job_specific_data",
-            },
-        )
-
-
-@whitelist_for_serdes(serializer=InstigatorStateSerializer)
+@whitelist_for_serdes(
+    old_storage_names={"JobState"},
+    storage_field_names={
+        "instigator_type": "job_type",
+        "instigator_data": "job_specific_data",
+    },
+)
 class InstigatorState(
     NamedTuple(
         "_InstigationState",
@@ -178,7 +123,7 @@ class InstigatorState(
             ("origin", ExternalInstigatorOrigin),
             ("instigator_type", InstigatorType),
             ("status", InstigatorStatus),
-            ("instigator_data", Optional[Union[ScheduleInstigatorData, SensorInstigatorData]]),
+            ("instigator_data", Optional[InstigatorData]),
         ],
     )
 ):
@@ -187,7 +132,7 @@ class InstigatorState(
         origin: ExternalInstigatorOrigin,
         instigator_type: InstigatorType,
         status: InstigatorStatus,
-        instigator_data: Optional[Union[ScheduleInstigatorData, SensorInstigatorData]] = None,
+        instigator_data: Optional[InstigatorData] = None,
     ):
         return super(InstigatorState, cls).__new__(
             cls,
@@ -198,47 +143,47 @@ class InstigatorState(
         )
 
     @property
-    def is_running(self):
+    def is_running(self) -> bool:
         return self.status != InstigatorStatus.STOPPED
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self.origin.instigator_name
 
     @property
-    def instigator_name(self):
+    def instigator_name(self) -> str:
         return self.origin.instigator_name
 
     @property
-    def repository_origin_id(self):
+    def repository_origin_id(self) -> str:
         return self.origin.external_repository_origin.get_id()
 
     @property
     def repository_selector(self) -> RepositorySelector:
         return RepositorySelector(
-            self.origin.external_repository_origin.repository_location_origin.location_name,
+            self.origin.external_repository_origin.code_location_origin.location_name,
             self.origin.external_repository_origin.repository_name,
         )
 
     @property
-    def repository_selector_id(self):
+    def repository_selector_id(self) -> str:
         return create_snapshot_id(self.repository_selector)
 
     @property
-    def instigator_origin_id(self):
+    def instigator_origin_id(self) -> str:
         return self.origin.get_id()
 
     @property
-    def selector_id(self):
+    def selector_id(self) -> str:
         return create_snapshot_id(
             InstigatorSelector(
-                self.origin.external_repository_origin.repository_location_origin.location_name,
+                self.origin.external_repository_origin.code_location_origin.location_name,
                 self.origin.external_repository_origin.repository_name,
                 self.origin.instigator_name,
             )
         )
 
-    def with_status(self, status):
+    def with_status(self, status: InstigatorStatus) -> "InstigatorState":
         check.inst_param(status, "status", InstigatorStatus)
         return InstigatorState(
             self.origin,
@@ -247,7 +192,7 @@ class InstigatorState(
             instigator_data=self.instigator_data,
         )
 
-    def with_data(self, instigator_data):
+    def with_data(self, instigator_data: InstigatorData) -> "InstigatorState":
         check_instigator_data(self.instigator_type, instigator_data)
         return InstigatorState(
             self.origin,
@@ -257,12 +202,7 @@ class InstigatorState(
         )
 
 
-register_serdes_tuple_fallbacks({"JobState": InstigatorState})
-# for internal backcompat
-JobState = InstigatorState
-
-
-@whitelist_for_serdes
+@whitelist_for_serdes(old_storage_names={"JobTickStatus"})
 class TickStatus(Enum):
     STARTED = "STARTED"
     SKIPPED = "SKIPPED"
@@ -270,55 +210,9 @@ class TickStatus(Enum):
     FAILURE = "FAILURE"
 
 
-register_serdes_enum_fallbacks({"JobTickStatus": TickStatus})
-# for internal backcompat
-JobTickStatus = TickStatus
-
-
-class TickSerializer(DefaultNamedTupleSerializer):
-    @classmethod
-    def value_from_storage_dict(
-        cls,
-        storage_dict: Dict[str, Any],
-        klass: Type,
-        args_for_class: Mapping[str, Parameter],
-        whitelist_map: WhitelistMap,
-        descent_path: str,
-    ) -> NamedTuple:
-        klass_kwargs = {}
-        for key, value in storage_dict.items():
-            unpacked = unpack_inner_value(value, whitelist_map, f"{descent_path}.{key}")
-            if key in args_for_class:
-                klass_kwargs[key] = unpacked
-            elif key == "job_tick_data":
-                # For backcompat, we store tick_data as job_tick_data
-                klass_kwargs["tick_data"] = unpacked
-
-        return klass(**klass_kwargs)
-
-    @classmethod
-    def value_to_storage_dict(
-        cls,
-        value: NamedTuple,
-        whitelist_map: WhitelistMap,
-        descent_path: str,
-    ) -> Dict[str, Any]:
-        storage = super().value_to_storage_dict(
-            value,
-            whitelist_map,
-            descent_path,
-        )
-        # For backcompat, we store:
-        # tick_data as job_tick_data
-        return replace_storage_keys(
-            storage,
-            {
-                "tick_data": "job_tick_data",
-            },
-        )
-
-
-@whitelist_for_serdes(serializer=TickSerializer)
+@whitelist_for_serdes(
+    old_storage_names={"JobTick"}, storage_field_names={"tick_data": "job_tick_data"}
+)
 class InstigatorTick(NamedTuple("_InstigatorTick", [("tick_id", int), ("tick_data", "TickData")])):
     def __new__(cls, tick_id: int, tick_data: "TickData"):
         return super(InstigatorTick, cls).__new__(
@@ -327,72 +221,72 @@ class InstigatorTick(NamedTuple("_InstigatorTick", [("tick_id", int), ("tick_dat
             check.inst_param(tick_data, "tick_data", TickData),
         )
 
-    def with_status(self, status, **kwargs):
+    def with_status(self, status: TickStatus, **kwargs: object):
         check.inst_param(status, "status", TickStatus)
         return self._replace(tick_data=self.tick_data.with_status(status, **kwargs))
 
-    def with_reason(self, skip_reason):
+    def with_reason(self, skip_reason: str) -> "InstigatorTick":
         check.opt_str_param(skip_reason, "skip_reason")
         return self._replace(tick_data=self.tick_data.with_reason(skip_reason))
 
-    def with_run_info(self, run_id=None, run_key=None):
+    def with_run_info(self, run_id: Optional[str] = None, run_key: Optional[str] = None):
         return self._replace(tick_data=self.tick_data.with_run_info(run_id, run_key))
 
-    def with_cursor(self, cursor):
+    def with_cursor(self, cursor: Optional[str]) -> "InstigatorTick":
         return self._replace(tick_data=self.tick_data.with_cursor(cursor))
 
-    def with_origin_run(self, origin_run_id):
+    def with_origin_run(self, origin_run_id: str) -> "InstigatorTick":
         return self._replace(tick_data=self.tick_data.with_origin_run(origin_run_id))
 
-    def with_log_key(self, log_key):
+    def with_log_key(self, log_key: Sequence[str]) -> "InstigatorTick":
         return self._replace(tick_data=self.tick_data.with_log_key(log_key))
 
     @property
-    def instigator_origin_id(self):
+    def instigator_origin_id(self) -> str:
         return self.tick_data.instigator_origin_id
 
     @property
-    def selector_id(self):
+    def selector_id(self) -> Optional[str]:
         return self.tick_data.selector_id
 
     @property
-    def instigator_name(self):
+    def instigator_name(self) -> str:
         return self.tick_data.instigator_name
 
     @property
-    def instigator_type(self):
+    def instigator_type(self) -> InstigatorType:
         return self.tick_data.instigator_type
 
     @property
-    def timestamp(self):
+    def timestamp(self) -> float:
         return self.tick_data.timestamp
 
     @property
-    def status(self):
+    def status(self) -> TickStatus:
         return self.tick_data.status
 
     @property
-    def run_ids(self):
+    def run_ids(self) -> Sequence[str]:
         return self.tick_data.run_ids
 
     @property
-    def run_keys(self):
+    def run_keys(self) -> Sequence[str]:
         return self.tick_data.run_keys
 
     @property
-    def error(self):
+    def error(self) -> Optional[SerializableErrorInfo]:
         return self.tick_data.error
 
     @property
-    def skip_reason(self):
+    def skip_reason(self) -> Optional[str]:
         return self.tick_data.skip_reason
 
     @property
-    def cursor(self):
+    def cursor(self) -> Optional[str]:
         return self.tick_data.cursor
 
     @property
-    def origin_run_ids(self):
+    def origin_run_ids(self) -> Optional[Sequence[str]]:
         return self.tick_data.origin_run_ids
 
     @property
@@ -420,61 +314,14 @@ class InstigatorTick(NamedTuple("_InstigatorTick", [("tick_id", int), ("tick_dat
         return self.tick_data.status == TickStatus.SUCCESS
 
 
-register_serdes_tuple_fallbacks({"JobTick": InstigatorTick})
-# for internal backcompat
-JobTick = InstigatorTick
-
-
-class TickDataSerializer(DefaultNamedTupleSerializer):
-    @classmethod
-    def value_from_storage_dict(
-        cls,
-        storage_dict: Dict[str, Any],
-        klass: Type,
-        args_for_class: Mapping[str, Parameter],
-        whitelist_map: WhitelistMap,
-        descent_path: str,
-    ) -> NamedTuple:
-        klass_kwargs = {}
-        for key, value in storage_dict.items():
-            unpacked = unpack_inner_value(value, whitelist_map, f"{descent_path}.{key}")
-            if key in args_for_class:
-                klass_kwargs[key] = unpacked
-            elif key == "job_origin_id":
-                # For backcompat, we store instigator_origin_id as job_origin_id
-                klass_kwargs["instigator_origin_id"] = unpacked
-            elif key == "job_name":
-                # For backcompat, we store instigator_name as job_name
-                klass_kwargs["instigator_name"] = unpacked
-            elif key == "job_type":
-                # For backcompat, we store instigator_type as job_type
-                klass_kwargs["instigator_type"] = unpacked
-
-        return klass(**klass_kwargs)
-
-    @classmethod
-    def value_to_storage_dict(
-        cls,
-        value: NamedTuple,
-        whitelist_map: WhitelistMap,
-        descent_path: str,
-    ) -> Dict[str, Any]:
-        storage = super().value_to_storage_dict(
-            value,
-            whitelist_map,
-            descent_path,
-        )
-        return replace_storage_keys(
-            storage,
-            {
-                "instigator_origin_id": "job_origin_id",
-                "instigator_name": "job_name",
-                "instigator_type": "job_type",
-            },
-        )
-
-
-@whitelist_for_serdes(serializer=TickDataSerializer)
+@whitelist_for_serdes(
+    old_storage_names={"JobTickData"},
+    storage_field_names={
+        "instigator_origin_id": "job_origin_id",
+        "instigator_name": "job_name",
+        "instigator_type": "job_type",
+    },
+)
 class TickData(
     NamedTuple(
         "_TickData",
@@ -496,8 +343,7 @@ class TickData(
         ],
     )
 ):
-    """
-    This class defines the data that is serialized and stored for each schedule/sensor tick. We
+    """This class defines the data that is serialized and stored for each schedule/sensor tick. We
     depend on the storage implementation to provide tick ids, and therefore separate all other
     data into this serializable class that can be stored independently of the id.
 
@@ -553,7 +399,13 @@ class TickData(
             log_key=log_key,
         )
 
-    def with_status(self, status, error=None, timestamp=None, failure_count=None):
+    def with_status(
+        self,
+        status: TickStatus,
+        error: Optional[SerializableErrorInfo] = None,
+        timestamp: Optional[float] = None,
+        failure_count: Optional[int] = None,
+    ) -> "TickData":
         return TickData(
             **merge_dicts(
                 self._asdict(),
@@ -568,7 +420,9 @@ class TickData(
             )
         )
 
-    def with_run_info(self, run_id=None, run_key=None):
+    def with_run_info(
+        self, run_id: Optional[str] = None, run_key: Optional[str] = None
+    ) -> "TickData":
         check.opt_str_param(run_id, "run_id")
         check.opt_str_param(run_key, "run_key")
         return TickData(
@@ -589,7 +443,7 @@ class TickData(
             )
         )
 
-    def with_failure_count(self, failure_count):
+    def with_failure_count(self, failure_count: int) -> "TickData":
         return TickData(
             **merge_dicts(
                 self._asdict(),
@@ -599,19 +453,19 @@ class TickData(
             )
         )
 
-    def with_reason(self, skip_reason):
+    def with_reason(self, skip_reason: Optional[str]) -> "TickData":
         return TickData(
             **merge_dicts(
                 self._asdict(), {"skip_reason": check.opt_str_param(skip_reason, "skip_reason")}
             )
         )
 
-    def with_cursor(self, cursor):
+    def with_cursor(self, cursor: Optional[str]) -> "TickData":
         return TickData(
             **merge_dicts(self._asdict(), {"cursor": check.opt_str_param(cursor, "cursor")})
         )
 
-    def with_origin_run(self, origin_run_id):
+    def with_origin_run(self, origin_run_id: str) -> "TickData":
         check.str_param(origin_run_id, "origin_run_id")
         return TickData(
             **merge_dicts(
@@ -620,7 +474,7 @@ class TickData(
             )
         )
 
-    def with_log_key(self, log_key):
+    def with_log_key(self, log_key: Sequence[str]) -> "TickData":
         return TickData(
             **merge_dicts(
                 self._asdict(),
@@ -629,12 +483,13 @@ class TickData(
         )
 
 
-register_serdes_tuple_fallbacks({"JobTickData": TickData})
-# for internal backcompat
-JobTickData = TickData
-
-
-def _validate_tick_args(instigator_type, status, run_ids=None, error=None, skip_reason=None):
+def _validate_tick_args(
+    instigator_type: InstigatorType,
+    status: TickStatus,
+    run_ids: Optional[Sequence[str]] = None,
+    error: Optional[SerializableErrorInfo] = None,
+    skip_reason: Optional[str] = None,
+) -> None:
     check.inst_param(instigator_type, "instigator_type", InstigatorType)
     check.inst_param(status, "status", TickStatus)
 
