@@ -1,7 +1,7 @@
 import dagster._check as check
 from dagster._config import ConfigSchemaSnapshot, snap_from_config_type
 from dagster._core.definitions.pipeline_definition import PipelineDefinition
-from dagster._utils import merge_dicts
+from dagster._utils.merger import merge_dicts
 
 
 def build_config_schema_snapshot(pipeline_def):
