@@ -1,4 +1,4 @@
-from typing import AbstractSet, Any, Mapping, NamedTuple, Optional, Sequence, Set, Union, cast
+from typing import AbstractSet, Any, Dict, Mapping, NamedTuple, Optional, Sequence, Union, cast
 
 from dagster import _check as check
 from dagster._config import (
@@ -24,7 +24,10 @@ from dagster._config import (
 from dagster._core.definitions.events import AssetKey
 from dagster._core.definitions.job_definition import JobDefinition
 from dagster._core.definitions.metadata import (
-    MetadataEntry,
+    MetadataFieldSerializer,
+    MetadataValue,
+    RawMetadataValue,
+    normalize_metadata,
 )
 from dagster._core.definitions.pipeline_definition import (
     PipelineDefinition,
@@ -32,25 +35,24 @@ from dagster._core.definitions.pipeline_definition import (
 )
 from dagster._core.utils import toposort_flatten
 from dagster._serdes import (
-    DefaultNamedTupleSerializer,
     create_snapshot_id,
     deserialize_value,
-    unpack_inner_value,
     whitelist_for_serdes,
 )
+from dagster._serdes.serdes import NamedTupleSerializer
 
 from .config_types import build_config_schema_snapshot
 from .dagster_types import DagsterTypeNamespaceSnapshot, build_dagster_type_namespace_snapshot
 from .dep_snapshot import (
     DependencyStructureSnapshot,
-    build_dep_structure_snapshot_from_icontains_solids,
+    build_dep_structure_snapshot_from_graph_def,
 )
 from .mode import ModeDefSnap, build_mode_def_snap
-from .solid import (
-    CompositeSolidDefSnap,
-    SolidDefinitionsSnapshot,
-    SolidDefSnap,
-    build_solid_definitions_snapshot,
+from .node import (
+    GraphDefSnap,
+    NodeDefsSnapshot,
+    OpDefSnap,
+    build_node_defs_snapshot,
 )
 
 
@@ -59,78 +61,40 @@ def create_pipeline_snapshot_id(snapshot: "PipelineSnapshot") -> str:
     return create_snapshot_id(snapshot)
 
 
-class PipelineSnapshotSerializer(DefaultNamedTupleSerializer):
-    @classmethod
-    def skip_when_empty(cls) -> Set[str]:
-        return {"metadata"}  # Maintain stable snapshot ID for back-compat purposes
-
-    @classmethod
-    def value_from_storage_dict(
-        cls,
-        storage_dict,
-        klass,
-        args_for_class,
-        whitelist_map,
-        descent_path,
-    ):
-        # unpack all stored fields
-        unpacked_dict = {
-            key: unpack_inner_value(value, whitelist_map, f"{descent_path}.{key}")
-            for key, value in storage_dict.items()
-        }
-        # called by the serdes layer, delegates to helper method with expanded kwargs
-        return _pipeline_snapshot_from_storage(**unpacked_dict)
+class PipelineSnapshotSerializer(NamedTupleSerializer["PipelineSnapshot"]):
+    # v0
+    # v1:
+    #     - lineage added
+    # v2:
+    #     - graph_def_name
+    # v3:
+    #     - metadata added
+    # v4:
+    #     - add kwargs so that if future versions add new args, this version of deserialization will
+    #     be able to ignore them. previously, new args would be passed to old versions and cause
+    #     deserialization errors.
+    def before_unpack(
+        self,
+        **packed: Any,
+    ) -> Dict[str, Any]:
+        if packed.get("graph_def_name") is None:
+            packed["graph_def_name"] = packed["name"]
+        if packed.get("metadata") is None:
+            packed["metadata"] = {}
+        if packed.get("lineage_snapshot") is None:
+            packed["lineage_snapshot"] = None
+        return packed
 
 
-def _pipeline_snapshot_from_storage(
-    name: str,
-    description: Optional[str],
-    tags: Optional[Mapping[str, Any]],
-    config_schema_snapshot: ConfigSchemaSnapshot,
-    dagster_type_namespace_snapshot: DagsterTypeNamespaceSnapshot,
-    solid_definitions_snapshot: SolidDefinitionsSnapshot,
-    dep_structure_snapshot: DependencyStructureSnapshot,
-    mode_def_snaps: Sequence[ModeDefSnap],
-    lineage_snapshot: Optional["PipelineSnapshotLineage"] = None,
-    graph_def_name: Optional[str] = None,
-    metadata: Optional[Sequence[MetadataEntry]] = None,
-    **kwargs,  # pylint: disable=unused-argument
-) -> "PipelineSnapshot":
-    """
-    v0
-    v1:
-        - lineage added
-    v2:
-        - graph_def_name
-    v3:
-        - metadata added
-    v4:
-        - add kwargs so that if future versions add new args, this version of deserialization will
-        be able to ignore them. previously, new args would be passed to old versions and cause
-        deserialization errors.
-    """
-    if graph_def_name is None:
-        graph_def_name = name
-
-    if metadata is None:
-        metadata = []
-
-    return PipelineSnapshot(
-        name=name,
-        description=description,
-        tags=tags,
-        metadata=metadata,
-        config_schema_snapshot=config_schema_snapshot,
-        dagster_type_namespace_snapshot=dagster_type_namespace_snapshot,
-        solid_definitions_snapshot=solid_definitions_snapshot,
-        dep_structure_snapshot=dep_structure_snapshot,
-        mode_def_snaps=mode_def_snaps,
-        lineage_snapshot=lineage_snapshot,
-        graph_def_name=graph_def_name,
-    )
-
-
-@whitelist_for_serdes(serializer=PipelineSnapshotSerializer)
+# Note that unlike other serdes-whitelisted objects that hold metadata, the field here has always
+# been called `metadata` instead of `metadata_entries`, so we don't need to rename the field for
+# serialization.
+@whitelist_for_serdes(
+    serializer=PipelineSnapshotSerializer,
+    skip_when_empty_fields={"metadata"},
+    field_serializers={"metadata": MetadataFieldSerializer},
+    storage_field_names={"node_defs_snapshot": "solid_definitions_snapshot"},
+)
 class PipelineSnapshot(
     NamedTuple(
         "_PipelineSnapshot",
@@ -140,12 +104,12 @@ class PipelineSnapshot(
             ("tags", Mapping[str, Any]),
             ("config_schema_snapshot", ConfigSchemaSnapshot),
             ("dagster_type_namespace_snapshot", DagsterTypeNamespaceSnapshot),
-            ("solid_definitions_snapshot", SolidDefinitionsSnapshot),
+            ("node_defs_snapshot", NodeDefsSnapshot),
             ("dep_structure_snapshot", DependencyStructureSnapshot),
             ("mode_def_snaps", Sequence[ModeDefSnap]),
             ("lineage_snapshot", Optional["PipelineSnapshotLineage"]),
             ("graph_def_name", str),
-            ("metadata", Sequence[MetadataEntry]),
+            ("metadata", Mapping[str, MetadataValue]),
         ],
     )
 ):
@@ -156,12 +120,12 @@ class PipelineSnapshot(
         tags: Optional[Mapping[str, Any]],
         config_schema_snapshot: ConfigSchemaSnapshot,
         dagster_type_namespace_snapshot: DagsterTypeNamespaceSnapshot,
-        solid_definitions_snapshot: SolidDefinitionsSnapshot,
+        node_defs_snapshot: NodeDefsSnapshot,
         dep_structure_snapshot: DependencyStructureSnapshot,
         mode_def_snaps: Sequence[ModeDefSnap],
         lineage_snapshot: Optional["PipelineSnapshotLineage"],
         graph_def_name: str,
-        metadata: Optional[Sequence[MetadataEntry]],
+        metadata: Optional[Mapping[str, RawMetadataValue]],
     ):
         return super(PipelineSnapshot, cls).__new__(
             cls,
@@ -176,8 +140,8 @@ class PipelineSnapshot(
                 "dagster_type_namespace_snapshot",
                 DagsterTypeNamespaceSnapshot,
             ),
-            solid_definitions_snapshot=check.inst_param(
-                solid_definitions_snapshot, "solid_definitions_snapshot", SolidDefinitionsSnapshot
+            node_defs_snapshot=check.inst_param(
+                node_defs_snapshot, "node_defs_snapshot", NodeDefsSnapshot
             ),
             dep_structure_snapshot=check.inst_param(
                 dep_structure_snapshot, "dep_structure_snapshot", DependencyStructureSnapshot
@@ -189,7 +153,9 @@ class PipelineSnapshot(
                 lineage_snapshot, "lineage_snapshot", PipelineSnapshotLineage
             ),
             graph_def_name=check.str_param(graph_def_name, "graph_def_name"),
-            metadata=check.opt_sequence_param(metadata, "metadata"),
+            metadata=normalize_metadata(
+                check.opt_mapping_param(metadata, "metadata", key_type=str)
+            ),
         )
 
     @classmethod
@@ -201,16 +167,16 @@ class PipelineSnapshot(
                 parent_snapshot_id=create_pipeline_snapshot_id(
                     cls.from_pipeline_def(pipeline_def.parent_pipeline_def)
                 ),
-                solid_selection=sorted(pipeline_def.solid_selection),
-                solids_to_execute=pipeline_def.solids_to_execute,
+                node_selection=sorted(pipeline_def.node_selection),
+                nodes_to_execute=pipeline_def.nodes_to_execute,
             )
         if isinstance(pipeline_def, JobDefinition) and pipeline_def.op_selection_data:
             lineage = PipelineSnapshotLineage(
                 parent_snapshot_id=create_pipeline_snapshot_id(
                     cls.from_pipeline_def(pipeline_def.op_selection_data.parent_job_def)
                 ),
-                solid_selection=sorted(pipeline_def.op_selection_data.op_selection),
-                solids_to_execute=pipeline_def.op_selection_data.resolved_op_selection,
+                node_selection=sorted(pipeline_def.op_selection_data.op_selection),
+                nodes_to_execute=pipeline_def.op_selection_data.resolved_op_selection,
             )
         if isinstance(pipeline_def, JobDefinition) and pipeline_def.asset_selection_data:
             lineage = PipelineSnapshotLineage(
@@ -227,10 +193,8 @@ class PipelineSnapshot(
             metadata=pipeline_def.metadata,
             config_schema_snapshot=build_config_schema_snapshot(pipeline_def),
             dagster_type_namespace_snapshot=build_dagster_type_namespace_snapshot(pipeline_def),
-            solid_definitions_snapshot=build_solid_definitions_snapshot(pipeline_def),
-            dep_structure_snapshot=build_dep_structure_snapshot_from_icontains_solids(
-                pipeline_def.graph
-            ),
+            node_defs_snapshot=build_node_defs_snapshot(pipeline_def),
+            dep_structure_snapshot=build_dep_structure_snapshot_from_graph_def(pipeline_def.graph),
             mode_def_snaps=[
                 build_mode_def_snap(md, pipeline_def.get_run_config_schema(md.name).config_type.key)
                 for md in pipeline_def.mode_definitions
@@ -239,32 +203,32 @@ class PipelineSnapshot(
             graph_def_name=pipeline_def.graph.name,
         )
 
-    def get_node_def_snap(self, solid_def_name: str) -> Union[SolidDefSnap, CompositeSolidDefSnap]:
-        check.str_param(solid_def_name, "solid_def_name")
-        for solid_def_snap in self.solid_definitions_snapshot.solid_def_snaps:
-            if solid_def_snap.name == solid_def_name:
-                return solid_def_snap
+    def get_node_def_snap(self, node_def_name: str) -> Union[OpDefSnap, GraphDefSnap]:
+        check.str_param(node_def_name, "node_def_name")
+        for node_def_snap in self.node_defs_snapshot.op_def_snaps:
+            if node_def_snap.name == node_def_name:
+                return node_def_snap
 
-        for comp_solid_def_snap in self.solid_definitions_snapshot.composite_solid_def_snaps:
-            if comp_solid_def_snap.name == solid_def_name:
-                return comp_solid_def_snap
+        for graph_def_snap in self.node_defs_snapshot.graph_def_snaps:
+            if graph_def_snap.name == node_def_name:
+                return graph_def_snap
 
         check.failed("not found")
 
-    def has_solid_name(self, solid_name: str) -> bool:
-        check.str_param(solid_name, "solid_name")
-        for solid_snap in self.dep_structure_snapshot.solid_invocation_snaps:
-            if solid_snap.solid_name == solid_name:
+    def has_node_name(self, node_name: str) -> bool:
+        check.str_param(node_name, "node_name")
+        for node_invocation_snap in self.dep_structure_snapshot.node_invocation_snaps:
+            if node_invocation_snap.node_name == node_name:
                 return True
         return False
 
-    def get_config_type_from_solid_def_snap(
+    def get_config_type_from_node_def_snap(
         self,
-        solid_def_snap: Union[SolidDefSnap, CompositeSolidDefSnap],
+        node_def_snap: Union[OpDefSnap, GraphDefSnap],
     ) -> Optional[ConfigType]:
-        check.inst_param(solid_def_snap, "solid_def_snap", (SolidDefSnap, CompositeSolidDefSnap))
-        if solid_def_snap.config_field_snap:
-            config_type_key = solid_def_snap.config_field_snap.type_key
+        check.inst_param(node_def_snap, "node_def_snap", (OpDefSnap, GraphDefSnap))
+        if node_def_snap.config_field_snap:
+            config_type_key = node_def_snap.config_field_snap.type_key
             if self.config_schema_snapshot.has_config_snap(config_type_key):
                 return construct_config_type_from_snap(
                     self.config_schema_snapshot.get_config_snap(config_type_key),
@@ -273,18 +237,18 @@ class PipelineSnapshot(
         return None
 
     @property
-    def solid_names(self) -> Sequence[str]:
-        return [ss.solid_name for ss in self.dep_structure_snapshot.solid_invocation_snaps]
+    def node_names(self) -> Sequence[str]:
+        return [ss.node_name for ss in self.dep_structure_snapshot.node_invocation_snaps]
 
     @property
-    def solid_names_in_topological_order(self) -> Sequence[str]:
+    def node_names_in_topological_order(self) -> Sequence[str]:
         upstream_outputs = {}
 
-        for solid_invocation_snap in self.dep_structure_snapshot.solid_invocation_snaps:
-            solid_name = solid_invocation_snap.solid_name
-            upstream_outputs[solid_name] = {
-                upstream_output_snap.solid_name
-                for input_dep_snap in solid_invocation_snap.input_dep_snaps
+        for node_invocation_snap in self.dep_structure_snapshot.node_invocation_snaps:
+            node_name = node_invocation_snap.node_name
+            upstream_outputs[node_name] = {
+                upstream_output_snap.node_name
+                for input_dep_snap in node_invocation_snap.input_dep_snaps
                 for upstream_output_snap in input_dep_snap.upstream_output_snaps
             }
 
@@ -444,17 +408,22 @@ def construct_config_type_from_snap(
         return _construct_map_from_snap(config_type_snap, config_snap_map)
     elif config_type_snap.kind == ConfigTypeKind.NONEABLE:
         return _construct_noneable_from_snap(config_type_snap, config_snap_map)
-    check.failed("Could not evaluate config type snap kind: {}".format(config_type_snap.kind))
+    check.failed(f"Could not evaluate config type snap kind: {config_type_snap.kind}")
 
 
-@whitelist_for_serdes
+@whitelist_for_serdes(
+    storage_field_names={
+        "node_selection": "solid_selection",
+        "nodes_to_execute": "solids_to_execute",
+    },
+)
 class PipelineSnapshotLineage(
     NamedTuple(
         "_PipelineSnapshotLineage",
         [
             ("parent_snapshot_id", str),
-            ("solid_selection", Optional[Sequence[str]]),
-            ("solids_to_execute", Optional[AbstractSet[str]]),
+            ("node_selection", Optional[Sequence[str]]),
+            ("nodes_to_execute", Optional[AbstractSet[str]]),
             ("asset_selection", Optional[AbstractSet[AssetKey]]),
         ],
     )
@@ -462,15 +431,15 @@ class PipelineSnapshotLineage(
     def __new__(
         cls,
         parent_snapshot_id: str,
-        solid_selection: Optional[Sequence[str]] = None,
-        solids_to_execute: Optional[AbstractSet[str]] = None,
+        node_selection: Optional[Sequence[str]] = None,
+        nodes_to_execute: Optional[AbstractSet[str]] = None,
         asset_selection: Optional[AbstractSet[AssetKey]] = None,
     ):
-        check.opt_set_param(solids_to_execute, "solids_to_execute", of_type=str)
+        check.opt_set_param(nodes_to_execute, "nodes_to_execute", of_type=str)
         return super(PipelineSnapshotLineage, cls).__new__(
             cls,
             check.str_param(parent_snapshot_id, parent_snapshot_id),
-            solid_selection,
-            solids_to_execute,
+            node_selection,
+            nodes_to_execute,
             asset_selection,
         )
