@@ -1,4 +1,4 @@
-"""scheduler update
+"""scheduler update.
 
 Revision ID: 5c18fd3c2957
 Revises: c39c047fa021
@@ -6,12 +6,10 @@ Create Date: 2020-06-10 10:00:57.793622
 
 """
 from alembic import op
+from dagster._core.storage.migration.utils import get_currently_upgrading_instance, has_table
 from sqlalchemy import inspect
 
-from dagster._core.storage.migration.utils import get_currently_upgrading_instance, has_table
-
 # alembic magic breaks pylint
-# pylint: disable=no-member
 
 
 # revision identifiers, used by Alembic.
