@@ -1,4 +1,4 @@
-"""create run_id idx
+"""create run_id idx.
 
 Revision ID: c34498c29964
 Revises: 07f83cc13695
@@ -8,8 +8,6 @@ Create Date: 2020-06-11 10:40:25.216776
 from alembic import op
 from sqlalchemy import inspect
 
-# pylint: disable=no-member
-
 # revision identifiers, used by Alembic.
 revision = "c34498c29964"
 down_revision = "07f83cc13695"
@@ -22,7 +20,7 @@ def upgrade():
     has_tables = inspector.get_table_names()
     if "event_logs" in has_tables:
         indices = [x.get("name") for x in inspector.get_indexes("event_logs")]
-        if not "idx_run_id" in indices:
+        if "idx_run_id" not in indices:
             op.create_index("idx_run_id", "event_logs", ["run_id"], unique=False)
 
 
