@@ -1,4 +1,4 @@
-"""change varchar to text
+"""change varchar to text.
 
 Revision ID: 4ea2b1f6f67b
 Revises: 8ccbed5060b8
@@ -7,11 +7,8 @@ Create Date: 2021-01-14 12:39:53.493651
 """
 import sqlalchemy as sa
 from alembic import op
-
 from dagster._core.storage.migration.utils import has_table
 
-# pylint: disable=no-member
-
 # revision identifiers, used by Alembic.
 revision = "4ea2b1f6f67b"
 down_revision = "8ccbed5060b8"
