@@ -1,4 +1,4 @@
-"""change varchar to text
+"""change varchar to text.
 
 Revision ID: 0da417ae1b81
 Revises: 4ea2b1f6f67b
@@ -7,11 +7,8 @@ Create Date: 2021-01-14 12:29:33.410870
 """
 import sqlalchemy as sa
 from alembic import op
-
 from dagster._core.storage.migration.utils import has_table
 
-# pylint: disable=no-member
-
 # revision identifiers, used by Alembic.
 revision = "0da417ae1b81"
 down_revision = "4ea2b1f6f67b"
