@@ -1,4 +1,4 @@
-"""extract asset_keys index columns
+"""extract asset_keys index columns.
 
 Revision ID: 1a2d72f6b24e
 Revises: 7f2b1a4ca7a5
