@@ -1,4 +1,4 @@
-"""add kvs table
+"""add kvs table.
 
 Revision ID: 5e139331e376
 Revises: 6860f830e40c
@@ -7,8 +7,7 @@ Create Date: 2022-06-06 15:48:51.559562
 """
 import sqlalchemy as db
 from alembic import op
-
-from dagster._core.storage.migration.utils import has_column, has_index, has_table
+from dagster._core.storage.migration.utils import has_index, has_table
 
 # revision identifiers, used by Alembic.
 revision = "5e139331e376"
