@@ -1,27 +1,28 @@
 import logging
 from collections import defaultdict
 from contextlib import contextmanager
-from typing import Callable
+from typing import Callable, Optional, cast
 
+from sqlalchemy.engine import Engine
 from sqlalchemy.pool import NullPool
 
 from dagster._core.storage.event_log.base import EventLogCursor
 from dagster._core.storage.sql import create_engine, get_alembic_config, stamp_alembic_rev
 from dagster._core.storage.sqlite import create_in_memory_conn_string
 from dagster._serdes import ConfigurableClass
+from dagster._serdes.config_class import ConfigurableClassData
 
 from .schema import SqlEventLogStorageMetadata
 from .sql_event_log import SqlEventLogStorage
 
 
 class InMemoryEventLogStorage(SqlEventLogStorage, ConfigurableClass):
-    """
-    In memory only event log storage. Used by ephemeral DagsterInstance or for testing purposes.
+    """In memory only event log storage. Used by ephemeral DagsterInstance or for testing purposes.
 
     WARNING: Dagit and other core functionality will not work if this is used on a real DagsterInstance
     """
 
-    def __init__(self, inst_data=None, preload=None):
+    def __init__(self, inst_data: Optional[ConfigurableClassData] = None, preload=None):
         self._inst_data = inst_data
         self._engine = None
         self._conn = None
@@ -62,7 +63,8 @@ class InMemoryEventLogStorage(SqlEventLogStorage, ConfigurableClass):
     def has_table(self, table_name: str) -> bool:
         if not self._conn:
             self._create_connection()
-        return bool(self._engine.dialect.has_table(self._conn, table_name))
+        engine = cast(Engine, self._engine)
+        return bool(engine.dialect.has_table(self._conn, table_name))
 
     @property
     def inst_data(self):
