@@ -2,8 +2,10 @@ import logging
 import os
 from collections import defaultdict
 from contextlib import contextmanager
+from typing import Any, Mapping, Optional
 
 from sqlalchemy.pool import NullPool
+from typing_extensions import Self
 from watchdog.events import PatternMatchingEventHandler
 from watchdog.observers import Observer
 
@@ -23,7 +25,7 @@ from dagster._serdes import ConfigurableClass, ConfigurableClassData
 from dagster._utils import mkdir_p
 
 from ..schema import SqlEventLogStorageMetadata
-from ..sql_event_log import SqlEventLogStorage
+from ..sql_event_log import SqlDbConnection, SqlEventLogStorage
 
 SQLITE_EVENT_LOG_FILENAME = "event_log"
 
@@ -49,7 +51,7 @@ class ConsolidatedSqliteEventLogStorage(SqlEventLogStorage, ConfigurableClass):
     The ``base_dir`` param tells the event log storage where on disk to store the database.
     """
 
-    def __init__(self, base_dir, inst_data=None):
+    def __init__(self, base_dir, inst_data: Optional[ConfigurableClassData] = None):
         self._base_dir = check.str_param(base_dir, "base_dir")
         self._conn_string = create_db_conn_string(base_dir, SQLITE_EVENT_LOG_FILENAME)
         self._secondary_index_cache = {}
@@ -70,8 +72,10 @@ class ConsolidatedSqliteEventLogStorage(SqlEventLogStorage, ConfigurableClass):
     def config_type(cls):
         return {"base_dir": StringSource}
 
-    @staticmethod
-    def from_config_value(inst_data, config_value):
+    @classmethod
+    def from_config_value(
+        cls, inst_data: ConfigurableClassData, config_value: Mapping[str, Any]
+    ) -> Self:
         return ConsolidatedSqliteEventLogStorage(inst_data=inst_data, **config_value)
 
     def _init_db(self):
@@ -102,7 +106,7 @@ class ConsolidatedSqliteEventLogStorage(SqlEventLogStorage, ConfigurableClass):
         finally:
             conn.close()
 
-    def run_connection(self, run_id):
+    def run_connection(self, run_id: Optional[str]) -> SqlDbConnection:
         return self._connect()
 
     def index_connection(self):
@@ -113,7 +117,7 @@ class ConsolidatedSqliteEventLogStorage(SqlEventLogStorage, ConfigurableClass):
         return bool(engine.dialect.has_table(engine.connect(), table_name))
 
     def get_db_path(self):
-        return os.path.join(self._base_dir, "{}.db".format(SQLITE_EVENT_LOG_FILENAME))
+        return os.path.join(self._base_dir, f"{SQLITE_EVENT_LOG_FILENAME}.db")
 
     def upgrade(self):
         alembic_config = get_alembic_config(__file__)
