@@ -24,9 +24,7 @@ InputLoadFn: TypeAlias = Union[
 
 
 class InputManager(ABC):
-    """
-    Base interface for classes that are responsible for loading solid inputs.
-    """
+    """Base interface for classes that are responsible for loading solid inputs."""
 
     @abstractmethod
     def load_input(self, context: "InputContext") -> object:
@@ -241,6 +239,7 @@ class _InputManagerDecoratorCallable:
             required_resource_keys=self.required_resource_keys,
         )
 
-        update_wrapper(root_input_manager_def, wrapped=load_fn)
+        # `update_wrapper` typing cannot currently handle a Union of Callables correctly
+        update_wrapper(root_input_manager_def, wrapped=load_fn)  # type: ignore
 
         return root_input_manager_def
