@@ -82,9 +82,9 @@ class LocalComputeLogManager(CapturedLogManager, ComputeLogManager, Configurable
             "polling_timeout": Field(Float, is_required=False),
         }
 
-    @staticmethod
+    @classmethod
     def from_config_value(
-        inst_data: Optional[ConfigurableClassData], config_value
+        cls, inst_data: Optional[ConfigurableClassData], config_value
     ) -> "LocalComputeLogManager":
         return LocalComputeLogManager(inst_data=inst_data, **config_value)
 
@@ -315,7 +315,7 @@ class LocalComputeLogManager(CapturedLogManager, ComputeLogManager, Configurable
 
     def download_url(self, run_id: str, key: str, io_type: ComputeIOType):
         check.inst_param(io_type, "io_type", ComputeIOType)
-        return "/download/{}/{}/{}".format(run_id, key, io_type.value)
+        return f"/download/{run_id}/{key}/{io_type.value}"
 
     def on_subscribe(self, subscription: "LogSubscription") -> None:
         self._subscription_manager.add_subscription(subscription)
