@@ -1,10 +1,12 @@
 from contextlib import contextmanager
+from typing import Iterator, Sequence
 
 import sqlalchemy as db
 from alembic import op
 from sqlalchemy import inspect
 
 import dagster._check as check
+from dagster._core.instance import DagsterInstance
 from dagster._core.storage.sql import get_current_timestamp
 
 
@@ -12,22 +14,22 @@ def get_inspector():
     return inspect(op.get_bind())
 
 
-def get_table_names():
+def get_table_names() -> Sequence[str]:
     return get_inspector().get_table_names()
 
 
-def has_table(table_name):
+def has_table(table_name: str) -> bool:
     return table_name in get_table_names()
 
 
-def has_column(table_name, column_name):
+def has_column(table_name: str, column_name: str) -> bool:
     if not has_table(table_name):
         return False
     columns = [x.get("name") for x in get_inspector().get_columns(table_name)]
     return column_name in columns
 
 
-def has_index(table_name, index_name):
+def has_index(table_name: str, index_name: str) -> bool:
     if not has_table(table_name):
         return False
     indexes = [x.get("name") for x in get_inspector().get_indexes(table_name)]
@@ -42,30 +44,30 @@ _UPGRADING_INSTANCE = None
 
 
 @contextmanager
-def upgrading_instance(instance):
-    global _UPGRADING_INSTANCE  # pylint: disable=global-statement,global-variable-not-assigned
+def upgrading_instance(instance: DagsterInstance) -> Iterator[None]:
+    global _UPGRADING_INSTANCE  # noqa: PLW0603
     check.invariant(_UPGRADING_INSTANCE is None, "update already in progress")
     try:
-        _UPGRADING_INSTANCE = instance
+        _UPGRADING_INSTANCE = instance  # noqa: PLW0603
         yield
     finally:
         _UPGRADING_INSTANCE = None
 
 
-def get_currently_upgrading_instance():
+def get_currently_upgrading_instance() -> DagsterInstance:
     global _UPGRADING_INSTANCE  # noqa: PLW0602
     check.invariant(_UPGRADING_INSTANCE is not None, "currently upgrading instance not set")
-    return _UPGRADING_INSTANCE
+    return _UPGRADING_INSTANCE  # type: ignore  # (possible none)
 
 
 # alembic magic breaks pylint
-# pylint: disable=no-member
+
 
 # These intentionally use the schema at the time of the 0.10.0 release, to be used
 # during the 0.10.0 new tables migration
 
 
-def create_0_10_0_run_tables():
+def create_0_10_0_run_tables() -> None:
     if not has_table("runs"):
         return
 
@@ -88,7 +90,7 @@ def create_0_10_0_run_tables():
         )
 
 
-def create_0_10_0_event_log_tables():
+def create_0_10_0_event_log_tables() -> None:
     if not has_table("event_logs"):
         return
 
@@ -110,7 +112,7 @@ def create_0_10_0_event_log_tables():
         )
 
 
-def create_0_10_0_schedule_tables():
+def create_0_10_0_schedule_tables() -> None:
     if not has_table("schedules") and not has_table("jobs"):
         return
 
@@ -153,7 +155,7 @@ def create_0_10_0_schedule_tables():
         op.drop_table("schedule_ticks")
 
 
-def create_bulk_actions_table():
+def create_bulk_actions_table() -> None:
     if not has_table("runs"):
         return
 
@@ -171,7 +173,7 @@ def create_bulk_actions_table():
         op.create_index("idx_bulk_actions_status", "bulk_actions", ["status"], unique=False)
 
 
-def add_asset_materialization_columns():
+def add_asset_materialization_columns() -> None:
     if not has_table("asset_keys"):
         return
 
@@ -182,7 +184,7 @@ def add_asset_materialization_columns():
     op.add_column("asset_keys", db.Column("last_run_id", db.String(255)))
 
 
-def add_asset_details_column():
+def add_asset_details_column() -> None:
     if not has_table("asset_keys"):
         return
 
@@ -192,7 +194,7 @@ def add_asset_details_column():
     op.add_column("asset_keys", db.Column("asset_details", db.Text))
 
 
-def extract_asset_keys_idx_columns():
+def extract_asset_keys_idx_columns() -> None:
     if not has_table("asset_keys"):
         return
 
@@ -205,7 +207,7 @@ def extract_asset_keys_idx_columns():
     op.add_column("asset_keys", db.Column("tags", db.TEXT))
 
 
-def create_event_log_event_idx():
+def create_event_log_event_idx() -> None:
     if not has_table("event_logs"):
         return
 
@@ -220,7 +222,7 @@ def create_event_log_event_idx():
     )
 
 
-def create_run_range_indices():
+def create_run_range_indices() -> None:
     if not has_table("runs"):
         return
 
@@ -240,7 +242,7 @@ def create_run_range_indices():
     )
 
 
-def add_run_record_start_end_timestamps():
+def add_run_record_start_end_timestamps() -> None:
     if not has_table("runs"):
         return
 
@@ -251,7 +253,7 @@ def add_run_record_start_end_timestamps():
     op.add_column("runs", db.Column("end_time", db.Float))
 
 
-def drop_run_record_start_end_timestamps():
+def drop_run_record_start_end_timestamps() -> None:
     if not has_table("runs"):
         return
 
@@ -262,7 +264,7 @@ def drop_run_record_start_end_timestamps():
     op.drop_column("runs", "end_time")
 
 
-def create_schedule_secondary_index_table():
+def create_schedule_secondary_index_table() -> None:
     if not has_table("jobs"):
         return
 
@@ -276,7 +278,7 @@ def create_schedule_secondary_index_table():
         )
 
 
-def create_instigators_table():
+def create_instigators_table() -> None:
     if not has_table("instigators") and not has_table("jobs"):
         # not a schedule storage db
         return
@@ -301,7 +303,7 @@ def create_instigators_table():
         op.add_column("job_ticks", db.Column("selector_id", db.String(255)))
 
 
-def create_tick_selector_index():
+def create_tick_selector_index() -> None:
     if not has_table("job_ticks"):
         return
 
@@ -313,7 +315,7 @@ def create_tick_selector_index():
     )
 
 
-def add_id_based_event_indices():
+def add_id_based_event_indices() -> None:
     if not has_table("event_logs"):
         return
 
@@ -365,7 +367,7 @@ def add_id_based_event_indices():
             )
 
 
-def drop_id_based_event_indices():
+def drop_id_based_event_indices() -> None:
     if not has_table("event_logs"):
         return
 
@@ -413,7 +415,7 @@ def drop_id_based_event_indices():
             )
 
 
-def add_cached_status_data_column():
+def add_cached_status_data_column() -> None:
     if not has_table("asset_keys"):
         return
 
@@ -423,7 +425,7 @@ def add_cached_status_data_column():
     op.add_column("asset_keys", db.Column("cached_status_data", db.Text))
 
 
-def add_run_job_index():
+def add_run_job_index() -> None:
     if not has_table("runs"):
         return
 
@@ -440,7 +442,7 @@ def add_run_job_index():
         )
 
 
-def drop_run_job_index():
+def drop_run_job_index() -> None:
     if not has_table("runs"):
         return
 
