@@ -1,4 +1,4 @@
-from typing import Dict, List, NamedTuple, Optional, Sequence, Set, Tuple, cast
+from typing import Dict, List, NamedTuple, Optional, Sequence, Set, Tuple
 
 from dagster import (
     AssetKey,
@@ -26,15 +26,34 @@ from dagster._core.storage.tags import (
     MULTIDIMENSIONAL_PARTITION_PREFIX,
     get_dimension_from_partition_tag,
 )
-from dagster._serdes import deserialize_json_to_dagster_namedtuple, whitelist_for_serdes
+from dagster._serdes import whitelist_for_serdes
+from dagster._serdes.errors import DeserializationError
+from dagster._serdes.serdes import deserialize_value
 
 CACHEABLE_PARTITION_TYPES = (
     TimeWindowPartitionsDefinition,
     MultiPartitionsDefinition,
     StaticPartitionsDefinition,
+    DynamicPartitionsDefinition,
 )
 
 
+def is_cacheable_partition_type(partitions_def: PartitionsDefinition) -> bool:
+    check.inst_param(partitions_def, "partitions_def", PartitionsDefinition)
+    if not isinstance(partitions_def, CACHEABLE_PARTITION_TYPES):
+        return False
+    if isinstance(partitions_def, MultiPartitionsDefinition):
+        return all(
+            is_cacheable_partition_type(dimension_def.partitions_def)
+            for dimension_def in partitions_def.partitions_defs
+        )
+    return (
+        partitions_def.name is not None
+        if isinstance(partitions_def, DynamicPartitionsDefinition)
+        else True
+    )
+
+
 @whitelist_for_serdes
 class AssetStatusCacheValue(
     NamedTuple(
@@ -44,12 +63,12 @@ class AssetStatusCacheValue(
             ("partitions_def_id", Optional[str]),
             ("serialized_materialized_partition_subset", Optional[str]),
             ("serialized_failed_partition_subset", Optional[str]),
+            ("serialized_in_progress_partition_subset", Optional[str]),
             ("earliest_in_progress_materialization_event_id", Optional[int]),
         ],
     )
 ):
-    """
-    Set of asset fields that reflect partition materialization status. This is used to display
+    """Set of asset fields that reflect partition materialization status. This is used to display
     global partition status in the asset view.
 
     Properties:
@@ -62,6 +81,8 @@ class AssetStatusCacheValue(
             unpartitioned.
         serialized_failed_partition_subset (Optional(str)): The serialized representation of the failed
             partition subsets, up to the latest storage id. None if the asset is unpartitioned.
+        serialized_in_progress_partition_subset (Optional(str)): The serialized representation of the
+            in progress partition subsets, up to the latest storage id. None if the asset is unpartitioned.
         earliest_in_progress_materialization_event_id (Optional(int)): The event id of the earliest
             materialization planned event for a run that is still in progress. This is used to check
             on the status of runs that are still in progress.
@@ -73,17 +94,19 @@ class AssetStatusCacheValue(
         partitions_def_id: Optional[str] = None,
         serialized_materialized_partition_subset: Optional[str] = None,
         serialized_failed_partition_subset: Optional[str] = None,
+        serialized_in_progress_partition_subset: Optional[str] = None,
         earliest_in_progress_materialization_event_id: Optional[int] = None,
     ):
         check.int_param(latest_storage_id, "latest_storage_id")
         check.opt_str_param(partitions_def_id, "partitions_def_id")
-        check.opt_inst_param(
-            serialized_materialized_partition_subset,
-            "serialized_materialized_partition_subset",
-            str,
+        check.opt_str_param(
+            serialized_materialized_partition_subset, "serialized_materialized_partition_subset"
         )
-        check.opt_inst_param(
-            serialized_failed_partition_subset, "serialized_failed_partition_subset", str
+        check.opt_str_param(
+            serialized_failed_partition_subset, "serialized_failed_partition_subset"
+        )
+        check.opt_str_param(
+            serialized_in_progress_partition_subset, "serialized_in_progress_partition_subset"
         )
         return super(AssetStatusCacheValue, cls).__new__(
             cls,
@@ -91,16 +114,18 @@ class AssetStatusCacheValue(
             partitions_def_id,
             serialized_materialized_partition_subset,
             serialized_failed_partition_subset,
+            serialized_in_progress_partition_subset,
             earliest_in_progress_materialization_event_id,
         )
 
     @staticmethod
-    def from_db_string(db_string):
+    def from_db_string(db_string: str) -> Optional["AssetStatusCacheValue"]:
         if not db_string:
             return None
 
-        cached_data = deserialize_json_to_dagster_namedtuple(db_string)
-        if not isinstance(cached_data, AssetStatusCacheValue):
+        try:
+            cached_data = deserialize_value(db_string, AssetStatusCacheValue)
+        except DeserializationError:
             return None
 
         return cached_data
@@ -121,6 +146,14 @@ class AssetStatusCacheValue(
 
         return partitions_def.deserialize_subset(self.serialized_failed_partition_subset)
 
+    def deserialize_in_progress_partition_subsets(
+        self, partitions_def: PartitionsDefinition
+    ) -> PartitionsSubset:
+        if not self.serialized_in_progress_partition_subset:
+            return partitions_def.empty_subset()
+
+        return partitions_def.deserialize_subset(self.serialized_in_progress_partition_subset)
+
 
 def get_materialized_multipartitions(
     instance: DagsterInstance, asset_key: AssetKey, partitions_def: MultiPartitionsDefinition
@@ -162,22 +195,9 @@ def get_validated_partition_keys(
             & partition_keys
         )
     elif isinstance(partitions_def, MultiPartitionsDefinition):
-        partition_keys_by_dimension = {
-            dim.name: dim.partitions_def.get_partition_keys(
-                dynamic_partitions_store=dynamic_partitions_store
-            )
-            for dim in partitions_def.partitions_defs
-        }
-        validated_partitions = set()
-        for partition_key in partition_keys:
-            multipartition_key = partitions_def.get_partition_key_from_str(partition_key)
-            if all(
-                key in partition_keys_by_dimension.get(dim, [])
-                for dim, key in cast(
-                    MultiPartitionKey, multipartition_key
-                ).keys_by_dimension.items()
-            ):
-                validated_partitions.add(partition_key)
+        validated_partitions = partitions_def.filter_valid_partition_keys(
+            partition_keys, dynamic_partitions_store
+        )
     else:
         if not isinstance(partitions_def, TimeWindowPartitionsDefinition):
             check.failed("Unexpected partitions definition type {partitions_def}")
@@ -192,12 +212,12 @@ def _build_status_cache(
     asset_key: AssetKey,
     latest_storage_id: int,
     partitions_def: Optional[PartitionsDefinition],
+    dynamic_partitions_store: DynamicPartitionsStore,
 ) -> AssetStatusCacheValue:
-    """
-    This method refreshes the asset status cache for a given asset key. It recalculates
+    """This method refreshes the asset status cache for a given asset key. It recalculates
     the materialized partition subset for the asset key and updates the cache value.
     """
-    if not partitions_def or not isinstance(partitions_def, CACHEABLE_PARTITION_TYPES):
+    if not partitions_def or not is_cacheable_partition_type(partitions_def):
         return AssetStatusCacheValue(latest_storage_id=latest_storage_id)
 
     materialized_keys: Sequence[str]
@@ -216,30 +236,40 @@ def _build_status_cache(
 
     serialized_materialized_partition_subset = (
         serialized_materialized_partition_subset.with_partition_keys(
-            get_validated_partition_keys(instance, partitions_def, set(materialized_keys))
+            get_validated_partition_keys(
+                dynamic_partitions_store, partitions_def, set(materialized_keys)
+            )
         )
     )
 
-    failed_subset, cursor = _build_failed_partition_subset(instance, asset_key, partitions_def)
+    failed_subset, in_progress_subset, cursor = build_failed_and_in_progress_partition_subset(
+        instance, asset_key, partitions_def, dynamic_partitions_store
+    )
 
     return AssetStatusCacheValue(
         latest_storage_id=latest_storage_id,
-        partitions_def_id=partitions_def.serializable_unique_identifier,
+        partitions_def_id=partitions_def.get_serializable_unique_identifier(
+            dynamic_partitions_store=dynamic_partitions_store
+        ),
         serialized_materialized_partition_subset=serialized_materialized_partition_subset.serialize(),
         serialized_failed_partition_subset=failed_subset.serialize(),
+        serialized_in_progress_partition_subset=in_progress_subset.serialize(),
         earliest_in_progress_materialization_event_id=cursor,
     )
 
 
-def _build_failed_partition_subset(
-    instance: DagsterInstance, asset_key: AssetKey, partitions_def: PartitionsDefinition
-) -> Tuple[PartitionsSubset, Optional[int]]:
+def build_failed_and_in_progress_partition_subset(
+    instance: DagsterInstance,
+    asset_key: AssetKey,
+    partitions_def: PartitionsDefinition,
+    dynamic_partitions_store: DynamicPartitionsStore,
+) -> Tuple[PartitionsSubset, PartitionsSubset, Optional[int]]:
     incomplete_materializations = instance.event_log_storage.get_latest_asset_partition_materialization_attempts_without_materializations(
         asset_key
     )
 
     if not incomplete_materializations:
-        return partitions_def.empty_subset(), None
+        return partitions_def.empty_subset(), partitions_def.empty_subset(), None
 
     finished_runs = {
         r.run_id: r.status
@@ -252,6 +282,7 @@ def _build_failed_partition_subset(
     }
 
     new_failed_partitions = set()
+    in_progress_partitions = set()
     cursor = None
     for partition, (run_id, event_id) in incomplete_materializations.items():
         if run_id in finished_runs:
@@ -259,27 +290,36 @@ def _build_failed_partition_subset(
             if status == DagsterRunStatus.FAILURE:
                 new_failed_partitions.add(partition)
         else:
+            in_progress_partitions.add(partition)
             # If the run is not finished, keep track of the event id so we can check on it next time
             if cursor is None or event_id < cursor:
                 cursor = event_id
 
     return (
         partitions_def.empty_subset().with_partition_keys(
-            get_validated_partition_keys(instance, partitions_def, new_failed_partitions)
+            get_validated_partition_keys(
+                dynamic_partitions_store, partitions_def, new_failed_partitions
+            )
         )
         if new_failed_partitions
         else partitions_def.empty_subset(),
+        partitions_def.empty_subset().with_partition_keys(
+            get_validated_partition_keys(instance, partitions_def, in_progress_partitions)
+        )
+        if in_progress_partitions
+        else partitions_def.empty_subset(),
         cursor,
     )
 
 
-def _get_updated_failed_partition_subset(
+def _get_updated_failed_and_in_progress_partition_subset(
     instance: DagsterInstance,
     asset_key: AssetKey,
     partitions_def: PartitionsDefinition,
     current_cached_subset: PartitionsSubset,
     unevaluated_event_records: Sequence[EventLogRecord],
-) -> Tuple[PartitionsSubset, Optional[int]]:
+    dynamic_partitions_store: DynamicPartitionsStore,
+) -> Tuple[PartitionsSubset, PartitionsSubset, Optional[int]]:
     current_failed_partitions = set(current_cached_subset.get_partition_keys())
 
     cursor = None
@@ -299,6 +339,7 @@ def _get_updated_failed_partition_subset(
                 current_failed_partitions.discard(event.partition)
 
     new_failed_partitions = set()
+    in_progress_partitions = set()
     if incomplete_materialization_records:
         finished_runs = {
             r.run_id: r.status
@@ -317,6 +358,7 @@ def _get_updated_failed_partition_subset(
                 if finished_runs[record.run_id] == DagsterRunStatus.FAILURE:
                     new_failed_partitions.add(partition)
             else:
+                in_progress_partitions.add(partition)
                 if cursor is None or record.storage_id <= cursor:
                     # If the run is not finished, keep track of the event id so we can check on it next time
                     cursor = record.storage_id
@@ -327,6 +369,9 @@ def _get_updated_failed_partition_subset(
                 instance, partitions_def, new_failed_partitions | current_failed_partitions
             )
         ),
+        partitions_def.empty_subset().with_partition_keys(
+            get_validated_partition_keys(instance, partitions_def, in_progress_partitions)
+        ),
         cursor,
     )
 
@@ -336,9 +381,9 @@ def _get_updated_status_cache(
     asset_key: AssetKey,
     current_status_cache_value: AssetStatusCacheValue,
     partitions_def: Optional[PartitionsDefinition],
+    dynamic_partitions_store: DynamicPartitionsStore,
 ) -> AssetStatusCacheValue:
-    """
-    This method accepts the current asset status cache value, and fetches unevaluated
+    """This method accepts the current asset status cache value, and fetches unevaluated
     records from the event log. It then updates the cache value with the new materializations.
     """
     # if earliest_in_progress_materialization_event_id is set, we fetch all events including the
@@ -371,12 +416,14 @@ def _get_updated_status_cache(
     unevaluated_event_records.extend(list(unevaluated_materialization_event_records))
 
     latest_storage_id = max([record.storage_id for record in unevaluated_event_records])
-    if not partitions_def or not isinstance(partitions_def, CACHEABLE_PARTITION_TYPES):
+    if not partitions_def or not is_cacheable_partition_type(partitions_def):
         return AssetStatusCacheValue(latest_storage_id=latest_storage_id)
 
     check.invariant(
         current_status_cache_value.partitions_def_id
-        == partitions_def.serializable_unique_identifier
+        == partitions_def.get_serializable_unique_identifier(
+            dynamic_partitions_store=dynamic_partitions_store
+        )
     )
     materialized_subset: PartitionsSubset = (
         partitions_def.deserialize_subset(
@@ -399,7 +446,9 @@ def _get_updated_status_cache(
             check.failed("Expected materialization or materialization planned event")
 
     materialized_subset = materialized_subset.with_partition_keys(
-        get_validated_partition_keys(instance, partitions_def, newly_materialized_partitions)
+        get_validated_partition_keys(
+            dynamic_partitions_store, partitions_def, newly_materialized_partitions
+        )
     )
 
     failed_subset: PartitionsSubset = (
@@ -411,12 +460,17 @@ def _get_updated_status_cache(
         else partitions_def.empty_subset()
     )
 
-    failed_subset, new_cursor = _get_updated_failed_partition_subset(
+    (
+        failed_subset,
+        in_progress_subset,
+        new_cursor,
+    ) = _get_updated_failed_and_in_progress_partition_subset(
         instance,
         asset_key,
         partitions_def,
         failed_subset,
         unevaluated_event_records,
+        dynamic_partitions_store=dynamic_partitions_store,
     )
 
     return AssetStatusCacheValue(
@@ -424,6 +478,7 @@ def _get_updated_status_cache(
         partitions_def_id=current_status_cache_value.partitions_def_id,
         serialized_materialized_partition_subset=materialized_subset.serialize(),
         serialized_failed_partition_subset=failed_subset.serialize(),
+        serialized_in_progress_partition_subset=in_progress_subset.serialize(),
         earliest_in_progress_materialization_event_id=new_cursor,
     )
 
@@ -445,13 +500,18 @@ def _fetch_stored_asset_status_cache_value(
 def _get_fresh_asset_status_cache_value(
     instance: DagsterInstance,
     asset_key: AssetKey,
+    dynamic_partitions_store: DynamicPartitionsStore,
     partitions_def: Optional[PartitionsDefinition] = None,
 ) -> Optional[AssetStatusCacheValue]:
     cached_status_data = _fetch_stored_asset_status_cache_value(instance, asset_key)
 
     updated_cache_value = None
     if cached_status_data is None or cached_status_data.partitions_def_id != (
-        partitions_def.serializable_unique_identifier if partitions_def else None
+        partitions_def.get_serializable_unique_identifier(
+            dynamic_partitions_store=dynamic_partitions_store
+        )
+        if partitions_def
+        else None
     ):
         planned_event_records = instance.get_event_records(
             event_records_filter=EventRecordsFilter(
@@ -480,6 +540,7 @@ def _get_fresh_asset_status_cache_value(
                 asset_key=asset_key,
                 partitions_def=partitions_def,
                 latest_storage_id=latest_storage_id,
+                dynamic_partitions_store=dynamic_partitions_store,
             )
     else:
         updated_cache_value = _get_updated_status_cache(
@@ -487,6 +548,7 @@ def _get_fresh_asset_status_cache_value(
             asset_key=asset_key,
             partitions_def=partitions_def,
             current_status_cache_value=cached_status_data,
+            dynamic_partitions_store=dynamic_partitions_store,
         )
 
     return updated_cache_value
@@ -496,8 +558,16 @@ def get_and_update_asset_status_cache_value(
     instance: DagsterInstance,
     asset_key: AssetKey,
     partitions_def: Optional[PartitionsDefinition] = None,
+    dynamic_partitions_loader: Optional[DynamicPartitionsStore] = None,
 ) -> Optional[AssetStatusCacheValue]:
-    updated_cache_value = _get_fresh_asset_status_cache_value(instance, asset_key, partitions_def)
+    updated_cache_value = _get_fresh_asset_status_cache_value(
+        instance=instance,
+        asset_key=asset_key,
+        partitions_def=partitions_def,
+        dynamic_partitions_store=dynamic_partitions_loader
+        if dynamic_partitions_loader
+        else instance,
+    )
     if updated_cache_value:
         instance.update_asset_cached_status_data(asset_key, updated_cache_value)
 
