@@ -52,10 +52,9 @@ from dagster._core.storage.tags import (
 )
 from dagster._daemon.types import DaemonHeartbeat
 from dagster._serdes import (
-    deserialize_as,
-    serialize_dagster_namedtuple,
+    deserialize_value,
+    serialize_value,
 )
-from dagster._serdes.serdes import deserialize_json_to_dagster_namedtuple
 from dagster._seven import JSONDecodeError
 from dagster._utils import PrintFn, utc_datetime_from_timestamp
 from dagster._utils.merger import merge_dicts
@@ -93,7 +92,7 @@ class SnapshotType(Enum):
     EXECUTION_PLAN = "EXECUTION_PLAN"
 
 
-class SqlRunStorage(RunStorage):  # pylint: disable=no-init
+class SqlRunStorage(RunStorage):
     """Base class for SQL based run storages."""
 
     @abstractmethod
@@ -138,11 +137,11 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
         partition = pipeline_run.tags.get(PARTITION_NAME_TAG) if has_tags else None
         partition_set = pipeline_run.tags.get(PARTITION_SET_TAG) if has_tags else None
 
-        runs_insert = RunsTable.insert().values(  # pylint: disable=no-value-for-parameter
+        runs_insert = RunsTable.insert().values(
             run_id=pipeline_run.run_id,
             pipeline_name=pipeline_run.pipeline_name,
             status=pipeline_run.status.value,
-            run_body=serialize_dagster_namedtuple(pipeline_run),
+            run_body=serialize_value(pipeline_run),
             snapshot_id=pipeline_run.pipeline_snapshot_id,
             partition=partition,
             partition_set=partition_set,
@@ -156,7 +155,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
             tags_to_insert = pipeline_run.tags_for_storage()
             if tags_to_insert:
                 conn.execute(
-                    RunTagsTable.insert(),  # pylint: disable=no-value-for-parameter
+                    RunTagsTable.insert(),
                     [
                         dict(run_id=pipeline_run.run_id, key=k, value=v)
                         for k, v in tags_to_insert.items()
@@ -199,10 +198,10 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
 
         with self.connect() as conn:
             conn.execute(
-                RunsTable.update()  # pylint: disable=no-value-for-parameter
+                RunsTable.update()
                 .where(RunsTable.c.run_id == run_id)
                 .values(
-                    run_body=serialize_dagster_namedtuple(run.with_status(new_pipeline_status)),
+                    run_body=serialize_value(run.with_status(new_pipeline_status)),
                     status=new_pipeline_status.value,
                     update_timestamp=now,
                     **kwargs,
@@ -210,7 +209,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
             )
 
     def _row_to_run(self, row: SqlAlchemyRow) -> DagsterRun:
-        run = deserialize_as(row["run_body"], DagsterRun)
+        run = deserialize_value(row["run_body"], DagsterRun)
         status = DagsterRunStatus(row["status"])
         # NOTE: the status column is more trustworthy than the status in the run body, since concurrent
         # writes (e.g.  handle_run_event and add_tags) can cause the status in the body to be out of
@@ -575,12 +574,10 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
 
         with self.connect() as conn:
             conn.execute(
-                RunsTable.update()  # pylint: disable=no-value-for-parameter
+                RunsTable.update()
                 .where(RunsTable.c.run_id == run_id)
                 .values(
-                    run_body=serialize_dagster_namedtuple(
-                        run.with_tags(merge_dicts(current_tags, new_tags))
-                    ),
+                    run_body=serialize_value(run.with_tags(merge_dicts(current_tags, new_tags))),
                     partition=partition,
                     partition_set=partition_set,
                     update_timestamp=pendulum.now("UTC"),
@@ -595,18 +592,18 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
 
             for tag in existing_tags:
                 conn.execute(
-                    RunTagsTable.update()  # pylint: disable=no-value-for-parameter
+                    RunTagsTable.update()
                     .where(db.and_(RunTagsTable.c.run_id == run_id, RunTagsTable.c.key == tag))
                     .values(value=new_tags[tag])
                 )
 
             if added_tags:
                 conn.execute(
-                    RunTagsTable.insert(),  # pylint: disable=no-value-for-parameter
+                    RunTagsTable.insert(),
                     [dict(run_id=run_id, key=tag, value=new_tags[tag]) for tag in added_tags],
                 )
 
-    def get_run_group(self, run_id: str) -> Tuple[str, Iterable[DagsterRun]]:
+    def get_run_group(self, run_id: str) -> Tuple[str, Sequence[DagsterRun]]:
         check.str_param(run_id, "run_id")
         pipeline_run = self._get_run_by_id(run_id)
         if not pipeline_run:
@@ -861,14 +858,10 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
         check.inst_param(snapshot_type, "snapshot_type", SnapshotType)
 
         with self.connect() as conn:
-            snapshot_insert = (
-                SnapshotsTable.insert().values(  # pylint: disable=no-value-for-parameter
-                    snapshot_id=snapshot_id,
-                    snapshot_body=zlib.compress(
-                        serialize_dagster_namedtuple(snapshot_obj).encode("utf-8")
-                    ),
-                    snapshot_type=snapshot_type.value,
-                )
+            snapshot_insert = SnapshotsTable.insert().values(
+                snapshot_id=snapshot_id,
+                snapshot_body=zlib.compress(serialize_value(snapshot_obj).encode("utf-8")),
+                snapshot_type=snapshot_type.value,
             )
             conn.execute(snapshot_insert)
             return snapshot_id
@@ -900,7 +893,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
 
         row = self.fetchone(query)
 
-        return defensively_unpack_pipeline_snapshot_query(logging, row) if row else None  # type: ignore
+        return defensively_unpack_execution_plan_snapshot_query(logging, row) if row else None  # type: ignore
 
     def get_run_partition_data(self, runs_filter: RunsFilter) -> Sequence[RunPartitionData]:
         if self.has_built_index(RUN_PARTITIONS) and self.has_run_stats_index_cols():
@@ -1008,18 +1001,16 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
         return len(results) > 0
 
     def mark_index_built(self, migration_name: str) -> None:
-        query = (
-            SecondaryIndexMigrationTable.insert().values(  # pylint: disable=no-value-for-parameter
-                name=migration_name,
-                migration_completed=datetime.now(),
-            )
+        query = SecondaryIndexMigrationTable.insert().values(
+            name=migration_name,
+            migration_completed=datetime.now(),
         )
         with self.connect() as conn:
             try:
                 conn.execute(query)
             except db_exc.IntegrityError:
                 conn.execute(
-                    SecondaryIndexMigrationTable.update()  # pylint: disable=no-value-for-parameter
+                    SecondaryIndexMigrationTable.update()
                     .where(SecondaryIndexMigrationTable.c.name == migration_name)
                     .values(migration_completed=datetime.now())
                 )
@@ -1045,21 +1036,21 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
             # insert, or update if already present
             try:
                 conn.execute(
-                    DaemonHeartbeatsTable.insert().values(  # pylint: disable=no-value-for-parameter
+                    DaemonHeartbeatsTable.insert().values(
                         timestamp=utc_datetime_from_timestamp(daemon_heartbeat.timestamp),
                         daemon_type=daemon_heartbeat.daemon_type,
                         daemon_id=daemon_heartbeat.daemon_id,
-                        body=serialize_dagster_namedtuple(daemon_heartbeat),
+                        body=serialize_value(daemon_heartbeat),
                     )
                 )
             except db_exc.IntegrityError:
                 conn.execute(
-                    DaemonHeartbeatsTable.update()  # pylint: disable=no-value-for-parameter
+                    DaemonHeartbeatsTable.update()
                     .where(DaemonHeartbeatsTable.c.daemon_type == daemon_heartbeat.daemon_type)
-                    .values(  # pylint: disable=no-value-for-parameter
+                    .values(
                         timestamp=utc_datetime_from_timestamp(daemon_heartbeat.timestamp),
                         daemon_id=daemon_heartbeat.daemon_id,
-                        body=serialize_dagster_namedtuple(daemon_heartbeat),
+                        body=serialize_value(daemon_heartbeat),
                     )
                 )
 
@@ -1068,23 +1059,23 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
             rows = conn.execute(db.select([DaemonHeartbeatsTable.c.body]))
             heartbeats = []
             for row in rows:
-                heartbeats.append(deserialize_as(row.body, DaemonHeartbeat))
+                heartbeats.append(deserialize_value(row.body, DaemonHeartbeat))
             return {heartbeat.daemon_type: heartbeat for heartbeat in heartbeats}
 
     def wipe(self) -> None:
         """Clears the run storage."""
         with self.connect() as conn:
             # https://stackoverflow.com/a/54386260/324449
-            conn.execute(RunsTable.delete())  # pylint: disable=no-value-for-parameter
-            conn.execute(RunTagsTable.delete())  # pylint: disable=no-value-for-parameter
-            conn.execute(SnapshotsTable.delete())  # pylint: disable=no-value-for-parameter
-            conn.execute(DaemonHeartbeatsTable.delete())  # pylint: disable=no-value-for-parameter
-            conn.execute(BulkActionsTable.delete())  # pylint: disable=no-value-for-parameter
+            conn.execute(RunsTable.delete())
+            conn.execute(RunTagsTable.delete())
+            conn.execute(SnapshotsTable.delete())
+            conn.execute(DaemonHeartbeatsTable.delete())
+            conn.execute(BulkActionsTable.delete())
 
     def wipe_daemon_heartbeats(self) -> None:
         with self.connect() as conn:
             # https://stackoverflow.com/a/54386260/324449
-            conn.execute(DaemonHeartbeatsTable.delete())  # pylint: disable=no-value-for-parameter
+            conn.execute(DaemonHeartbeatsTable.delete())
 
     def get_backfills(
         self,
@@ -1105,13 +1096,13 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
             query = query.limit(limit)
         query = query.order_by(BulkActionsTable.c.id.desc())
         rows = self.fetchall(query)
-        return [deserialize_as(row[0], PartitionBackfill) for row in rows]
+        return [deserialize_value(row[0], PartitionBackfill) for row in rows]
 
     def get_backfill(self, backfill_id: str) -> Optional[PartitionBackfill]:
         check.str_param(backfill_id, "backfill_id")
         query = db.select([BulkActionsTable.c.body]).where(BulkActionsTable.c.key == backfill_id)
         row = self.fetchone(query)
-        return deserialize_as(row[0], PartitionBackfill) if row else None
+        return deserialize_value(row[0], PartitionBackfill) if row else None
 
     def add_backfill(self, partition_backfill: PartitionBackfill) -> None:
         check.inst_param(partition_backfill, "partition_backfill", PartitionBackfill)
@@ -1119,7 +1110,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
             key=partition_backfill.backfill_id,
             status=partition_backfill.status.value,
             timestamp=utc_datetime_from_timestamp(partition_backfill.backfill_timestamp),
-            body=serialize_dagster_namedtuple(cast(NamedTuple, partition_backfill)),
+            body=serialize_value(cast(NamedTuple, partition_backfill)),
         )
 
         if self.has_bulk_actions_selector_cols():
@@ -1127,9 +1118,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
             values["action_type"] = partition_backfill.bulk_action_type.value
 
         with self.connect() as conn:
-            conn.execute(
-                BulkActionsTable.insert().values(**values)  # pylint: disable=no-value-for-parameter
-            )
+            conn.execute(BulkActionsTable.insert().values(**values))
 
     def update_backfill(self, partition_backfill: PartitionBackfill) -> None:
         check.inst_param(partition_backfill, "partition_backfill", PartitionBackfill)
@@ -1140,11 +1129,11 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
             )
         with self.connect() as conn:
             conn.execute(
-                BulkActionsTable.update()  # pylint: disable=no-value-for-parameter
+                BulkActionsTable.update()
                 .where(BulkActionsTable.c.key == backfill_id)
                 .values(
                     status=partition_backfill.status.value,
-                    body=serialize_dagster_namedtuple(partition_backfill),
+                    body=serialize_value(partition_backfill),
                 )
             )
 
@@ -1171,7 +1160,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
                 conn.execute(KeyValueStoreTable.insert().values(db_values))
             except db_exc.IntegrityError:
                 conn.execute(
-                    KeyValueStoreTable.update()  # pylint: disable=no-value-for-parameter
+                    KeyValueStoreTable.update()
                     .where(KeyValueStoreTable.c.key.in_(pairs.keys()))
                     .values(value=db.sql.case(pairs, value=KeyValueStoreTable.c.key))
                 )
@@ -1181,14 +1170,14 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
         new_label = job_origin.external_repository_origin.get_label()
         with self.connect() as conn:
             conn.execute(
-                RunsTable.update()  # pylint: disable=no-value-for-parameter
+                RunsTable.update()
                 .where(RunsTable.c.run_id == run.run_id)
                 .values(
-                    run_body=serialize_dagster_namedtuple(run.with_job_origin(job_origin)),
+                    run_body=serialize_value(run.with_job_origin(job_origin)),
                 )
             )
             conn.execute(
-                RunTagsTable.update()  # pylint: disable=no-value-for-parameter
+                RunTagsTable.update()
                 .where(RunTagsTable.c.run_id == run.run_id)
                 .where(RunTagsTable.c.key == REPOSITORY_LABEL_TAG)
                 .values(value=new_label)
@@ -1198,15 +1187,15 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
 GET_PIPELINE_SNAPSHOT_QUERY_ID = "get-pipeline-snapshot"
 
 
-def defensively_unpack_pipeline_snapshot_query(
+def defensively_unpack_execution_plan_snapshot_query(
     logger: logging.Logger, row: SqlAlchemyRow
-) -> Optional[PipelineSnapshot]:
+) -> Optional[Union[ExecutionPlanSnapshot, PipelineSnapshot]]:
     # no checking here because sqlalchemy returns a special
     # row proxy and don't want to instance check on an internal
     # implementation detail
 
     def _warn(msg: str) -> None:
-        logger.warning("get-pipeline-snapshot: {msg}".format(msg=msg))
+        logger.warning(f"get-pipeline-snapshot: {msg}")
 
     if not isinstance(row[0], bytes):
         _warn("First entry in row is not a binary type.")
@@ -1225,7 +1214,7 @@ def defensively_unpack_pipeline_snapshot_query(
         return None
 
     try:
-        return deserialize_json_to_dagster_namedtuple(decoded_str)  # type: ignore
+        return deserialize_value(decoded_str, (ExecutionPlanSnapshot, PipelineSnapshot))
     except JSONDecodeError:
         _warn("Could not parse json in snapshot table.")
         return None
