@@ -1,5 +1,5 @@
 """As an open source project, we collect usage statistics to inform development priorities.
-For more information, check out the docs at https://docs.dagster.io/install#telemetry'.
+For more information, check out the docs at https://docs.dagster.io/getting-started/telemetry'.
 
 To see the logs we send, inspect $DAGSTER_HOME/logs/ if $DAGSTER_HOME is set or ~/.dagster/logs/
 
@@ -20,10 +20,22 @@ import sys
 import uuid
 from functools import wraps
 from logging.handlers import RotatingFileHandler
-from typing import Mapping, NamedTuple, Optional
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Mapping,
+    NamedTuple,
+    Optional,
+    Sequence,
+    TypeVar,
+    Union,
+    overload,
+)
 
 import click
 import yaml
+from typing_extensions import ParamSpec
 
 import dagster._check as check
 from dagster._core.definitions.pipeline_base import IPipeline
@@ -40,6 +52,10 @@ from dagster._core.instance import DagsterInstance
 from dagster._utils.merger import merge_dicts
 from dagster.version import __version__ as dagster_module_version
 
+if TYPE_CHECKING:
+    from dagster._core.host_representation.external import ExternalPipeline, ExternalRepository
+    from dagster._core.workspace.context import IWorkspaceProcessContext
+
 TELEMETRY_STR = ".telemetry"
 INSTANCE_ID_STR = "instance_id"
 ENABLED_STR = "enabled"
@@ -63,6 +79,7 @@ TELEMETRY_WHITELISTED_FUNCTIONS = {
     "execute_execute_command",
     "execute_launch_command",
     "_daemon_run_command",
+    "execute_materialize_command",
 }
 
 KNOWN_CI_ENV_VAR_KEYS = {
@@ -76,26 +93,45 @@ KNOWN_CI_ENV_VAR_KEYS = {
     "BUILDKITE",  # https://buildkite.com/docs/pipelines/environment-variables
 }
 
+P = ParamSpec("P")
+T = TypeVar("T")
+T_Callable = TypeVar("T_Callable", bound=Callable[..., Any])
 
-def telemetry_wrapper(metadata):
-    """
-    Wrapper around functions that are logged. Will log the function_name, client_time, and
+
+@overload
+def telemetry_wrapper(target_fn: T_Callable) -> T_Callable:
+    ...
+
+
+@overload
+def telemetry_wrapper(
+    *, metadata: Optional[Mapping[str, str]]
+) -> Callable[[Callable[P, T]], Callable[P, T]]:
+    ...
+
+
+def telemetry_wrapper(
+    target_fn: Optional[T_Callable] = None, *, metadata: Optional[Mapping[str, str]] = None
+) -> Union[T_Callable, Callable[[Callable[P, T]], Callable[P, T]]]:
+    """Wrapper around functions that are logged. Will log the function_name, client_time, and
     elapsed_time, and success.
 
     Wrapped function must be in the list of whitelisted function, and must have a DagsterInstance
     parameter named 'instance' in the signature.
     """
-    if callable(metadata):
-        return _telemetry_wrapper(metadata)
+    if target_fn is not None:
+        return _telemetry_wrapper(target_fn)
 
-    def _wraps(f):
+    def _wraps(f: Callable[P, T]) -> Callable[P, T]:
         return _telemetry_wrapper(f, metadata)
 
     return _wraps
 
 
-def _telemetry_wrapper(f, metadata=None):
-    metadata = check.opt_dict_param(metadata, "metadata", key_type=str, value_type=str)
+def _telemetry_wrapper(
+    f: Callable[P, T], metadata: Optional[Mapping[str, str]] = None
+) -> Callable[P, T]:
+    metadata = check.opt_mapping_param(metadata, "metadata", key_type=str, value_type=str)
 
     if f.__name__ not in TELEMETRY_WHITELISTED_FUNCTIONS:
         raise DagsterInvariantViolationError(
@@ -115,7 +151,7 @@ def _telemetry_wrapper(f, metadata=None):
         ) from e
 
     @wraps(f)
-    def wrap(*args, **kwargs):
+    def wrap(*args: P.args, **kwargs: P.kwargs) -> T:
         instance = _check_telemetry_instance_param(args, kwargs, instance_index)
         start_time = datetime.datetime.now()
         log_action(
@@ -139,12 +175,12 @@ def _telemetry_wrapper(f, metadata=None):
     return wrap
 
 
-def get_python_version():
+def get_python_version() -> str:
     version = sys.version_info
-    return "{}.{}.{}".format(version.major, version.minor, version.micro)
+    return f"{version.major}.{version.minor}.{version.micro}"
 
 
-def get_is_known_ci_env():
+def get_is_known_ci_env() -> bool:
     # Many CI tools will use `CI` key which lets us know for sure it's a CI env
     if os.environ.get("CI") is True:
         return True
@@ -176,8 +212,7 @@ class TelemetryEntry(
         ],
     )
 ):
-    """
-    Schema for telemetry logs.
+    """Schema for telemetry logs.
 
     Currently, log entries are coerced to the same schema to enable storing all entries in one DB
     table with unified schema.
@@ -234,7 +269,7 @@ class TelemetryEntry(
         )
 
 
-def _dagster_home_if_set():
+def _dagster_home_if_set() -> Optional[str]:
     dagster_home_path = os.getenv("DAGSTER_HOME")
 
     if not dagster_home_path:
@@ -243,9 +278,8 @@ def _dagster_home_if_set():
     return os.path.expanduser(dagster_home_path)
 
 
-def get_or_create_dir_from_dagster_home(target_dir):
-    """
-    If $DAGSTER_HOME is set, return $DAGSTER_HOME/<target_dir>/
+def get_or_create_dir_from_dagster_home(target_dir: str) -> str:
+    """If $DAGSTER_HOME is set, return $DAGSTER_HOME/<target_dir>/
     Otherwise, return ~/.dagster/<target_dir>/.
 
     The 'logs' directory is used to cache logs before upload
@@ -270,9 +304,8 @@ def get_or_create_dir_from_dagster_home(target_dir):
     return dagster_home_logs_path
 
 
-def get_log_queue_dir():
-    """
-    Get the directory where we store log queue files, creating the directory if needed.
+def get_log_queue_dir() -> str:
+    """Get the directory where we store log queue files, creating the directory if needed.
 
     The log queue directory is used to temporarily store logs during upload. This is to prevent
     dropping events or double-sending events that occur during the upload process.
@@ -291,10 +324,12 @@ def get_log_queue_dir():
     return dagster_home_logs_queue_path
 
 
-def _check_telemetry_instance_param(args, kwargs, instance_index):
+def _check_telemetry_instance_param(
+    args: Sequence[object], kwargs: Mapping[str, object], instance_index: int
+) -> DagsterInstance:
     if "instance" in kwargs:
         return check.inst_param(
-            kwargs["instance"],
+            kwargs["instance"],  # type: ignore
             "instance",
             DagsterInstance,
             "'instance' parameter passed as keyword argument must be a DagsterInstance",
@@ -302,7 +337,7 @@ def _check_telemetry_instance_param(args, kwargs, instance_index):
     else:
         check.invariant(len(args) > instance_index)
         return check.inst_param(
-            args[instance_index],
+            args[instance_index],  # type: ignore
             "instance",
             DagsterInstance,
             "'instance' argument at position {position} must be a DagsterInstance".format(
@@ -311,7 +346,7 @@ def _check_telemetry_instance_param(args, kwargs, instance_index):
         )
 
 
-def _get_telemetry_logger():
+def _get_telemetry_logger() -> logging.Logger:
     # If a concurrently running process deleted the logging directory since the
     # last action, we need to make sure to re-create the directory
     # (the logger does not do this itself.)
@@ -339,7 +374,7 @@ def _get_telemetry_logger():
 
 
 # For use in test teardown
-def cleanup_telemetry_logger():
+def cleanup_telemetry_logger() -> None:
     logger = logging.getLogger("dagster_telemetry_logger")
     if len(logger.handlers) == 0:
         return
@@ -350,12 +385,12 @@ def cleanup_telemetry_logger():
     logger.removeHandler(handler)
 
 
-def write_telemetry_log_line(log_line):
+def write_telemetry_log_line(log_line: object) -> None:
     logger = _get_telemetry_logger()
     logger.info(json.dumps(log_line))
 
 
-def _get_instance_telemetry_info(instance):
+def _get_instance_telemetry_info(instance: DagsterInstance):
     from dagster._core.storage.runs import SqlRunStorage
 
     check.inst_param(instance, "instance", DagsterInstance)
@@ -370,11 +405,11 @@ def _get_instance_telemetry_info(instance):
     return (dagster_telemetry_enabled, instance_id, run_storage_id)
 
 
-def _get_instance_telemetry_enabled(instance):
+def _get_instance_telemetry_enabled(instance: DagsterInstance) -> bool:
     return instance.telemetry_enabled
 
 
-def get_or_set_instance_id():
+def get_or_set_instance_id() -> str:
     instance_id = _get_telemetry_instance_id()
     if instance_id is None:
         instance_id = _set_telemetry_instance_id()
@@ -382,7 +417,7 @@ def get_or_set_instance_id():
 
 
 # Gets the instance_id at $DAGSTER_HOME/.telemetry/id.yaml
-def _get_telemetry_instance_id():
+def _get_telemetry_instance_id() -> Optional[str]:
     telemetry_id_path = os.path.join(get_or_create_dir_from_dagster_home(TELEMETRY_STR), "id.yaml")
     if not os.path.exists(telemetry_id_path):
         return
@@ -399,7 +434,7 @@ def _get_telemetry_instance_id():
 
 
 # Sets the instance_id at $DAGSTER_HOME/.telemetry/id.yaml
-def _set_telemetry_instance_id():
+def _set_telemetry_instance_id() -> str:
     click.secho(TELEMETRY_TEXT)
     click.secho(SLACK_PROMPT)
 
@@ -414,18 +449,37 @@ def _set_telemetry_instance_id():
         return "<<unable_to_write_instance_id>>"
 
 
-def hash_name(name):
+def hash_name(name: str) -> str:
     return hashlib.sha256(name.encode("utf-8")).hexdigest()
 
 
-def log_external_repo_stats(instance, source, external_repo, external_pipeline=None):
+def log_external_repo_stats(
+    instance: DagsterInstance,
+    source: str,
+    external_repo: "ExternalRepository",
+    external_pipeline: Optional["ExternalPipeline"] = None,
+):
     from dagster._core.host_representation.external import ExternalPipeline, ExternalRepository
+    from dagster._core.host_representation.external_data import (
+        ExternalAssetNode,
+        ExternalDynamicPartitionsDefinitionData,
+    )
 
     check.inst_param(instance, "instance", DagsterInstance)
     check.str_param(source, "source")
     check.inst_param(external_repo, "external_repo", ExternalRepository)
     check.opt_inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
 
+    def _get_num_dynamic_partitioned_assets(
+        external_asset_nodes: Sequence[ExternalAssetNode],
+    ) -> int:
+        return sum(
+            1
+            for asset in external_asset_nodes
+            if asset.partitions_def_data
+            and isinstance(asset.partitions_def_data, ExternalDynamicPartitionsDefinitionData)
+        )
+
     if _get_instance_telemetry_enabled(instance):
         instance_id = get_or_set_instance_id()
 
@@ -435,7 +489,11 @@ def log_external_repo_stats(instance, source, external_repo, external_pipeline=N
         num_pipelines_in_repo = len(external_repo.get_all_external_jobs())
         num_schedules_in_repo = len(external_repo.get_external_schedules())
         num_sensors_in_repo = len(external_repo.get_external_sensors())
-        num_assets_in_repo = len(external_repo.get_external_asset_nodes())
+        external_asset_nodes = external_repo.get_external_asset_nodes()
+        num_assets_in_repo = len(external_asset_nodes)
+        num_dynamic_partitioned_assets_in_repo = _get_num_dynamic_partitioned_assets(
+            external_asset_nodes
+        )
 
         write_telemetry_log_line(
             TelemetryEntry(
@@ -452,17 +510,36 @@ def log_external_repo_stats(instance, source, external_repo, external_pipeline=N
                     "num_assets_in_repo": str(num_assets_in_repo),
                     "repo_hash": repo_hash,
                     "location_name_hash": location_name_hash,
+                    "num_dynamic_partitioned_assets_in_repo": str(
+                        num_dynamic_partitioned_assets_in_repo
+                    ),
                 },
             )._asdict()
         )
 
 
-def log_repo_stats(instance, source, pipeline=None, repo=None):
+def log_repo_stats(
+    instance: DagsterInstance,
+    source: str,
+    pipeline: Optional[IPipeline] = None,
+    repo: Optional[ReconstructableRepository] = None,
+) -> None:
+    from dagster._core.definitions.assets import AssetsDefinition
+    from dagster._core.definitions.partition import DynamicPartitionsDefinition
+
     check.inst_param(instance, "instance", DagsterInstance)
     check.str_param(source, "source")
     check.opt_inst_param(pipeline, "pipeline", IPipeline)
     check.opt_inst_param(repo, "repo", ReconstructableRepository)
 
+    def _get_num_dynamic_partitioned_assets(asset_defs: Sequence[AssetsDefinition]) -> int:
+        return sum(
+            1
+            for asset in asset_defs
+            if asset.partitions_def
+            and isinstance(asset.partitions_def, DynamicPartitionsDefinition)
+        )
+
     if _get_instance_telemetry_enabled(instance):
         instance_id = get_or_set_instance_id()
 
@@ -473,10 +550,9 @@ def log_repo_stats(instance, source, pipeline=None, repo=None):
             num_pipelines_in_repo = len(repository.pipeline_names)
             num_schedules_in_repo = len(repository.schedule_defs)
             num_sensors_in_repo = len(repository.sensor_defs)
-            all_assets = list(
-                repository._assets_defs_by_key.values()  # pylint: disable=protected-access
-            )
+            all_assets = list(repository.assets_defs_by_key.values())
             num_assets_in_repo = len(all_assets)
+            num_dynamic_partitioned_assets_in_repo = _get_num_dynamic_partitioned_assets(all_assets)
         elif isinstance(repo, ReconstructableRepository):
             pipeline_name_hash = ""
             repository = repo.get_definition()
@@ -484,17 +560,17 @@ def log_repo_stats(instance, source, pipeline=None, repo=None):
             num_pipelines_in_repo = len(repository.pipeline_names)
             num_schedules_in_repo = len(repository.schedule_defs)
             num_sensors_in_repo = len(repository.sensor_defs)
-            all_assets = list(
-                repository._assets_defs_by_key.values()  # pylint: disable=protected-access
-            )
+            all_assets = list(repository.assets_defs_by_key.values())
             num_assets_in_repo = len(all_assets)
+            num_dynamic_partitioned_assets_in_repo = _get_num_dynamic_partitioned_assets(all_assets)
         else:
-            pipeline_name_hash = hash_name(pipeline.get_definition().name)
-            repo_hash = hash_name(get_ephemeral_repository_name(pipeline.get_definition().name))
+            pipeline_name_hash = hash_name(pipeline.get_definition().name)  # type: ignore
+            repo_hash = hash_name(get_ephemeral_repository_name(pipeline.get_definition().name))  # type: ignore
             num_pipelines_in_repo = 1
             num_schedules_in_repo = 0
             num_sensors_in_repo = 0
             num_assets_in_repo = 0
+            num_dynamic_partitioned_assets_in_repo = 0
 
         write_telemetry_log_line(
             TelemetryEntry(
@@ -510,12 +586,17 @@ def log_repo_stats(instance, source, pipeline=None, repo=None):
                     "num_sensors_in_repo": str(num_sensors_in_repo),
                     "num_assets_in_repo": str(num_assets_in_repo),
                     "repo_hash": repo_hash,
+                    "num_dynamic_partitioned_assets_in_repo": str(
+                        num_dynamic_partitioned_assets_in_repo
+                    ),
                 },
             )._asdict()
         )
 
 
-def log_workspace_stats(instance, workspace_process_context):
+def log_workspace_stats(
+    instance: DagsterInstance, workspace_process_context: "IWorkspaceProcessContext"
+) -> None:
     from dagster._core.workspace.context import IWorkspaceProcessContext
 
     check.inst_param(instance, "instance", DagsterInstance)
@@ -525,18 +606,18 @@ def log_workspace_stats(instance, workspace_process_context):
 
     request_context = workspace_process_context.create_request_context()
 
-    for repo_location in request_context.repository_locations:
-        for external_repo in repo_location.get_repositories().values():
+    for code_location in request_context.code_locations:
+        for external_repo in code_location.get_repositories().values():
             log_external_repo_stats(instance, source="dagit", external_repo=external_repo)
 
 
 def log_action(
-    instance,
-    action,
-    client_time=None,
-    elapsed_time=None,
-    metadata=None,
-):
+    instance: DagsterInstance,
+    action: str,
+    client_time: Optional[datetime.datetime] = None,
+    elapsed_time: Optional[datetime.timedelta] = None,
+    metadata: Optional[Mapping[str, str]] = None,
+) -> None:
     check.inst_param(instance, "instance", DagsterInstance)
     if client_time is None:
         client_time = datetime.datetime.now()
@@ -560,13 +641,13 @@ def log_action(
         )
 
 
-def log_dagster_event(event: DagsterEvent, pipeline_context: PlanOrchestrationContext):
+def log_dagster_event(event: DagsterEvent, pipeline_context: PlanOrchestrationContext) -> None:
     if not any((event.is_step_start, event.is_step_success, event.is_step_failure)):
         return
 
     metadata = {
         "run_id_hash": hash_name(pipeline_context.run_id),
-        "step_key_hash": hash_name(event.step_key),
+        "step_key_hash": hash_name(event.step_key),  # type: ignore
     }
 
     if event.is_step_start:
@@ -574,7 +655,7 @@ def log_dagster_event(event: DagsterEvent, pipeline_context: PlanOrchestrationCo
     elif event.is_step_success:
         action = STEP_SUCCESS_EVENT
         if isinstance(event.event_specific_data, StepSuccessData):  # make mypy happy
-            metadata["duration_ms"] = event.event_specific_data.duration_ms
+            metadata["duration_ms"] = event.event_specific_data.duration_ms  # type: ignore
     else:  # event.is_step_failure
         action = STEP_FAILURE_EVENT
 
@@ -590,7 +671,7 @@ TELEMETRY_TEXT = """
   %(telemetry)s
 
   As an open source project, we collect usage statistics to inform development priorities. For more
-  information, read https://docs.dagster.io/install#telemetry.
+  information, read https://docs.dagster.io/getting-started/telemetry.
 
   We will not see or store solid definitions, pipeline definitions, modes, resources, context, or
   any data that is processed within solids and pipelines.
