@@ -43,7 +43,7 @@ def load_type_input_schema_dict(value):
         with open(file_options["path"], "rb") as ff:
             return pickle.load(ff)
     else:
-        check.failed("Unsupported key {key}".format(key=file_type))
+        check.failed(f"Unsupported key {file_type}")
 
 
 def define_any_input_schema():
