@@ -24,7 +24,12 @@ from dagster._config import (
     Noneable as ConfigNoneable,
 )
 from dagster._core.definitions.events import DynamicOutput, Output, TypeCheck
-from dagster._core.definitions.metadata import MetadataEntry, RawMetadataValue, normalize_metadata
+from dagster._core.definitions.metadata import (
+    MetadataEntry,
+    MetadataValue,
+    RawMetadataValue,
+    normalize_metadata,
+)
 from dagster._core.errors import DagsterInvalidDefinitionError, DagsterInvariantViolationError
 from dagster._serdes import whitelist_for_serdes
 from dagster._seven import is_subclass
@@ -143,18 +148,16 @@ class DagsterType(RequiresResources):
         self.is_builtin = check.bool_param(is_builtin, "is_builtin")
         check.invariant(
             self.display_name is not None,
-            "All types must have a valid display name, got None for key {}".format(key),
+            f"All types must have a valid display name, got None for key {key}",
         )
 
         self.kind = check.inst_param(kind, "kind", DagsterTypeKind)
 
         self._typing_type = typing_type
 
-        metadata_entries = check.opt_list_param(
-            metadata_entries, "metadata_entries", of_type=MetadataEntry
-        )
-        self._metadata_entries = normalize_metadata(
-            check.opt_mapping_param(metadata, "metadata", key_type=str), metadata_entries
+        self._metadata = normalize_metadata(
+            check.opt_mapping_param(metadata, "metadata", key_type=str),
+            check.opt_list_param(metadata_entries, "metadata_entries", of_type=MetadataEntry),
         )
 
     @public
@@ -187,8 +190,8 @@ class DagsterType(RequiresResources):
         return _RUNTIME_MAP[builtin_enum]
 
     @property
-    def metadata_entries(self) -> t.Sequence[MetadataEntry]:
-        return self._metadata_entries
+    def metadata(self) -> t.Mapping[str, MetadataValue]:
+        return self._metadata
 
     @public
     @property
@@ -209,7 +212,7 @@ class DagsterType(RequiresResources):
         # TODO: docstring and body inconsistent-- can this be None or not?
         check.invariant(
             self._name is not None,
-            "unique_name requested but is None for type {}".format(self.display_name),
+            f"unique_name requested but is None for type {self.display_name}",
         )
         return self._name
 
@@ -460,7 +463,7 @@ class _Nothing(DagsterType):
         if value is not None:
             return TypeCheck(
                 success=False,
-                description="Value must be None, got a {value_type}".format(value_type=type(value)),
+                description=f"Value must be None, got a {type(value)}",
             )
 
         return TypeCheck(success=True)
@@ -770,8 +773,7 @@ as_dagster_type are registered here so that we can remap the Python types to run
 def make_python_type_usable_as_dagster_type(
     python_type: TypingType[t.Any], dagster_type: DagsterType
 ) -> None:
-    """
-    Take any existing python type and map it to a dagster type (generally created with
+    """Take any existing python type and map it to a dagster type (generally created with
     :py:class:`DagsterType <dagster.DagsterType>`) This can only be called once
     on a given python type.
     """
@@ -853,7 +855,7 @@ def resolve_dagster_type(dagster_type: object) -> DagsterType:
 
     check.invariant(
         not (isinstance(dagster_type, type) and is_subclass(dagster_type, DagsterType)),
-        "Do not pass runtime type classes. Got {}".format(dagster_type),
+        f"Do not pass runtime type classes. Got {dagster_type}",
     )
 
     # First, check to see if we're using Dagster's generic output type to do the type catching.
@@ -923,7 +925,7 @@ def is_dynamic_output_annotation(dagster_type: object) -> bool:
 
     check.invariant(
         not (isinstance(dagster_type, type) and is_subclass(dagster_type, ConfigType)),
-        "Do not pass runtime type classes. Got {}".format(dagster_type),
+        f"Do not pass runtime type classes. Got {dagster_type}",
     )
 
     if dagster_type == DynamicOutput or get_origin(dagster_type) == DynamicOutput:
@@ -943,9 +945,7 @@ def is_generic_output_annotation(dagster_type: object) -> bool:
 
 
 def resolve_python_type_to_dagster_type(python_type: t.Type) -> DagsterType:
-    """
-    Resolves a Python type to a Dagster type.
-    """
+    """Resolves a Python type to a Dagster type."""
     check.inst_param(python_type, "python_type", type)
 
     if python_type in _PYTHON_TYPE_TO_DAGSTER_TYPE_MAPPING_REGISTRY:
