@@ -20,12 +20,12 @@ class LoadableTargetOrigin(
 ):
     def __new__(
         cls,
-        executable_path=None,
-        python_file=None,
-        module_name=None,
-        working_directory=None,
-        attribute=None,
-        package_name=None,
+        executable_path: Optional[str] = None,
+        python_file: Optional[str] = None,
+        module_name: Optional[str] = None,
+        working_directory: Optional[str] = None,
+        attribute: Optional[str] = None,
+        package_name: Optional[str] = None,
     ):
         return super(LoadableTargetOrigin, cls).__new__(
             cls,
