@@ -35,7 +35,7 @@ class _TypedPythonSet(DagsterType):
     def __init__(self, item_dagster_type):
         self.item_type = item_dagster_type
         super(_TypedPythonSet, self).__init__(
-            key="TypedPythonSet.{}".format(item_dagster_type.key),
+            key=f"TypedPythonSet.{item_dagster_type.key}",
             name=None,
             loader=(TypedSetLoader(item_dagster_type) if item_dagster_type.loader else None),
             type_check_fn=self.type_check_method,
@@ -62,7 +62,7 @@ class _TypedPythonSet(DagsterType):
 
     @property
     def display_name(self):
-        return "Set[{}]".format(self.item_type.display_name)
+        return f"Set[{self.item_type.display_name}]"
 
     @property
     def inner_types(self):
