@@ -2,7 +2,7 @@ from dagster import (
     Output,
     _check as check,
 )
-from dagster._core.definitions import In, InputDefinition, OpDefinition, lambda_solid
+from dagster._core.definitions import In, InputDefinition, OpDefinition, op
 from dagster._core.definitions.output import Out
 
 
@@ -25,7 +25,7 @@ def _compute_fn(context, inputs):
 def define_stub_solid(name, value):
     check.str_param(name, "name")
 
-    @lambda_solid(name=name)
+    @op(name=name)
     def _stub():
         return value
 
