@@ -1,9 +1,13 @@
+import os
 from abc import ABC, abstractmethod
 from typing import NamedTuple, Optional, Sequence
 
+import tomli
+
 from dagster._core.host_representation.origin import (
-    GrpcServerRepositoryLocationOrigin,
-    RepositoryLocationOrigin,
+    CodeLocationOrigin,
+    GrpcServerCodeLocationOrigin,
+    ManagedGrpcPythonEnvCodeLocationOrigin,
 )
 
 from .load import (
@@ -16,8 +20,8 @@ from .load import (
 
 class WorkspaceLoadTarget(ABC):
     @abstractmethod
-    def create_origins(self) -> Sequence[RepositoryLocationOrigin]:
-        """Reloads the RepositoryLocationOrigins for this workspace."""
+    def create_origins(self) -> Sequence[CodeLocationOrigin]:
+        """Reloads the CodeLocationOrigins for this workspace."""
 
 
 class CompositeTarget(
@@ -33,10 +37,32 @@ class CompositeTarget(
 class WorkspaceFileTarget(
     NamedTuple("WorkspaceFileTarget", [("paths", Sequence[str])]), WorkspaceLoadTarget
 ):
-    def create_origins(self):
+    def create_origins(self) -> Sequence[CodeLocationOrigin]:
         return location_origins_from_yaml_paths(self.paths)
 
 
+def get_origins_from_toml(path: str) -> Sequence[ManagedGrpcPythonEnvCodeLocationOrigin]:
+    with open(path, "rb") as f:
+        data = tomli.load(f)
+        if not isinstance(data, dict):
+            return []
+
+        dagster_block = data.get("tool", {}).get("dagster", {})
+        if "module_name" in dagster_block:
+            return ModuleTarget(
+                module_name=dagster_block["module_name"],
+                attribute=None,
+                working_directory=os.getcwd(),
+                location_name=None,
+            ).create_origins()
+        return []
+
+
+class PyProjectFileTarget(NamedTuple("PyProjectFileTarget", [("path", str)]), WorkspaceLoadTarget):
+    def create_origins(self) -> Sequence[CodeLocationOrigin]:
+        return get_origins_from_toml(self.path)
+
+
 class PythonFileTarget(
     NamedTuple(
         "PythonFileTarget",
@@ -49,7 +75,7 @@ class PythonFileTarget(
     ),
     WorkspaceLoadTarget,
 ):
-    def create_origins(self):
+    def create_origins(self) -> Sequence[ManagedGrpcPythonEnvCodeLocationOrigin]:
         return [
             location_origin_from_python_file(
                 python_file=self.python_file,
@@ -72,7 +98,7 @@ class ModuleTarget(
     ),
     WorkspaceLoadTarget,
 ):
-    def create_origins(self):
+    def create_origins(self) -> Sequence[ManagedGrpcPythonEnvCodeLocationOrigin]:
         return [
             location_origin_from_module_name(
                 self.module_name,
@@ -95,7 +121,7 @@ class PackageTarget(
     ),
     WorkspaceLoadTarget,
 ):
-    def create_origins(self):
+    def create_origins(self) -> Sequence[ManagedGrpcPythonEnvCodeLocationOrigin]:
         return [
             location_origin_from_package_name(
                 self.package_name,
@@ -118,9 +144,9 @@ class GrpcServerTarget(
     ),
     WorkspaceLoadTarget,
 ):
-    def create_origins(self):
+    def create_origins(self) -> Sequence[GrpcServerCodeLocationOrigin]:
         return [
-            GrpcServerRepositoryLocationOrigin(
+            GrpcServerCodeLocationOrigin(
                 port=self.port,
                 socket=self.socket,
                 host=self.host,
@@ -131,5 +157,5 @@ class GrpcServerTarget(
 
 #  Utility target for graphql commands that do not require a workspace, e.g. downloading schema
 class EmptyWorkspaceTarget(NamedTuple("EmptyWorkspaceTarget", []), WorkspaceLoadTarget):
-    def create_origins(self):
+    def create_origins(self) -> Sequence[CodeLocationOrigin]:
         return []
