@@ -1,28 +1,29 @@
 import os
 import sys
-import threading
-import time
-import warnings
-from contextlib import ExitStack
+from typing import Optional
 
 import click
-import pendulum
 
 from dagster import __version__ as dagster_version
-from dagster._cli.workspace.cli_target import get_workspace_load_target, workspace_target_argument
-from dagster._core.instance import DagsterInstance
+from dagster._cli.workspace.cli_target import (
+    ClickArgMapping,
+    ClickArgValue,
+    get_workspace_load_target,
+    workspace_target_argument,
+)
+from dagster._core.instance import DagsterInstance, InstanceRef
 from dagster._core.telemetry import telemetry_wrapper
 from dagster._daemon.controller import (
     DEFAULT_DAEMON_HEARTBEAT_TOLERANCE_SECONDS,
-    DagsterDaemonController,
-    all_daemons_healthy,
+    DagsterDaemonController as DagsterDaemonController,
     all_daemons_live,
     daemon_controller_from_instance,
     debug_daemon_heartbeats,
     get_daemon_statuses,
 )
 from dagster._daemon.daemon import get_telemetry_daemon_session_id
-from dagster._utils.interrupts import capture_interrupts, raise_interrupts_as
+from dagster._serdes import deserialize_value
+from dagster._utils.interrupts import capture_interrupts
 
 
 def _get_heartbeat_tolerance():
@@ -36,21 +37,48 @@ def _get_heartbeat_tolerance():
     name="run",
     help="Run any daemons configured on the DagsterInstance.",
 )
+@click.option(
+    "--code-server-log-level",
+    help="Set the log level for any code servers spun up by the daemon.",
+    show_default=True,
+    default="warning",
+    type=click.Choice(
+        ["critical", "error", "warning", "info", "debug", "trace"], case_sensitive=False
+    ),
+)
+@click.option(
+    "--instance-ref",
+    type=click.STRING,
+    required=False,
+    hidden=True,
+)
 @workspace_target_argument
-def run_command(**kwargs):
-    with capture_interrupts():
-        with DagsterInstance.get() as instance:
-            _daemon_run_command(instance, kwargs)
+def run_command(
+    code_server_log_level: str,
+    instance_ref: Optional[str],
+    **kwargs: ClickArgValue,
+) -> None:
+    try:
+        with capture_interrupts():
+            with DagsterInstance.from_ref(
+                deserialize_value(instance_ref, InstanceRef)
+            ) if instance_ref else DagsterInstance.get() as instance:
+                _daemon_run_command(instance, code_server_log_level, kwargs)
+    except KeyboardInterrupt:
+        return  # Exit cleanly on interrupt
 
 
 @telemetry_wrapper(metadata={"DAEMON_SESSION_ID": get_telemetry_daemon_session_id()})
-def _daemon_run_command(instance: DagsterInstance, kwargs):
+def _daemon_run_command(
+    instance: DagsterInstance, code_server_log_level: str, kwargs: ClickArgMapping
+) -> None:
     workspace_load_target = get_workspace_load_target(kwargs)
 
     with daemon_controller_from_instance(
         instance,
         workspace_load_target=workspace_load_target,
         heartbeat_tolerance_seconds=_get_heartbeat_tolerance(),
+        code_server_log_level=code_server_log_level,
     ) as controller:
         controller.check_daemon_loop()
 
@@ -59,7 +87,7 @@ def _daemon_run_command(instance: DagsterInstance, kwargs):
     name="liveness-check",
     help="Check for recent heartbeats from the daemon.",
 )
-def liveness_check_command():
+def liveness_check_command() -> None:
     with DagsterInstance.get() as instance:
         if all_daemons_live(instance, heartbeat_tolerance_seconds=_get_heartbeat_tolerance()):
             click.echo("Daemon live")
@@ -72,7 +100,7 @@ def liveness_check_command():
     name="wipe",
     help="Wipe all heartbeats from storage.",
 )
-def wipe_command():
+def wipe_command() -> None:
     with DagsterInstance.get() as instance:
         instance.wipe_daemon_heartbeats()
         click.echo("Daemon heartbeats wiped")
@@ -82,7 +110,7 @@ def wipe_command():
     name="heartbeat",
     help="Read and write a heartbeat",
 )
-def debug_heartbeat_command():
+def debug_heartbeat_command() -> None:
     with DagsterInstance.get() as instance:
         debug_daemon_heartbeats(instance)
 
@@ -91,7 +119,7 @@ def debug_heartbeat_command():
     name="heartbeat-dump",
     help="Log all heartbeat statuses",
 )
-def debug_heartbeat_dump_command():
+def debug_heartbeat_dump_command() -> None:
     with DagsterInstance.get() as instance:
         for daemon_status in get_daemon_statuses(instance, instance.get_required_daemon_types()):
             click.echo(daemon_status)
@@ -100,11 +128,11 @@ def debug_heartbeat_dump_command():
 @click.group(
     commands={"heartbeat": debug_heartbeat_command, "heartbeat-dump": debug_heartbeat_dump_command}
 )
-def debug_group():
-    "Daemon debugging utils"
+def debug_group() -> None:
+    """Daemon debugging utils."""
 
 
-def create_dagster_daemon_cli():
+def create_dagster_daemon_cli() -> click.Group:
     commands = {
         "run": run_command,
         "liveness-check": liveness_check_command,
@@ -115,7 +143,7 @@ def create_dagster_daemon_cli():
     @click.group(commands=commands)
     @click.version_option(version=dagster_version)
     def group():
-        "CLI tools for working with the dagster daemon process."
+        """CLI tools for working with the dagster daemon process."""
 
     return group
 
@@ -123,5 +151,5 @@ def create_dagster_daemon_cli():
 cli = create_dagster_daemon_cli()
 
 
-def main():
+def main() -> None:
     cli(obj={})  # pylint:disable=E1123
