@@ -6,12 +6,15 @@ from abc import ABC, abstractmethod
 from collections import deque
 from contextlib import AbstractContextManager
 from threading import Event
-from typing import Generator, Generic, TypeVar, Union
+from typing import TYPE_CHECKING, Generator, Generic, Optional, TypeVar, Union
 
 import pendulum
+from typing_extensions import TypeAlias
 
-from dagster import DagsterInstance
-from dagster import _check as check
+from dagster import (
+    DagsterInstance,
+    _check as check,
+)
 from dagster._core.scheduler.scheduler import DagsterDaemonScheduler
 from dagster._core.telemetry import DAEMON_ALIVE, log_action
 from dagster._core.workspace.context import IWorkspaceProcessContext
@@ -22,6 +25,9 @@ from dagster._daemon.types import DaemonHeartbeat
 from dagster._scheduler.scheduler import execute_scheduler_iteration_loop
 from dagster._utils.error import SerializableErrorInfo, serializable_error_info_from_exc_info
 
+if TYPE_CHECKING:
+    from pendulum.datetime import DateTime
+
 
 def get_default_daemon_logger(daemon_name) -> logging.Logger:
     return logging.getLogger(f"dagster.daemon.{daemon_name}")
@@ -36,12 +42,16 @@ def get_telemetry_daemon_session_id() -> str:
     return _telemetry_daemon_session_id
 
 
-TDaemonGenerator = Generator[Union[None, SerializableErrorInfo], None, None]
+# DaemonIterator = Iterator[Union[None, SerializableErrorInfo]]
+DaemonIterator: TypeAlias = Generator[Union[None, SerializableErrorInfo], None, None]
 
 TContext = TypeVar("TContext", bound=IWorkspaceProcessContext)
 
 
 class DagsterDaemon(AbstractContextManager, ABC, Generic[TContext]):
+    _logger: logging.Logger
+    _last_heartbeat_time: Optional["DateTime"]
+
     def __init__(self):
         self._logger = get_default_daemon_logger(type(self).__name__)
 
@@ -56,9 +66,7 @@ class DagsterDaemon(AbstractContextManager, ABC, Generic[TContext]):
     @classmethod
     @abstractmethod
     def daemon_type(cls) -> str:
-        """
-        returns: str
-        """
+        """returns: str."""
 
     def __exit__(self, _exception_type, _exception_value, _traceback):
         pass
@@ -68,13 +76,13 @@ class DagsterDaemon(AbstractContextManager, ABC, Generic[TContext]):
         workspace_process_context: TContext,
         daemon_uuid: str,
         daemon_shutdown_event: Event,
-        heartbeat_interval_seconds: int,
+        heartbeat_interval_seconds: float,
         error_interval_seconds: int,
     ):
         from dagster._core.telemetry_upload import uploading_logging_thread
 
         with uploading_logging_thread():
-            daemon_generator = self.core_loop(workspace_process_context)
+            daemon_generator = self.core_loop(workspace_process_context, daemon_shutdown_event)
 
             try:
                 while not daemon_shutdown_event.is_set():
@@ -84,7 +92,8 @@ class DagsterDaemon(AbstractContextManager, ABC, Generic[TContext]):
                             self._errors.appendleft((result, pendulum.now("UTC")))
                     except StopIteration:
                         self._logger.error(
-                            "Daemon loop finished without raising an error - daemon loops should run forever until they are interrupted."
+                            "Daemon loop finished without raising an error - daemon loops should"
+                            " run forever until they are interrupted."
                         )
                         break
                     except Exception:
@@ -94,7 +103,9 @@ class DagsterDaemon(AbstractContextManager, ABC, Generic[TContext]):
                         )
                         self._errors.appendleft((error_info, pendulum.now("UTC")))
                         daemon_generator.close()
-                        daemon_generator = self.core_loop(workspace_process_context)
+                        daemon_generator = self.core_loop(
+                            workspace_process_context, daemon_shutdown_event
+                        )
                     finally:
                         try:
                             self._check_add_heartbeat(
@@ -115,10 +126,10 @@ class DagsterDaemon(AbstractContextManager, ABC, Generic[TContext]):
     def _check_add_heartbeat(
         self,
         instance: DagsterInstance,
-        daemon_uuid,
-        heartbeat_interval_seconds,
-        error_interval_seconds,
-    ):
+        daemon_uuid: str,
+        heartbeat_interval_seconds: float,
+        error_interval_seconds: int,
+    ) -> None:
         error_max_time = pendulum.now("UTC").subtract(seconds=error_interval_seconds)
 
         while len(self._errors):
@@ -148,10 +159,12 @@ class DagsterDaemon(AbstractContextManager, ABC, Generic[TContext]):
             and last_stored_heartbeat.daemon_id != daemon_uuid
         ):
             self._logger.error(
-                "Another %s daemon is still sending heartbeats. You likely have multiple "
-                "daemon processes running at once, which is not supported. "
-                "Last heartbeat daemon id: %s, "
-                "Current daemon_id: %s",
+                (
+                    "Another %s daemon is still sending heartbeats. You likely have multiple "
+                    "daemon processes running at once, which is not supported. "
+                    "Last heartbeat daemon id: %s, "
+                    "Current daemon_id: %s"
+                ),
                 daemon_type,
                 last_stored_heartbeat.daemon_id,
                 daemon_uuid,
@@ -182,10 +195,11 @@ class DagsterDaemon(AbstractContextManager, ABC, Generic[TContext]):
     def core_loop(
         self,
         workspace_process_context: TContext,
-    ) -> TDaemonGenerator:
-        """
-        Execute the daemon loop, which should be a generator function that never finishes.
-        Should periodically yield so that the controller can check for heartbeats. Yields can be either NoneType or a SerializableErrorInfo.
+        shutdown_event: Event,
+    ) -> DaemonIterator:
+        """Execute the daemon loop, which should be a generator function that never finishes.
+        Should periodically yield so that the controller can check for heartbeats. Yields can be
+        either NoneType or a SerializableErrorInfo.
 
         returns: generator (SerializableErrorInfo).
         """
@@ -199,34 +213,37 @@ class IntervalDaemon(DagsterDaemon[TContext], ABC):
     def core_loop(
         self,
         workspace_process_context: TContext,
-    ) -> TDaemonGenerator:
+        shutdown_event: Event,
+    ) -> DaemonIterator:
         while True:
             start_time = time.time()
             try:
+                yield None  # Heartbeat once at the beginning to kick things off
                 yield from self.run_iteration(workspace_process_context)
             except Exception:
                 error_info = serializable_error_info_from_exc_info(sys.exc_info())
                 self._logger.error("Caught error:\n%s", error_info)
                 yield error_info
             while time.time() - start_time < self.interval_seconds:
+                shutdown_event.wait(0.5)
                 yield None
-                time.sleep(0.5)
             yield None
 
     @abstractmethod
-    def run_iteration(self, workspace_process_context: TContext) -> TDaemonGenerator:
+    def run_iteration(self, workspace_process_context: TContext) -> DaemonIterator:
         ...
 
 
 class SchedulerDaemon(DagsterDaemon):
     @classmethod
-    def daemon_type(cls):
+    def daemon_type(cls) -> str:
         return "SCHEDULER"
 
     def core_loop(
         self,
         workspace_process_context: IWorkspaceProcessContext,
-    ) -> TDaemonGenerator:
+        shutdown_event: Event,
+    ) -> DaemonIterator:
         scheduler = workspace_process_context.instance.scheduler
         if not isinstance(scheduler, DagsterDaemonScheduler):
             check.failed(f"Expected DagsterDaemonScheduler, got {scheduler}")
@@ -236,43 +253,46 @@ class SchedulerDaemon(DagsterDaemon):
             self._logger,
             scheduler.max_catchup_runs,
             scheduler.max_tick_retries,
+            shutdown_event,
         )
 
 
 class SensorDaemon(DagsterDaemon):
     @classmethod
-    def daemon_type(cls):
+    def daemon_type(cls) -> str:
         return "SENSOR"
 
     def core_loop(
         self,
         workspace_process_context: IWorkspaceProcessContext,
-    ) -> TDaemonGenerator:
+        shutdown_event: Event,
+    ) -> DaemonIterator:
         yield from execute_sensor_iteration_loop(
             workspace_process_context,
             self._logger,
+            shutdown_event,
         )
 
 
 class BackfillDaemon(IntervalDaemon):
     @classmethod
-    def daemon_type(cls):
+    def daemon_type(cls) -> str:
         return "BACKFILL"
 
     def run_iteration(
         self,
         workspace_process_context: IWorkspaceProcessContext,
-    ) -> TDaemonGenerator:
+    ) -> DaemonIterator:
         yield from execute_backfill_iteration(workspace_process_context, self._logger)
 
 
 class MonitoringDaemon(IntervalDaemon):
     @classmethod
-    def daemon_type(cls):
+    def daemon_type(cls) -> str:
         return "MONITORING"
 
     def run_iteration(
         self,
         workspace_process_context: IWorkspaceProcessContext,
-    ) -> TDaemonGenerator:
+    ) -> DaemonIterator:
         yield from execute_monitoring_iteration(workspace_process_context, self._logger)
