@@ -1,6 +1,7 @@
 import logging
 import sys
 import time
+from typing import Iterator, Optional
 
 from dagster import (
     DagsterInstance,
@@ -15,12 +16,15 @@ from dagster._core.storage.pipeline_run import (
     RunsFilter,
 )
 from dagster._core.workspace.context import IWorkspace, IWorkspaceProcessContext
-from dagster._utils.error import serializable_error_info_from_exc_info
+from dagster._utils import DebugCrashFlags
+from dagster._utils.error import SerializableErrorInfo, serializable_error_info_from_exc_info
 
 RESUME_RUN_LOG_MESSAGE = "Launching a new run worker to resume run"
 
 
-def monitor_starting_run(instance: DagsterInstance, run, logger):
+def monitor_starting_run(
+    instance: DagsterInstance, run: DagsterRun, logger: logging.Logger
+) -> None:
     check.invariant(run.status == DagsterRunStatus.STARTING)
     run_stats = instance.get_run_stats(run.run_id)
 
@@ -29,17 +33,25 @@ def monitor_starting_run(instance: DagsterInstance, run, logger):
     )
     if time.time() - launch_time >= instance.run_monitoring_start_timeout_seconds:
         msg = (
-            f"Run {run.run_id} has been running for {time.time() - launch_time} seconds, which is"
-            f" longer than the timeout of {instance.run_monitoring_start_timeout_seconds} seconds"
-            " to start. Marking run failed"
+            "Run timed out due to taking longer than"
+            f" {instance.run_monitoring_start_timeout_seconds} seconds to start."
         )
+
+        debug_info = None
+        try:
+            debug_info = instance.run_launcher.get_run_worker_debug_info(run)
+        except Exception:
+            logger.exception("Failure fetching debug info for failed run worker")
+
+        if debug_info:
+            msg = msg + f"\n{debug_info}"
+
         logger.info(msg)
+
         instance.report_run_failed(run, msg)
 
-    # TODO: consider attempting to resume the run, if the run worker is in a bad status
 
-
-def count_resume_run_attempts(instance: DagsterInstance, run_id: str):
+def count_resume_run_attempts(instance: DagsterInstance, run_id: str) -> int:
     events = instance.all_logs(run_id, of_type=DagsterEventType.ENGINE_EVENT)
     return len([event for event in events if event.message == RESUME_RUN_LOG_MESSAGE])
 
@@ -49,7 +61,7 @@ def monitor_started_run(
     workspace: IWorkspace,
     run: DagsterRun,
     logger: logging.Logger,
-):
+) -> None:
     check.invariant(run.status == DagsterRunStatus.STARTED)
     check_health_result = instance.run_launcher.check_run_worker_health(run)
     if check_health_result.status not in [WorkerStatus.RUNNING, WorkerStatus.SUCCESS]:
@@ -77,16 +89,24 @@ def monitor_started_run(
                 attempt_number,
             )
         else:
+            debug_info = ""
+            try:
+                debug_info = instance.run_launcher.get_run_worker_debug_info(run)
+            except Exception:
+                logger.exception("Failure fetching debug info for failed run worker")
+
             if instance.run_launcher.supports_resume_run:
                 msg = (
                     f"Detected run worker status {check_health_result}. Marking run {run.run_id} as"
                     " failed, because it has surpassed the configured maximum attempts to resume"
                     f" the run: {instance.run_monitoring_max_resume_run_attempts}."
+                    + (f"\n{debug_info}" if debug_info else "")
                 )
             else:
                 msg = (
                     f"Detected run worker status {check_health_result}. Marking run {run.run_id} as"
                     " failed."
+                    + (f"\n{debug_info}" if debug_info else "")
                 )
             logger.info(msg)
             instance.report_run_failed(run, msg)
@@ -95,8 +115,8 @@ def monitor_started_run(
 def execute_monitoring_iteration(
     workspace_process_context: IWorkspaceProcessContext,
     logger: logging.Logger,
-    _debug_crash_flags=None,
-):
+    _debug_crash_flags: Optional[DebugCrashFlags] = None,
+) -> Iterator[Optional[SerializableErrorInfo]]:
     instance = workspace_process_context.instance
     check.invariant(
         instance.run_launcher.supports_check_run_worker_health, "Must use a supported run launcher"
