@@ -3,14 +3,14 @@ import threading
 import time
 from concurrent.futures import ThreadPoolExecutor, as_completed
 from contextlib import ExitStack
-from typing import Dict, Iterable, List, Optional, Sequence
+from typing import Dict, Iterable, Iterator, List, Optional, Sequence
 
 from dagster import (
     DagsterEvent,
     DagsterEventType,
     _check as check,
 )
-from dagster._core.errors import DagsterRepositoryLocationLoadError, DagsterUserCodeUnreachableError
+from dagster._core.errors import DagsterCodeLocationLoadError, DagsterUserCodeUnreachableError
 from dagster._core.events import EngineEventData
 from dagster._core.instance import DagsterInstance
 from dagster._core.launcher import LaunchRunContext
@@ -27,21 +27,19 @@ from dagster._core.storage.pipeline_run import (
 from dagster._core.storage.tags import PRIORITY_TAG
 from dagster._core.workspace.context import IWorkspaceProcessContext
 from dagster._core.workspace.workspace import IWorkspace
-from dagster._daemon.daemon import IntervalDaemon, TDaemonGenerator
-from dagster._utils import len_iter
+from dagster._daemon.daemon import DaemonIterator, IntervalDaemon
 from dagster._utils.error import serializable_error_info_from_exc_info
 from dagster._utils.tags import TagConcurrencyLimitsCounter
 
 
 class QueuedRunCoordinatorDaemon(IntervalDaemon):
-    """
-    Used with the QueuedRunCoordinator on the instance. This process finds queued runs from the run
+    """Used with the QueuedRunCoordinator on the instance. This process finds queued runs from the run
     store and launches them.
     """
 
-    def __init__(self, interval_seconds):
+    def __init__(self, interval_seconds) -> None:
         self._exit_stack = ExitStack()
-        self._executor = None
+        self._executor: Optional[ThreadPoolExecutor] = None
         self._location_timeouts_lock = threading.Lock()
         self._location_timeouts: Dict[str, float] = {}
         super().__init__(interval_seconds)
@@ -59,18 +57,18 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
 
     def __exit__(self, _exception_type, _exception_value, _traceback):
         self._executor = None
-        self._exit_stack.pop_all()
+        self._exit_stack.close()
         super().__exit__(_exception_type, _exception_value, _traceback)
 
     @classmethod
-    def daemon_type(cls):
+    def daemon_type(cls) -> str:
         return "QUEUED_RUN_COORDINATOR"
 
     def run_iteration(
         self,
         workspace_process_context: IWorkspaceProcessContext,
         fixed_iteration_time: Optional[float] = None,  # used for tests
-    ) -> TDaemonGenerator:
+    ) -> DaemonIterator:
         run_coordinator = workspace_process_context.instance.run_coordinator
         if not isinstance(run_coordinator, QueuedRunCoordinator):
             check.failed(f"Expected QueuedRunCoordinator, got {run_coordinator}")
@@ -96,7 +94,7 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
         runs_to_dequeue: List[DagsterRun],
         run_queue_config: RunQueueConfig,
         fixed_iteration_time: Optional[float],
-    ):
+    ) -> Iterator[None]:
         if run_coordinator.dequeue_use_threads:
             yield from self._dequeue_runs_iter_threaded(
                 workspace_process_context,
@@ -135,7 +133,7 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
         max_workers: Optional[int],
         run_queue_config: RunQueueConfig,
         fixed_iteration_time: Optional[float],
-    ):
+    ) -> Iterator[None]:
         num_dequeued_runs = 0
 
         for future in as_completed(
@@ -162,7 +160,7 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
         runs_to_dequeue: List[DagsterRun],
         run_queue_config: RunQueueConfig,
         fixed_iteration_time: Optional[float],
-    ):
+    ) -> Iterator[None]:
         num_dequeued_runs = 0
         workspace = workspace_process_context.create_request_context()
         for run in runs_to_dequeue:
@@ -195,13 +193,13 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
         in_progress_runs = self._get_in_progress_runs(instance)
 
         max_concurrent_runs_enabled = max_concurrent_runs != -1  # setting to -1 disables the limit
-        max_runs_to_launch = max_concurrent_runs - len_iter(in_progress_runs)
+        max_runs_to_launch = max_concurrent_runs - len(in_progress_runs)
         if max_concurrent_runs_enabled:
             # Possibly under 0 if runs were launched without queuing
             if max_runs_to_launch <= 0:
                 self._logger.info(
                     "{} runs are currently in progress. Maximum is {}, won't launch more.".format(
-                        len_iter(in_progress_runs), max_concurrent_runs
+                        len(in_progress_runs), max_concurrent_runs
                     )
                 )
                 return []
@@ -257,18 +255,18 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
 
         return batch
 
-    def _get_queued_runs(self, instance):
+    def _get_queued_runs(self, instance: DagsterInstance) -> Sequence[DagsterRun]:
         queued_runs_filter = RunsFilter(statuses=[DagsterRunStatus.QUEUED])
 
         # Reversed for fifo ordering
         runs = instance.get_runs(filters=queued_runs_filter)[::-1]
         return runs
 
-    def _get_in_progress_runs(self, instance: DagsterInstance) -> Iterable[DagsterRun]:
+    def _get_in_progress_runs(self, instance: DagsterInstance) -> Sequence[DagsterRun]:
         return instance.get_runs(filters=RunsFilter(statuses=IN_PROGRESS_RUN_STATUSES))
 
     def _priority_sort(self, runs: Iterable[DagsterRun]) -> Sequence[DagsterRun]:
-        def get_priority(run):
+        def get_priority(run: DagsterRun) -> int:
             priority_tag_value = run.tags.get(PRIORITY_TAG, "0")
             try:
                 return int(priority_tag_value)
@@ -278,7 +276,7 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
         # sorted is stable, so fifo is maintained
         return sorted(runs, key=get_priority, reverse=True)
 
-    def _is_location_pausing_dequeues(self, location_name, now):
+    def _is_location_pausing_dequeues(self, location_name: str, now: float) -> bool:
         with self._location_timeouts_lock:
             return (
                 location_name in self._location_timeouts
@@ -345,7 +343,7 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
                 )
                 return False
             elif run_queue_config.max_user_code_failure_retries and isinstance(
-                e, (DagsterUserCodeUnreachableError, DagsterRepositoryLocationLoadError)
+                e, (DagsterUserCodeUnreachableError, DagsterCodeLocationLoadError)
             ):
                 if location_name:
                     with self._location_timeouts_lock:
