@@ -1,4 +1,3 @@
-import datetime
 import logging
 import os
 import sys
@@ -7,9 +6,23 @@ import time
 from collections import defaultdict
 from concurrent.futures import Future, ThreadPoolExecutor
 from contextlib import ExitStack
-from typing import Dict, Generator, List, Mapping, NamedTuple, Optional, Sequence, Union
+from types import TracebackType
+from typing import (
+    TYPE_CHECKING,
+    Dict,
+    Iterator,
+    List,
+    Mapping,
+    NamedTuple,
+    Optional,
+    Sequence,
+    Type,
+    Union,
+    cast,
+)
 
 import pendulum
+from typing_extensions import Self, TypeAlias
 
 import dagster._check as check
 import dagster._seven as seven
@@ -18,9 +31,9 @@ from dagster._core.definitions.selector import PipelineSelector
 from dagster._core.definitions.sensor_definition import DefaultSensorStatus, SensorExecutionData
 from dagster._core.definitions.utils import validate_tags
 from dagster._core.errors import DagsterError
+from dagster._core.host_representation.code_location import CodeLocation
 from dagster._core.host_representation.external import ExternalPipeline, ExternalSensor
 from dagster._core.host_representation.external_data import ExternalTargetData
-from dagster._core.host_representation.repository_location import RepositoryLocation
 from dagster._core.instance import DagsterInstance
 from dagster._core.scheduler.instigation import (
     InstigatorState,
@@ -35,31 +48,30 @@ from dagster._core.storage.tags import RUN_KEY_TAG, SENSOR_NAME_TAG
 from dagster._core.telemetry import SENSOR_RUN_CREATED, hash_name, log_action
 from dagster._core.workspace.context import IWorkspaceProcessContext
 from dagster._scheduler.stale import resolve_stale_or_missing_assets
+from dagster._utils import DebugCrashFlags, SingleInstigatorDebugCrashFlags
 from dagster._utils.error import SerializableErrorInfo, serializable_error_info_from_exc_info
 from dagster._utils.merger import merge_dicts
 
+if TYPE_CHECKING:
+    from pendulum.datetime import DateTime
+
 MIN_INTERVAL_LOOP_TIME = 5
 
 FINISHED_TICK_STATES = [TickStatus.SKIPPED, TickStatus.SUCCESS, TickStatus.FAILURE]
 
-TDaemonGenerator = Generator[Union[None, SerializableErrorInfo], None, None]
+TDaemonGenerator: TypeAlias = Iterator[Optional[SerializableErrorInfo]]
 
 
 class DagsterSensorDaemonError(DagsterError):
     """Error when running the SensorDaemon."""
 
 
-class SkippedSensorRun(
-    NamedTuple(
-        "SkippedSensorRun",
-        [
-            ("run_key", Optional[str]),
-            ("existing_run", DagsterRun),
-        ],
-    )
-):
+class SkippedSensorRun(NamedTuple):
     """Placeholder for runs that are skipped during the run_key idempotence check."""
 
+    run_key: Optional[str]
+    existing_run: DagsterRun
+
 
 class SensorLaunchContext:
     def __init__(
@@ -82,21 +94,21 @@ class SensorLaunchContext:
             self._purge_settings[day_offset].add(status)
 
     @property
-    def status(self):
+    def status(self) -> TickStatus:
         return self._tick.status
 
     @property
-    def logger(self):
+    def logger(self) -> logging.Logger:
         return self._logger
 
     @property
-    def run_count(self):
+    def run_count(self) -> int:
         return len(self._tick.run_ids)
 
-    def update_state(self, status, **kwargs):
-        skip_reason = kwargs.get("skip_reason")
-        cursor = kwargs.get("cursor")
-        origin_run_id = kwargs.get("origin_run_id")
+    def update_state(self, status: TickStatus, **kwargs: object):
+        skip_reason = cast(Optional[str], kwargs.get("skip_reason"))
+        cursor = cast(Optional[str], kwargs.get("cursor"))
+        origin_run_id = cast(Optional[str], kwargs.get("origin_run_id"))
         if "skip_reason" in kwargs:
             del kwargs["skip_reason"]
 
@@ -120,16 +132,16 @@ class SensorLaunchContext:
         if origin_run_id:
             self._tick = self._tick.with_origin_run(origin_run_id)
 
-    def add_run_info(self, run_id=None, run_key=None):
+    def add_run_info(self, run_id: Optional[str] = None, run_key: Optional[str] = None) -> None:
         self._tick = self._tick.with_run_info(run_id, run_key)
 
-    def add_log_info(self, log_key):
+    def add_log_info(self, log_key: Sequence[str]) -> None:
         self._tick = self._tick.with_log_key(log_key)
 
-    def set_should_update_cursor_on_failure(self, should_update_cursor_on_failure: bool):
+    def set_should_update_cursor_on_failure(self, should_update_cursor_on_failure: bool) -> None:
         self._should_update_cursor_on_failure = should_update_cursor_on_failure
 
-    def _write(self):
+    def _write(self) -> None:
         self._instance.update_tick(self._tick)
 
         if self._tick.status not in FINISHED_TICK_STATES:
@@ -146,19 +158,19 @@ class SensorLaunchContext:
             state = self._instance.get_instigator_state(
                 self._external_sensor.get_external_origin_id(), self._external_sensor.selector_id
             )
-            last_run_key = state.instigator_data.last_run_key if state.instigator_data else None
+            last_run_key = state.instigator_data.last_run_key if state.instigator_data else None  # type: ignore  # (possible none)
             if self._tick.run_keys and should_update_cursor_and_last_run_key:
                 last_run_key = self._tick.run_keys[-1]
 
-            cursor = state.instigator_data.cursor if state.instigator_data else None
+            cursor = state.instigator_data.cursor if state.instigator_data else None  # type: ignore  # (possible none)
             if should_update_cursor_and_last_run_key:
                 cursor = self._tick.cursor
 
             marked_timestamp = max(
-                self._tick.timestamp, state.instigator_data.last_tick_start_timestamp or 0
+                self._tick.timestamp, state.instigator_data.last_tick_start_timestamp or 0  # type: ignore  # (possible none)
             )
             self._instance.update_instigator_state(
-                state.with_data(
+                state.with_data(  # type: ignore  # (possible none)
                     SensorInstigatorData(
                         last_tick_timestamp=self._tick.timestamp,
                         last_run_key=last_run_key,
@@ -169,10 +181,15 @@ class SensorLaunchContext:
                 )
             )
 
-    def __enter__(self):
+    def __enter__(self) -> Self:
         return self
 
-    def __exit__(self, exception_type, exception_value, traceback):
+    def __exit__(
+        self,
+        exception_type: Type[BaseException],
+        exception_value: Exception,
+        traceback: TracebackType,
+    ) -> None:
         if exception_type and isinstance(exception_value, KeyboardInterrupt):
             return
 
@@ -194,7 +211,9 @@ class SensorLaunchContext:
             )
 
 
-def _check_for_debug_crash(debug_crash_flags, key):
+def _check_for_debug_crash(
+    debug_crash_flags: Optional[SingleInstigatorDebugCrashFlags], key: str
+) -> None:
     if not debug_crash_flags:
         return
 
@@ -214,10 +233,9 @@ def execute_sensor_iteration_loop(
     workspace_process_context: IWorkspaceProcessContext,
     logger: logging.Logger,
     shutdown_event: threading.Event,
-    until=None,
+    until: Optional[float] = None,
 ) -> TDaemonGenerator:
-    """
-    Helper function that performs sensor evaluations on a tighter loop, while reusing grpc locations
+    """Helper function that performs sensor evaluations on a tighter loop, while reusing grpc locations
     within a given daemon interval.  Rather than relying on the daemon machinery to run the
     iteration loop every 30 seconds, sensors are continuously evaluated, every 5 seconds. We rely on
     each sensor definition's min_interval to check that sensor evaluations are spaced appropriately.
@@ -278,7 +296,7 @@ def execute_sensor_iteration(
     sensor_tick_futures: Optional[Dict[str, Future]] = None,
     sensor_state_lock: Optional[threading.Lock] = None,
     log_verbose_checks: bool = True,
-    debug_crash_flags=None,
+    debug_crash_flags: Optional[DebugCrashFlags] = None,
 ):
     instance = workspace_process_context.instance
 
@@ -301,9 +319,9 @@ def execute_sensor_iteration(
 
     sensors: Dict[str, ExternalSensor] = {}
     for location_entry in workspace_snapshot.values():
-        repo_location = location_entry.repository_location
-        if repo_location:
-            for repo in repo_location.get_repositories().values():
+        code_location = location_entry.code_location
+        if code_location:
+            for repo in code_location.get_repositories().values():
                 for sensor in repo.get_external_sensors():
                     selector_id = sensor.selector_id
                     if sensor.get_current_instigator_state(
@@ -325,26 +343,26 @@ def execute_sensor_iteration(
 
         for sensor_state in unloadable_sensor_states.values():
             sensor_name = sensor_state.origin.instigator_name
-            repo_location_origin = (
-                sensor_state.origin.external_repository_origin.repository_location_origin
+            code_location_origin = (
+                sensor_state.origin.external_repository_origin.code_location_origin
             )
 
-            repo_location_name = repo_location_origin.location_name
+            location_name = code_location_origin.location_name
             repo_name = sensor_state.origin.external_repository_origin.repository_name
             if (
-                repo_location_name not in workspace_snapshot
-                or not workspace_snapshot[repo_location_name].repository_location
+                location_name not in workspace_snapshot
+                or not workspace_snapshot[location_name].code_location
             ):
                 logger.warning(
                     f"Sensor {sensor_name} was started from a location "
-                    f"{repo_location_name} that can no longer be found in the workspace. "
+                    f"{location_name} that can no longer be found in the workspace. "
                     "You can turn off this sensor in the Dagit UI from the Status tab."
                 )
             elif not check.not_none(  # checked above
-                workspace_snapshot[repo_location_name].repository_location
+                workspace_snapshot[location_name].code_location
             ).has_repository(repo_name):
                 logger.warning(
-                    f"Could not find repository {repo_name} in location {repo_location_name} to "
+                    f"Could not find repository {repo_name} in location {location_name} to "
                     + f"run sensor {sensor_name}. If this repository no longer exists, you can "
                     + "turn off the sensor in the Dagit UI from the Status tab.",
                 )
@@ -423,7 +441,7 @@ def _process_tick(
     external_sensor: ExternalSensor,
     sensor_state: InstigatorState,
     sensor_state_lock: threading.Lock,
-    sensor_debug_crash_flags,
+    sensor_debug_crash_flags: Optional[SingleInstigatorDebugCrashFlags],
     tick_retention_settings,
 ):
     # evaluate the tick immediately, but from within a thread.  The main thread should be able to
@@ -447,7 +465,7 @@ def _process_tick_generator(
     external_sensor: ExternalSensor,
     sensor_state: InstigatorState,
     sensor_state_lock: threading.Lock,
-    sensor_debug_crash_flags,
+    sensor_debug_crash_flags: Optional[SingleInstigatorDebugCrashFlags],
     tick_retention_settings,
 ):
     instance = workspace_process_context.instance
@@ -513,8 +531,8 @@ def _mark_sensor_state_for_tick(
     instance: DagsterInstance,
     external_sensor: ExternalSensor,
     sensor_state: InstigatorState,
-    now: datetime.datetime,
-):
+    now: "DateTime",
+) -> None:
     instigator_data = _sensor_instigator_data(sensor_state)
     instance.update_instigator_state(
         sensor_state.with_data(
@@ -536,20 +554,20 @@ def _evaluate_sensor(
     context: SensorLaunchContext,
     external_sensor: ExternalSensor,
     state: InstigatorState,
-    sensor_debug_crash_flags=None,
+    sensor_debug_crash_flags: Optional[SingleInstigatorDebugCrashFlags] = None,
 ):
     instance = workspace_process_context.instance
     context.logger.info(f"Checking for new runs for sensor: {external_sensor.name}")
 
     sensor_origin = external_sensor.get_external_origin()
     repository_handle = external_sensor.handle.repository_handle
-    repo_location = workspace_process_context.create_request_context().get_repository_location(
-        sensor_origin.external_repository_origin.repository_location_origin.location_name
+    code_location = workspace_process_context.create_request_context().get_code_location(
+        sensor_origin.external_repository_origin.code_location_origin.location_name
     )
 
     instigator_data = _sensor_instigator_data(state)
 
-    sensor_runtime_data = repo_location.get_external_sensor_execution_data(
+    sensor_runtime_data = code_location.get_external_sensor_execution_data(
         instance,
         repository_handle,
         external_sensor.name,
@@ -627,33 +645,35 @@ def _evaluate_sensor(
         instance, external_sensor, sensor_runtime_data.run_requests
     )
 
-    for run_request in sensor_runtime_data.run_requests:
-        if run_request.stale_assets_only:
-            stale_assets = resolve_stale_or_missing_assets(workspace_process_context, run_request, external_sensor)  # type: ignore
+    for raw_run_request in sensor_runtime_data.run_requests:
+        if raw_run_request.stale_assets_only:
+            stale_assets = resolve_stale_or_missing_assets(workspace_process_context, raw_run_request, external_sensor)  # type: ignore
             # asset selection is empty set after filtering for stale
             if len(stale_assets) == 0:
                 continue
             else:
-                run_request = run_request.with_replaced_attrs(
+                run_request = raw_run_request.with_replaced_attrs(
                     asset_selection=stale_assets, stale_assets_only=False
                 )
+        else:
+            run_request = raw_run_request
 
         target_data: ExternalTargetData = check.not_none(
             external_sensor.get_target_data(run_request.job_name)
         )
 
         pipeline_selector = PipelineSelector(
-            location_name=repo_location.name,
+            location_name=code_location.name,
             repository_name=sensor_origin.external_repository_origin.repository_name,
             pipeline_name=target_data.pipeline_name,
             solid_selection=target_data.solid_selection,
             asset_selection=run_request.asset_selection,
         )
-        external_pipeline = repo_location.get_external_pipeline(pipeline_selector)
+        external_pipeline = code_location.get_external_pipeline(pipeline_selector)
         run = _get_or_create_sensor_run(
             context,
             instance,
-            repo_location,
+            code_location,
             external_sensor,
             external_pipeline,
             run_request,
@@ -671,9 +691,7 @@ def _evaluate_sensor(
 
         error_info = None
         try:
-            context.logger.info(
-                "Launching run for {sensor_name}".format(sensor_name=external_sensor.name)
-            )
+            context.logger.info(f"Launching run for {external_sensor.name}")
             instance.submit_run(run.run_id, workspace_process_context.create_request_context())
             context.logger.info(
                 "Completed launch of run {run_id} for {sensor_name}".format(
@@ -769,16 +787,16 @@ def _fetch_existing_runs(
 def _get_or_create_sensor_run(
     context: SensorLaunchContext,
     instance: DagsterInstance,
-    repo_location: RepositoryLocation,
+    code_location: CodeLocation,
     external_sensor: ExternalSensor,
     external_pipeline: ExternalPipeline,
     run_request: RunRequest,
     target_data: ExternalTargetData,
     existing_runs_by_key: Mapping[str, DagsterRun],
-):
+) -> Union[DagsterRun, SkippedSensorRun]:
     if not run_request.run_key:
         return _create_sensor_run(
-            instance, repo_location, external_sensor, external_pipeline, run_request, target_data
+            instance, code_location, external_sensor, external_pipeline, run_request, target_data
         )
 
     run = existing_runs_by_key.get(run_request.run_key)
@@ -798,21 +816,21 @@ def _get_or_create_sensor_run(
     context.logger.info(f"Creating new run for {external_sensor.name}")
 
     return _create_sensor_run(
-        instance, repo_location, external_sensor, external_pipeline, run_request, target_data
+        instance, code_location, external_sensor, external_pipeline, run_request, target_data
     )
 
 
 def _create_sensor_run(
     instance: DagsterInstance,
-    repo_location: RepositoryLocation,
+    code_location: CodeLocation,
     external_sensor: ExternalSensor,
     external_pipeline: ExternalPipeline,
     run_request: RunRequest,
     target_data: ExternalTargetData,
-):
+) -> DagsterRun:
     from dagster._daemon.daemon import get_telemetry_daemon_session_id
 
-    external_execution_plan = repo_location.get_external_execution_plan(
+    external_execution_plan = code_location.get_external_execution_plan(
         external_pipeline,
         run_request.run_config,
         target_data.mode,
@@ -837,7 +855,7 @@ def _create_sensor_run(
             "DAEMON_SESSION_ID": get_telemetry_daemon_session_id(),
             "SENSOR_NAME_HASH": hash_name(external_sensor.name),
             "pipeline_name_hash": hash_name(external_pipeline.name),
-            "repo_hash": hash_name(repo_location.name),
+            "repo_hash": hash_name(code_location.name),
         },
     )
 
