@@ -2,14 +2,21 @@
 
 import os
 import sys
-from contextlib import ExitStack
-from typing import Generator, Optional, Sequence, Union
+from contextlib import contextmanager
+from typing import Generator, Iterator, Optional, Sequence, Tuple, Union
 
 import pendulum
 
 import dagster._check as check
 from dagster._core.definitions import ScheduleEvaluationContext
 from dagster._core.definitions.events import AssetKey
+from dagster._core.definitions.job_definition import JobDefinition
+from dagster._core.definitions.multi_dimensional_partitions import MultiPartitionsDefinition
+from dagster._core.definitions.partition import (
+    DynamicPartitionsDefinition,
+    PartitionedConfig,
+    PartitionsDefinition,
+)
 from dagster._core.definitions.reconstruct import ReconstructablePipeline
 from dagster._core.definitions.repository_definition import RepositoryDefinition
 from dagster._core.definitions.sensor_definition import SensorEvaluationContext
@@ -34,6 +41,7 @@ from dagster._core.host_representation.external_data import (
     ExternalPipelineSubsetResult,
     ExternalScheduleExecutionErrorData,
     ExternalSensorExecutionErrorData,
+    job_name_for_external_partition_set_name,
 )
 from dagster._core.instance import DagsterInstance
 from dagster._core.instance.ref import InstanceRef
@@ -43,7 +51,7 @@ from dagster._core.snap.execution_plan_snapshot import (
 )
 from dagster._core.storage.pipeline_run import DagsterRun
 from dagster._grpc.types import ExecutionPlanSnapshotArgs
-from dagster._serdes import deserialize_as
+from dagster._serdes import deserialize_value
 from dagster._serdes.ipc import IPCErrorMessage
 from dagster._seven import nullcontext
 from dagster._utils import start_termination_thread
@@ -120,13 +128,12 @@ def core_execute_run(
         raise
 
     # Reload the run to verify that its status didn't change while the pipeline was loaded
-    dagster_run = check.not_none(
+    pipeline_run = check.not_none(
         instance.get_run_by_id(pipeline_run.run_id),
         f"Pipeline run with id '{pipeline_run.run_id}' was deleted after the run worker started.",
     )
 
     try:
-        dagster_run = check.not_none(instance.get_run_by_id(dagster_run.run_id))
         yield from execute_run_iterator(
             recon_pipeline, pipeline_run, instance, resume_from_failure=resume_from_failure
         )
@@ -150,6 +157,19 @@ def core_execute_run(
         raise
 
 
+@contextmanager
+def _instance_from_ref_for_dynamic_partitions(
+    instance_ref: Optional[InstanceRef], partitions_def: PartitionsDefinition
+) -> Iterator[Optional[DagsterInstance]]:
+    # Certain gRPC servers do not have access to the instance, so we only attempt to instantiate
+    # the instance when necessary for dynamic partitions: https://github.com/dagster-io/dagster/issues/12440
+
+    with DagsterInstance.from_ref(instance_ref) if (
+        instance_ref and (_partitions_def_contains_dynamic_partitions_def(partitions_def))
+    ) else nullcontext() as instance:
+        yield instance
+
+
 def _run_in_subprocess(
     serialized_execute_run_args,
     recon_pipeline,
@@ -159,14 +179,16 @@ def _run_in_subprocess(
 ):
     start_termination_thread(termination_event)
     try:
-        execute_run_args = deserialize_as(serialized_execute_run_args, ExecuteExternalPipelineArgs)
+        execute_run_args = deserialize_value(
+            serialized_execute_run_args, ExecuteExternalPipelineArgs
+        )
 
         with (
             DagsterInstance.from_ref(execute_run_args.instance_ref)
             if execute_run_args.instance_ref
             else nullcontext()
         ) as instance:
-            instance = check.not_none(instance)
+            instance = check.not_none(instance)  # noqa: PLW2901
             pipeline_run = instance.get_run_by_id(execute_run_args.pipeline_run_id)
 
             if not pipeline_run:
@@ -196,7 +218,7 @@ def _run_in_subprocess(
 
     run_event_handler(
         instance.report_engine_event(
-            "Started process for run (pid: {pid}).".format(pid=pid),
+            f"Started process for run (pid: {pid}).",
             pipeline_run,
             EngineEventData.in_process(pid),
         )
@@ -220,7 +242,7 @@ def _run_in_subprocess(
         if not closed:
             run_event_handler(
                 instance.report_engine_event(
-                    "Process for run exited (pid: {pid}).".format(pid=pid),
+                    f"Process for run exited (pid: {pid}).",
                     pipeline_run,
                 )
             )
@@ -253,12 +275,14 @@ def get_external_pipeline_subset_result(
             op_selection=solid_selection,
             asset_selection=frozenset(asset_selection) if asset_selection else None,
         )
+        external_pipeline_data = external_pipeline_data_from_def(definition)
+        return ExternalPipelineSubsetResult(
+            success=True, external_pipeline_data=external_pipeline_data
+        )
     except Exception:
         return ExternalPipelineSubsetResult(
             success=False, error=serializable_error_info_from_exc_info(sys.exc_info())
         )
-    external_pipeline_data = external_pipeline_data_from_def(definition)
-    return ExternalPipelineSubsetResult(success=True, external_pipeline_data=external_pipeline_data)
 
 
 def get_external_schedule_execution(
@@ -268,20 +292,34 @@ def get_external_schedule_execution(
     scheduled_execution_timestamp: Optional[float],
     scheduled_execution_timezone: Optional[str],
 ):
-    schedule_def = repo_def.get_schedule_def(schedule_name)
-    scheduled_execution_time = (
-        pendulum.from_timestamp(
-            scheduled_execution_timestamp,
-            tz=check.not_none(scheduled_execution_timezone),
-        )
-        if scheduled_execution_timestamp
-        else None
-    )
+    from dagster._core.execution.resources_init import get_transitive_required_resource_keys
 
-    with ScheduleEvaluationContext(
-        instance_ref, scheduled_execution_time, repo_def.name, schedule_name
-    ) as schedule_context:
-        try:
+    try:
+        schedule_def = repo_def.get_schedule_def(schedule_name)
+        scheduled_execution_time = (
+            pendulum.from_timestamp(
+                scheduled_execution_timestamp,
+                tz=check.not_none(scheduled_execution_timezone),
+            )
+            if scheduled_execution_timestamp
+            else None
+        )
+
+        required_resource_keys = get_transitive_required_resource_keys(
+            schedule_def.required_resource_keys, repo_def.get_top_level_resources()
+        )
+        resources_to_build = {
+            k: v
+            for k, v in repo_def.get_top_level_resources().items()
+            if k in required_resource_keys
+        }
+        with ScheduleEvaluationContext(
+            instance_ref,
+            scheduled_execution_time,
+            repo_def.name,
+            schedule_name,
+            resources=resources_to_build,
+        ) as schedule_context:
             with user_code_error_boundary(
                 ScheduleExecutionError,
                 lambda: "Error occurred during the execution function for schedule {schedule_name}".format(
@@ -289,10 +327,10 @@ def get_external_schedule_execution(
                 ),
             ):
                 return schedule_def.evaluate_tick(schedule_context)
-        except Exception:
-            return ExternalScheduleExecutionErrorData(
-                serializable_error_info_from_exc_info(sys.exc_info())
-            )
+    except Exception:
+        return ExternalScheduleExecutionErrorData(
+            serializable_error_info_from_exc_info(sys.exc_info())
+        )
 
 
 def get_external_sensor_execution(
@@ -303,22 +341,30 @@ def get_external_sensor_execution(
     last_run_key: Optional[str],
     cursor: Optional[str],
 ):
-    sensor_def = repo_def.get_sensor_def(sensor_name)
+    from dagster._core.execution.resources_init import get_transitive_required_resource_keys
 
-    with ExitStack() as stack:
-        sensor_context = stack.enter_context(
-            SensorEvaluationContext(
-                instance_ref,
-                last_completion_time=last_completion_timestamp,
-                last_run_key=last_run_key,
-                cursor=cursor,
-                repository_name=repo_def.name,
-                repository_def=repo_def,
-                sensor_name=sensor_name,
-            )
+    try:
+        sensor_def = repo_def.get_sensor_def(sensor_name)
+
+        required_resource_keys = get_transitive_required_resource_keys(
+            sensor_def.required_resource_keys, repo_def.get_top_level_resources()
         )
+        resources_to_build = {
+            k: v
+            for k, v in repo_def.get_top_level_resources().items()
+            if k in required_resource_keys
+        }
 
-        try:
+        with SensorEvaluationContext(
+            instance_ref,
+            last_completion_time=last_completion_timestamp,
+            last_run_key=last_run_key,
+            cursor=cursor,
+            repository_name=repo_def.name,
+            repository_def=repo_def,
+            sensor_name=sensor_name,
+            resources=resources_to_build,
+        ) as sensor_context:
             with user_code_error_boundary(
                 SensorExecutionError,
                 lambda: "Error occurred during the execution of evaluation_fn for sensor {sensor_name}".format(
@@ -326,55 +372,84 @@ def get_external_sensor_execution(
                 ),
             ):
                 return sensor_def.evaluate_tick(sensor_context)
-        except Exception:
-            return ExternalSensorExecutionErrorData(
-                serializable_error_info_from_exc_info(sys.exc_info())
-            )
+    except Exception:
+        return ExternalSensorExecutionErrorData(
+            serializable_error_info_from_exc_info(sys.exc_info())
+        )
+
+
+def _partitions_def_contains_dynamic_partitions_def(partitions_def: PartitionsDefinition) -> bool:
+    if isinstance(partitions_def, DynamicPartitionsDefinition):
+        return True
+    if isinstance(partitions_def, MultiPartitionsDefinition):
+        return any(
+            _partitions_def_contains_dynamic_partitions_def(dimension.partitions_def)
+            for dimension in partitions_def.partitions_defs
+        )
+    return False
+
+
+def _get_job_partitions_and_config_for_partition_set_name(
+    repo_def: RepositoryDefinition,
+    partition_set_name: str,
+) -> Tuple[JobDefinition, PartitionsDefinition, PartitionedConfig]:
+    job_name = job_name_for_external_partition_set_name(partition_set_name)
+    job_def = repo_def.get_job(job_name)
+    assert job_def.partitions_def and job_def.partitioned_config, (
+        f"Job {job_def.name} corresponding to external partition set {partition_set_name} does not"
+        " have a partitions_def"
+    )
+    return job_def, job_def.partitions_def, job_def.partitioned_config
 
 
 def get_partition_config(
     repo_def: RepositoryDefinition,
     partition_set_name: str,
-    partition_name: str,
-    instance: Optional[DagsterInstance] = None,
-):
-    partition_set_def = repo_def.get_partition_set_def(partition_set_name)
-    partition = partition_set_def.get_partition(partition_name, instance)
+    partition_key: str,
+    instance_ref: Optional[InstanceRef] = None,
+) -> Union[ExternalPartitionConfigData, ExternalPartitionExecutionErrorData]:
     try:
-        with user_code_error_boundary(
-            PartitionExecutionError,
-            lambda: "Error occurred during the evaluation of the `run_config_for_partition` function for partition set {partition_set_name}".format(
-                partition_set_name=partition_set_def.name
-            ),
-        ):
-            run_config = partition_set_def.run_config_for_partition(partition)
-            return ExternalPartitionConfigData(name=partition.name, run_config=run_config)
+        (
+            _,
+            partitions_def,
+            partitioned_config,
+        ) = _get_job_partitions_and_config_for_partition_set_name(repo_def, partition_set_name)
+
+        with _instance_from_ref_for_dynamic_partitions(instance_ref, partitions_def) as instance:
+            partition = partitions_def.get_partition(
+                partition_key, dynamic_partitions_store=instance
+            )
+
+            with user_code_error_boundary(
+                PartitionExecutionError,
+                lambda: f"Error occurred during the evaluation of the `run_config_for_partition` function for partition set {partition_set_name}",
+            ):
+                run_config = partitioned_config.get_run_config_for_partition_key(
+                    partition_key, dynamic_partitions_store=instance
+                )
+                return ExternalPartitionConfigData(name=partition.name, run_config=run_config)
     except Exception:
         return ExternalPartitionExecutionErrorData(
             serializable_error_info_from_exc_info(sys.exc_info())
         )
 
 
-def _get_target_for_partition_execution_error(partition_set_def):
-    if partition_set_def.job_name:
-        return f"partitioned config on job '{partition_set_def.job_name}'"
-    else:
-        return f"partition set '{partition_set_def.name}'"
-
-
 def get_partition_names(
     repo_def: RepositoryDefinition,
     partition_set_name: str,
-):
-    partition_set_def = repo_def.get_partition_set_def(partition_set_name)
+) -> Union[ExternalPartitionNamesData, ExternalPartitionExecutionErrorData]:
     try:
+        (
+            job_def,
+            partitions_def,
+            _,
+        ) = _get_job_partitions_and_config_for_partition_set_name(repo_def, partition_set_name)
+
         with user_code_error_boundary(
             PartitionExecutionError,
-            lambda: f"Error occurred during the execution of the partition generation function for {_get_target_for_partition_execution_error(partition_set_def)}",
+            lambda: f"Error occurred during the execution of the partition generation function for partitioned config on job '{job_def.name}'",
         ):
-            return ExternalPartitionNamesData(
-                partition_names=partition_set_def.get_partition_names()
-            )
+            return ExternalPartitionNamesData(partition_names=partitions_def.get_partition_keys())
     except Exception:
         return ExternalPartitionExecutionErrorData(
             serializable_error_info_from_exc_info(sys.exc_info())
@@ -385,18 +460,30 @@ def get_partition_tags(
     repo_def: RepositoryDefinition,
     partition_set_name: str,
     partition_name: str,
-    instance: Optional[DagsterInstance] = None,
+    instance_ref: Optional[InstanceRef] = None,
 ):
-    partition_set_def = repo_def.get_partition_set_def(partition_set_name)
-
-    partition = partition_set_def.get_partition(partition_name, dynamic_partitions_store=instance)
     try:
-        with user_code_error_boundary(
-            PartitionExecutionError,
-            lambda: f"Error occurred during the evaluation of the `tags_for_partition` function for {_get_target_for_partition_execution_error(partition_set_def)}",
-        ):
-            tags = partition_set_def.tags_for_partition(partition)
-            return ExternalPartitionTagsData(name=partition.name, tags=tags)
+        (
+            job_def,
+            partitions_def,
+            partitioned_config,
+        ) = _get_job_partitions_and_config_for_partition_set_name(repo_def, partition_set_name)
+
+        # Certain gRPC servers do not have access to the instance, so we only attempt to instantiate
+        # the instance when necessary for dynamic partitions: https://github.com/dagster-io/dagster/issues/12440
+
+        with _instance_from_ref_for_dynamic_partitions(instance_ref, partitions_def) as instance:
+            partition = partitions_def.get_partition(
+                partition_name, dynamic_partitions_store=instance
+            )
+            with user_code_error_boundary(
+                PartitionExecutionError,
+                lambda: f"Error occurred during the evaluation of the `tags_for_partition` function for partitioned config on job '{job_def.name}'",
+            ):
+                tags = partitioned_config.get_tags_for_partition_key(
+                    partition.name, job_name=job_def.name, dynamic_partitions_store=instance
+                )
+                return ExternalPartitionTagsData(name=partition.name, tags=tags)
     except Exception:
         return ExternalPartitionExecutionErrorData(
             serializable_error_info_from_exc_info(sys.exc_info())
@@ -434,45 +521,55 @@ def get_external_execution_plan_snapshot(
 
 
 def get_partition_set_execution_param_data(
-    repo_definition: RepositoryDefinition,
+    repo_def: RepositoryDefinition,
     partition_set_name: str,
     partition_names: Sequence[str],
-    instance: Optional[DagsterInstance] = None,
+    instance_ref: Optional[InstanceRef] = None,
 ) -> Union[ExternalPartitionSetExecutionParamData, ExternalPartitionExecutionErrorData]:
-    partition_set_def = repo_definition.get_partition_set_def(partition_set_name)
+    (
+        job_def,
+        partitions_def,
+        partitioned_config,
+    ) = _get_job_partitions_and_config_for_partition_set_name(repo_def, partition_set_name)
+
     try:
-        with user_code_error_boundary(
-            PartitionExecutionError,
-            lambda: f"Error occurred during the partition generation for {_get_target_for_partition_execution_error(partition_set_def)}",
-        ):
-            all_partitions = partition_set_def.get_partitions(dynamic_partitions_store=instance)
-        partitions = [
-            partition for partition in all_partitions if partition.name in partition_names
-        ]
-
-        partition_data = []
-        for partition in partitions:
-
-            def _error_message_fn(partition_name):
-                return (
-                    lambda: f"Error occurred during the partition config and tag generation for '{partition_name}' in {_get_target_for_partition_execution_error(partition_set_def)}"
-                )
-
+        with _instance_from_ref_for_dynamic_partitions(instance_ref, partitions_def) as instance:
             with user_code_error_boundary(
-                PartitionExecutionError, _error_message_fn(partition.name)
+                PartitionExecutionError,
+                lambda: f"Error occurred during the partition generation for partitioned config on job '{job_def.name}'",
             ):
-                run_config = partition_set_def.run_config_for_partition(partition)
-                tags = partition_set_def.tags_for_partition(partition)
+                all_partitions = partitions_def.get_partitions(dynamic_partitions_store=instance)
+                partitions = [
+                    partition for partition in all_partitions if partition.name in partition_names
+                ]
 
-            partition_data.append(
-                ExternalPartitionExecutionParamData(
-                    name=partition.name,
-                    tags=tags,
-                    run_config=run_config,
+            partition_data = []
+            for partition in partitions:
+
+                def _error_message_fn(partition_name: str):
+                    return (
+                        lambda: f"Error occurred during the partition config and tag generation for '{partition_name}' in partitioned config on job '{job_def.name}'"
+                    )
+
+                with user_code_error_boundary(
+                    PartitionExecutionError, _error_message_fn(partition.name)
+                ):
+                    run_config = partitioned_config.get_run_config_for_partition_key(
+                        partition.name, instance
+                    )
+                    tags = partitioned_config.get_tags_for_partition_key(
+                        partition.name, instance, job_name=job_def.name
+                    )
+
+                partition_data.append(
+                    ExternalPartitionExecutionParamData(
+                        name=partition.name,
+                        tags=tags,
+                        run_config=run_config,
+                    )
                 )
-            )
 
-        return ExternalPartitionSetExecutionParamData(partition_data=partition_data)
+            return ExternalPartitionSetExecutionParamData(partition_data=partition_data)
 
     except Exception:
         return ExternalPartitionExecutionErrorData(
