@@ -8,8 +8,8 @@ import sys
 import threading
 import time
 import uuid
+import warnings
 from concurrent.futures import ThreadPoolExecutor
-from contextlib import nullcontext
 from multiprocessing.synchronize import Event as MPEvent
 from subprocess import Popen
 from threading import Event as ThreadingEventType
@@ -36,11 +36,10 @@ from dagster._core.libraries import DagsterLibraryRegistry
 from dagster._core.origin import DEFAULT_DAGSTER_ENTRY_POINT, get_python_environment_entry_point
 from dagster._core.types.loadable_target_origin import LoadableTargetOrigin
 from dagster._core.workspace.autodiscovery import LoadableTarget
-from dagster._serdes import deserialize_as, serialize_dagster_namedtuple, whitelist_for_serdes
+from dagster._serdes import deserialize_value, serialize_value, whitelist_for_serdes
 from dagster._serdes.ipc import IPCErrorMessage, ipc_write_stream, open_ipc_subprocess
 from dagster._utils import (
     find_free_port,
-    frozenlist,
     get_run_crash_explanation,
     safe_tempfile_path_unmanaged,
 )
@@ -238,7 +237,7 @@ class DagsterApiServer(DagsterApiServicer):
         self._serializable_load_error = None
 
         self._entry_point = (
-            frozenlist(check.sequence_param(entry_point, "entry_point", of_type=str))
+            check.sequence_param(entry_point, "entry_point", of_type=str)
             if entry_point is not None
             else DEFAULT_DAGSTER_ENTRY_POINT
         )
@@ -382,7 +381,7 @@ class DagsterApiServer(DagsterApiServicer):
         return api_pb2.GetServerIdReply(server_id=self._server_id)  # type: ignore
 
     def ExecutionPlanSnapshot(self, request, _context):
-        execution_plan_args = deserialize_as(
+        execution_plan_args = deserialize_value(
             request.serialized_execution_plan_snapshot_args,
             ExecutionPlanSnapshotArgs,
         )
@@ -395,15 +394,13 @@ class DagsterApiServer(DagsterApiServicer):
             execution_plan_args,
         )
         return api_pb2.ExecutionPlanSnapshotReply(
-            serialized_execution_plan_snapshot=serialize_dagster_namedtuple(
-                execution_plan_snapshot_or_error
-            )
+            serialized_execution_plan_snapshot=serialize_value(execution_plan_snapshot_or_error)
         )
 
     def ListRepositories(self, request, _context) -> api_pb2.ListRepositoriesReply:  # type: ignore
         if self._serializable_load_error:
             return api_pb2.ListRepositoriesReply(  # type: ignore
-                serialized_list_repositories_response_or_error=serialize_dagster_namedtuple(
+                serialized_list_repositories_response_or_error=serialize_value(
                     self._serializable_load_error
                 )
             )
@@ -421,16 +418,16 @@ class DagsterApiServer(DagsterApiServicer):
         )
 
         return api_pb2.ListRepositoriesReply(  # type: ignore
-            serialized_list_repositories_response_or_error=serialize_dagster_namedtuple(response)
+            serialized_list_repositories_response_or_error=serialize_value(response)
         )
 
     def ExternalPartitionNames(self, request, _context) -> api_pb2.ExternalPartitionNamesReply:  # type: ignore
-        partition_names_args = deserialize_as(
+        partition_names_args = deserialize_value(
             request.serialized_partition_names_args,
             PartitionNamesArgs,
         )
         return api_pb2.ExternalPartitionNamesReply(  # type: ignore
-            serialized_external_partition_names_or_external_partition_execution_error=serialize_dagster_namedtuple(
+            serialized_external_partition_names_or_external_partition_execution_error=serialize_value(
                 get_partition_names(
                     self._get_repo_for_origin(partition_names_args.repository_origin),
                     partition_names_args.partition_set_name,
@@ -444,60 +441,57 @@ class DagsterApiServer(DagsterApiServicer):
         return api_pb2.ExternalNotebookDataReply(content=get_notebook_data(notebook_path))  # type: ignore
 
     def ExternalPartitionSetExecutionParams(self, request, _context):
-        args = deserialize_as(
+        args = deserialize_value(
             request.serialized_partition_set_execution_param_args,
             PartitionSetExecutionParamArgs,
         )
 
         instance_ref = args.instance_ref if args.instance_ref else self._instance_ref
 
-        with DagsterInstance.from_ref(instance_ref) if instance_ref else nullcontext() as instance:
-            serialized_data = serialize_dagster_namedtuple(
-                get_partition_set_execution_param_data(
-                    self._get_repo_for_origin(args.repository_origin),
-                    partition_set_name=args.partition_set_name,
-                    partition_names=args.partition_names,
-                    instance=instance,
-                )
+        serialized_data = serialize_value(
+            get_partition_set_execution_param_data(
+                self._get_repo_for_origin(args.repository_origin),
+                partition_set_name=args.partition_set_name,
+                partition_names=args.partition_names,
+                instance_ref=instance_ref,
             )
+        )
 
         yield from self._split_serialized_data_into_chunk_events(serialized_data)
 
     def ExternalPartitionConfig(self, request, _context):
-        args = deserialize_as(request.serialized_partition_args, PartitionArgs)
+        args = deserialize_value(request.serialized_partition_args, PartitionArgs)
 
         instance_ref = args.instance_ref if args.instance_ref else self._instance_ref
 
-        with DagsterInstance.from_ref(instance_ref) if instance_ref else nullcontext() as instance:
-            serialized_data = serialize_dagster_namedtuple(
-                get_partition_config(
-                    self._get_repo_for_origin(args.repository_origin),
-                    args.partition_set_name,
-                    args.partition_name,
-                    instance=instance,
-                )
+        serialized_data = serialize_value(
+            get_partition_config(
+                self._get_repo_for_origin(args.repository_origin),
+                args.partition_set_name,
+                args.partition_name,
+                instance_ref=instance_ref,
             )
+        )
 
         return api_pb2.ExternalPartitionConfigReply(
             serialized_external_partition_config_or_external_partition_execution_error=serialized_data
         )
 
     def ExternalPartitionTags(self, request, _context) -> api_pb2.ExternalPartitionTagsReply:  # type: ignore
-        partition_args = deserialize_as(request.serialized_partition_args, PartitionArgs)
+        partition_args = deserialize_value(request.serialized_partition_args, PartitionArgs)
 
         instance_ref = (
             partition_args.instance_ref if partition_args.instance_ref else self._instance_ref
         )
 
-        with DagsterInstance.from_ref(instance_ref) if instance_ref else nullcontext() as instance:
-            serialized_data = serialize_dagster_namedtuple(
-                get_partition_tags(
-                    self._get_repo_for_origin(partition_args.repository_origin),
-                    partition_args.partition_set_name,
-                    partition_args.partition_name,
-                    instance=instance,
-                )
+        serialized_data = serialize_value(
+            get_partition_tags(
+                self._get_repo_for_origin(partition_args.repository_origin),
+                partition_args.partition_set_name,
+                partition_args.partition_name,
+                instance_ref=instance_ref,
             )
+        )
 
         return api_pb2.ExternalPartitionTagsReply(  # type: ignore
             serialized_external_partition_tags_or_external_partition_execution_error=serialized_data
@@ -506,13 +500,13 @@ class DagsterApiServer(DagsterApiServicer):
     def ExternalPipelineSubsetSnapshot(
         self, request: Any, _context
     ) -> api_pb2.ExternalPipelineSubsetSnapshotReply:  # type: ignore
-        pipeline_subset_snapshot_args = deserialize_as(
+        pipeline_subset_snapshot_args = deserialize_value(
             request.serialized_pipeline_subset_snapshot_args,
             PipelineSubsetSnapshotArgs,
         )
 
         return api_pb2.ExternalPipelineSubsetSnapshotReply(  # type: ignore
-            serialized_external_pipeline_subset_result=serialize_dagster_namedtuple(
+            serialized_external_pipeline_subset_result=serialize_value(
                 get_external_pipeline_subset_result(
                     self._get_repo_for_origin(
                         pipeline_subset_snapshot_args.pipeline_origin.external_repository_origin
@@ -526,19 +520,19 @@ class DagsterApiServer(DagsterApiServicer):
 
     def _get_serialized_external_repository_data(self, request):
         try:
-            repository_origin = deserialize_as(
+            repository_origin = deserialize_value(
                 request.serialized_repository_python_origin,
                 ExternalRepositoryOrigin,
             )
 
-            return serialize_dagster_namedtuple(
+            return serialize_value(
                 external_repository_data_from_def(
                     self._get_repo_for_origin(repository_origin),
                     defer_snapshots=request.defer_snapshots,
                 )
             )
         except Exception:
-            return serialize_dagster_namedtuple(
+            return serialize_value(
                 ExternalRepositoryErrorData(serializable_error_info_from_exc_info(sys.exc_info()))
             )
 
@@ -550,17 +544,17 @@ class DagsterApiServer(DagsterApiServicer):
 
     def ExternalJob(self, request, _context) -> api_pb2.ExternalJobReply:  # type: ignore
         try:
-            repository_origin = deserialize_as(
+            repository_origin = deserialize_value(
                 request.serialized_repository_origin,
                 ExternalRepositoryOrigin,
             )
 
             job_def = self._get_repo_for_origin(repository_origin).get_pipeline(request.job_name)
-            ser_job_data = serialize_dagster_namedtuple(external_pipeline_data_from_def(job_def))
+            ser_job_data = serialize_value(external_pipeline_data_from_def(job_def))
             return api_pb2.ExternalJobReply(serialized_job_data=ser_job_data)  # type: ignore
         except Exception:
             return api_pb2.ExternalJobReply(  # type: ignore
-                serialized_error=serialize_dagster_namedtuple(
+                serialized_error=serialize_value(
                     serializable_error_info_from_exc_info(sys.exc_info())
                 )
             )
@@ -601,11 +595,11 @@ class DagsterApiServer(DagsterApiServicer):
             )
 
     def ExternalScheduleExecution(self, request, _context):
-        args = deserialize_as(
+        args = deserialize_value(
             request.serialized_external_schedule_execution_args,
             ExternalScheduleExecutionArgs,
         )
-        serialized_schedule_data = serialize_dagster_namedtuple(
+        serialized_schedule_data = serialize_value(
             get_external_schedule_execution(
                 self._get_repo_for_origin(args.repository_origin),
                 args.instance_ref,
@@ -618,12 +612,12 @@ class DagsterApiServer(DagsterApiServicer):
         yield from self._split_serialized_data_into_chunk_events(serialized_schedule_data)
 
     def ExternalSensorExecution(self, request, _context):
-        args = deserialize_as(
+        args = deserialize_value(
             request.serialized_external_sensor_execution_args,
             SensorExecutionArgs,
         )
 
-        serialized_sensor_data = serialize_dagster_namedtuple(
+        serialized_sensor_data = serialize_value(
             get_external_sensor_execution(
                 self._get_repo_for_origin(args.repository_origin),
                 args.instance_ref,
@@ -640,13 +634,13 @@ class DagsterApiServer(DagsterApiServicer):
         try:
             self._shutdown_once_executions_finish_event.set()
             return api_pb2.ShutdownServerReply(  # type: ignore
-                serialized_shutdown_server_result=serialize_dagster_namedtuple(
+                serialized_shutdown_server_result=serialize_value(
                     ShutdownServerResult(success=True, serializable_error_info=None)
                 )
             )
         except:
             return api_pb2.ShutdownServerReply(  # type: ignore
-                serialized_shutdown_server_result=serialize_dagster_namedtuple(
+                serialized_shutdown_server_result=serialize_value(
                     ShutdownServerResult(
                         success=False,
                         serializable_error_info=serializable_error_info_from_exc_info(
@@ -661,7 +655,7 @@ class DagsterApiServer(DagsterApiServicer):
         message = None
         serializable_error_info = None
         try:
-            cancel_execution_request = deserialize_as(
+            cancel_execution_request = deserialize_value(
                 request.serialized_cancel_execution_request,
                 CancelExecutionRequest,
             )
@@ -675,7 +669,7 @@ class DagsterApiServer(DagsterApiServicer):
             serializable_error_info = serializable_error_info_from_exc_info(sys.exc_info())
 
         return api_pb2.CancelExecutionReply(
-            serialized_cancel_execution_result=serialize_dagster_namedtuple(
+            serialized_cancel_execution_result=serialize_value(
                 CancelExecutionResult(
                     success=success,
                     message=message,
@@ -685,7 +679,7 @@ class DagsterApiServer(DagsterApiServicer):
         )
 
     def CanCancelExecution(self, request, _context) -> api_pb2.CanCancelExecutionReply:  # type: ignore
-        can_cancel_execution_request = deserialize_as(
+        can_cancel_execution_request = deserialize_value(
             request.serialized_can_cancel_execution_request,
             CanCancelExecutionRequest,
         )
@@ -696,7 +690,7 @@ class DagsterApiServer(DagsterApiServicer):
             )
 
         return api_pb2.CanCancelExecutionReply(  # type: ignore
-            serialized_can_cancel_execution_result=serialize_dagster_namedtuple(
+            serialized_can_cancel_execution_result=serialize_value(
                 CanCancelExecutionResult(can_cancel=can_cancel)
             )
         )
@@ -704,7 +698,7 @@ class DagsterApiServer(DagsterApiServicer):
     def StartRun(self, request, _context) -> api_pb2.StartRunReply:  # type: ignore
         if self._shutdown_once_executions_finish_event.is_set():
             return api_pb2.StartRunReply(  # type: ignore
-                serialized_start_run_result=serialize_dagster_namedtuple(
+                serialized_start_run_result=serialize_value(
                     StartRunResult(
                         success=False,
                         message="Tried to start a run on a server after telling it to shut down",
@@ -714,7 +708,7 @@ class DagsterApiServer(DagsterApiServicer):
             )
 
         try:
-            execute_external_pipeline_args = deserialize_as(
+            execute_external_pipeline_args = deserialize_value(
                 request.serialized_execute_run_args,
                 ExecuteExternalPipelineArgs,
             )
@@ -730,7 +724,7 @@ class DagsterApiServer(DagsterApiServicer):
 
         except:
             return api_pb2.StartRunReply(  # type: ignore
-                serialized_start_run_result=serialize_dagster_namedtuple(
+                serialized_start_run_result=serialize_value(
                     StartRunResult(
                         success=False,
                         message=None,
@@ -808,7 +802,7 @@ class DagsterApiServer(DagsterApiServicer):
                 self._clear_run(run_id)
 
         return api_pb2.StartRunReply(  # type: ignore
-            serialized_start_run_result=serialize_dagster_namedtuple(
+            serialized_start_run_result=serialize_value(
                 StartRunResult(
                     success=success,
                     message=message,
@@ -819,7 +813,7 @@ class DagsterApiServer(DagsterApiServicer):
 
     def GetCurrentImage(self, request, _context):
         return api_pb2.GetCurrentImageReply(
-            serialized_current_image=serialize_dagster_namedtuple(
+            serialized_current_image=serialize_value(
                 GetCurrentImageResult(
                     current_image=self._container_image, serializable_error_info=None
                 )
@@ -829,7 +823,7 @@ class DagsterApiServer(DagsterApiServicer):
     def GetCurrentRuns(self, request, context) -> api_pb2.GetCurrentRunsReply:  # type: ignore
         with self._execution_lock:
             return api_pb2.GetCurrentRunsReply(  # type: ignore
-                serialized_current_runs=serialize_dagster_namedtuple(
+                serialized_current_runs=serialize_value(
                     GetCurrentRunsResult(
                         current_runs=list(self._executions.keys()), serializable_error_info=None
                     )
@@ -1010,7 +1004,7 @@ class DagsterGrpcServer:
         self.server.start()
 
         # Note: currently this is hardcoded as serving, since both services are cohosted
-        # pylint: disable=no-member
+
         self._health_servicer.set("DagsterApi", health_pb2.HealthCheckResponse.SERVING)
 
         if self._ipc_output_file:
@@ -1038,11 +1032,7 @@ class CouldNotStartServerProcess(Exception):
     def __init__(self, port=None, socket=None):
         super(CouldNotStartServerProcess, self).__init__(
             "Could not start server with "
-            + (
-                "port {port}".format(port=port)
-                if port is not None
-                else "socket {socket}".format(socket=socket)
-            )
+            + (f"port {port}" if port is not None else f"socket {socket}")
         )
 
 
@@ -1098,24 +1088,24 @@ def open_server_process(
 
     executable_path = loadable_target_origin.executable_path if loadable_target_origin else None
 
-    subprocess_args = (
-        get_python_environment_entry_point(executable_path or sys.executable)
-        + ["api", "grpc"]
-        + ["--lazy-load-user-code"]
-        + (["--port", str(port)] if port else [])
-        + (["--socket", socket] if socket else [])
-        + (["-n", str(max_workers)] if max_workers else [])
-        + (["--heartbeat"] if heartbeat else [])
-        + (["--heartbeat-timeout", str(heartbeat_timeout)] if heartbeat_timeout else [])
-        + (["--fixed-server-id", fixed_server_id] if fixed_server_id else [])
-        + (["--override-system-timezone", mocked_system_timezone] if mocked_system_timezone else [])
-        + (["--log-level", log_level])
-        # only use the Python environment if it has been explicitly set in the workspace
-        + (["--use-python-environment-entry-point"] if executable_path else [])
-        + (["--inject-env-vars-from-instance"])
-        + (["--instance-ref", serialize_dagster_namedtuple(instance_ref)])
-        + (["--location-name", location_name] if location_name else [])
-    )
+    subprocess_args = [
+        *get_python_environment_entry_point(executable_path or sys.executable),
+        *["api", "grpc"],
+        *["--lazy-load-user-code"],
+        *(["--port", str(port)] if port else []),
+        *(["--socket", socket] if socket else []),
+        *(["-n", str(max_workers)] if max_workers else []),
+        *(["--heartbeat"] if heartbeat else []),
+        *(["--heartbeat-timeout", str(heartbeat_timeout)] if heartbeat_timeout else []),
+        *(["--fixed-server-id", fixed_server_id] if fixed_server_id else []),
+        *(["--override-system-timezone", mocked_system_timezone] if mocked_system_timezone else []),
+        *(["--log-level", log_level]),
+        # only use the Python environment if it has been explicitly set in the workspace,
+        *(["--use-python-environment-entry-point"] if executable_path else []),
+        *(["--inject-env-vars-from-instance"]),
+        *(["--instance-ref", serialize_value(instance_ref)]),
+        *(["--location-name", location_name] if location_name else []),
+    ]
 
     if loadable_target_origin:
         subprocess_args += loadable_target_origin.get_cli_args()
@@ -1199,6 +1189,7 @@ class GrpcServerProcess:
         cwd: Optional[str] = None,
         log_level: str = "INFO",
         env: Optional[Dict[str, str]] = None,
+        wait_on_exit=False,
     ):
         self.port = None
         self.socket = None
@@ -1261,17 +1252,54 @@ class GrpcServerProcess:
         else:
             self.server_process = server_process
 
+        self._wait_on_exit = wait_on_exit
+
+        self._waited = False
+        self._shutdown = False
+
     @property
     def pid(self):
         return self.server_process.pid
 
     def wait(self, timeout=30):
+        self._waited = True
         if self.server_process.poll() is None:
             seven.wait_for_process(self.server_process, timeout=timeout)
 
-    def create_ephemeral_client(self):
-        from dagster._grpc.client import EphemeralDagsterGrpcClient
+    def __enter__(self):
+        return self
 
-        return EphemeralDagsterGrpcClient(
-            port=self.port, socket=self.socket, server_process=self.server_process
-        )
+    def __exit__(self, _exception_type, _exception_value, _traceback):
+        self.shutdown_server()
+
+        if self._wait_on_exit:
+            self.wait()
+
+    def shutdown_server(self):
+        if not self._shutdown:
+            self._shutdown = True
+            if self.server_process.poll() is None:
+                try:
+                    self.create_client().shutdown_server()
+                except DagsterUserCodeUnreachableError:
+                    pass
+
+    def __del__(self):
+        if not self._shutdown:
+            warnings.warn(
+                "GrpcServerProcess is being destroyed without signalling to server that "
+                "it should shut down. This may result in server processes living longer than "
+                "they need to. To fix this, wrap the GrpcServerProcess in a contextmanager or "
+                "call shutdown_server on it."
+            )
+
+        if not self._waited and os.getenv("STRICT_GRPC_SERVER_PROCESS_WAIT"):
+            warnings.warn(
+                "GrpcServerProcess is being destroyed without waiting for the process to "
+                + "fully terminate. This can cause test instability."
+            )
+
+    def create_client(self):
+        from dagster._grpc.client import DagsterGrpcClient
+
+        return DagsterGrpcClient(port=self.port, socket=self.socket)
