@@ -1,11 +1,11 @@
 import hashlib
 from typing import NamedTuple
 
-from .serdes import serialize_dagster_namedtuple
+from .serdes import serialize_value
 
 
 def create_snapshot_id(snapshot: NamedTuple) -> str:
-    json_rep = serialize_dagster_namedtuple(snapshot)
+    json_rep = serialize_value(snapshot)
     return hash_str(json_rep)
 
 
@@ -17,4 +17,4 @@ def hash_str(in_str: str) -> str:
 
 def serialize_pp(value: NamedTuple) -> str:
     """Serialize and pretty print."""
-    return serialize_dagster_namedtuple(value, indent=2, separators=(",", ": "))
+    return serialize_value(value, indent=2, separators=(",", ": "))
