@@ -5,38 +5,28 @@ import pendulum
 
 _IS_PENDULUM_2 = (
     hasattr(pendulum, "__version__")
-    and getattr(packaging.version.parse(getattr(pendulum, "__version__")), "major")
-    == 2  # pylint: disable=no-member
+    and getattr(packaging.version.parse(getattr(pendulum, "__version__")), "major") == 2
 )
 
 
 @contextmanager
 def mock_pendulum_timezone(override_timezone):
     if _IS_PENDULUM_2:
-        with pendulum.tz.test_local_timezone(  # pylint: disable=no-member
-            pendulum.tz.timezone(override_timezone)  # pylint: disable=no-member
-        ):
+        with pendulum.tz.test_local_timezone(pendulum.tz.timezone(override_timezone)):
             yield
     else:
-        with pendulum.tz.LocalTimezone.test(  # pylint: disable=no-member
-            pendulum.Timezone.load(override_timezone)  # pylint: disable=no-member
-        ):
+        with pendulum.tz.LocalTimezone.test(pendulum.Timezone.load(override_timezone)):
             yield
 
 
 def create_pendulum_time(year, month, day, *args, **kwargs):
     return (
-        pendulum.datetime(  # pylint: disable=no-member, pendulum-create
-            year, month, day, *args, **kwargs
-        )
+        pendulum.datetime(year, month, day, *args, **kwargs)
         if _IS_PENDULUM_2
-        else pendulum.create(  # pylint: disable=no-member, pendulum-create
-            year, month, day, *args, **kwargs
-        )
+        else pendulum.create(year, month, day, *args, **kwargs)
     )
 
 
-# pylint: disable=no-member
 PendulumDateTime = (
     pendulum.DateTime if _IS_PENDULUM_2 else pendulum.Pendulum  # type: ignore[attr-defined]
 )
