@@ -4,7 +4,7 @@ import sys
 
 # From https://stackoverflow.com/questions/4716533/how-to-attach-debugger-to-a-python-subproccess
 class ForkedPdb(pdb.Pdb):
-    """A pdb subclass that may be used from a forked multiprocessing child
+    """A pdb subclass that may be used from a forked multiprocessing child.
 
     **Examples**:
 
