@@ -63,7 +63,7 @@ def create_test_pipeline_execution_context(
     )
     mode_def = ModeDefinition(logger_defs=loggers)
     pipeline_def = PipelineDefinition(
-        name="test_legacy_context", solid_defs=[], mode_defs=[mode_def]
+        name="test_legacy_context", node_defs=[], mode_defs=[mode_def]
     )
     run_config: Dict[str, Dict[str, Dict]] = {"loggers": {key: {} for key in loggers}}
     pipeline_run = DagsterRun(pipeline_name="test_legacy_context", run_config=run_config)
@@ -109,7 +109,7 @@ def build_pipeline_with_input_stubs(
     stub_solid_defs = []
 
     for solid_name, input_dict in inputs.items():
-        if not pipeline_def.has_solid_named(solid_name):
+        if not pipeline_def.has_node_named(solid_name):
             raise DagsterInvariantViolationError(
                 (
                     "You are injecting an input value for solid {solid_name} "
@@ -117,7 +117,7 @@ def build_pipeline_with_input_stubs(
                 ).format(solid_name=solid_name, pipeline_name=pipeline_def.name)
             )
 
-        solid = pipeline_def.solid_named(solid_name)
+        solid = pipeline_def.get_node_named(solid_name)
         for input_name, input_value in input_dict.items():
             stub_solid_def = define_stub_solid(
                 "__stub_{solid_name}_{input_name}".format(
@@ -130,7 +130,7 @@ def build_pipeline_with_input_stubs(
 
     return PipelineDefinition(
         name=pipeline_def.name + "_stubbed",
-        solid_defs=[*pipeline_def.top_level_solid_defs, *stub_solid_defs],
+        node_defs=[*pipeline_def.top_level_node_defs, *stub_solid_defs],
         mode_defs=pipeline_def.mode_definitions,
         dependencies=deps,  # type: ignore
     )
@@ -382,9 +382,9 @@ def execute_solid(
 
     result = execute_pipeline(
         PipelineDefinition(
-            name="ephemeral_{}_solid_pipeline".format(solid_def.name),
-            solid_defs=solid_defs,
-            dependencies=dependencies,  # type: ignore
+            name=f"ephemeral_{solid_def.name}_solid_pipeline",
+            node_defs=solid_defs,
+            dependencies=dependencies,
             mode_defs=[mode_def] if mode_def else None,
         ),
         run_config=run_config,
@@ -423,9 +423,9 @@ class FilesystemTestScheduler(Scheduler, ConfigurableClass):
     def config_type(cls):
         return {"base_dir": str}
 
-    @staticmethod
+    @classmethod
     def from_config_value(
-        inst_data: object, config_value: Mapping[str, object]
+        cls, inst_data: object, config_value: Mapping[str, object]
     ) -> "FilesystemTestScheduler":
         artifacts_dir = cast(str, config_value["base_dir"])
         return FilesystemTestScheduler(artifacts_dir=artifacts_dir, inst_data=inst_data)
