@@ -72,7 +72,7 @@ class TestPostgresInstance:
             TestPostgresInstance.dagster_postgres_installed(),
             "dagster_postgres must be installed to test with postgres",
         )
-        from dagster_postgres.utils import get_conn_string  # pylint: disable=import-error
+        from dagster_postgres.utils import get_conn_string
 
         return get_conn_string(
             **dict(
@@ -92,7 +92,7 @@ class TestPostgresInstance:
             TestPostgresInstance.dagster_postgres_installed(),
             "dagster_postgres must be installed to test with postgres",
         )
-        from dagster_postgres.run_storage import PostgresRunStorage  # pylint: disable=import-error
+        from dagster_postgres.run_storage import PostgresRunStorage
 
         storage = PostgresRunStorage.create_clean_storage(
             conn_string, should_autocreate_tables=should_autocreate_tables
@@ -106,7 +106,7 @@ class TestPostgresInstance:
             TestPostgresInstance.dagster_postgres_installed(),
             "dagster_postgres must be installed to test with postgres",
         )
-        from dagster_postgres.event_log import (  # pylint: disable=import-error
+        from dagster_postgres.event_log import (
             PostgresEventLogStorage,
         )
 
@@ -122,7 +122,7 @@ class TestPostgresInstance:
             TestPostgresInstance.dagster_postgres_installed(),
             "dagster_postgres must be installed to test with postgres",
         )
-        from dagster_postgres.schedule_storage.schedule_storage import (  # pylint: disable=import-error
+        from dagster_postgres.schedule_storage.schedule_storage import (
             PostgresScheduleStorage,
         )
 
@@ -148,7 +148,7 @@ class TestPostgresInstance:
         )
         conn_args = check.opt_dict_param(conn_args, "conn_args") if conn_args else {}
 
-        from dagster_postgres.utils import wait_for_connection  # pylint: disable=import-error
+        from dagster_postgres.utils import wait_for_connection
 
         if BUILDKITE:
             yield TestPostgresInstance.conn_string(
@@ -175,7 +175,7 @@ class TestPostgresInstance:
         except subprocess.CalledProcessError as ex:
             err_text = ex.output.decode("utf-8")
             raise PostgresDockerError(
-                "Failed to launch docker container(s) via docker-compose: {}".format(err_text),
+                f"Failed to launch docker container(s) via docker-compose: {err_text}",
                 ex,
             ) from ex
 
@@ -205,9 +205,9 @@ class TestPostgresInstance:
                 yield conn_str
         except PostgresDockerError as ex:
             warnings.warn(
-                "Error launching Dockerized Postgres: {}".format(ex), RuntimeWarning, stacklevel=3
+                f"Error launching Dockerized Postgres: {ex}", RuntimeWarning, stacklevel=3
             )
-            pytest.skip("Skipping due to error launching Dockerized Postgres: {}".format(ex))
+            pytest.skip(f"Skipping due to error launching Dockerized Postgres: {ex}")
 
 
 def is_postgres_running(service_name):
@@ -219,7 +219,7 @@ def is_postgres_running(service_name):
                 "container",
                 "ps",
                 "-f",
-                "name={}".format(service_name),
+                f"name={service_name}",
                 "-f",
                 "status=running",
             ],
