@@ -6,7 +6,7 @@ import pytest
 
 from dagster._core.host_representation import (
     ExternalRepositoryOrigin,
-    ManagedGrpcPythonEnvRepositoryLocationOrigin,
+    ManagedGrpcPythonEnvCodeLocationOrigin,
 )
 from dagster._core.scheduler.instigation import (
     InstigatorState,
@@ -22,8 +22,7 @@ from dagster._utils.error import SerializableErrorInfo
 
 
 class TestScheduleStorage:
-    """
-    You can extend this class to easily run these set of tests on any schedule storage. When extending,
+    """You can extend this class to easily run these set of tests on any schedule storage. When extending,
     you simply need to override the `schedule_storage` fixture and return your implementation of
     `ScheduleStorage`.
 
@@ -34,7 +33,7 @@ class TestScheduleStorage:
         __test__ = True
 
         @pytest.fixture(scope='function', name='storage')
-        def schedule_storage(self):  # pylint: disable=arguments-differ
+        def schedule_storage(self):
             return MyStorageImplementation()
     ```
     """
@@ -56,7 +55,7 @@ class TestScheduleStorage:
     @staticmethod
     def fake_repo_target():
         return ExternalRepositoryOrigin(
-            ManagedGrpcPythonEnvRepositoryLocationOrigin(
+            ManagedGrpcPythonEnvCodeLocationOrigin(
                 LoadableTargetOrigin(
                     executable_path=sys.executable, module_name="fake", attribute="fake"
                 ),
@@ -98,7 +97,7 @@ class TestScheduleStorage:
         schedule = schedules[0]
         assert schedule.instigator_name == "my_schedule"
         assert schedule.instigator_data.cron_schedule == "* * * * *"
-        assert schedule.instigator_data.start_timestamp == None
+        assert schedule.instigator_data.start_timestamp is None
 
     def test_add_multiple_schedules(self, storage):
         assert storage
@@ -130,7 +129,7 @@ class TestScheduleStorage:
         schedule = storage.get_instigator_state(state.instigator_origin_id, state.selector_id)
 
         assert schedule.instigator_name == "my_schedule"
-        assert schedule.instigator_data.start_timestamp == None
+        assert schedule.instigator_data.start_timestamp is None
 
     def test_get_schedule_state_not_found(self, storage):
         assert storage
@@ -184,7 +183,7 @@ class TestScheduleStorage:
         schedule = schedules[0]
         assert schedule.instigator_name == "my_schedule"
         assert schedule.status == InstigatorStatus.STOPPED
-        assert schedule.instigator_data.start_timestamp == None
+        assert schedule.instigator_data.start_timestamp is None
 
     def test_update_schedule_not_found(self, storage):
         assert storage
@@ -256,7 +255,7 @@ class TestScheduleStorage:
         assert tick.timestamp == current_time
         assert tick.status == TickStatus.STARTED
         assert tick.run_ids == []
-        assert tick.error == None
+        assert tick.error is None
 
     def test_update_tick_to_success(self, storage):
         assert storage
@@ -276,7 +275,7 @@ class TestScheduleStorage:
         assert tick.timestamp == current_time
         assert tick.status == TickStatus.SUCCESS
         assert tick.run_ids == ["1234"]
-        assert tick.error == None
+        assert tick.error is None
 
     def test_update_tick_to_skip(self, storage):
         assert storage
@@ -296,7 +295,7 @@ class TestScheduleStorage:
         assert tick.timestamp == current_time
         assert tick.status == TickStatus.SKIPPED
         assert tick.run_ids == []
-        assert tick.error == None
+        assert tick.error is None
 
     def test_update_tick_to_failure(self, storage):
         assert storage
@@ -472,7 +471,7 @@ class TestScheduleStorage:
         assert tick.timestamp == current_time
         assert tick.status == TickStatus.STARTED
         assert tick.run_ids == []
-        assert tick.error == None
+        assert tick.error is None
 
     def test_get_sensor_tick(self, storage):
         assert storage
@@ -515,7 +514,7 @@ class TestScheduleStorage:
         assert tick.timestamp == current_time
         assert tick.status == TickStatus.SUCCESS
         assert tick.run_ids == ["1234"]
-        assert tick.error == None
+        assert tick.error is None
 
     def test_update_sensor_tick_to_skip(self, storage):
         assert storage
@@ -536,7 +535,7 @@ class TestScheduleStorage:
         assert tick.timestamp == current_time
         assert tick.status == TickStatus.SKIPPED
         assert tick.run_ids == []
-        assert tick.error == None
+        assert tick.error is None
 
     def test_update_sensor_tick_to_failure(self, storage):
         assert storage
